/**
 *  Copyright (C) 2012  plegal  (plegal@appert44.org)
 *  @file         CAdrRadio.cpp
 *  @brief        Classe pour les adresses radio des composants WAVENIS
 *  @version      0.1
 *  @date         17 déc. 2012 21:27:53
 *
 *  Description detaillee du fichier CAdrRadio.cpp
 *  Fabrication   gcc (Ubuntu/Linaro 4.6.3-1ubuntu5) 4.6.3 
 *  @todo         Liste des choses restant a faire.
 *  @bug          17 déc. 2012 21:27:53 - Aucun pour l'instant
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Includes system C
#include <stdio.h>        // sprintf
// Includes system C++
// A décommenter si besoin cout, cin, ...
// #include <iostream>
// using namespace std;

// Includes qt

// Includes application
#include "CAdrRadio.h"
#include "CWNExceptions.h"

/// Espace de noms pour les classes de la librairie WAVENIS
namespace WAVENIS {

/**
 * Constructeurpar défaut d'une adresse radio pour les tableaux d'adresses.
 */
CAdrRadio::CAdrRadio()
{
	  _MR_GET(Taille);
	  m_Taille = SIZEADRRADIO;
	  this->strAdrRadio.assign("");
}

/// Constructeur d'une adresse radio à partir du tableau des octets de cette adresse
/// L'adresse radio est aussi stockée dans une chaîne accessible par la méthode ToString()
/// \arg Le format de la chaîne est : "aabb..xx"
/// \param AdrRadio : tableau des SIZEADRRADIO octets de l'adresse
/// Gestion des erreurs: format incorrect
/// \ see CARFMTException
CAdrRadio::CAdrRadio(Byte AdrRadio[SIZEADRRADIO]) {
  _MR_GET(Taille);
  m_Taille = SIZEADRRADIO;
  char Conv[80];
  this->strAdrRadio.assign("");
  for ( int i=0; i<Taille; i++ ) {
    this->bAdrRadio[i] = AdrRadio[i];     // copier l'octet
    sprintf(Conv,"%02x",AdrRadio[i]);     // format d'une sous-chaîne aa
    this->strAdrRadio.append(Conv);
  }
}

/// Constructeur d'une adresse radio à partir d'une chaîne.
/// \arg Le format de la chaîne est : "aabb..xx"
/// L'adresse chaîne est convertie en tableau d'octets.
/// \param AdrRadio : la chaîne contenant l'adresse radio
/// Gestion des exceptions
/// \par format incorrect : taille trop grande ou des caractères != [0..9] et [a(A)..f(F)].
CAdrRadio::CAdrRadio( string AdrRadio) try {
  _MR_GET(Taille);
  m_Taille = SIZEADRRADIO;
  // format 48 bits aabbccddeeff que des digits 0 à 9 et a(A)à f(F)
  int i;
  for ( i=0; i<(int)AdrRadio.size(); i++ )
	if ( !isxdigit(AdrRadio.c_str()[i]) )
      throw CARFMTException();
  if ( i != 12 )
    throw CARFMTException();
  this->strAdrRadio.assign(AdrRadio);
  // Pour les sous chaînes
  string sub;
  for ( int i=0; i<Taille; i++ ) {
    sub.assign(AdrRadio.substr((2*i),2));   // aa extrait de la sous chaîne 0xaa
    this->bAdrRadio[i] = 0;
    for ( int j=0; j<2; j++ ) {               // 2 caractères ascii par sous chaîne numérique
      Byte b = toupper(sub.c_str()[j]);       // passer en majuscule pour les digits de a à f
      this->bAdrRadio[i] += (b-(b<='9' && b>='0' ?'0':('A'-10)))<<(4*(1-j));  // conversion ASCII binaire de chaque digit hexa
    }
  }
}
catch ( CARFMTException& e ) {
  // Rien de spécial à faire ici. Pas de déallocation
}

/**
 * Destructeur
 */
CAdrRadio::~CAdrRadio()
{
}

// Methodes publiques
// ex :
/**
 * Exemple de description d'une methode
 *
 * @param un_parametre  Description d'un parametre
 * @return              Description du retour
 */
// ReturnType CAdrRadio::NomMethode(Type parametre)
// {
// }

/// Retrouver l'adresse radio sous la forme d'une chaîne facilement affichable
/// \arg Le format de la chaîne est : "aabb..xx"
/// \return L'adresse radio sous la forme d'une chaîne
string CAdrRadio::ToString() {
  return this->strAdrRadio;
}

/// Retrouver l'adresse radio sous la forme d'un tableau d'octets
/// \return Le tableau des octets de l'adresse radio
Byte* CAdrRadio::ToBytes() {
  return this->bAdrRadio;
}

/// propriété en lecture de la taille de l'adresse Radio
/// \return La taille de l'adresse Radio
int CAdrRadio::getTaille() const {
  return m_Taille;
}

// Methodes protegees

// Methodes privees

} // namespace WAVENIS

// Programme principal
// Si c'est la classe principale du projet, on place une fonction main()
// Dans ce cas, on peut supprimer les fichiers de tests unitaires
// ex :
// int main(int argc, char *argv[])
// {
//    // Construction de l'instance de la classe principale
//    cout << "Construction de l'instance de la classe principale CAdrRadio" << endl;
//    CAdrRadio *cadrradio = new CAdrRadio();
//    // Lancement de l'activité principale
//    return 0;
// }


