///////////////////////////////////////////////////////////////////////////////////////
// Projet       : classe CConfig
// Auteur       : MR
// Date         : Février 2010
// Version      : v1
// Description  : Classe pour la configuration matériel du dispositif
// Le format du fichier est :
//    clé=valeur soit
//    MAXRETRYREQ=3
//    MAXRETRYRES=3
//    MAXRETRYREC=3
//    MAXRETRYTO=3
/////////////////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <fstream>
using namespace std;

#include <string.h>
#include <stdlib.h>

//#include "SerialPort.h"
#include "CWNExceptions.h"
#include "CConfig.h"
#include "commun.h"

//using namespace RS232_v1;

namespace WAVENIS {

/// Constructeur
/// \param FichConfig : le nom complet du fichier de configuration
/// \arg Le format de ce fichier est de la forme clé=valeur
/// Les paramètres enregistrés sont :
/// MaxRetryREQ : le nombre de tentatives de ré émission d'une trame REQ
/// MaxRetryRES : le nombre de tentatives de réception d'un acquittement de haut niveau RES
/// MaxRetryREC : le nombre de tentatives de réception de la réponse d'un module distant RECEIVED
/// MaxRetryTO  : le nombre de tentatives de réception lors d'un dépassement du délai d'attente dans la réception d'une trame.
/// ReadTimeout : le délai d'attente dans la réception d'une trame exprimé en nombre de fois 0,1s (ex: 22 --> 2,2s)
CConfig::CConfig(string FichConfig) {
  _MR_CC_GET_SET(MaxRetryREQ);
  _MR_CC_GET_SET(MaxRetryRES);
  _MR_CC_GET_SET(MaxRetryREC);
  _MR_CC_GET_SET(MaxRetryTO);
  _MR_CC_GET_SET(ReadTimeout);
  this->FichConfig.assign(FichConfig);
  // Ouvrir le fichier de configuration
  this->fConfig.open(FichConfig.c_str());
  // Ouverture OK
  if ( !fConfig.fail()) {
    string Ligne;
    getline(fConfig,Ligne);
    this->MaxRetryREQ = atoi(Ligne.substr(strlen("MaxRetryREQ")+1,3).c_str());
    getline(fConfig,Ligne);
    this->MaxRetryRES = atoi(Ligne.substr(strlen("MaxRetryRES")+1,3).c_str());
    getline(fConfig,Ligne);
    this->MaxRetryREC = atoi(Ligne.substr(strlen("MaxRetryREC")+1,3).c_str());
    getline(fConfig,Ligne);
    this->MaxRetryTO = atoi(Ligne.substr(strlen("MaxRetryTO")+1,3).c_str());
    getline(fConfig,Ligne);
    this->ReadTimeout = atoi(Ligne.substr(strlen("ReadTimeout")+1,12).c_str());
  }
  else
    throw new CCFILEException();
}

CConfig::~CConfig() {
}
/// Retrouver le nombre maximum de tentatives en cas d'erreur lors de la réception d'une trame RECEIVED
/// \return Le nombre de tentatives
Byte CConfig::getMaxRetryREC() const {
  return m_MaxRetryREC;
}

/// Retrouver le nombre maximum de tentatives en cas d'erreurs lors de lémission d'une trame REQ
/// \return Le nombre de tentatives
Byte CConfig::getMaxRetryREQ() const {
  return m_MaxRetryREQ;
}

/// Retrouver le nombre maximum de tentatives en cas d'erreur lors de la réception d'une trame RES
/// \return Le nombre de tentatives
Byte CConfig::getMaxRetryRES() const {
  return m_MaxRetryRES;
}

/// Retrouver le nombre maximum de tentatives lors d'erreur de timeout dans la lecture d'une trame sur le port série.
/// \return Le nombre de tentatives
Byte CConfig::getMaxRetryTO() const {
  return m_MaxRetryTO;
}

int CConfig::getReadTimeout() const {
  return m_ReadTimeout;
}

/// Fixer le nombre maximum de tentatives en cas d'erreurs lors de la réception d'une trame RECEIVED
/// \param m_MaxRetryREC : Le nombre de tentatives autorisées
void CConfig::setMaxRetryREC(Byte m_MaxRetryREC) {
  this->m_MaxRetryREC = m_MaxRetryREC;
}

/// Fixer le nombre maximum de tentatives en cas d'erreur lors de l'émission d'une trame REQ
/// \param m_MaxRetryREQ : Le nombre de tentatives autorisées
void CConfig::setMaxRetryREQ(Byte m_MaxRetryREQ) {
  this->m_MaxRetryREQ = m_MaxRetryREQ;
}

/// Fixer le nombre maximum de tentatives en cas d'erreurs lors de la réception d'une trame RES
/// \param m_MaxRetryRES : Le nombre de tentatives autorisées
void CConfig::setMaxRetryRES(Byte m_MaxRetryRES) {
  this->m_MaxRetryRES = m_MaxRetryRES;
}

/// Fixer le nombre maximum de tentatives lors d'erreur de timeout dans la lecture d'une trame sur le port série.
/// \param m_MaxRetryTO : Le nombre de tentatives autorisées
void CConfig::setMaxRetryTO(Byte m_MaxRetryTO) {
  this->m_MaxRetryTO = m_MaxRetryTO;
}

void CConfig::setReadTimeout(int m_ReadTimeout) {
  this->m_ReadTimeout = m_ReadTimeout;
}


}
