///////////////////////////////////////////////////////////////////////////////////////
// Projet       : classe CConfig
// Auteur       : MR
// Date         : Février 2010
// Version      : v1
// Description  : Classe pour la configuration matériel du dispositif
// Le format du fichier est :
//    clé=valeur soit
//    MAXRETRYREQ=3
//    MAXRETRYRES=3
//    MAXRETRYREC=3
//    MAXRETRYTO=3
/////////////////////////////////////////////////////////////////////////////////////////

#ifndef CCONFIG_H_
#define CCONFIG_H_

#include <iostream>
#include <fstream>
using namespace std;

#include "Property.h"     // pour créer des properties pour les attributs
#include "commun.h"

//using namespace RS232_v1;

namespace WAVENIS {

/// macros pour l'initialisation des objets Property : get et set de la classe CAdrRadio
#define _MR_CC_GET_SET(a) \
  a.setContainer(this); \
  a.setter(&CConfig::set##a);\
  a.getter(&CConfig::get##a);

/// Classe d'encapsulation des paramètres de configuration des matériels WAVENIS.
/// La configuration est inscrite dans un fichier texte dans un format clé=Valeur
class CConfig {
public:
    CConfig(string FichConfig);
    virtual ~CConfig();
    Byte getMaxRetryREC() const;
    Byte getMaxRetryREQ() const;
    Byte getMaxRetryRES() const;
    Byte getMaxRetryTO() const;
    void setMaxRetryREC(Byte m_MaxRetryREC);
    void setMaxRetryREQ(Byte m_MaxRetryREQ);
    void setMaxRetryRES(Byte m_MaxRetryRES);
    void setMaxRetryTO(Byte m_MaxRetryTO);
    int getReadTimeout() const ;
    void setReadTimeout(int m_ReadTimeout);

  // implémentation d'une property pour m_MaxRetryREQ
  /// Obtient ou fixe le nombre de tentatives d'envoi d'une commande REQ avant l'abandon en cas d'erreurs.
  property<CConfig,Byte,READ_WRITE> MaxRetryREQ;
  // implémentation d'une property pour m_MaxRetryRES
  /// Obtient ou fixe le nombre de tentatives de réception d'un acquittement de haut niveau (trame REQ_) avant l'abandon en cas d'erreurs.
  property<CConfig,Byte,READ_WRITE> MaxRetryRES;
  // implémentation d'une property pour m_MaxRetryREQ
  /// Obtient ou fixe le nombre de tentatives de réception d'une trame de réponse (trame RECEIVED_) avant l'abandon en cas d'erreurs.
  property<CConfig,Byte,READ_WRITE> MaxRetryREC;
  // implémentation d'une property pour m_MaxRetryREQ.
  /// Obtient ou fixe le nombre de tentatives de lecture d'une trame avant l'abandon en cas de dépassement de timeout.
  property<CConfig,Byte,READ_WRITE> MaxRetryTO;
  // implémentation d'une property pour m_ReadTimeout.
  /// Obtient ou fixe le nombre de tentatives de lecture d'une trame avant l'abandon en cas de dépassement de timeout.
  property<CConfig,int,READ_WRITE> ReadTimeout;

  string FichConfig;

private:
  ifstream fConfig;

  Byte m_MaxRetryREQ;
  Byte m_MaxRetryRES;
  Byte m_MaxRetryREC;
  Byte m_MaxRetryTO;
  int m_ReadTimeout;
};

}
#endif /* CCONFIG_H_ */
