/**
 *  Copyright (C) 2012  plegal  (plegal@appert44.org)
 *  @file         CDallas.cpp
 *  @brief        Pilote pour les périphériques DALLAS dérivés de CWaveTherm
 *  @version      0.1
 *  @date         17 déc. 2012 21:07:28
 *
 *  Description detaillee du fichier CDallas.cpp
 *  Fabrication   gcc (Ubuntu/Linaro 4.6.3-1ubuntu5) 4.6.3 
 *  @todo         Liste des choses restant a faire.
 *  @bug          17 déc. 2012 21:07:28 - Aucun pour l'instant
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Includes system C
#include <string.h>
#include <math.h>
// Includes system C++
// A décommenter si besoin cout, cin, ...
// #include <iostream>
// using namespace std;

// Includes qt
// serial driver api include
//#include <serial_driver_event.h>
//#include <serial_driver_status.h>
//// protocol api include
//#include <protocol_command_types.h>
//#include <protocol_response_types.h>
//// domain include
//#include "loader_serial_driver.h"
//#include "loader_protocol.h"
//#include "serial_driver_types.h"
#include "protocol_types.h"	// RadioAddress
//#include "protocol_types_utils.h"

// Includes application
#include "CDallas.h"

namespace WAVENIS {

/**
 * Constructeur
 */
CDallas::CDallas( CWavePort* pwp, CAdrRadio* AdresseRadio )
: CWaveTherm ( pwp, AdresseRadio ) {
  this->NTures = 48;          // 24 mesures de 16 bits
}

CDallas::CDallas( CWavePort* pwp, RadioAddress radioAddress )
: CWaveTherm ( pwp, radioAddress ) {
  this->NTures = 48;          // 24 mesures de 16 bits
}
/**
 * Destructeur
 */
CDallas::~CDallas()
{
}

// Methodes publiques
// ex :
/**
 * Exemple de description d'une methode
 *
 * @param un_parametre  Description d'un parametre
 * @return              Description du retour
 */
// ReturnType CDallas::NomMethode(Type parametre)
// {
// }

/// Retrouver la mesure brute de la température de la voie choisie
/// Un module wavetherm peut supporter 2 capteurs de températures
/// \param Voie : le numéro du capteur cible. Seules les valeurs 1 ou 2 sont possibles. Dans le cas contraire, une exception CWTVIException est levée.
/// \par Lors de la mesure, les deux voies étant lues simultanément, les 2 mesures sont systématiquement stockées.
/// \par Gestion des exceptions:
/// \arg CWTVIException : le numéro du capteur demandé est en dehors de la plage autorisée [1,2].
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \arg CWPCRException : erreur dans la commande Radio
/// \return la mesure sur 2 octets relevée sur le capteur numéro "Voie".
int CDallas::TemperatureBrute ( int Voie ) {
  if ( Voie > 2 || Voie <= 0 ) {
    throw CWTVIException();
  }
  cout << "value call" << endl;
  Value();
  return m_TemperatureBrute[Voie-1];
}

/// Retrouver la donnée brute du périphérique, c'est à dire la mesure sur 2 octets relevée sur chaque capteur interne, soit 4 octets.
/// \par Note :  en cas d'absence du capteur, la valeur brute est 0x4FFF
/// Un module wavetherm peut supporter 2 capteurs de température
/// \par Le format de la température mesurée est sur 16 bits :
/// \arg SSSSeeee eeeedddd avec :
/// -# SSSS : le signe --> 0 (>0) ou F (<0)
/// -# eeee.eeee la partie entière (8bits) de la température
/// -# dddd la partie décimale (4bits) en multiple de 1/16 ( 2 pow(-4) )
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \arg CWPCRException : erreur dans la commande Radio
/// \return Les 4 octets de mesures 0xAAAA 0xBBBB ordonnées en MSB LSB
int CDallas::Value ( ) {
  this->NbDonneesSpecifiques = 0;     // aucune donnée spécifique
  try {
	  cout << "CmdParamCtrl" << endl;
    this->CmdParamCtrl( (enum CModuleFinal::CMDAPP) REQ_CURRENT_TEMP );
  }
  catch (CWPCPCException& e ) {
    throw e;
  }
  catch ( CWPCNSException& e ){
    throw e;
  }
  catch ( CWPCRIException& e ){
    throw e;
  }
  catch ( CWPCRException& e ){
    throw e;
  }
  this->m_TemperatureBrute[VOIE1] = (PDUe[SIZEADRRADIO+1+1+1]<<8) + (PDUe[SIZEADRRADIO+1+1+1+1]);
  this->m_TemperatureBrute[VOIE2] = (PDUe[SIZEADRRADIO+1+1+1+2]<<8) + (PDUe[SIZEADRRADIO+1+1+1+1+2]);
  return (int)*(&PDUe[SIZEADRRADIO+1+1+1]);
}

/// Convertir la mesure brute en température en °C XX,yyyy
/// \par Le format de la température mesurée est sur 16 bits :
/// \arg SSSSeeee eeeedddd avec :
/// -# SSSS : le signe --> 0 (>0) ou F (<0)
/// -# eeee.eeee la partie entière (8bits) de la température
/// -# dddd la partie décimale (4bits) en multiple de 1/16 ( 2 pow(-4) )
/// \param TempBrute : la mesure brute à convertir en température réelle
/// \return La température réelle associée.
double CDallas::Temperature ( int TempBrute ) {
   double T;
   int Unite = (TempBrute>>4)&0xFF;
   Byte Dec = TempBrute & 0xF;
   if ( (TempBrute & 0xF000) == 0xF000 )
     Unite = -((0xFF - Unite + 1) & 0xFF);
   T = Unite + Dec*0.0625;
   return T;
}

/// Convertir une température réelle en mesure brute sur 2 octets
/// La température réelle est un flottant XX.yyyy
/// \par Le format de la température mesurée est sur 16 bits :
/// \arg SSSSeeee eeeedddd avec :
/// -# SSSS : le signe --> 0 (>0) ou F (<0)
/// -# eeee.eeee la partie entière (8bits) de la température
/// -# dddd la partie décimale (4bits) en multiple de 1/16 ( 2 pow(-4) )
/// \param TempReelle : la température réelle à convertir.
/// \return La valeur brute sur 16 bits de la température réelle.
int CDallas::TemperatureInt ( float TempReelle ) {
  // exemple TempReelle = 22,03
  int T=0;
  int EntTempReelle = floor(TempReelle);                          // 22
  int DecTempReelle = floor((TempReelle - EntTempReelle)*10000);  // 300
  int N=0;
  int i;
  for ( i=15; i!=0; i--) {
     N = floor(((i*10000)/16.0));
     if ( DecTempReelle >= N ) break;
  }
  T = (TempReelle > 0 ? 0:(0xF<<12)) + (((EntTempReelle & 0xFF) >> 4)<<8) + ((EntTempReelle & 0x0F)<<4) + i;
  return T;
}

/// Lire la table de détection des seuils en mode datalogging
/// \par Structure de la table. l'enregistrement pour un capteur est :
/// \arg Numéro du capteur (1 octet)
/// \arg Date de la détection (6 octet)
/// \arg Durée du dépassement (1 octet) : cette durée est exprimée en multiple de la période de mesure
/// \arg Valeur moyenne : valeur moyenne des mesures effectuée entre le début et la fin du dépassement. La taille de ce champ est fonction du format de la mesure. seul la classe dérivée en connaît la valeur.
/// \par La table de détection est une FIFO des 5 derniers enregistrements.
/// \par En sortie, la PDUe contient la table. La classe dérivée extraira les données car elle seule connaît la taille du champ "valeur moyenne".
/// \see DetectionSeuilsTable
/// \return l'adresse de la table de détection des seuils.
DETECTTABLE* CDallas::ReadDetectionSeuilsTable( void ) {
  CWaveTherm::ReadDetectionSeuilsTable();
  memset(DetectionSeuilsTable,0xFF,sizeof(DetectionSeuilsTable));
  // il y a max 5 enregistrements
  for ( int j=0; j<5; j++ ) {
    // copier le numéro du capteur
    this->DetectionSeuilsTable[j].NumCapteur = PDUe[SIZEADRRADIO+1+j*(1+6+2+2)];
    // copier la date
    for ( int i=0; i< SIZEDATE; i++ )
      this->DetectionSeuilsTable[j].Date[i] = PDUe[SIZEADRRADIO+1+1+(j*(1+6+2+2))+i];
    // copier la durée du dépassement
    for ( int i=0; i<2; i++ )
      this->DetectionSeuilsTable[j].Duree[i] = PDUe[SIZEADRRADIO+1+1+SIZEADRRADIO+(j*(1+6+2+2))+i];
    // copier la valeur moyenne des mesures pendant le dépassement
    for ( int i=0; i<2; i++ )
      this->DetectionSeuilsTable[j].Moyenne[i] = PDUe[SIZEADRRADIO+1+1+SIZEADRRADIO+2+(j*(1+6+2+2))+i];
  }
  return this->DetectionSeuilsTable;
}

/// Retrouver le seuil haut d'alarme du capteur 1.
/// \par Lorsque ce seuil est dépassé, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x1A0 soit 26°
/// \return Le seuil haut d'alarme.
int CDallas::getHighThreshold_Alarm_Sensor1() {
  _MR_MF_CMDREADPARAM(HIGHTHRESHOLD_ALARM_SENSOR1,SIZEREQ_HIGHTHRESHOLD_ALARM_SENSOR1);
  this->m_HighThreshold_Alarm_Sensor1[0] = this->PDUe[SIZEADRRADIO+1+1+1+1];
  this->m_HighThreshold_Alarm_Sensor1[1] = this->PDUe[SIZEADRRADIO+1+1+1+1+1];
  int Seuil = (this->m_HighThreshold_Alarm_Sensor1[0]<<8) + this->m_HighThreshold_Alarm_Sensor1[1];
  return Seuil;
}

/// Retrouver le seuil haut d'alarme du capteur 1.
/// \par Lorsque ce seuil est dépassé, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x1A0 soit 26°
/// \return Le seuil haut d'alarme.
int CDallas::getHighThreshold_Alarm_Sensor2() {
  _MR_MF_CMDREADPARAM(HIGHTHRESHOLD_ALARM_SENSOR2,SIZEREQ_HIGHTHRESHOLD_ALARM_SENSOR2);
  this->m_HighThreshold_Alarm_Sensor2[0] = this->PDUe[SIZEADRRADIO+1+1+1+1];
  this->m_HighThreshold_Alarm_Sensor2[1] = this->PDUe[SIZEADRRADIO+1+1+1+1+1];
  int Seuil = (this->m_HighThreshold_Alarm_Sensor2[0]<<8) + this->m_HighThreshold_Alarm_Sensor2[1];
  return Seuil;
}

/// Retrouver la durée du dépassement de seuil haut qui fixe l'émission d'une alarme pour le capteur 1.
/// \par Selon le mode de détrection cette durée joue un rôle différent :
/// \arg Mode successif : il y a alarme si le dépassement a eu lieu pendant une durée continue supérieure à l'Excess_Time fixé. Dans ce cas il faut que le dépassement ait duré de façon ininterrompue plus que l'excess_time.
/// \arg Mode cumulatif : il y a alarme si le dépassement a eu lieu pendant une durée cumulée supérieure à l'Excess_Time fixé. Dans ce cas c'est la somme des durées pendant lesquelles le dépassement a eu lieu qui est prise en compte par le module.
/// Cette durée est exprimé en multiple de la période de mesure de détection des seuils (measurement_period_detection_time) indépendante de la période de mesure en mode datalogging. Cependant pour des porblèmes de consommation, il est souhaitable que la période de mesuer en mode datalogging soit un multiple de la période de mesure de détection des seuils.
/// \par Par défaut la durée est de 4 période de mesures.
/// \return la durée du dépassement
/// \see getMeasurement_period_threshold_detection
Byte CDallas::getHighThreshold_Excess_Time_Sensor1() {
  _MR_MF_CMDREADPARAM(HIGHTHRESHOLD_EXCESS_TIME_SENSOR1,SIZEREQ_HIGHTHRESHOLD_EXCESS_TIME_SENSOR1);
  m_HighThreshold_Excess_Time_Sensor1 = this->PDUe[SIZEADRRADIO+1+1+1+1];
  return m_HighThreshold_Excess_Time_Sensor1;
}

/// Retrouver la durée du dépassement de seuil haut qui fixe l'émission d'une alarme pour le capteur 2.
/// \par Selon le mode de détection cette durée joue un rôle différent :
/// \arg Mode successif : il y a alarme si le dépassement a eu lieu pendant une durée continue supérieure à l'Excess_Time fixé. Dans ce cas il faut que le dépassement ait duré de façon ininterrompue plus que l'excess_time.
/// \arg Mode cumulatif : il y a alarme si le dépassement a eu lieu pendant une durée cumulée supérieure à l'Excess_Time fixé. Dans ce cas c'est la somme des durées pendant lesquelles le dépassement a eu lieu qui est prise en compte par le module.
/// Cette durée est exprimé en multiple de la période de mesure de détection des seuils (measurement_period_detection_time) indépendante de la période de mesure en mode datalogging. Cependant pour des porblèmes de consommation, il est souhaitable que la période de mesuer en mode datalogging soit un multiple de la période de mesure de détection des seuils.
/// \par Par défaut la durée est de 4 période de mesures.
/// \return la durée du dépassement
/// \see getMeasurement_period_threshold_detection
Byte CDallas::getHighThreshold_Excess_Time_Sensor2() {
  _MR_MF_CMDREADPARAM(HIGHTHRESHOLD_EXCESS_TIME_SENSOR2,SIZEREQ_HIGHTHRESHOLD_EXCESS_TIME_SENSOR2);
  m_HighThreshold_Excess_Time_Sensor2 = this->PDUe[SIZEADRRADIO+1+1+1+1];
  return m_HighThreshold_Excess_Time_Sensor2;
}

/// Retrouver le seuil bas d'alarme du capteur 1.
/// \par Lorsque la mesure descend au dessous de ce seuil, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x100 soit 16°
/// \return Le seuil haut d'alarme.
int CDallas::getLowThreshold_Alarm_Sensor1() {
  _MR_MF_CMDREADPARAM(LOWTHRESHOLD_ALARM_SENSOR1,SIZEREQ_LOWTHRESHOLD_ALARM_SENSOR1);
  this->m_LowThreshold_Alarm_Sensor1[0] = this->PDUe[SIZEADRRADIO+1+1+1+1];
  this->m_LowThreshold_Alarm_Sensor1[1] = this->PDUe[SIZEADRRADIO+1+1+1+1+1];
  int Seuil = (this->m_LowThreshold_Alarm_Sensor1[0]<<8) + this->m_LowThreshold_Alarm_Sensor1[1];
  return Seuil;
}

/// Retrouver le seuil bas d'alarme du capteur 2.
/// \par Lorsque la mesure descend au dessous de ce seuil, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x100 soit 16°
/// \return Le seuil haut d'alarme.
int CDallas::getLowThreshold_Alarm_Sensor2() {
  _MR_MF_CMDREADPARAM(LOWTHRESHOLD_ALARM_SENSOR2,SIZEREQ_LOWTHRESHOLD_ALARM_SENSOR2);
  this->m_LowThreshold_Alarm_Sensor2[0] = this->PDUe[SIZEADRRADIO+1+1+1+1];
  this->m_LowThreshold_Alarm_Sensor2[1] = this->PDUe[SIZEADRRADIO+1+1+1+1+1];
  int Seuil = (this->m_LowThreshold_Alarm_Sensor2[0]<<8) + this->m_LowThreshold_Alarm_Sensor2[1];
  return Seuil;
}

/// Retrouver la durée du dépassement de seuil bas qui fixe l'émission d'une alarme pour le capteur 1.
/// \par Selon le mode de détection cette durée joue un rôle différent :
/// \arg Mode successif : il y a alarme si le dépassement a eu lieu pendant une durée continue supérieure à l'Excess_Time fixé. Dans ce cas il faut que le dépassement ait duré de façon ininterrompue plus que l'excess_time.
/// \arg Mode cumulatif : il y a alarme si le dépassement a eu lieu pendant une durée cumulée supérieure à l'Excess_Time fixé. Dans ce cas c'est la somme des durées pendant lesquelles le dépassement a eu lieu qui est prise en compte par le module.
/// Cette durée est exprimé en multiple de la période de mesure de détection des seuils (measurement_period_detection_time) indépendante de la période de mesure en mode datalogging. Cependant pour des porblèmes de consommation, il est souhaitable que la période de mesuer en mode datalogging soit un multiple de la période de mesure de détection des seuils.
/// \par Par défaut la durée est de 4 période de mesures.
/// \return la durée du dépassement
/// \see getMeasurement_period_threshold_detection
Byte CDallas::getLowThreshold_Excess_Time_Sensor1() {
  _MR_MF_CMDREADPARAM(LOWTHRESHOLD_EXCESS_TIME_SENSOR1,SIZEREQ_LOWTHRESHOLD_EXCESS_TIME_SENSOR1);
  m_LowThreshold_Excess_Time_Sensor1 = this->PDUe[SIZEADRRADIO+1+1+1+1];
  return m_LowThreshold_Excess_Time_Sensor1;
}

/// Retrouver la durée du dépassement de seuil bas qui fixe l'émission d'une alarme pour le capteur 2.
/// \par Selon le mode de détection cette durée joue un rôle différent :
/// \arg Mode successif : il y a alarme si le dépassement a eu lieu pendant une durée continue supérieure à l'Excess_Time fixé. Dans ce cas il faut que le dépassement ait duré de façon ininterrompue plus que l'excess_time.
/// \arg Mode cumulatif : il y a alarme si le dépassement a eu lieu pendant une durée cumulée supérieure à l'Excess_Time fixé. Dans ce cas c'est la somme des durées pendant lesquelles le dépassement a eu lieu qui est prise en compte par le module.
/// Cette durée est exprimé en multiple de la période de mesure de détection des seuils (measurement_period_detection_time) indépendante de la période de mesure en mode datalogging. Cependant pour des porblèmes de consommation, il est souhaitable que la période de mesuer en mode datalogging soit un multiple de la période de mesure de détection des seuils.
/// \par Par défaut la durée est de 4 période de mesures.
/// \return la durée du dépassement
/// \see getMeasurement_period_threshold_detection
Byte CDallas::getLowThreshold_Excess_Time_Sensor2() {
  _MR_MF_CMDREADPARAM(LOWTHRESHOLD_EXCESS_TIME_SENSOR2,SIZEREQ_LOWTHRESHOLD_EXCESS_TIME_SENSOR2);
  m_LowThreshold_Excess_Time_Sensor2 = this->PDUe[SIZEADRRADIO+1+1+1+1];
  return m_LowThreshold_Excess_Time_Sensor2;
}

/// Retrouver la période de mesure pendant laquelle il y a détection des dépassement de seuil.
/// \return La période exprimée en minutes.
Byte CDallas::getMeasurement_period_threshold_detection() {
  _MR_MF_CMDREADPARAM(MEASUREMENT_PERIOD_THRESHOLD_DETECTION,SIZEREQ_MEASUREMENT_PERIOD_THRESHOLD_DETECTION);
  m_Measurement_period_threshold_detection = this->PDUe[SIZEADRRADIO+1+1+1+1];
  return m_Measurement_period_threshold_detection;
}

int CDallas::getParameter_A_sensor1() {
  _MR_MF_CMDREADPARAM(PARAMETER_A_SENSOR1,SIZEREQ_PARAMETER_A_SENSOR1);
  this->m_Parameter_A_sensor1[0] = this->PDUe[SIZEADRRADIO+1+1+1+1];
  this->m_Parameter_A_sensor1[1] = this->PDUe[SIZEADRRADIO+1+1+1+1+1];
  int Param = (this->m_Parameter_A_sensor1[0]<<8) + this->m_Parameter_A_sensor1[1];
  return Param;
}

int CDallas::getParameter_A_sensor2() {
  _MR_MF_CMDREADPARAM(PARAMETER_A_SENSOR2,SIZEREQ_PARAMETER_A_SENSOR2);
  this->m_Parameter_A_sensor2[0] = this->PDUe[SIZEADRRADIO+1+1+1+1];
  this->m_Parameter_A_sensor2[1] = this->PDUe[SIZEADRRADIO+1+1+1+1+1];
  int Param = (this->m_Parameter_A_sensor2[0]<<8) + this->m_Parameter_A_sensor2[1];
  return Param;
}

int CDallas::getParameter_B_sensor1() {
  _MR_MF_CMDREADPARAM(PARAMETER_B_SENSOR1,SIZEREQ_PARAMETER_B_SENSOR1);
  this->m_Parameter_B_sensor1[0] = this->PDUe[SIZEADRRADIO+1+1+1+1];
  this->m_Parameter_B_sensor1[1] = this->PDUe[SIZEADRRADIO+1+1+1+1+1];
  int Param = (this->m_Parameter_B_sensor1[0]<<8) + this->m_Parameter_B_sensor1[1];
  return Param;
}

int CDallas::getParameter_B_sensor2() {
  _MR_MF_CMDREADPARAM(PARAMETER_B_SENSOR2,SIZEREQ_PARAMETER_B_SENSOR2);
  this->m_Parameter_B_sensor2[0] = this->PDUe[SIZEADRRADIO+1+1+1+1];
  this->m_Parameter_B_sensor2[1] = this->PDUe[SIZEADRRADIO+1+1+1+1+1];
  int Param = (this->m_Parameter_B_sensor2[0]<<8) + this->m_Parameter_B_sensor2[1];
  return Param;
}

/// Fixer le seuil haut d'alarme du capteur 1.
/// \par Lorsque ce seuil est dépassé, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x1A0 soit 26°
/// \param HighThreshold_Alarm_Sensor1 : le seuil haut d'alarme souhaité exprimé en mesure brute.
void CDallas::_setHighThreshold_Alarm_Sensor1(Byte HighThreshold_Alarm_Sensor1[2]) {
  for ( int i=0; i<2; i++ )
    this->m_HighThreshold_Alarm_Sensor1[i] = HighThreshold_Alarm_Sensor1[i];
  Byte Tmp[2+1];
  Tmp[0] = 2;
  for ( int i=0; i<2; i++ )
    Tmp[i+1] = HighThreshold_Alarm_Sensor1[i];
  _MR_WT_CMDWRITEPARAM(HIGHTHRESHOLD_ALARM_SENSOR1, SIZEREQ_HIGHTHRESHOLD_ALARM_SENSOR1, Tmp);
}

/// Fixer le seuil haut d'alarme du capteur 1.
/// \par Lorsque ce seuil est dépassé, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x1A0 soit 26°
/// \param HighThreshold_Alarm_Sensor1 : le seuil haut d'alarme souhaité exprimé en réel.
void CDallas::setHighThreshold_Alarm_Sensor1(float HighThreshold_Alarm_Sensor1) {
  int Temp = this->TemperatureInt(HighThreshold_Alarm_Sensor1);
  for ( int i=0; i<2; i++ )
    this->m_HighThreshold_Alarm_Sensor1[1-i] = (Byte) ((Temp >> (i*8)) & 0xFF);
  Byte Tmp[2+1];
  Tmp[0] = 2;
  for ( int i=0; i<2; i++ )
    Tmp[i+1] = this->m_HighThreshold_Alarm_Sensor1[i];
  _MR_WT_CMDWRITEPARAM(HIGHTHRESHOLD_ALARM_SENSOR1, SIZEREQ_HIGHTHRESHOLD_ALARM_SENSOR1, Tmp);
}

/// Fixer le seuil haut d'alarme du capteur 2.
/// \par Lorsque ce seuil est dépassé, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x1A0 soit 26°
/// \param HighThreshold_Alarm_Sensor2 : le seuil haut d'alarme souhaité.
void CDallas::_setHighThreshold_Alarm_Sensor2(Byte HighThreshold_Alarm_Sensor2[2]) {
  for ( int i=0; i<2; i++ )
    this->m_HighThreshold_Alarm_Sensor2[i] = HighThreshold_Alarm_Sensor2[i];
}

/// Fixer le seuil haut d'alarme du capteur 2.
/// \par Lorsque ce seuil est dépassé, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x1A0 soit 26°
/// \param HighThreshold_Alarm_Sensor2 : le seuil haut d'alarme souhaité exprimé en réel.
void CDallas::setHighThreshold_Alarm_Sensor2(float HighThreshold_Alarm_Sensor2) {
  int Temp = this->TemperatureInt(HighThreshold_Alarm_Sensor2);
  for ( int i=0; i<2; i++ )
    this->m_HighThreshold_Alarm_Sensor2[1-i] = (Byte) ((Temp >> (i*8)) & 0xFF);
  Byte Tmp[2+1];
  Tmp[0] = 2;
  for ( int i=0; i<2; i++ )
    Tmp[i+1] = this->m_HighThreshold_Alarm_Sensor2[i];
  _MR_WT_CMDWRITEPARAM(HIGHTHRESHOLD_ALARM_SENSOR2, SIZEREQ_HIGHTHRESHOLD_ALARM_SENSOR2, Tmp);
}

/// Fixer la durée du dépassement de seuil haut qui fixe l'émission d'une alarme pour le capteur 1.
/// \par Selon le mode de détrection cette durée joue un rôle différent :
/// \arg Mode successif : il y a alarme si le dépassement a eu lieu pendant une durée continue supérieure à l'Excess_Time fixé. Dans ce cas il faut que le dépassement ait duré de façon ininterrompue plus que l'excess_time.
/// \arg Mode cumulatif : il y a alarme si le dépassement a eu lieu pendant une durée cumulée supérieure à l'Excess_Time fixé. Dans ce cas c'est la somme des durées pendant lesquelles le dépassement a eu lieu qui est prise en compte par le module.
/// Cette durée est exprimé en multiple de la période de mesure de détection des seuils (measurement_period_detection_time) indépendante de la période de mesure en mode datalogging. Cependant pour des porblèmes de consommation, il est souhaitable que la période de mesuer en mode datalogging soit un multiple de la période de mesure de détection des seuils.
/// \par Par défaut la durée est de 4 période de mesures.
/// \param HighThreshold_Excess_Time_Sensor1 : la durée du dépassement autorisé avant alarme
/// \see getMeasurement_period_threshold_detection
void CDallas::setHighThreshold_Excess_Time_Sensor1(Byte HighThreshold_Excess_Time_Sensor1) {
    this->m_HighThreshold_Excess_Time_Sensor1 = HighThreshold_Excess_Time_Sensor1;
}

/// Fixer la durée du dépassement de seuil haut qui fixe l'émission d'une alarme pour le capteur 1.
/// \par Selon le mode de détrection cette durée joue un rôle différent :
/// \arg Mode successif : il y a alarme si le dépassement a eu lieu pendant une durée continue supérieure à l'Excess_Time fixé. Dans ce cas il faut que le dépassement ait duré de façon ininterrompue plus que l'excess_time.
/// \arg Mode cumulatif : il y a alarme si le dépassement a eu lieu pendant une durée cumulée supérieure à l'Excess_Time fixé. Dans ce cas c'est la somme des durées pendant lesquelles le dépassement a eu lieu qui est prise en compte par le module.
/// Cette durée est exprimé en multiple de la période de mesure de détection des seuils (measurement_period_detection_time) indépendante de la période de mesure en mode datalogging. Cependant pour des porblèmes de consommation, il est souhaitable que la période de mesuer en mode datalogging soit un multiple de la période de mesure de détection des seuils.
/// \par Par défaut la durée est de 4 période de mesures.
/// \param HighThreshold_Excess_Time_Sensor2 : la durée du dépassement autorisé avant alarme
/// \see getMeasurement_period_threshold_detection
void CDallas::setHighThreshold_Excess_Time_Sensor2(Byte HighThreshold_Excess_Time_Sensor2) {
    this->m_HighThreshold_Excess_Time_Sensor2 = HighThreshold_Excess_Time_Sensor2;
}

/// Fixer le seuil bas d'alarme du capteur 1.
/// \par Lorsque ce seuil est dépassé, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x1A0 soit 26°
/// \param LowThreshold_Alarm_Sensor1 : le seuil haut d'alarme souhaité.
void CDallas::_setLowThreshold_Alarm_Sensor1(Byte LowThreshold_Alarm_Sensor1[2]) {
  for ( int i=0; i<2; i++ )
    this->m_LowThreshold_Alarm_Sensor1[i] = LowThreshold_Alarm_Sensor1[i];
}

/// Fixer le seuil bas d'alarme du capteur 1.
/// \par Lorsque ce seuil est dépassé, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x1A0 soit 26°
/// \param LowThreshold_Alarm_Sensor1 : le seuil haut d'alarme souhaité.
void CDallas::setLowThreshold_Alarm_Sensor1(float LowThreshold_Alarm_Sensor1) {
  int Temp = this->TemperatureInt(LowThreshold_Alarm_Sensor1);
  for ( int i=0; i<2; i++ )
    this->m_LowThreshold_Alarm_Sensor1[1-i] = (Byte) ((Temp >> (i*8)) & 0xFF);
  Byte Tmp[2+1];
  Tmp[0] = 2;
  for ( int i=0; i<2; i++ )
    Tmp[i+1] = this->m_LowThreshold_Alarm_Sensor1[i];
  _MR_WT_CMDWRITEPARAM(LOWTHRESHOLD_ALARM_SENSOR1, SIZEREQ_LOWTHRESHOLD_ALARM_SENSOR1, Tmp);
}

/// Fixer le seuil bas d'alarme du capteur 2.
/// \par Lorsque ce seuil est dépassé, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x1A0 soit 26°
/// \param LowThreshold_Alarm_Sensor2 : le seuil haut d'alarme souhaité.
void CDallas::_setLowThreshold_Alarm_Sensor2(Byte LowThreshold_Alarm_Sensor2[2]) {
  for ( int i=0; i<2; i++ )
    this->m_LowThreshold_Alarm_Sensor2[i] = LowThreshold_Alarm_Sensor2[i];
}

/// Fixer le seuil bas d'alarme du capteur 1.
/// \par Lorsque ce seuil est dépassé, la date est enregistrée et une alarme est émise.
/// Par défaut ce seuil est fixé à 0x1A0 soit 26°
/// \param LowThreshold_Alarm_Sensor2 : le seuil haut d'alarme souhaité.
void CDallas::setLowThreshold_Alarm_Sensor2(float LowThreshold_Alarm_Sensor2) {
  int Temp = this->TemperatureInt(LowThreshold_Alarm_Sensor2);
  for ( int i=0; i<2; i++ )
    this->m_LowThreshold_Alarm_Sensor2[1-i] = (Byte) ((Temp >> (i*8)) & 0xFF);
  Byte Tmp[2+1];
  Tmp[0] = 2;
  for ( int i=0; i<2; i++ )
    Tmp[i+1] = this->m_LowThreshold_Alarm_Sensor2[i];
  _MR_WT_CMDWRITEPARAM(LOWTHRESHOLD_ALARM_SENSOR2, SIZEREQ_LOWTHRESHOLD_ALARM_SENSOR2, Tmp);
}

/// Fixer la durée du dépassement de seuil bas qui fixe l'émission d'une alarme pour le capteur 1.
/// \par Selon le mode de détrection cette durée joue un rôle différent :
/// \arg Mode successif : il y a alarme si le dépassement a eu lieu pendant une durée continue supérieure à l'Excess_Time fixé. Dans ce cas il faut que le dépassement ait duré de façon ininterrompue plus que l'excess_time.
/// \arg Mode cumulatif : il y a alarme si le dépassement a eu lieu pendant une durée cumulée supérieure à l'Excess_Time fixé. Dans ce cas c'est la somme des durées pendant lesquelles le dépassement a eu lieu qui est prise en compte par le module.
/// Cette durée est exprimé en multiple de la période de mesure de détection des seuils (measurement_period_detection_time) indépendante de la période de mesure en mode datalogging. Cependant pour des porblèmes de consommation, il est souhaitable que la période de mesuer en mode datalogging soit un multiple de la période de mesure de détection des seuils.
/// \par Par défaut la durée est de 4 période de mesures.
/// \param LowThreshold_Excess_Time_Sensor1 : la durée du dépassement autorisé avant alarme
/// \see getMeasurement_period_threshold_detection
void CDallas::setLowThreshold_Excess_Time_Sensor1(Byte LowThreshold_Excess_Time_Sensor1) {
    this->m_LowThreshold_Excess_Time_Sensor1 = LowThreshold_Excess_Time_Sensor1;
}

/// Fixer la durée du dépassement de seuil bas qui fixe l'émission d'une alarme pour le capteur 2.
/// \par Selon le mode de détrection cette durée joue un rôle différent :
/// \arg Mode successif : il y a alarme si le dépassement a eu lieu pendant une durée continue supérieure à l'Excess_Time fixé. Dans ce cas il faut que le dépassement ait duré de façon ininterrompue plus que l'excess_time.
/// \arg Mode cumulatif : il y a alarme si le dépassement a eu lieu pendant une durée cumulée supérieure à l'Excess_Time fixé. Dans ce cas c'est la somme des durées pendant lesquelles le dépassement a eu lieu qui est prise en compte par le module.
/// Cette durée est exprimé en multiple de la période de mesure de détection des seuils (measurement_period_detection_time) indépendante de la période de mesure en mode datalogging. Cependant pour des porblèmes de consommation, il est souhaitable que la période de mesuer en mode datalogging soit un multiple de la période de mesure de détection des seuils.
/// \par Par défaut la durée est de 4 période de mesures.
/// \param LowThreshold_Excess_Time_Sensor2 : la durée du dépassement autorisé avant alarme
/// \see getMeasurement_period_threshold_detection
void CDallas::setLowThreshold_Excess_Time_Sensor2(Byte LowThreshold_Excess_Time_Sensor2) {
    this->m_LowThreshold_Excess_Time_Sensor2 = LowThreshold_Excess_Time_Sensor2;
}

/// Fixer la période de mesure pendant laquelle il y a détection des dépassement de seuil.
/// \param Measurement_period_threshold_detection : La période exprimée en minutes.
void CDallas::setMeasurement_period_threshold_detection(Byte Measurement_period_threshold_detection) {
    this->m_Measurement_period_threshold_detection = Measurement_period_threshold_detection;
}

/// Fixer la valeur du paramètre A pour le capteur 1
/// \param Parameter_A_sensor1 : la valeur du paramètre A
void CDallas::setParameter_A_sensor1(Byte Parameter_A_sensor1[2]) {
  for ( int i=0; i<2; i++ )
    this->m_Parameter_A_sensor1[i] = Parameter_A_sensor1[i];
}

/// Fixer la valeur du paramètre A pour le capteur 2
/// \param Parameter_A_sensor2 : la valeur du paramètre A
void CDallas::setParameter_A_sensor2(Byte Parameter_A_sensor2[2]) {
  for ( int i=0; i<2; i++ )
    this->m_Parameter_A_sensor2[i] = Parameter_A_sensor2[i];
}

/// Fixer la valeur du paramètre B pour le capteur 1
/// \param Parameter_B_sensor1 : la valeur du paramètre B
void CDallas::setParameter_B_sensor1(Byte Parameter_B_sensor1[2]) {
  for ( int i=0; i<2; i++ )
    this->m_Parameter_B_sensor1[i] = Parameter_B_sensor1[i];
}

/// Fixer la valeur du paramètre B pour le capteur 2
/// \param Parameter_B_sensor2 : la valeur du paramètre B
void CDallas::setParameter_B_sensor2(Byte Parameter_B_sensor2[2]) {
  for ( int i=0; i<2; i++ )
    this->m_Parameter_B_sensor2[i] = Parameter_B_sensor2[i];
}

// Methodes protegees

// Methodes privees

} // namespace WAVENIS

// Programme principal
// Si c'est la classe principale du projet, on place une fonction main()
// Dans ce cas, on peut supprimer les fichiers de tests unitaires
// ex :
// int main(int argc, char *argv[])
// {
//    // Construction de l'instance de la classe principale
//    cout << "Construction de l'instance de la classe principale CDallas" << endl;
//    CDallas *cdallas = new CDallas();
//    // Lancement de l'activité principale
//    return 0;
// }



