/*
 *  Copyright (C) 2012  plegal  (plegal@appert44.org)
 *  @file         CDallasTest.cpp
 *  Classe        CDallas
 *  @note         Implementation de la classe en charge des tests unitaires
 */

// Includes system C
// Includes system C++
#include <cppunit/CompilerOutputter.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/TestResult.h>
#include <cppunit/TestResultCollector.h>
#include <cppunit/TestRunner.h>
#include <cppunit/BriefTestProgressListener.h>
#include <string>
using namespace std;
// Includes qt


// Includes application
#include "protocol_types.h"
#include "CDallasTest.h"
#include "CConfig.h"
#include "CAdrRadio.h"
#include "CWavePort.h"


using namespace WAVENIS;

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION(CDallasTest);

// Exemple d'assertions possibles
// # CPPUNIT_ASSERT(condition)
// Assertions that a condition is true.
// # CPPUNIT_ASSERT_MESSAGE(message, condition)
// Assertion with a user specified message.
// # CPPUNIT_FAIL(message)
// Fails with the specified message
// # CPPUNIT_ASSERT_EQUAL(expected, actual)
// Asserts that two values are equals.
// # CPPUNIT_ASSERT_EQUAL_MESSAGE(message, expected, actual)
// Asserts that two values are equals, provides additional message on failure
// # CPPUNIT_ASSERT_DOUBLES_EQUAL(expected, actual, delta)

// setUp() to initialize the variables you will use for test
void CDallasTest::setUp() {
}

// tearDown() to release any permanent resources you allocated in setUp()
void CDallasTest::tearDown() {
}

// Suite des tests unitaires

void CDallasTest::testConstructor() {
//	// Construction de l'instance de classe a tester
//	cout << "load config.conf" << endl;
//	CConfig config("config.conf");
//
//	cout << "new adrRadio" << endl;
//	CAdrRadio *adrRadioPort = new CAdrRadio("000000000000");
//
//	cout << "new adrRadioDallas" << endl;
//	CAdrRadio *adrRadioDallas = new CAdrRadio("10194C300559");
//
//	cout << "new waveport" << endl;
//	CWavePort *cwaveport = new CWavePort(config, (string) "/dev/ttyUSB0",
//			adrRadioPort, NULL);
//
//	cout << "new cdallas" << endl;
//	CDallas *cdallas = new CDallas(cwaveport, adrRadioDallas);
//	CPPUNIT_ASSERT(cdallas != NULL);
//
//	cout << "delete cdallas" << endl;
//	delete cdallas;
//
//	cout << "delete cwaveport" << endl;
//	delete cwaveport;
//
//	cout << "delete adrRadioDallas" << endl;
//	delete adrRadioDallas;
//
//	cout << "delete adrRadio" << endl;
//	delete adrRadioPort;
}

void CDallasTest::testUnitaire1() {
	// Construction de l'instance de classe a tester
	cout << "load config.conf" << endl;
	CConfig config("config.conf");
	cout << "new adrRadio" << endl;
	CAdrRadio *adrRadioPort = new CAdrRadio("000000000000");

	cout << "new adrRadioDallas" << endl;
//	CAdrRadio *adrRadioDallas = new CAdrRadio("10194C300559");

	RadioAddress dallasRadioAddress = (RadioAddress){"10194C300559"};

	cout << "new waveport" << endl;
	CWavePort *cwaveport = new CWavePort(config, (string) "/dev/ttyUSB0",
			adrRadioPort, NULL);

	cout << "new cdallas" << endl;
	CDallas *cdallas = new CDallas(cwaveport, dallasRadioAddress);
	CPPUNIT_ASSERT(cdallas != NULL);

	cout << "SerialDriver and Protocol load" << endl;
	bool ret = cwaveport->SerialDriverAndProtocolLoad();
	CPPUNIT_ASSERT_MESSAGE("Serial Driver & Protocol Load", ret == true);

	cout << "SerialDriver and Protocol Instanciation" << endl;
	ret = cwaveport->SerialDriverAndProtocolInstanciation();
	CPPUNIT_ASSERT_MESSAGE("Serial Driver & Protocol Instanciation",
			ret == true);

	cout << "Protocol Init" << endl;
	int status = cwaveport->ProtocolInit();
	CPPUNIT_ASSERT_MESSAGE("Protocol Init", status == OK_PROTOCOL_SUCCESS);

	cout << "Protocol Open" << endl;
	status = cwaveport->ProtocolOpen();
	CPPUNIT_ASSERT_MESSAGE("Protocol Open", status == OK_PROTOCOL_SUCCESS);

	int temperatureBrute = cdallas->TemperatureBrute(1);
	printf("temperature brute : %4x \n",temperatureBrute);

	double temperature = cdallas->Temperature(temperatureBrute);
	printf("temperature : %f \n",temperature);

	cout << "Protocol Close" << endl;
	cwaveport->ProtocolClose();
	cout << "SerialDriver and Protocol Suppression" << endl;
	cwaveport->SerialDriverAndProtocolSuppression();
	cout << "SerialDriver and Protocol Free" << endl;
	cwaveport->SerialDriverAndProtocolFree();
	cout << "delete cdallas" << endl;
	delete cdallas;
	cout << "delete cwaveport" << endl;
	delete cwaveport;
//	cout << "delete adrRadioDallas" << endl;
//	delete adrRadioDallas;
	cout << "delete adrRadio" << endl;
	delete adrRadioPort;

	// Test unitaire d'une methode publique de la classe
	// Utilisation des macros CPPUNIT_ASSERT, CPPUNIT_ASSERT_EQUAL, etc.
}

// the main method
int main(int argc, char* argv[]) {
	// informs test-listener about testresults
	CPPUNIT_NS::TestResult testresult;

	// register listener for collecting the test-results
	CPPUNIT_NS::TestResultCollector collectedresults;
	testresult.addListener(&collectedresults);

	// register listener for per-test progress output
	CPPUNIT_NS::BriefTestProgressListener progress;
	testresult.addListener(&progress);

	// insert test-suite at test-runner by registry
	CPPUNIT_NS::TestRunner testrunner;
	testrunner.addTest(
			CPPUNIT_NS::TestFactoryRegistry::getRegistry().makeTest());
	testrunner.run(testresult);

	// output results in compiler-format
	CPPUNIT_NS::CompilerOutputter compileroutputter(&collectedresults,
			std::cerr);
	compileroutputter.write();

	// for hudson
	std::ofstream file("CDallas-cppunit-report.xml");
	CPPUNIT_NS::XmlOutputter xmloutputter(&collectedresults, file);
	xmloutputter.write();
	file.close();

	// return 0 if tests were successful
	return collectedresults.wasSuccessful() ? 0 : 1;
}

