///////////////////////////////////////////////////////////////////////////////////////
// Projet     : classe CDateTime
// Auteur     : MR
// Date       : Avril 2010
// Version    : v4
/////////////////////////////////////////////////////////////////////////////////////////

#include "string.h"      // pour memset
#include <stdio.h>       // sprintf

using namespace std;

#include "CWNExceptions.h"
#include "CDateTime.h"

namespace WAVENIS {

/// Constructeur
/// \param sDateTime : la date au format texte JSJJMMAAAAHHMM
/// \par exemple : Lu210420101241 Lundi 21 avril 2010 12h41
/// \arg JS : le jour de la semaine
///-# Di Dimanche=0
///-# lu Lundi=1
///-# Ma Mardi=2
///-# Me Mecredi=3
///-# Je Jeudi=4
///-# Ve Vendredi=5
///-# Sa Samedi=6
/// \arg JJMMAAAA : jour mois année (toujours à partir de 2000)
/// \arg HHMM : heure minute(s)
CDateTime::CDateTime ( string sDateTime ) {
  string _Semaine[7] = {"Di","Lu","Ma","Me","Je","Ve","Sa"};
  for ( int i=0; i<7; i++ )
    this->Semaine[i] = _Semaine[i];
  this->Taille = SIZEDATE;
  this->m_JourSemaine = sDateTime.substr(0,2);
  this->m_Jour = sDateTime.substr(2*JJ+2,2);
  this->m_Mois = sDateTime.substr(2*MM+2,2);
  this->m_Annee = sDateTime.substr((AA+1)*2,4);
  this->m_Heure = sDateTime.substr((HH+1)*2,2);
  this->m_Minutes = sDateTime.substr((MN+1)*2,2);
  this->bDateTime[JS] = IndiceJourSemaine(this->m_JourSemaine);
  this->bDateTime[JJ] = (this->m_Jour.c_str()[0]-'0')*10 + this->m_Jour.c_str()[1]-'0';
  this->bDateTime[MM] = (this->m_Mois.c_str()[0]-'0')*10 + this->m_Mois.c_str()[1]-'0';
  this->bDateTime[AA] = (this->m_Annee.c_str()[2]-'0')*10 + this->m_Annee.c_str()[3]-'0';
  this->bDateTime[HH] = (this->m_Heure.c_str()[0]-'0')*10 + this->m_Heure.c_str()[1]-'0';
  this->bDateTime[MN] = (this->m_Minutes.c_str()[0]-'0')*10 + this->m_Minutes.c_str()[1]-'0';
  if (    sDateTime.length() >  14
       || this->bDateTime[JS] > 6
       || this->bDateTime[JJ] > 30
       || this->bDateTime[JJ] < 1
       || this->bDateTime[MM] < 1
       || this->bDateTime[MM] > 12
       || this->m_Annee.substr(0,2).compare("20")
       || this->bDateTime[HH] > 23
       || this->bDateTime[MN] > 59
     ) {
    memset(this->bDateTime,0,SIZEDATE);
    this->m_JourSemaine.assign("");
    this->m_Jour.assign("");
    this->m_Mois.assign("");
    this->m_Annee.assign("");
    this->m_Heure.assign("");
    this->m_Minutes.assign("");
    throw CDTFMTException ();
  }
  this->m_strDateTime.assign(sDateTime);
}

/// Constructeur
/// \param bDateTime : la date au format tableau d'octets
/// \par exemple : Lu210420101241 correspond au tableau {0x01,0x15,0x04,0x0A,0x0C,0x29}
/// \arg Le jour de la semaine
///-# Di Dimanche=0
///-# lu Lundi=1
///-# Ma Mardi=2
///-# Me Mecredi=3
///-# Je Jeudi=4
///-# Ve Vendredi=5
///-# Sa Samedi=6
/// \arg JJMMAAAA : jour mois année (toujours à partir de 2000)
/// \arg HHMM : heure minute(s)
CDateTime::CDateTime ( Byte* bDateTime ) {
  string _Semaine[7] = {"Di","Lu","Ma","Me","Je","Ve","Sa"};
  for ( int i=0; i<7; i++ )
    this->Semaine[i] = _Semaine[i];
  this->Taille = SIZEDATE;
  this->bDateTime[JS] = bDateTime[JS];
  this->bDateTime[JJ] = bDateTime[JJ];
  this->bDateTime[MM] = bDateTime[MM];
  this->bDateTime[AA] = bDateTime[AA];
  this->bDateTime[HH] = bDateTime[HH];
  this->bDateTime[MN] = bDateTime[MN];
  this->m_strDateTime.assign("");
  char Txt[5];
  Txt[2]=0;
  sprintf(Txt,"%s",this->Semaine[bDateTime[JS]].c_str());
//  sprintf(Txt,"%02d",bDateTime[JS]);
  this->m_strDateTime.append(Txt);
  sprintf(Txt,"%02d",bDateTime[JJ]);
  this->m_strDateTime.append(Txt);
  sprintf(Txt,"%02d",bDateTime[MM]);
  this->m_strDateTime.append(Txt);
  Txt[4]=0;
  sprintf(Txt,"20%02d",bDateTime[AA]);
  this->m_strDateTime.append(Txt);
  sprintf(Txt,"%02d",bDateTime[HH]);
  this->m_strDateTime.append(Txt);
  sprintf(Txt,"%02d",bDateTime[MN]);
  this->m_strDateTime.append(Txt);
  this->m_JourSemaine = m_strDateTime.substr(0,2);
  this->m_Jour = m_strDateTime.substr(2*JJ+2,2);
  this->m_Mois = m_strDateTime.substr(2*MM+2,2);
  this->m_Annee = m_strDateTime.substr((AA+1)*2,4);
  this->m_Heure = m_strDateTime.substr((HH+1)*2,2);
  this->m_Minutes = m_strDateTime.substr((MN+1)*2,2);

  if (    m_strDateTime.length() >  14
       || this->bDateTime[JS] > 6
       || this->bDateTime[JJ] > 30
       || this->bDateTime[JJ] < 1
       || this->bDateTime[MM] < 1
       || this->bDateTime[MM] > 12
       || this->bDateTime[HH] > 23
       || this->bDateTime[MN] > 59
     ) {
    memset(this->bDateTime,0,SIZEDATE);
    this->m_JourSemaine.assign("");
    this->m_Jour.assign("");
    this->m_Mois.assign("");
    this->m_Annee.assign("");
    this->m_Heure.assign("");
    this->m_Minutes.assign("");
    throw CDTFMTException ();
  }
}

CDateTime::~CDateTime() {
}

/// Retrouver la chaîne date heure au format choisi
/// \return La date et l'heure au format choisi soit : JSJJMMAAAAHHMM
/// \par exemple : Lu210420101241 Lundi 21 avril 2010 12h41
/// \arg JS : le jour de la semaine
///-# Di Dimanche=0
///-# lu Lundi=1
///-# Ma Mardi=2
///-# Me Mecredi=3
///-# Je Jeudi=4
///-# Ve Vendredi=5
///-# Sa Samedi=6
/// \arg JJMMAAAA : jour mois année (toujours à partir de 2000)
/// \arg HHMM : heure minute(s)
string CDateTime::ToString ( ) {
  return this->m_strDateTime;
}

Byte* CDateTime::ToBytes ( ) {
  return this->bDateTime;
}

/// Retrouver l'indice du jour de la semaine dans le tableau des jours de la semaine
Byte CDateTime::IndiceJourSemaine ( string js ) {
  int i = 0;
  while ( js.compare(this->Semaine[i++]) && i < 7) ;
  return i-1;
}

}
