/**
 *  Copyright (C) 2012  plegal  (plegal@appert44.org)
 *  @file         CModuleFinal.cpp
 *  @brief        Pilote pour les éléments communs à tous les modules finaux WAVENIS
 *  @version      0.1
 *  @date         17 déc. 2012 21:05:37
 *
 *  Description detaillee du fichier CModuleFinal.cpp
 *  Fabrication   gcc (Ubuntu/Linaro 4.6.3-1ubuntu5) 4.6.3 
 *  @todo         Liste des choses restant a faire.
 *  @bug          17 déc. 2012 21:05:37 - Aucun pour l'instant
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Includes system C
#include <stdio.h>
// Includes system C++
// A décommenter si besoin cout, cin, ...
// #include <iostream>
// using namespace std;

// Includes qt

// Includes application

#include "protocol_types.h"	// Payload

#include "CModuleFinal.h"
#include "CWNExceptions.h"

/// Espace de noms pour les classes de la librairie WAVENIS
namespace WAVENIS {

/// Constructeur 1 :
/// \arg l'adresse radio du module final associé a été lu dans la BD
/// \arg Le nom du port série associé sera le port par défaut
CModuleFinal::CModuleFinal(CWavePort* pwp, CAdrRadio* AdresseRadio) {
	// stocker l'adresse radio du module
	this->AdresseRadio = AdresseRadio;
	this->NbParamReq = MAXPARAM + 1;
	// toutes les PDU à émettre contient l'adresse radio du module
	for (int i = 0; i < SIZEADRRADIO; i++)
		this->PDUe[i] = this->AdresseRadio->ToBytes()[i];
	this->pwp = pwp;
	// récupérer le flux de traçage
	this->Stream = pwp->getStream();
}

CModuleFinal::CModuleFinal(CWavePort* pwp, RadioAddress radioAddress) {
	// stocker l'adresse radio du module

	this->m_radioAddress = radioAddress;

//  this->NbParamReq = MAXPARAM+1;
//  // toutes les PDU à émettre contient l'adresse radio du module
//  for ( int i=0 ; i< SIZEADRRADIO ; i++ )
//    this->PDUe[i] = this->AdresseRadio->ToBytes()[i];
	this->pwp = pwp;
	// récupérer le flux de traçage
	this->Stream = pwp->getStream();
}

CModuleFinal::~CModuleFinal() {
	// TODO Auto-generated destructor stub
}

/// Effectuer une lecture de la version du firmware
/// La lecture retourne après l'adresse radio du module et la commande CMDAPPACK :
/// \arg en 1 : le caractère 'V'
/// \arg en 2 et 3 : le mode de transmission LSB MSB
/// \arg en 4 et 5 : la version du firmware LSB MSB
/// \return La version du firmware.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
int CModuleFinal::getfirmware_version(void) {
	this->NbDonneesSpecifiques = 0;       // pas de données spécifiques
	_MR_MF_CMDPARAMCTRL(REQ_GET_FIRMWARE_VERSION);
	char Txt[10];
	sprintf(Txt, "%02d.%02d", this->PDUe[SIZEADRRADIO + 1 + 1 + 3],
			this->PDUe[SIZEADRRADIO + 1 + 1 + 2]);
	firmware_version.assign(Txt);
	return this->PDUe[SIZEADRRADIO + 1 + 1 + 2]
			+ (this->PDUe[SIZEADRRADIO + 1 + 1 + 3] << 8);
}

/// Effectuer une lecture du type du module
/// la lecture retourne après l'adresse radio du module et la commande CMDAPPACK :
/// \arg en 1 : le module type
/// \arg en 2 : RSSI
/// \arg en 3 : awakening_period
/// \arg en 4 : Equipement type qui est toujours égal à Module type.
/// \return Le type du module
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
Byte CModuleFinal::getModule_type(void) {
	this->NbDonneesSpecifiques = 0;       // pas de données spécifiques
	_MR_MF_CMDPARAMCTRL(REQ_READ_MODULE_TYPE);
	return this->PDUe[SIZEADRRADIO + 1 + 1];
}

/// Effectuer une lecture de la date du module
/// la lecture retourne après l'adresse radio du module et la commande CMDAPPACK la date du module au format texte soit : JSJJMMAAAAHHMM
/// \arg exemple : Ma250520101021 soit Mardi 25 mai 20910 10:21
/// Le format interne WAVENIS est :
/// \arg octet 1 : le jour
/// \arg octet 2 : le mois
/// \arg octet 3 : l'année dont la valeur complète est 2000+octet3
/// \arg octet 4 : le jour de la semaine de 0 pour Dimanche à 6 pour samedi
/// \arg octet 5 : l'heure
/// \arg octet 6 : la minute
/// \return La date et l'heure du module au format texte fixé.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
/// \see CDateTime
CDateTime CModuleFinal::getDate_time(void) {
	this->NbDonneesSpecifiques = 0;       // pas de données spécifiques
	_MR_MF_CMDPARAMCTRL(REQ_READ_DATE);
	for (int i = 0; i < SIZEDATE; i++)
		m_date_time[i] = this->PDUe[SIZEADRRADIO + 1 + i];
	this->Date = new CDateTime(m_date_time);
	return *Date;
}

/// Fixer la date et l'heure du module. le format WAVENIS est :
/// \arg octet 1 : le jour
/// \arg octet 2 : le mois
/// \arg octet 3 : l'année dont la valeur complète est 2000+octet3
/// \arg octet 4 : le jour de la semaine de 0 pour Dimanche à 6 pour samedi
/// \arg octet 5 : l'heure
/// \arg octet 6 : la minute
/// \param Date : l'objet date contenant la date à appliquer sous la forme d'une chaîne JSJJMMAAAAHHMN.
/// \arg exemple : Ma250520101021 soit Mardi 25 mai 20910 10:21
/// \par Gestion des exceptions:
/// \arg CMFUPDTException : erreur de mise à jour de la date du module.
/// \see CMFUPDTException
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setDate_time(CDateTime Date) {
	this->NbDonneesSpecifiques = SIZEDATE;
	for (int i = 0; i < SIZEDATE; i++)
		this->PDUe[SIZEADRRADIO + 1 + i] = Date.ToBytes()[i];
	_MR_MF_CMDPARAMCTRL(REQ_WRITE_DATE);
	if (this->PDUe[SIZEADRRADIO + 1] == CWavePort::UPDERROR) {
		Tracer("Erreur à la mise à jour de la date", this->PDUe,
				SIZEADRRADIO + 1 + SIZEDATE);
		throw new CMFUPDTException();
	}
	for (int i = 0; i < SIZEDATE; i++)
		m_date_time[i] = this->PDUe[SIZEADRRADIO + 1 + i];
}

/// Retrouver l'adresse du module vers qui émettre l'alarme
/// \return L'adresse radio 6 octets
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
CAdrRadio CModuleFinal::getAddress_Recipient_Alarm() {
	_MR_MF_CMDREADPARAM(ADDRESS_RECIPIENT_ALARM,
			SIZEREQ_ADDRESS_RECIPIENT_ALARM);
	for (int i = 0; i < PDUe[SIZEADRRADIO + 1 + 1 + 1]; i++) {
		m_Address_Recipient_Alarm[i] = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1 + i];
	}
	//Address_Recipient_Alarm = new CAdrRadio(m_Address_Recipient_Alarm);
	return *Address_Recipient_Alarm;
}

/// Retrouver l'octet de status du module précisant la configuration du module.
/// Les éléments de configuration sont inscrits par un bit de cet octet.
/// cet octet doit être interprété selon le module, donc dans la classe dérivée.
/// A ce niveau il est impossible d'en faire l'analyse bit par bit.
/// \return Un octet contenant des informations de donfiguration inscrite dans chacun de ses 8 bits.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
/// \see SetAlarm
Byte CModuleFinal::getApplication_Status() {
	_MR_MF_CMDREADPARAM(APPLICATION_STATUS, SIZEREQ_APPLICATION_STATUS);
	m_Application_Status = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_Application_Status;
}

/// Retrouver la date de fin de vie de la batterie. cela permet d'en prévoir le remplacement.
/// \return La date sous la forme de 6 octets :
/// \arg octet 1 : le jour
/// \arg octet 2 : le mois
/// \arg octet 3 : l'année dont la valeur complète est 2000+octet3
/// \arg octet 4 : le jour de la semaine de 0 pour Dimanche à 6 pour samedi
/// \arg octet 5 : l'heure
/// \arg octet 6 : la minute
/// \return La date de fin de vie de la batterie sous la forme texte choisie.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
CDateTime CModuleFinal::getDate_End_of_Battery_life() {
	_MR_MF_CMDREADPARAM(DATE_END_OF_BATTERY_LIFE,
			SIZEREQ_DATE_END_OF_BATTERY_LIFE);

	for (int i = 0; i < PDUe[SIZEADRRADIO + 1 + 1 + 1]; i++) {
		m_Date_End_of_Battery_life[i] = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1 + i];
	}
	m_strDate_End_of_Battery_life = new CDateTime(m_Date_End_of_Battery_life);
	return *m_strDate_End_of_Battery_life;
}

/// Retrouver la configuration d'activation des fenêtres de temps selon le jour de la semaine.
/// Lorsque le mode time window est utilisé (voir application status), l'utilisateur peut choisir pour chaque jour de la semaine si le module sera actif ou pas.
/// Ainsi un module actif tous les jours de la semaine aura un octet enable_time de 0x7F car chaque bit représente un jour de la semaine.
/// \par Bit0 pour lundi à bit6 pour dimanche.
/// \par Un bit à 0 signifie module inactif ce jour
/// \par Un bit à 1 signifie module actif ce jour
/// \return Un octet contenant l'indication d'activité pour chaque jour de la semaine.
/// \see
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
Byte CModuleFinal::getEnable_Time() {
	_MR_MF_CMDREADPARAM(ENABLE_TIME, SIZEREQ_ENABLE_TIME);
	m_Enable_Time = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_Enable_Time;
}

/// Retrouver la configuration d'activation des périodes de réveil selon le jour de la semaine.
/// Lorsque le mode wake up est utilisé (voir application status), l'utilisateur peut choisir pour chaque jour de la semaine si
/// si le réveil de module sera actif ou pas.
/// ATTENTION, le principe actif/pas actif est inversé par rapport à celui de "enable_time" (fenêtre de temps)
/// \par Bit0 pour lundi à bit6 pour dimanche.
/// \par Un bit à 0 signifie réveil actif sur le module ce jour
/// \par Un bit à 1 signifie réveil inactif sur le module ce jour
/// Ainsi un module actif tous les jours de la semaine aura un octet enable_wakeup_periods de 0.
/// \return Un octet contenant l'indication de réveil pour chaque jour de la semaine.
/// \see getWakeUp_Status
/// \see setWakeUp_Status
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
Byte CModuleFinal::getEnable_WakeUp_Periods() {
	CmdReadParam(ENABLE_WAKEUP_PERIODS, SIZEREQ_ENABLE_WAKEUP_PERIODS);
	_MR_MF_CMDREADPARAM(ENABLE_WAKEUP_PERIODS, SIZEREQ_ENABLE_WAKEUP_PERIODS);
	m_Enable_WakeUp_Periods = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_Enable_WakeUp_Periods;
}

/// Retrouver le numéro de groupe auquel appartient le module dans le cas d'une interrogation par polling.
/// \return Un octet contenant l'indication de réveil pour chaque jour de la semaine.
Byte CModuleFinal::getGroup_Number() {
	_MR_MF_CMDREADPARAM(GROUP_NUMBER, SIZEREQ_GROUP_NUMBER);
	m_Group_Number = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_Group_Number;
}

/// Retrouver le nombre de répéteurs lorsqu'une réponse survient suite à une alarme déclenchée par le module.
///\par Pour chaque répeteur le module enregistre une adresse radio.
/// \return Le nombre de répéteurs. 3 au maximum.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
Byte CModuleFinal::getNumber_Repeaters_Alarm() {
	_MR_MF_CMDREADPARAM(NUMBER_REPEATERS_ALARM, SIZEREQ_NUMBER_REPEATERS_ALARM);
	m_Number_Repeaters_Alarm = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_Number_Repeaters_Alarm;
}

/// Retrouver le mode opératoire du module.
/// cet octet doit être interprété selon le module, donc dans la classe dérivée.
/// A ce niveau il est impossible d'en faire l'analyse bit par bit.
/// \return L'octet de mode du module.
Byte CModuleFinal::getOperating_Mode() {
	_MR_MF_CMDREADPARAM(OPERATING_MODE, SIZEREQ_OPERATING_MODE);
	m_Operating_Mode = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_Operating_Mode;
}

/// Retrouver l'adresse radio du premier répéteur dans le cas de réponse à une alarme déclenchée par le module.
/// \return L'adresse radio du répéteur.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
CAdrRadio CModuleFinal::getRepeater_Alarm1() {
	_MR_MF_CMDREADPARAM(REPEATER_ALARM1, SIZEREQ_REPEATER_ALARM1);
	for (int i = 0; i < PDUe[SIZEADRRADIO + 1 + 1 + 1]; i++) {
		m_Repeater_Alarm1[i] = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1 + i];
	}
	//Repeater_Alarm1 = new CAdrRadio(m_Repeater_Alarm1);
	return *Repeater_Alarm1;
}

/// Retrouver l'adresse radio du second répéteur dans le cas de réponse à une alarme déclenchée par le module.
/// \return L'adresse radio du répéteur.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
CAdrRadio CModuleFinal::getRepeater_Alarm2() {
	_MR_MF_CMDREADPARAM(REPEATER_ALARM2, SIZEREQ_REPEATER_ALARM2);
	for (int i = 0; i < PDUe[SIZEADRRADIO + 1 + 1 + 1]; i++) {
		m_Repeater_Alarm2[i] = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1 + i];
	}
	//Repeater_Alarm2 = new CAdrRadio(m_Repeater_Alarm2);
	return *Repeater_Alarm2;
}

/// Retrouver l'adresse radio du troisième répéteur dans le cas de réponse à une alarme déclenchée par le module.
/// \return L'adresse radio du répéteur.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
CAdrRadio CModuleFinal::getRepeater_Alarm3() {
	_MR_MF_CMDREADPARAM(REPEATER_ALARM3, SIZEREQ_REPEATER_ALARM3);
	for (int i = 0; i < PDUe[SIZEADRRADIO + 1 + 1 + 1]; i++) {
		m_Repeater_Alarm3[i] = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1 + i];
	}
	//Repeater_Alarm3 = new CAdrRadio(m_Repeater_Alarm3);
	return *Repeater_Alarm3;
}

/// Retrouver l'heure de lancement de la première fenêtre de temps (enable_time) lorsque le mode fenêtre de temps par jour est actif.
/// Cette  données est en heures entre 0 et 23.
/// \return L'heure de démarrage de la fenêtre de temps dans l'intervalle 0 à 23h.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
Byte CModuleFinal::getStart_Time1() {
	_MR_MF_CMDREADPARAM(START_TIME1, SIZEREQ_START_TIME1);
	m_Start_Time1 = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_Start_Time1;
}

/// Retrouver l'heure de lancement de la seconde fenêtre de temps (enable_time) lorsque le mode fenêtre de temps par jour est actif.
/// Cette  données est en heures entre 0 et 23.
/// \return L'heure de démarrage de la fenêtre de temps dans l'intervalle 0 à 23h.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
Byte CModuleFinal::getStart_Time2() {
	_MR_MF_CMDREADPARAM(START_TIME2, SIZEREQ_START_TIME2);
	m_Start_Time2 = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_Start_Time2;
}

/// Retrouver la période de réveil par défaut en mode Wakeup périodique.
/// Cette  données est en secondes et ne peut dépasser 10s.
/// \return La période de réveil en secondes.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
Byte CModuleFinal::getWakeUp_Period() {
	_MR_MF_CMDREADPARAM(WAKEUP_PERIOD, SIZEREQ_WAKEUP_PERIOD);
	m_WakeUp_Period = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_WakeUp_Period;
}

/// Retrouver la période de réveil pour la fenêtre 1 en mode Wakeup périodique.
/// Cette  données est en secondes et ne peut dépasser 10s.
/// \return La période de réveil en secondes.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
Byte CModuleFinal::getWakeUp_Period1() {
	_MR_MF_CMDREADPARAM(WAKEUP_PERIOD1, SIZEREQ_WAKEUP_PERIOD1);
	m_WakeUp_Period1 = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_WakeUp_Period1;
}

/// Retrouver la période de réveil pour la fenêtre 2 en mode Wakeup périodique.
/// Cette  données est en secondes et ne peut dépasser 10s.
/// \return La période de réveil en secondes.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
Byte CModuleFinal::getWakeUp_Period2() {
	_MR_MF_CMDREADPARAM(WAKEUP_PERIOD2, SIZEREQ_WAKEUP_PERIOD2);
	m_WakeUp_Period2 = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_WakeUp_Period2;
}

/// Retrouver le mode de réveil actif :
/// \arg Enable time window : bit0 (1 actif 0 inactif)
/// \arg Enable day of week : bit1 (1 actif 0 inactif)
/// Les 4 combinaisons sont possibles :
/// \arg 00 :réveil périodique sans distinction du jour de la semaine
/// \par Les paramètres à régler sont:
/// -# la période de réveil par défaut (setWakeUp_Period)
/// -# le mode de réveil (setWakeUp_Status) sur 00
/// \arg 01 :réveil périodique dans des fenêtres de temps spécifiques par jour de la semaine.
/// \par Les paramètres à régler sont :
/// -# la période de réveil par défaut (setWakeUp_Period)
/// -# les heures de départ des fenêtres de temps 1 et 2 (setStart_Time1 et setStart_Time2)
/// -# la période de réveil pour les fenêtres de temps 1 et 2 (setWakeUp_Period1 et setWakeUp_Period2)
/// -# le mode fenêtres de temps par jour de la semaine (setEnable_Time)
/// -# le mode de réveil (setWakeUp_Status) sur 01
/// \par Exemple :
/// \par Mode fenêtre de temps L Ma J ( valeur 0x0B) : les fenêtres de temps avec période de réveil s'appliqueront pour ces jours.
/// Pour les autres jours de la semaine le réveil périodique avec la période de réveil par défaut s'appliquera.
/// \arg 02 :réveil périodique pour certains jours et pas de réveil périodique pour les autres jours de la semaine.
/// \par Les paramètres à régler sont :
/// -# la période de réveil par défaut (setWakeUp_Period)
/// -# le mode réveil périodique par jour de la semaine (setEnable_Wakeup_Periods).
/// -# le mode de réveil (setWakeUp_Status) sur 02
/// \par Exemple :
/// \par Mode période de réveil périodique par jour de la semaine Ma Me S (setEnable_WakeUp_Periods) (valeur 0x59).
/// Les autres jours, le module se réveille toutes les 10s.
/// \arg 03 :réveil périodique dans des fenêtres de temps spécifiques pour certains jours de la semaine, réveil périodique
/// pour certains jours de la semaine et réveil périodique désactivé pour les autres jours de la semaine.
/// \par Les paramètres à régler sont :
/// -# la période de réveil par défaut (setWakeUp_Period)
/// -# les heures de départ des fenêtres de temps 1 et 2 (setStart_Time1 et setStart_Time2)
/// -# la période de réveil pour les fenêtres de temps 1 et 2 (setWakeUp_Period1 et setWakeUp_Period2)
/// -# le mode fenêtres de temps par jour de la semaine (setEnable_Time) avec les jours où la fenêtre de temps s'applique.
/// -# le mode réveil périodique (setEnable_Wakeup_Periods). Pour les jours sélectionnés ici (et non sélectionnés pour le mode fenre de temps) le réveil est périodique avec la période de réveil par défaut. Pour les uatres jours (autres que ceux sélectionnés pour le mode fenêtre de temps) le module se réveille toutes les 10 secondes.
/// -# le mode de réveil (setWakeUp_Status) sur 03
/// \see setWakeUp_Period
/// \see getWakeUp_Status
/// \see setWakeUp_Period1
/// \see setWakeUp_Period2
/// \see setStart_Time1
/// \see setStart_Time2
/// \see setEnable_Time
/// \see setEnable_Wakeup_Periods
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
Byte CModuleFinal::getWakeUp_Status() {
	_MR_MF_CMDREADPARAM(WAKEUP_STATUS, SIZEREQ_WAKEUP_STATUS);
	m_WakeUp_Status = PDUe[SIZEADRRADIO + 1 + 1 + 1 + 1];
	return m_WakeUp_Status;
}

/// Fixer l'adresse radio du destinataire d'une alarme déclenchée par le module.
/// \param Address_Recipient_Alarm : l'adresse radio du récepteur de l'alarme
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setAddress_Recipient_Alarm(
		CAdrRadio Address_Recipient_Alarm) {
	Byte Tmp[SIZEADRRADIO + 1];
	Tmp[0] = SIZEADRRADIO;
	for (int i = 0; i < SIZEADRRADIO; i++)
		Tmp[i + 1] = Address_Recipient_Alarm.ToBytes()[i];
	_MR_MF_CMDWRITEPARAM(ADDRESS_RECIPIENT_ALARM,
			SIZEREQ_ADDRESS_RECIPIENT_ALARM, Tmp);
	for (int i = 0; i < SIZEADRRADIO; i++)
		this->m_Address_Recipient_Alarm[i] =
				Address_Recipient_Alarm.ToBytes()[i];
}

/// Fixer l'octet de status du module précisant la configuration du module.
/// Les éléments de configuration sont inscrits par un bit de cet octet.
/// cet octet doit être interprété selon le module, donc dans la classe dérivée.
/// A ce niveau il est impossible d'en faire l'analyse bit par bit.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setApplication_Status(Byte m_Application_Status) {
	Byte Tmp[] = { 1, m_Application_Status }; // [0] = taille + 1 octet de données
	_MR_MF_CMDWRITEPARAM(APPLICATION_STATUS, SIZEREQ_APPLICATION_STATUS, Tmp);
	this->m_Application_Status = m_Application_Status;
}

/// Fixer la date de fin de vie de la batterie du module.
/// La date fournie est au format :
/// \arg octet 1 : le jour
/// \arg octet 2 : le mois
/// \arg octet 3 : l'année dont la valeur complète est 2000+octet3
/// \arg octet 4 : le jour de la semaine de 0 pour Dimanche à 6 pour samedi
/// \arg octet 5 : l'heure
/// \arg octet 6 : la minute
/// \param Date_End_of_Battery_life : la date souhaitée
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setDate_End_of_Battery_life(
		CDateTime Date_End_of_Battery_life) {
	Byte Tmp[SIZEDATE + 1];
	Tmp[0] = SIZEDATE;
	for (int i = 0; i < SIZEDATE; i++)
		Tmp[i + 1] = Date_End_of_Battery_life.ToBytes()[i];
	_MR_MF_CMDWRITEPARAM(DATE_END_OF_BATTERY_LIFE,
			SIZEREQ_DATE_END_OF_BATTERY_LIFE, Tmp);
	for (int i = 0; i < SIZEDATE; i++)
		this->m_Date_End_of_Battery_life[i] =
				Date_End_of_Battery_life.ToBytes()[i];
}

/// Fixer le mode fenêtre de temps jour par jour de la semaine
/// \par Bit0 pour lundi à bit6 pour dimanche.
/// \par Un bit à 0 signifie module inactif ce jour
/// \par Un bit à 1 signifie module actif ce jour
/// \param m_Enable_Time :  le mode souhaité
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setEnable_Time(Byte m_Enable_Time) {
	Byte Tmp[] = { 1, m_Enable_Time };
	_MR_MF_CMDWRITEPARAM(ENABLE_TIME, SIZEREQ_ENABLE_TIME, Tmp);
	this->m_Enable_Time = m_Enable_Time;
}

/// Fixer la configuration d'activation des périodes de réveil selon le jour de la semaine.
/// Lorsque le mode wake up est utilisé (voir setWakeUp_Status), l'utilisateur peut choisir pour chaque jour de la semaine si
/// si le réveil de module sera actif ou pas.
/// ATTENTION, le principe actif/pas actif est inversé par rapport à celui de "enable_time" (fenêtre de temps)
/// \par Bit0 pour lundi à bit6 pour dimanche.
/// \par Un bit à 0 signifie réveil actif sur le module ce jour
/// \par Un bit à 1 signifie réveil inactif sur le module ce jour
/// Ainsi un module actif tous les jours de la semaine aura un octet enable_wakeup_periods de 0.
/// \return Un octet contenant l'indication de réveil pour chaque jour de la semaine.
/// \see setWakeUp_Status
/// \param m_Enable_WakeUp_Periods :  le mode souhaité
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setEnable_WakeUp_Periods(Byte m_Enable_WakeUp_Periods) {
	Byte Tmp[] = { 1, m_Enable_WakeUp_Periods };
	_MR_MF_CMDWRITEPARAM(ENABLE_WAKEUP_PERIODS, SIZEREQ_ENABLE_WAKEUP_PERIODS,
			Tmp);
	this->m_Enable_WakeUp_Periods = m_Enable_WakeUp_Periods;
}

/// Fixer le numéro de groupe auquel appartient le module dans le cas d'une interrogation par polling.
/// \param m_Group_Number :  le numéro de groupe d'appartenance
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setGroup_Number(Byte m_Group_Number) {
	Byte Tmp[] = { 1, m_Group_Number };
	_MR_MF_CMDWRITEPARAM(GROUP_NUMBER, SIZEREQ_GROUP_NUMBER, Tmp);
	this->m_Group_Number = m_Group_Number;
}

/// Fixer le nombre de répéteurs lorsqu'une réponse survient suite à une alarme déclenchée par le module.
///\par Pour chaque répeteur le module enregistre une adresse radio.
/// \param  m_Number_Repeaters_Alarm : le nombre de répéteurs. 3 au maximum.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setNumber_Repeaters_Alarm(Byte m_Number_Repeaters_Alarm) {
	Byte Tmp[] = { 1, m_Number_Repeaters_Alarm };
	_MR_MF_CMDWRITEPARAM(NUMBER_REPEATERS_ALARM, SIZEREQ_NUMBER_REPEATERS_ALARM,
			Tmp);
	this->m_Number_Repeaters_Alarm = m_Number_Repeaters_Alarm;
}

/// Fixer le mode de fonctionnement du module.
/// La constitution de cet octet dépend de la classe dérivée. A ce niveau il est impossible d'en fixer les bits.
/// \param m_Operating_Mode : l'octet de mode du module.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setOperating_Mode(Byte m_Operating_Mode) {
	Byte Tmp[] = { 1, m_Operating_Mode };
	_MR_MF_CMDWRITEPARAM(OPERATING_MODE, SIZEREQ_OPERATING_MODE, Tmp);
	this->m_Operating_Mode = m_Operating_Mode;
}

/// Fixer l'adresse radio du premier répéteur dans le cas de réponse à une alarme déclenchée par le module.
/// \param Repeater_Alarm1 : l'adresse radio du répéteur.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setRepeater_Alarm1(CAdrRadio Repeater_Alarm1) {
	Byte Tmp[SIZEADRRADIO + 1];
	Tmp[0] = SIZEADRRADIO;
	for (int i = 0; i < SIZEADRRADIO; i++)
		Tmp[i + 1] = Repeater_Alarm1.ToBytes()[i];
	_MR_MF_CMDWRITEPARAM(REPEATER_ALARM1, SIZEREQ_REPEATER_ALARM1, Tmp);
	for (int i = 0; i < SIZEADRRADIO; i++)
		this->m_Repeater_Alarm1[i] = Repeater_Alarm1.ToBytes()[i];
}

/// Fixer l'adresse radio du second répéteur dans le cas de réponse à une alarme déclenchée par le module.
/// \param Repeater_Alarm2 : l'adresse radio du répéteur.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setRepeater_Alarm2(CAdrRadio Repeater_Alarm2) {
	Byte Tmp[SIZEADRRADIO + 1];
	Tmp[0] = SIZEADRRADIO;
	for (int i = 0; i < SIZEADRRADIO; i++)
		Tmp[i + 1] = Repeater_Alarm2.ToBytes()[i];
	_MR_MF_CMDWRITEPARAM(REPEATER_ALARM2, SIZEREQ_REPEATER_ALARM2, Tmp);
	for (int i = 0; i < SIZEADRRADIO; i++)
		this->m_Repeater_Alarm2[i] = Repeater_Alarm2.ToBytes()[i];
}

/// Fixer l'adresse radio du troisième répéteur dans le cas de réponse à une alarme déclenchée par le module.
/// \param Repeater_Alarm3 : l'adresse radio du répéteur.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setRepeater_Alarm3(CAdrRadio Repeater_Alarm3) {
	Byte Tmp[SIZEADRRADIO + 1];
	Tmp[0] = SIZEADRRADIO;
	for (int i = 0; i < SIZEADRRADIO; i++)
		Tmp[i + 1] = Repeater_Alarm3.ToBytes()[i];
	_MR_MF_CMDWRITEPARAM(REPEATER_ALARM3, SIZEREQ_REPEATER_ALARM3, Tmp);
	for (int i = 0; i < SIZEADRRADIO; i++)
		this->m_Repeater_Alarm3[i] = Repeater_Alarm3.ToBytes()[i];
}

/// Fixer l'heure de lancement de la première fenêtre de temps (enable_time) lorsque le mode fenêtre de temps par jour est actif.
/// Cette  données est en heures entre 0 et 23.
/// \param m_Start_Time1 : l'heure de démarrage de la fenêtre de temps dans l'intervalle 0 à 23h.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setStart_Time1(Byte m_Start_Time1) {
	Byte Tmp[] = { 1, m_Start_Time1 };
	_MR_MF_CMDWRITEPARAM(START_TIME1, SIZEREQ_START_TIME1, Tmp);
	this->m_Start_Time1 = m_Start_Time1;
}

/// Fixer l'heure de lancement de la seconde fenêtre de temps (enable_time) lorsque le mode fenêtre de temps par jour est actif.
/// Cette  données est en heures entre 0 et 23.
/// \param m_Start_Time2 : l'heure de démarrage de la fenêtre de temps dans l'intervalle 0 à 23h.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setStart_Time2(Byte m_Start_Time2) {
	Byte Tmp[] = { 1, m_Start_Time2 };
	_MR_MF_CMDWRITEPARAM(START_TIME2, SIZEREQ_START_TIME2, Tmp);
	this->m_Start_Time2 = m_Start_Time2;
}

/// Fixer la période de réveil par défaut en mode Wakeup périodique.
/// Cette  données est en secondes et ne peut dépasser 10s.
/// \param m_WakeUp_Period : la période de réveil en secondes.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setWakeUp_Period(Byte m_WakeUp_Period) {
	Byte Tmp[] = { 1, m_WakeUp_Period };
	_MR_MF_CMDWRITEPARAM(WAKEUP_PERIOD, SIZEREQ_WAKEUP_PERIOD, Tmp);
	this->m_WakeUp_Period = m_WakeUp_Period;
}

/// Fixer la période de réveil pour la fenêtre 1 en mode Wakeup périodique.
/// Cette  données est en secondes et ne peut dépasser 10s.
/// \param m_WakeUp_Period1 : la période de réveil en secondes.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setWakeUp_Period1(Byte m_WakeUp_Period1) {
	Byte Tmp[] = { 1, m_WakeUp_Period1 };
	_MR_MF_CMDWRITEPARAM(WAKEUP_PERIOD1, SIZEREQ_WAKEUP_PERIOD1, Tmp);
	this->m_WakeUp_Period1 = m_WakeUp_Period1;
}

/// Fixer la période de réveil pour la fenêtre 2 en mode Wakeup périodique.
/// Cette  données est en secondes et ne peut dépasser 10s.
/// \param m_WakeUp_Period2 : la période de réveil en secondes.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setWakeUp_Period2(Byte m_WakeUp_Period2) {
	Byte Tmp[] = { 1, m_WakeUp_Period2 };
	_MR_MF_CMDWRITEPARAM(WAKEUP_PERIOD2, SIZEREQ_WAKEUP_PERIOD2, Tmp);
	this->m_WakeUp_Period2 = m_WakeUp_Period2;
}

/// Fixer le mode de réveil actif :
/// \arg Enable time window : bit0 (1 actif 0 inactif)
/// \arg Enable day of week : bit1 (1 actif 0 inactif)
/// Les 4 combinaisons sont possibles :
/// \arg 00 :réveil périodique sans distinction du jour de la semaine
/// \par Les paramètres à régler sont:
/// -# la période de réveil par défaut (setWakeUp_Period)
/// -# le mode de réveil (setWakeUp_Status) sur 00
/// \arg 01 :réveil périodique dans des fenêtres de temps spécifiques par jour de la semaine.
/// \par Les paramètres à régler sont :
/// -# la période de réveil par défaut (setWakeUp_Period)
/// -# les heures de départ des fenêtres de temps 1 et 2 (setStart_Time1 et setStart_Time2)
/// -# la période de réveil pour les fenêtres de temps 1 et 2 (setWakeUp_Period1 et setWakeUp_Period2)
/// -# le mode fenêtres de temps par jour de la semaine (setEnable_Time)
/// -# le mode de réveil (setWakeUp_Status) sur 01
/// \par Exemple :
/// \par Mode fenêtre de temps L Ma J ( valeur 0x0B) : les fenêtres de temps avec période de réveil s'appliqueront pour ces jours.
/// Pour les autres jours de la semaine le réveil périodique avec la période de réveil par défaut s'appliquera.
/// \arg 02 :réveil périodique pour certains jours et pas de réveil périodique pour les autres jours de la semaine.
/// \par Les paramètres à régler sont :
/// -# la période de réveil par défaut (setWakeUp_Period)
/// -# le mode réveil périodique par jour de la semaine (setEnable_Wakeup_Periods).
/// -# le mode de réveil (setWakeUp_Status) sur 02
/// \par Exemple :
/// \par Mode période de réveil périodique par jour de la semaine Ma Me S (setEnable_WakeUp_Periods) (valeur 0x59).
/// Les autres jours, le module se réveille toutes les 10s.
/// \arg 03 :réveil périodique dans des fenêtres de temps spécifiques pour certains jours de la semaine, réveil périodique
/// pour certains jours de la semaine et réveil périodique désactivé pour les autres jours de la semaine.
/// \par Les paramètres à régler sont :
/// -# la période de réveil par défaut (setWakeUp_Period)
/// -# les heures de départ des fenêtres de temps 1 et 2 (setStart_Time1 et setStart_Time2)
/// -# la période de réveil pour les fenêtres de temps 1 et 2 (setWakeUp_Period1 et setWakeUp_Period2)
/// -# le mode fenêtres de temps par jour de la semaine (setEnable_Time) avec les jours où la fenêtre de temps s'applique.
/// -# le mode réveil périodique (setEnable_Wakeup_Periods). Pour les jours sélectionnés ici (et non sélectionnés pour le mode fenre de temps) le réveil est périodique avec la période de réveil par défaut. Pour les uatres jours (autres que ceux sélectionnés pour le mode fenêtre de temps) le module se réveille toutes les 10 secondes.
/// -# le mode de réveil (setWakeUp_Status) sur 03
/// \see setWakeUp_Period
/// \see getWakeUp_Status
/// \see setWakeUp_Period1
/// \see setWakeUp_Period2
/// \see setStart_Time1
/// \see setStart_Time2
/// \see setEnable_Time
/// \see setEnable_Wakeup_Periods
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::setWakeUp_Status(Byte m_WakeUp_Status) {
	Byte Tmp[] = { 1, m_WakeUp_Status };
	_MR_MF_CMDWRITEPARAM(WAKEUP_STATUS, SIZEREQ_WAKEUP_STATUS, Tmp);
	this->m_WakeUp_Status = m_WakeUp_Status;
}

/// Effectuer une requête spécifique en lecture relative aux paramètres de contrôle
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::CmdParamCtrl(enum CMDAPP Cmd) {
	this->PDUe[SIZEADRRADIO] = Cmd;
	try {
		//cout << "CmdRadioMd" << endl;
		printf("CmdRadioMd : %2x\n", Cmd);
		Payload payload = (Payload ) { { Cmd }, 1 };
		//this->pwp->CmdRadioMd ( this->PDUe,SIZEADRRADIO+1+this->NbDonneesSpecifiques );
		this->pwp->CmdRadioMd(payload, this->m_radioAddress);

		Payload* pl = (Payload*)(pwp->getResponse().command.command.payload);

		for (unsigned int i=0; i < pl->len ; i++)
		{
			this->PDUe[SIZEADRRADIO+i] = (Byte)pl->frame[i];
			printf("PDUe[%d]=%x\n",i,this->PDUe[i]);
		}
	} catch (CWPCPCException& e) {
		throw e;
	} catch (CWPCNSException& e) {
		throw e;
	} catch (CWPCRIException& e) {
		throw e;
	} catch (CWPCRException& e) {
		throw e;
	}
	// en retour les données sont PDUe
	// adresse radio suivie des données de la réponse
}

/// Effectuer une requête en lecture d'un seul paramètre interne du module.
/// La lecture des paramètres se fait par une requête REQ_READ_PARAM.
/// \par l'Adresse Radio de la réponse est dans AdresseRadio.ToBytes()[] et au delà on trouve :
/// \par PDUe[SIZEADRRADIO] : la commande d'acquittement CMDAPPACK 1 octet
/// \par PDUe[SIZEADRRADIO+1] : le nombre de  paramètres lus
/// \par pour chaque paramètre
/// \arg le numéro du paramètre1 octet
/// \arg la taille du paramètre
/// \arg les données[tailleparamètre]
/// \par Cette méthode permet d'interroger un paramètre unique, donc PDUe[SIZEADRRADIO+1] = 1
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CModuleFinal::CmdReadParam(enum PARAM NumParam, enum SIZEREQPARAM Taille) {
	NbDonneesSpecifiques = 2; // 2 données spécifiques numéro du param taille du paramètre
	int i = SIZEADRRADIO; // premier indice pour compléter la PDUe après Adresse Radio
	PDUe[i++] = REQ_READ_PARAMETER;         // la commande applicative
	PDUe[i++] = 1;                          // Un seul param lu
	PDUe[i++] = NumParam;                   // Numéro du paramètre à lire
	PDUe[i++] = Taille;                  // Taille en octets du paramètre à lire
	try {
		this->pwp->CmdRadioMd(this->PDUe,
				SIZEADRRADIO + 1 + 1 + NbDonneesSpecifiques);
	} catch (CWPCPCException& e) {
		throw e;
	} catch (CWPCNSException& e) {
		throw e;
	} catch (CWPCRIException& e) {
		throw e;
	} catch (CWPCRException& e) {
		throw e;
	}

}

/// Effectuer une requête en lecture de 1 à 9 paramètres internes du module.
/// La lecture des paramètres se fait par une requête REQ_READ_PARAM. Dans une requête il est possible de lire jusqu'à 9 paramètres.
/// \par l'Adresse Radio de la réponse est dans AdresseRadio[] et au delà on trouve :
/// \par PDUe[SIZEADRRADIO] : la commande d'acquittement CMDAPPACK 1 octet
/// \par PDUe[SIZEADRRADIO+1] : le nombre de  paramètres lus
/// \par pour chaque paramètre
/// \arg le numéro du paramètre1 octet
/// \arg la taille du paramètre
/// \arg les données[tailleparamètre]
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
/// \arg CMFTPRException : le nombre de paramètres demandé est incorrecte. Il doit rester dans l'intervalle [1,9].
/// \see CMFTPRException
/// \param NbParam :  nombre de paramètres à écrire (de 1 à 9 max).
/// \param NumParam :  un tableau des numéros des paramètres à lire. A l'indice 0 se trouve le nombre de paramètres à lire (de 1 à 9 max).
/// \param Taille :  un tableau des tailles des paramètres à lire. A l'indice 0 le nombre de paramètres à lire. A l'indice 1 la taille du premier paramètre, à l'indice 9 la taille du dernier paramètre.
/// \param Resultat :  un tableau pouvant recevoir les paramètres lus dans l'ordre de la demande.
/// la taille des tableaux est à la charge de l'appelant.
void CModuleFinal::CmdReadParams(int NbParam, enum PARAM NumParam[],
		enum SIZEREQPARAM Taille[], Byte Resultat[]) {
	if (NbParam < 1 || NbParam > 9)
		throw CMFTPRException(); // exception Trop de paramètres dans une requête de lecture
	NbDonneesSpecifiques = 2 * NbParam; // 2*Nb_param_à_lire (NbParam) Données spécifiques numéro du param taille du paramètre
	int i = SIZEADRRADIO; // premier indice pour compléter la PDUe après Adresse Radio
	PDUe[i++] = REQ_READ_PARAMETER;         // la commande applicative
	PDUe[i++] = NbParam;                          // Un seul param lu
	for (int j = 0; j < NbDonneesSpecifiques;) {
		PDUe[i++] = NumParam[j++];
		PDUe[i++] = Taille[j++];
	}
	try {
		this->pwp->CmdRadioMd(this->PDUe,
				SIZEADRRADIO + 1 + 1 + NbDonneesSpecifiques);
	} catch (CWPCPCException& e) {
		throw e;
	} catch (CWPCNSException& e) {
		throw e;
	} catch (CWPCRIException& e) {
		throw e;
	} catch (CWPCRException& e) {
		throw e;
	}
	int k = 0;                  // première entrée dans le tableau des résultats
	int Offset = 0;
	for (int i = 0; i < NbParam; i++) {
		for (int j = 0; j < Taille[i]; j++, k++) {
			Resultat[k] = PDUe[SIZEADRRADIO + 1 + 1 + (1 + 1) * (i + 1) + Offset
					+ j];
		}
		Offset += Taille[i];
	}
}

/// Effectuer une requête en écriture d'un seul paramètre interne du module.
/// \param NumParam : le tableau des numéros des paramètres à écrire.
/// \param Taille :  le tableau des tailles des paramètres à écrire
/// \param Data : le tableau des données à insérer dans la PDU en émission dans l'odre des paramètres à écrire.
/// \arg Data[0 à n] contient les données à insérer.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
/// \arg CMFTPWException : le nombre de paramètres à écrire est incorrect. Il doit rester dans l'intervalle [1,9].
/// \see CMFTPWException
void CModuleFinal::CmdWriteParam(enum PARAM NumParam, enum SIZEREQPARAM Taille,
		Byte Data[]) {
	NbDonneesSpecifiques = 2; // données spécifiques numéro du param taille du paramètre
	int i = SIZEADRRADIO; // premier indice pour compléter la PDUe après Adresse Radio
	PDUe[i++] = REQ_WRITE_PARAMETER;            // la commande applicative
	PDUe[i++] = 1;                              // Un seul param
	PDUe[i++] = NumParam;                     // le numéro du paramètre à écrire
	PDUe[i++] = Taille;             // La taille en octets du paramètre à écrire
	for (int j = 0; j < Taille; i++, j++)
		PDUe[i] = Data[j + 1];            // les données commencent à l'indice 1
	try {
		this->pwp->CmdRadioMd(this->PDUe,
				SIZEADRRADIO + 1 + 1 + NbDonneesSpecifiques + Data[0]);
	} catch (CWPCPCException& e) {
		throw e;
	} catch (CWPCNSException& e) {
		throw e;
	} catch (CWPCRIException& e) {
		throw e;
	} catch (CWPCRException& e) {
		throw e;
	}
	// en retour les données sont PDUe
}

/// Effectuer une requête en écriture d'un seul paramètre interne du module.
/// \param NbParam :  nombre de paramètres à écrire. Si ce nombre est en dehors de l'intervalle [1,9], une exception CMFTPWException est levée.
/// \param NumParam : le tableau des numéros des paramètres à écrire.
/// \param Taille :  le tableau des tailles des paramètres à écrire
/// \param Data : le tableau des données à insérer dans la PDU en émission dans l'odre des paramètres à écrire.
/// \par Note :
/// \par Dans les écritures, les valeurs souhaitées des paramètres ne sont pas vérifiées. Si une valeur à écrire est incorrecte,
/// l'écriture est considérée comme bonne mais le paramètre est incorrect.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
/// \arg CMFTPWException : le nombre de paramètres à écrire est incorrect. Il doit rester dans l'intervalle [1,9].
/// \see CMFTPWException
void CModuleFinal::CmdWriteParams(int NbParam, enum PARAM NumParam[],
		enum SIZEREQPARAM Taille[], Byte Data[]) {
	if (NbParam < 1 || NbParam > 9)
		throw CMFTPWException(); // exception nombre de paramètres à écrire en dehors de l'intervalle [1,9]
	// cumuler la taille des données à écrire
	int TailleData = 0;
	for (int i = 0; i < NbParam; i++) {
		TailleData += Taille[i];
	}
	NbDonneesSpecifiques = 2 * NbParam + TailleData; // pour chaque donnée 2 octets (Num et Size) + n octets de données
	int i = SIZEADRRADIO; // premier indice pour compléter la PDUe après Adresse Radio
	PDUe[i++] = REQ_WRITE_PARAMETER;            // la commande applicative
	PDUe[i++] = NbParam;                        // NbParam paramètres à écrire
	int l = 0;                                  // position dans le tableau Data
	for (int kk = 0, j = 0; kk < NbParam; j += (2 + Taille[kk++])) {
		PDUe[i++] = NumParam[kk];             // le numéro du paramètre
		PDUe[i++] = Taille[kk];             // la taille du paramètre
		for (int k = 0; k < Taille[kk]; k++, l++) // les n octets de données du paramètre
			PDUe[i++] = Data[l];
	}
	try {
		this->pwp->CmdRadioMd(this->PDUe,
				SIZEADRRADIO + 1 + 1 + NbDonneesSpecifiques);
	} catch (CWPCPCException& e) {
		throw e;
	} catch (CWPCNSException& e) {
		throw e;
	} catch (CWPCRIException& e) {
		throw e;
	} catch (CWPCRException& e) {
		throw e;
	}
	// en retour les données sont PDUe
}

/// Pour tracer les erreurs sur un flux disponible dans l'objet CWavePort utilisé pour communiquer.
/// \param Msg : un message préalable à afficher.
/// \param Trame : la trame reçue ou émise.
/// \param Taille : taille du tableau Trame
void CModuleFinal::Tracer(string Msg, Byte Trame[], int Taille) {
	if (Stream) {
		string Entete = "\nDANS CMODULEFINAL\n";
		fwrite(Entete.c_str(), Entete.length(), 1, Stream);
		if (Trame != NULL) {
			Msg.append(" --> ");
			fwrite(Msg.c_str(), Msg.length(), 1, Stream);
			Byte Hex[15];
			for (int i = 0; i < Taille; i++) {
				sprintf((char*) Hex, "0x%0X ", (int) Trame[i]);
				string strHex((char*) Hex);
				fwrite(strHex.c_str(), strHex.length(), 1, Stream);
			}
		} else
			fwrite(Msg.c_str(), Msg.length(), 1, Stream);
		string lf = "\n";
		fwrite(lf.c_str(), lf.length(), 1, Stream);
	}
}

} // namespace WAVENIS

// Programme principal
// Si c'est la classe principale du projet, on place une fonction main()
// Dans ce cas, on peut supprimer les fichiers de tests unitaires
// ex :
// int main(int argc, char *argv[])
// {
//    // Construction de l'instance de la classe principale
//    cout << "Construction de l'instance de la classe principale CModuleFinal" << endl;
//    CModuleFinal *cmodulefinal = new CModuleFinal();
//    // Lancement de l'activité principale
//    return 0;
// }

