/**
 *  Copyright (C) 2012  plegal  (plegal@appert44.org)
 *  @file         CModuleFinal.h
 *  @brief        Pilote pour les éléments communs à tous les modules finaux WAVENIS
 *  @version      0.1
 *  @date         17 déc. 2012 21:05:37
 *
 *  @note         Voir la description detaillee explicite dans le fichier
 *                CModuleFinal.cpp
 *                C++ Google Style :
 *                http://google-styleguide.googlecode.com/svn/trunk/cppguide.xml
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef _CMODULEFINAL_H
#define _CMODULEFINAL_H

// Includes system C
//#include <stdint.h>  // definition des types int8_t, int16_t ...

// Includes system C++
#include <string>
using namespace std;
// Includes qt

// Includes application
#include "CWavePort.h"
#include "CAdrRadio.h"
#include "CDateTime.h"
#include "commun.h"
#include "protocol_types.h"	// RadioAddress
// Constantes
// ex :
// const int kDaysInAWeek = 7;
#define MAXDATA       152
#define MAXPARAM      9

#define SIZEDATE      6

// Enumerations
// ex :
// enum Couleur
// {
//     kBlue = 0,
//     kWhite,
//     kRed,
// };

// Structures
// ex:
// struct UrlTableProperties
// {
//  string name;
//  int numEntries;
// }

// Declarations de classe avancees
// ex:
// class MaClasse;

/// Espace de noms pour les classes de la librairie WAVENIS
namespace WAVENIS {

class CAdrRadio;
class CWavePort;
class CDateTime;


/** @brief Pilote pour les éléments communs à tous les modules finaux WAVENIS
 *  Description detaillee de la classe.
 */
class CModuleFinal
{
#define _MR_MF_CMDPARAMCTRL(a) \
    try{ \
      CmdParamCtrl(a);\
    } \
    catch( CWPCPCException& e) {\
      _MR_MF_THROW(e);\
    } \
    catch( CWPCNSException& e) {\
      _MR_MF_THROW(e);\
    } \
    catch( CWPCRIException& e) {\
      _MR_MF_THROW(e);\
    }\
    catch ( CWPCRException& e ) {\
      _MR_MF_THROW(e);\
    }

#define _MR_MF_CMDREADPARAM(a,b) \
  try { \
    CmdReadParam((CModuleFinal::PARAM)a,(CModuleFinal::SIZEREQPARAM)b); \
  } \
  catch ( CWPCRPException& e ) { \
    _MR_MF_THROW(e);\
  } \
  catch ( CWPCNSException& e ) { \
    _MR_MF_THROW(e);\
  } \
  catch ( CWPCRIException& e ) { \
    _MR_MF_THROW(e);\
  }\
  catch (CWPCPCException& e) {\
    _MR_MF_THROW(e);\
  }


#define _MR_MF_CMDWRITEPARAM(a,b,c) \
  try { \
    CmdWriteParam(a,b,c); \
  } \
  catch ( CWPCWPException& e ) { \
    _MR_MF_THROW(e);\
  } \
  catch ( CWPCNSException& e ) { \
    _MR_MF_THROW(e);\
  } \
  catch ( CWPCRIException& e ) { \
    _MR_MF_THROW(e);\
  }\
  catch (CWPCPCException& e) {\
    _MR_MF_THROW(e);\
  }
#if 0
#define _MR_MF_THROW(a) \
    this->Tracer(a.ToString(),NULL,0); \
    throw(a);
#else
#define _MR_MF_THROW(a) \
    throw(a);
#endif


public :

	/// Enumération des commandes applicatives
	/// \arg Commandes standard READ/WRITE pour les paramètres
	/// \arg Commandes spécifiques pour certaines informations du module.
	enum CMDAPP {
	  REQ_READ_PARAMETER  = 0x10,
	  REQ_WRITE_PARAMETER = 0x11,
	  REQ_READ_MODULE_TYPE = 0x20,
	  REQ_GET_FIRMWARE_VERSION = 0x28,
	  REQ_READ_DATE  = 0x12,
	  REQ_WRITE_DATE = 0x13,
	  REQ_SET_ALARM  = 0x23
	};

	/// Enumération des acquittements des commandes applicatives
	/// \arg Commandes standard READ/WRITE pour les paramètres
	/// \arg Commandes spécifiques pour certaines informations du module.
	enum CMDAPPACK {
	  RES_READ_PARAMETER  = 0x90,
	  RES_WRITE_PARAMETER = 0x91,
	  RES_READ_MODULE_TYPE = 0xA0,
	  RES_GET_FIRMWARE_VERSION = 0xA8,
	  RES_READ_DATE_TIME  = 0x92,
	  RES_WRITE_DATE_TIME = 0x93,
	  RES_SET_ALARM  = 0xA3
	};

	/// Enumération des  tailles des requête applicatives
	/// \arg Commandes standard READ/WRITE pour les paramètres
	/// \arg Commandes spécifiques pour certaines informations du module.
	/// \see CMDAPP
	enum SIZEREQCMDAPP {
	  SIZEREQ_READ_PARAMETER  = 2,
	  SIZEREQ_WRITE_PARAMETER = 2,
	  SIZEREQ_READ_MODULE_TYPE = 2,
	  SIZEREQ_GET_FIRMWARE_VERSION = 2,
	  SIZEREQ_READ_DATE  = 2,
	  SIZEREQ_WRITE_DATE = 2,
	  SIZEREQ_SET_ALARM  = 2
	};

	/// Enumération des paramètres accessibles par une commande standard READ/WRITE
	enum PARAM {
	  OPERATING_MODE  = 0x01  ,
	  WAKEUP_STATUS = 0x02  ,
	  WAKEUP_PERIOD = 0x03  ,
	  START_TIME1  = 0x04  ,
	  WAKEUP_PERIOD1 = 0x05  ,
	  START_TIME2  = 0x06  ,
	  WAKEUP_PERIOD2 = 0x07  ,
	  ENABLE_TIME = 0x08  ,
	  ENABLE_WAKEUP_PERIODS = 0x09  ,
	  APPLICATION_STATUS = 0x20  ,
	  GROUP_NUMBER = 0x85  ,
	  DATE_END_OF_BATTERY_LIFE = 0x90  ,
	  FIRMWARE_VERSION  = 0xA1  ,
	  NUMBER_REPEATERS_ALARM  = 0xB0  ,
	  REPEATER_ALARM1  = 0xB1  ,
	  REPEATER_ALARM2  = 0xB2  ,
	  REPEATER_ALARM3  = 0xB3  ,
	  ADDRESS_RECIPIENT_ALARM = 0xB4
	};

	enum SIZEREQPARAM {
	  SIZEREQ_OPERATING_MODE = 1,
	  SIZEREQ_WAKEUP_STATUS = 1,
	  SIZEREQ_WAKEUP_PERIOD = 1,
	  SIZEREQ_START_TIME1 = 1,
	  SIZEREQ_WAKEUP_PERIOD1 = 1,
	  SIZEREQ_START_TIME2 = 1,
	  SIZEREQ_WAKEUP_PERIOD2 = 1,
	  SIZEREQ_ENABLE_TIME = 1,
	  SIZEREQ_ENABLE_WAKEUP_PERIODS = 1,
	  SIZEREQ_NUMBER_DATALOGGING_LOOP = 1,
	  SIZEREQ_APPLICATION_STATUS = 1,
	  SIZEREQ_GROUP_NUMBER = 1,
	  SIZEREQ_DATE_END_OF_BATTERY_LIFE = SIZEDATE,
	  SIZEREQ_FIRMWARE_VERSION= 1,
	  SIZEREQ_NUMBER_REPEATERS_ALARM= 1,
	  SIZEREQ_REPEATER_ALARM1= SIZEADRRADIO,
	  SIZEREQ_REPEATER_ALARM2= SIZEADRRADIO,
	  SIZEREQ_REPEATER_ALARM3= SIZEADRRADIO,
	  SIZEREQ_ADDRESS_RECIPIENT_ALARM = SIZEADRRADIO
	};

    /**
     * Constructeur
     */
	CModuleFinal ( CWavePort* pwp, CAdrRadio* AdresseRadio );
	CModuleFinal ( CWavePort* pwp, RadioAddress radioAddress );
    /**
     * Destructeur
     */
	virtual ~CModuleFinal();

    // Methodes publiques de la classe
    // ex : ReturnType NomMethode(Type);
	/// Obtenir la version du firmware
	  virtual int getfirmware_version( void );
	  /// Obtenir le type du module
	  virtual Byte getModule_type( void );
	  /// Obtenir la date et l'heure du module
	  virtual CDateTime getDate_time( void );
	  /// Fixer la date et l'heure du module
	  virtual void setDate_time( CDateTime Date );
	  /// Fixer le mode d'alarme du module. Cette méthode doit être définie dans chaque type de module
	  CAdrRadio getAddress_Recipient_Alarm();
	  virtual Byte getApplication_Status();
	  virtual void setApplication_Status(Byte m_Application_Status);
	  CDateTime getDate_End_of_Battery_life();
	  Byte getEnable_Time() ;
	  Byte getEnable_WakeUp_Periods();
	  Byte getGroup_Number();
	  Byte getNumber_Repeaters_Alarm();
	  Byte getOperating_Mode();
	  CAdrRadio getRepeater_Alarm1();
	  CAdrRadio getRepeater_Alarm2();
	  CAdrRadio getRepeater_Alarm3();
	  Byte getStart_Time1();
	  Byte getStart_Time2();
	  Byte getWakeUp_Period();
	  Byte getWakeUp_Period1();
	  Byte getWakeUp_Period2();
	  Byte getWakeUp_Status();
	  void setAddress_Recipient_Alarm(CAdrRadio Address_Recipient_Alarm);
	  void setDate_End_of_Battery_life(CDateTime Date_End_of_Battery_life);
	  void setEnable_Time(Byte m_Enable_Time);
	  void setEnable_WakeUp_Periods(Byte m_Enable_WakeUp_Periods);
	  void setGroup_Number(Byte m_Group_Number);
	  void setNumber_Repeaters_Alarm(Byte m_Number_Repeaters_Alarm);
	  void setOperating_Mode(Byte m_Operating_Mode);
	  void setRepeater_Alarm1(CAdrRadio Repeater_Alarm1);
	  void setRepeater_Alarm2(CAdrRadio Repeater_Alarm2);
	  void setRepeater_Alarm3(CAdrRadio Repeater_Alarm3);
	  void setStart_Time1(Byte m_Start_Time1);
	  void setStart_Time2(Byte m_Start_Time2);
	  void setWakeUp_Period(Byte m_WakeUp_Period);
	  void setWakeUp_Period1(Byte m_WakeUp_Period1);
	  void setWakeUp_Period2(Byte m_WakeUp_Period2);
	  void setWakeUp_Status(Byte m_WakeUp_Status);

	  // pour effectuer une requête relative à une série de 9 paramètres au maximum
	  /// Acces aux paramètres par commande standard READ
	  void CmdReadParams ( int NbParam, enum PARAM NumParam[], enum SIZEREQPARAM Taille[], Byte Resultat[] );
	  /// Acces aux paramètres par commande standard WRITE
	  void CmdWriteParams ( int NbParam, enum PARAM NumParam[], enum SIZEREQPARAM Taille[], Byte Data[] );

	  //
	  string firmware_version;
    // Pour les associations :
    // Methodes publiques setter/getter (mutateurs/accesseurs) des attributs prives
    // ex :
    // void setNomAttribut(Type nomAttribut) { nomAttribut_ = nomAttribut; }
    // Type getNomAttribut(void) const { return nomAttribut_; }

protected :
    // Attributs proteges
	  // le status de l'application est analysé dans les classes dérivées
	  Byte m_Application_Status;

	  // association avec un wavePort pour la communication
	  CWavePort *pwp;
	  // PDUe[0 à 5]= ADRRADIO puis CMDAPP puis suivent les données
	  Byte PDUe[MAXDATA + SIZEADRRADIO];
	  // le nombre de données spécifiques lors d'un accès en lecture ou en écriture à un paramètre de module distant.
	  Byte NbDonneesSpecifiques;
	  /// Acces aux paramétres par commande spécifique
	  void CmdParamCtrl(enum CMDAPP Cmd);
	  // pour effectuer une requête relative au paramètre NumParam
	  /// Acces aux paramètres par commande standard READ
	  void CmdReadParam( enum PARAM NumParam, enum SIZEREQPARAM Taille );
	  /// Acces aux paramètres par commande standard WRITE
	  void CmdWriteParam( enum PARAM NumParam, enum SIZEREQPARAM Taille, Byte Data[] );

    // Methode protegees
	  /// Pour tracer les erreurs avant un système plus sophistiqué
	 	    virtual void Tracer(string Msg, Byte Trame[], int Taille);

private :
    // Attributs prives
    // ex :
    // Type nomAttribut_;
	  // nombre de paramètres accédés dans une requête standard READ ou WRITE
	   int NbParamReq;
	   // les paramètres communs à tous les modules
	   // l'adresse radio du module
	   CAdrRadio* AdresseRadio;
	   RadioAddress m_radioAddress;
	   // la version du firmware
	   int m_firmware_version;
	   // le type du module
	   Byte m_moduletype;
	   // le type de transmission
	   int m_phyconfig;
	   // la date et l'heure codé
	   Byte m_date_time[6];
	   // La date et l'heure
	   CDateTime* Date;
	   // la date et l'heure dans un objet adhoc
	   //CDateTime m_date_time;
	   Byte m_Operating_Mode;
	   Byte m_WakeUp_Status;
	   Byte m_WakeUp_Period;
	   Byte m_Start_Time1;
	   Byte m_WakeUp_Period1;
	   Byte m_Start_Time2;
	   Byte m_WakeUp_Period2;
	   Byte m_Enable_Time;
	   Byte m_Enable_WakeUp_Periods;
	   Byte m_Group_Number;
	   CDateTime* m_strDate_End_of_Battery_life;
	   Byte m_Date_End_of_Battery_life[6];
	   Byte m_Firmware_Version;
	   Byte m_Number_Repeaters_Alarm;
	   Byte m_Repeater_Alarm1[6];
	   CAdrRadio* Repeater_Alarm1;
	   Byte m_Repeater_Alarm2[6];
	   CAdrRadio* Repeater_Alarm2;
	   Byte m_Repeater_Alarm3[6];
	   CAdrRadio* Repeater_Alarm3;
	   Byte m_Address_Recipient_Alarm[6];
	   CAdrRadio* Address_Recipient_Alarm;

	   /// Le flux de traçage
	   FILE *Stream;
    // Methodes privees
};

// Methodes publiques inline
// ex :
// inline void CModuleFinal::maMethodeInline(Type valeur)
// {
//   
// }
// inline Type CModuleFinal::monAutreMethode_inline_(void)
// {
//   return 0;
// }

} // namespace WAVENIS


#endif  // _CMODULEFINAL_H

