///////////////////////////////////////////////////////////////////////////////////////
// Projet     : classes d'exception de la hiérarchie WAVENIS
// Auteur     : MR
// Date       : Mars 2010
// Version    : v4
/////////////////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <stdio.h>

using namespace std;

#include "CAdrRadio.h"
#include "CWNExceptions.h"

//using namespace RS232_v1;
using namespace WAVENIS;

namespace WAVENIS {

/// Constructeur par défaut de la classe de base des erreurs
CWNException::CWNException ( )  {
  this->ErrMsg.assign("");
}

/// Destructeur de la classe de base des erreurs
CWNException::~CWNException ( ) {}

/// Permet d'afficher le message d'erreur lié à l'exception.
/// \return une chaîne contenant le message d'erreur
string CWNException::ToString ( ) {
  return this->ErrMsg;
}

/// Constructeur par défaut de la classe d'erreur CWPREQException.
/// \par Cette exception correspond à un dépassement du nombre d'essais possible lors de la requête vers un périphérique WAVENIS.
/// \arg Par défaut le nombre d'essais maxi est de 3. XXXX il faut envisager un paramétrage pour que le pilote s'adapte aux milieux plus ou moins bruités.
CWPREQException::CWPREQException ( )
: CWNException() {
  this->ErrMsg.assign("CWPREQException ** Nombre maxi d'essais REQ atteint.\n");
}

/// Destructeur de la classe CWPREQException
CWPREQException::~CWPREQException ( ) {}

/// Constructeur par défaut de la classe d'erreur CWPRESException.
/// \par Cette exception correspond à un dépassement du nombre d'essais possible lors de la réception de la confirmation du waveport qu'il a bien pris en compte la requête en retournant la commande RES associée.
/// \arg Par défaut le nombre d'essais maxi est de 3. XXXX il faut envisager un paramétrage pour que le pilote s'adapte aux milieux plus ou moins bruités.
CWPRESException::CWPRESException ( )
: CWNException() {
  this->ErrMsg.assign("CWPRESException ** Nombre maxi d'essais RES atteint.\n");
}

/// destructeur de la classe CWPRESException
CWPRESException::~CWPRESException ( ) {}

/// Constructeur par défaut de la classe d'erreur CWPRECException.
/// \par Cette exception correspond à un dépassement du nombre d'essais possible lors de la réception de la réponse d'un module distant en retournant la commande RECEIVED associée.
/// \arg Par défaut le nombre d'essais maxi est de 3. XXXX il faut envisager un paramétrage pour que le pilote s'adapte aux milieux plus ou moins bruités.
CWPRECException::CWPRECException ( )
: CWNException() {
  this->ErrMsg.assign("CWPRECException ** Nombre maxi d'essais RECEIVED atteint lors de la réponse d'un module distant.\n");
}

/// destructeur de la classe CWPRECException
CWPRECException::~CWPRECException ( ) {}

/// Constructeur de la classe d'erreur CWPCRPException (CommandeReadParam exception)
/// \par Cette exception correspond à un problème lors d'une tentative de lecture d'un paramètre fonctionnel du waveport.
/// \param e : l'exception de niveau inférieure reçue
/// \param NumParam :  le numéro du paramètre concerné
CWPCRPException::CWPCRPException ( CWNException& e, int NumParam )
: CWNException() {
  char Tmp[20];
  this->ErrMsg.assign(e.ToString());
  this->ErrMsg.append("\nCWPCRPException ** Erreur lecture du paramètre ");
  sprintf ( Tmp,"0x%02X\n", NumParam);
  this->ErrMsg.append(Tmp);
}

/// Constructeur de la classe d'erreur CWPCRPException (CommandeReadParam exception)
/// \par Cette exception correspond à un problème lors d'une tentative de lecture d'un paramètre fonctionnel du waveport.
/// \param e : l'exception de niveau inférieure reçue
/// \param NumParam :  le numéro du paramètre concerné
//CWPCRPException::CWPCRPException ( SPException& e, int NumParam )
//: CWNException() {
//  char Tmp[20];
//  this->ErrMsg.assign(e.ToString());
//  this->ErrMsg.append("\nCWPCRPException ** nErreur lecture du paramètre ");
//  sprintf ( Tmp,"0x%02X\n", NumParam);
//  this->ErrMsg.append(Tmp);
//}

/// destructeur de la classe CWPCRPException
CWPCRPException::~CWPCRPException ( ) {}

/// Constructeur de la classe d'erreur CWPCWPException (CommandeWriteParam exception)
/// \par Cette exception correspond à un problème lors d'une tentative d'écritured'un paramètre fonctionnel du waveport.
/// \param e : l'exception de niveau inférieure reçue
/// \param NumParam :  le numéro du paramètre concerné
CWPCWPException::CWPCWPException ( CWNException& e, int NumParam )
: CWNException() {
  char Tmp[20];
  this->ErrMsg.assign(e.ToString());
  this->ErrMsg.append("\nCWPCWPException ** Erreur d'écriture du paramètre ");
  sprintf ( Tmp,"0x%02X\n", NumParam);
  this->ErrMsg.append(Tmp);
}

/// Constructeur de la classe d'erreur CWPCWPException (CommandeWriteParam exception)
/// \par Cette exception correspond à un problème lors d'une tentative d'écritured'un paramètre fonctionnel du waveport.
/// \param e : l'exception de niveau inférieure reçue
/// \param NumParam :  le numéro du paramètre concerné
//CWPCWPException::CWPCWPException ( SPException& e, int NumParam )
//: CWNException() {
//  char Tmp[20];
//  this->ErrMsg.assign(e.ToString());
//  this->ErrMsg.append("\nCWPCWPException ** Erreur d'écriture du paramètre ");
//  sprintf ( Tmp,"0x%02X\n", NumParam);
//  this->ErrMsg.append(Tmp);
//}

/// destructeur de la classe CWPCWPException
CWPCWPException::~CWPCWPException ( ) {}

/// Constructeur de la classe d'erreur CWPCPCException (CommandeParamCtrl exception)
/// \par Cette exception correspond à un problème lors d'une tentative de lecture ou d'éciture d'un paramètre fonctionnel du waveport.
/// \arg C'est le code de commande <ui différentie une opération d'écriture d'une opération de lecture.
/// \param e : l'exception de niveau inférieure reçue
/// \param CmdParamCtrl :  la commande de contrôle du paramètre concerné
CWPCPCException::CWPCPCException ( CWNException e, int CmdParamCtrl )
: CWNException() {
  char Tmp[10];
  this->ErrMsg.assign(e.ToString());
  this->ErrMsg.append("\nCWPCPCException ** Erreur de requête de paramétrage de contrôle ");
  sprintf ( Tmp,"0x%02X\n", CmdParamCtrl);
  this->ErrMsg.append(Tmp);
}

/// Constructeur de la classe d'erreur CWPCPCException (CommandeParamCtrl exception)
/// \par Cette exception correspond à un problème lors d'une tentative de lecture ou d'éciture d'un paramètre fonctionnel du waveport.
/// \arg C'est le code de commande <ui différentie une opération d'écriture d'une opération de lecture.
/// \param e : l'exception de niveau inférieure reçue
/// \param CmdParamCtrl : la commande de contrôle du paramètre concerné
//CWPCPCException::CWPCPCException ( SPException e, int CmdParamCtrl )
//: CWNException() {
//  char Tmp[10];
//  this->ErrMsg.assign(e.ToString());
//  this->ErrMsg.append("\nCWPCPCException ** Erreur de requête de paramétrage de contrôle ");
//  sprintf ( Tmp,"0x%02X\n", CmdParamCtrl);
//  this->ErrMsg.append(Tmp);
//}

/// destructeur de la classe CWPCPCException
CWPCPCException::~CWPCPCException ( ) {}

/// Constructeur de la classe d'erreur CWPCNSException (CommandeNonSupportée exception)
/// \par Cette exception correspond à un problème lorsqu'une commande n'est pas supportée par un périphérique.
/// \param CmdNonSupportee :  la commande non supportée
CWPCNSException::CWPCNSException ( int CmdNonSupportee )
: CWNException() {
  char Tmp[80];
  sprintf ( Tmp,"CWPCNSException ** Commande 0x%02X non supportée\n", CmdNonSupportee);
  this->ErrMsg.append(Tmp);
}

/// Constructeur de la classe d'erreur CWPCNSException (CommandeNonSupportée exception)
/// \par Cette exception correspond à un problème lorsqu'une commande n'est pas supportée par un périphérique.
/// \param CmdNonSupportee :  la commande non supportée
/// \param NumParam : Le numéro du paramètre concerné par la commande non supportée.
CWPCNSException::CWPCNSException ( int CmdNonSupportee, int NumParam )
: CWNException() {
  char Tmp[80];
  sprintf ( Tmp,"CWPCNSException ** Commande 0x%02X non supportée pour le paramètre 0x%02X.\n", CmdNonSupportee, NumParam);
  this->ErrMsg.append(Tmp);
}

/// Constructeur de la classe d'erreur CWPCNSException (CommandeNonSupportée exception)
/// \par Cette exception correspond à un problème lorsqu'une commande de service n'est pas supportée par un périphérique.
/// \param AdresseRadio : l'adresse radio du périphérique cible.
/// \param ReqType : le type de requête de service effectué
CWPCNSException::CWPCNSException ( CAdrRadio* AdresseRadio, unsigned char ReqType )
: CWNException () {
  char Tmp[80];
  sprintf ( Tmp,"CWPCNSException ** Type de requête 0x%02X incorrecte pour l'adresse radio 0x%02X 0x%02X 0x%02X 0x%02X 0x%02X 0x%02X.\n"
               ,ReqType,AdresseRadio->ToBytes()[0],AdresseRadio->ToBytes()[1],AdresseRadio->ToBytes()[2],AdresseRadio->ToBytes()[3]
               ,AdresseRadio->ToBytes()[4],AdresseRadio->ToBytes()[5] );
  this->ErrMsg.append(Tmp);
}

/// destructeur de la classe CWPCNSException
CWPCNSException::~CWPCNSException ( ) {}

/// Constructeur de la classe d'erreur CWPCRIException (CommandeResInconnue exception)
/// \par Cette exception correspond à un problème lorsque la réponse du périphérique contient une commande RES inconnue.
/// \param CmdReq : la commande de requête effectuée.
/// \param CmdResRecue : la commande d' acquittement retournée inconnue.
CWPCRIException::CWPCRIException ( int CmdReq, int CmdResRecue )
: CWNException() {
  char Tmp[80];
  sprintf ( Tmp,"CWPCRIException ** Commande RES reçue : 0x%02X\t en réponse à la commande REQ 0x02%x", CmdResRecue, CmdReq );
  this->ErrMsg.append(Tmp);
}

/// Constructeur de la classe d'erreur CWPCRIException (CommandeResInconnue exception)
/// \par Cette exception correspond à un problème lorsque la réponse du périphérique contient une commande RES inconnue.
/// \param CmdReq : la commande de requête effectuée.
/// \param CmdResRecue : la commande d' acquittement retournée inconnue.
/// \param NumParam : le numéro du paramètre concerné
CWPCRIException::CWPCRIException ( int CmdReq, int CmdResRecue, int NumParam )
: CWNException() {
  char Tmp[80];
  sprintf ( Tmp,"CWPCRIException ** Commande RES reçue : 0x%02X en réponse à la commande REQ : 0x%02X pour le paramètre 0x%02X\t", CmdResRecue, CmdReq, NumParam );
  this->ErrMsg.append(Tmp);
}

/// destructeur de la classe CWPCRIException
CWPCRIException::~CWPCRIException ( ) {}

/// Constructeur par défaut de la classe d'erreur CWPCRException (CommandeRadio exception)
/// \par Cette exception correspond à un problème lors de requêtes destinées à communiquer avec des modules distants par Radio.
CWPCRException::CWPCRException ( )
: CWNException() {
}

/// Constructeur de la classe d'erreur CWPCRException (CommandeRadio exception)
/// \par Cette exception correspond à un problème lors de requêtes destinées à communiquer avec des modules distants par Radio.
/// \param e : l'exception de niveau inférieur
/// \param Cmd : le code commande de la requête concernée
/// \param AdresseRadio :  l'adresse radio do module distant concerné.
CWPCRException::CWPCRException ( CWNException& e, int Cmd, CAdrRadio* AdresseRadio )
: CWNException() {
  char Tmp[256];
  this->ErrMsg.assign(e.ToString());
  sprintf ( Tmp,"CWPCRException ** Erreur de requête :0x%02X pour le module distant d'adresse radio: 0x%02X 0x%02X 0x%02X 0x%02X 0x%02X 0x%02X.\n"
               , Cmd,AdresseRadio->ToBytes()[0],AdresseRadio->ToBytes()[1],AdresseRadio->ToBytes()[2]
               ,AdresseRadio->ToBytes()[3],AdresseRadio->ToBytes()[4],AdresseRadio->ToBytes()[5] );
  this->ErrMsg.append(Tmp);
}

/// Constructeur de la classe d'erreur CWPCRException (CommandeRadio exception)
/// \par Cette exception correspond à un problème lors de requêtes destinées à communiquer avec des modules distants par Radio.
/// \param e : l'exception de niveau inférieur
/// \param Cmd : le code commande de la requête concernée
/// \param AdresseRadio :  l'adresse radio du module distant concerné.
//CWPCRException::CWPCRException ( SPException& e, int Cmd, CAdrRadio* AdresseRadio )
//: CWNException() {
//  char Tmp[80];
//  this->ErrMsg.assign(e.ToString());
//  sprintf ( Tmp,"CWPCRException ** Erreur de requête :0x%02X pour le module distant d'adresse radio: 0x%02X 0x%02X 0x%02X 0x%02X 0x%02X 0x%02X.\n"
//               , Cmd,AdresseRadio->ToBytes()[0],AdresseRadio->ToBytes()[1],AdresseRadio->ToBytes()[2]
//               ,AdresseRadio->ToBytes()[3],AdresseRadio->ToBytes()[4],AdresseRadio->ToBytes()[5] );
//  this->ErrMsg.append(Tmp);
//}

/// destructeur de la classe CWPCRException
CWPCRException::~CWPCRException ( ) {}


/// Constructeur de la classe d'erreur CWPSIException (StatusIncorrect exception)
/// \par Cette exception correspond à un status incorrect lors de la confirmation du waveport.
/// Le code Cmd reçu n'est pas le code RES attendu mais un code d'erreur
/// \param CmdRes : le code commande RES attendue
CWPSIException::CWPSIException ( int CmdRes )
: CWNException() {
  char Tmp[80];
  sprintf ( Tmp,"CWPSIException ** Status incorrect dans une trame RES de cmd: 0x%02X\n", CmdRes );
  this->ErrMsg.append(Tmp);
}

/// destructeur de la classe CWPSIException
CWPSIException::~CWPSIException ( ) {}

/// Constructeur de la classe d'erreur CWPCSException (CommandeService exception)
/// \par Cette exception se produit lorsqu'un problème se présente dans une requête de service.
/// \param e : l'exception de niveau inférieur
/// \param AdresseRadio :  l'adresse radio do module distant concerné.
/// \param ReqType : le type de la requête de service.
CWPCSException::CWPCSException ( CWNException e, CAdrRadio* AdresseRadio, unsigned char ReqType )
: CWNException() {
  char Tmp[80];
  this->ErrMsg.assign(e.ToString());
  sprintf ( Tmp,"CWPCSException ** Type de requête :0x%02X pour le module distant d'adresse radio: 0x%02X 0x%02X 0x%02X 0x%02X 0x%02X 0x%02X.\n"
               , ReqType,AdresseRadio->ToBytes()[0],AdresseRadio->ToBytes()[1],AdresseRadio->ToBytes()[2]
               , AdresseRadio->ToBytes()[3],AdresseRadio->ToBytes()[4],AdresseRadio->ToBytes()[5] );
  this->ErrMsg.append(Tmp);
}

/// destructeur de la classe CWPCSException
CWPCSException::~CWPCSException() {}

/// Constructeur de la classe d'erreur CWTVIException (VoieIncorrecte exception)
/// \par Cette exception se produit lorsque le numéro de capteur cible est incorrect.
/// \par Le numéro de cpateur doit être 1 ou 2 pour les périphériques Wavetherm.
CWTVIException::CWTVIException ( )
: CWNException() {
  this->ErrMsg.assign("CWTVIException ** Le numéro du capteur demandé est incorrect. [1,2] seulement.\n");
}

/// destructeur de la classe CWPCSException
CWTVIException::~CWTVIException() {}

/// Constructeur de la classe d'erreur CWTSAException (SetAlarm exception)
/// \par Cette exception se produit lorsque la mise à jour du mode alarme a échoué.
CWTSAException::CWTSAException ( )
: CWNException() {
  this->ErrMsg.assign("CWTSAException ** La mise à jour du mode alarme a échoué.\n");
}

/// destructeur de la classe CWTSAException
CWTSAException::~CWTSAException() {}


/// Constructeur de la classe d'erreur CMFTPRException (TropParamRead exception)
/// \par Cette exception se produit lorsque le nombre de paramètres demandés dans une lecture multiple est en dehors de l'intervalle [1,9].
CMFTPRException::CMFTPRException ( )
: CWNException() {
  this->ErrMsg.assign("CMFTPRException ** Nombre de paramètres demandés en lecture de 1 à 9 seulement.\n");
}

/// destructeur de la classe CWTSAException
CMFTPRException::~CMFTPRException() {}

/// Constructeur de la classe d'erreur CMFTPWException (TropParamWrite exception)
/// \par Cette exception se produit lorsque le nombre de paramètres demandés dans écrituremultiple est en dehors de l'intervalle [1,9].
CMFTPWException::CMFTPWException ( )
: CWNException() {
  this->ErrMsg.assign("CMFTPWException ** Nombre de paramètres à écrire de 1 à 9 seulement.\n");
}

/// destructeur de la classe CWTSAException
CMFTPWException::~CMFTPWException() {}

/// Constructeur de la classe d'erreur CWTEUDException (ErreurUserData exception)
/// \par Cette exception se produit lorsqu'une erreur se produit dans l'écriture de données utilisateur.
CWTEUDException::CWTEUDException ( )
: CWNException() {
  this->ErrMsg.assign("CWTEUDException ** Erreur dans l'écriture de données utilisateur.\n");
}

/// destructeur de la classe CWTSAException
CWTEUDException::~CWTEUDException() {}

/// Constructeur de la classe CDTFMTException.
CDTFMTException::CDTFMTException()
: CWNException(){
  this->ErrMsg.append("CDTException ** Format de date incorrect. le format est JSJJMMAAAAHHMN -> Jour de la semaine(Lu Ma Me Je Ve Sa Di),Mois, Année, heure, minutes.");
}

CDTFMTException::~CDTFMTException() {}


/// Constructeur de la classe CDTFMTException.
CMFUPDTException::CMFUPDTException()
: CWNException(){
  this->ErrMsg.append("CMFUPDTException ** Erreur de mise à jour de la date du module.");
}

CMFUPDTException::~CMFUPDTException() {}

/// Constructeur de la classe CARFMTException.
CARFMTException::CARFMTException()
: CWNException() {
  this->ErrMsg.append("CARFMTException ** Format d'adresse radio incorrect. le format est AABBCCDDEEFF.");
}

CARFMTException::~CARFMTException() {}

/// Constructeur de la classe CARFMTException.
CCFILEException::CCFILEException()
: CWNException() {
  this->ErrMsg.append("CCFILEException ** Fichier de configuration inconnu.");
}

CCFILEException::~CCFILEException() {}

}
