//////////////////////////////////////////////////////////////////////////////////////
// Projet     : classes d'exception de la hiérarchie WAVENIS
// Auteur     : MR
// Date       : Février 2010
// Version    : v1
// Description: Classes d'exception de la librairie WAVENIS
/////////////////////////////////////////////////////////////////////////////////////////

#ifndef CWNEXCEPTIONS_H_
#define CWNEXCEPTIONS_H_

#include <iostream>
using namespace std;

//#include "SerialPort.h"
//
//using namespace RS232_v1;

namespace WAVENIS {

class CAdrRadio;

/// Classe de base pour les exceptions des classes WAVENIS
class CWNException {
public:

  CWNException ( );
  virtual ~CWNException();
  string ToString();

protected:
  string ErrMsg;

};

/// Classe d'exception pour la classe CWavePort : une requête REQ s'est mal passée.
class CWPREQException : public CWNException {
public:
  CWPREQException ( );
  virtual ~CWPREQException();

};

/// Classe d'exception pour la classe CWavePort : une confirmation RES a été mal reçue.
class CWPRESException : public CWNException {
public:
  CWPRESException ( );
  virtual ~CWPRESException();

};

/// Classe d'exception pour la classe CWavePort : la réponse du module distant est incorrecte.
class CWPRECException : public CWNException {
public:
  CWPRECException ( );
  virtual ~CWPRECException();

};

/// Classe d'exception pour la classe CWavePort : la lecture d'un paramètre s'est mal passée.
class CWPCRPException : public CWNException {
public:
  CWPCRPException ( CWNException& e, int NumParam );
//  CWPCRPException ( SPException& e, int NumParam );
  virtual ~CWPCRPException();

};

/// Classe d'exception pour la classe CWavePort : l'écriture d'un paramètre s'est mal passée.
class CWPCWPException : public CWNException {
public:
  CWPCWPException ( CWNException& e, int NumParam );
//  CWPCWPException ( SPException& e, int NumParam );
  virtual ~CWPCWPException();

};

/// Classe d'exception pour la classe CWavePort : l'accés à un paramètre de contrôle s'est mal passé.
class CWPCPCException : public CWNException {
public:
  CWPCPCException ( CWNException e, int CmdParamCtrl );
//  CWPCPCException ( SPException e, int CmdParamCtrl );
  virtual ~CWPCPCException();

};

/// Classe d'exception pour la classe CWavePort : la commande tentée sur le module n'est pas supportée par celui-ci.
class CWPCNSException : public CWNException {
public:
  CWPCNSException ( int CmdNonSupportee );
  CWPCNSException ( int CmdNonSupportee, int NumParam );
  CWPCNSException ( CAdrRadio* AdrRadio, unsigned char ReqType );
  virtual ~CWPCNSException();

};

/// Classe d'exception pour la classe CWavePort : la réponse du périphérique contient contient une commande RES inconnue.
class CWPCRIException : public CWNException {
public:
  CWPCRIException ( int CmdReq, int CmdResRecue );
  CWPCRIException ( int CmdReq, int CmdResRecue, int NumParam );
  virtual ~CWPCRIException();

};

/// Classe d'exception pour la classe CWavePort : une commande radio s'est mal passée
class CWPCRException : public CWNException {
public:
  CWPCRException ( );
  CWPCRException ( CWNException& e, int Cmd, CAdrRadio* AdresseRadio );
//  CWPCRException ( SPException& e, int Cmd, CAdrRadio* AdresseRadio );
  virtual ~CWPCRException();

};

/// Classe d'exception pour la classe CWavePort : la réponse du périphérique contient un Status incorrect.
class CWPSIException : public CWNException {
public:
  CWPSIException ( int CmdRes );
  virtual ~CWPSIException();

};

/// Classe d'exception pour la classe CWavePort : la commande de service a échoué
class CWPCSException : public CWNException {
public:
  CWPCSException ( CWNException e, CAdrRadio* AdrRadio, unsigned char ReqType );
  virtual ~CWPCSException();

};

/// Classe d'exception pour la classe CWaveTherm : la voie demandée pour un périphérique Wavetherm est incorrecte.
class CWTVIException : public CWNException {
public:
  CWTVIException ( );
  virtual ~CWTVIException();

};

/// Classe d'exception pour la classe CWaveTherm : l'installation de la gestion des alarmes a échoué.
class CWTSAException : public CWNException {
public:
  CWTSAException ( );
  virtual ~CWTSAException();

};

/// Classe d'exception pour la classe CModuleFinal : la demande de lecture porte sur trop de paramètres. 9 paramètres au maximum.
class CMFTPRException : public CWNException {
public:
  CMFTPRException ( );
  virtual ~CMFTPRException();

};

/// Classe d'exception pour la classe CModuleFinal : la demande d'écriture sur trop de paramètres. 9 paramètres au maximum.
class CMFTPWException : public CWNException {
public:
  CMFTPWException ( );
  virtual ~CMFTPWException();

};

/// Classe d'exception pour la classe CWaveTherm : la zone des données utilisateur accédée est incorrecte.
class CWTEUDException : public CWNException {
public:
  CWTEUDException ( );
  virtual ~CWTEUDException();

};

/// Classe d'exception pour la classe CDateTime : erreur de format.
/// \par le format choisi est : JSJJMMAAAAHHMM
/// \par exemple : Lu2104201012:41 Lundi 21 avril 2010 12h41
/// \arg JS : le jour de la semaine
///-# lu Lundi
///-# Ma Mardi
///-# Me Mecredi
///-# Je Jeudi
///-# Ve Vendredi
///-# Sa Samedi
///-# Di Dimanche
/// \arg JJMMAAAA : jour mois année (toujours à partir de 2000)
/// \arg HHMM : heure minute(s)
class CDTFMTException : public CWNException {
public:
  CDTFMTException ( );
  virtual ~CDTFMTException();

};

/// Classe d'exception pour la classe CModuleFinal : erreur de mise à jour de la date.
class CMFUPDTException : public CWNException {
public:
  CMFUPDTException ( );
  virtual ~CMFUPDTException();

};

/// Classe d'exception pour la classe CAdrRadio : erreur de format.
/// \par le format choisi est : AABBCCDDEEFF soit 48 bits
/// \par exemple : 05190930040C
class CARFMTException : public CWNException {
public:
  CARFMTException ( );
  virtual ~CARFMTException();

};

/// Classe d'exception pour la classe CConfig : erreur de fichier.
/// \par Le fichier de configutraztion n'a pas étét trouvé.
class CCFILEException : public CWNException {
public:
  CCFILEException ( );
  virtual ~CCFILEException();

};

}

#endif /* CWNEXCEPTIONS_H_ */
