/**
 *  Copyright (C) 2012  plegal  (plegal@appert44.org)
 *  @file         CWavePort.cpp
 *  @brief        Pilote pour le transmetteur RS232 de WAVENIS
 *  @version      0.1
 *  @date         17 déc. 2012 21:20:15
 *
 *  Description detaillee du fichier CWavePort.cpp
 *  Fabrication   gcc (Ubuntu/Linaro 4.6.3-1ubuntu5) 4.6.3 
 *  @todo         Liste des choses restant a faire.
 *  @bug          17 déc. 2012 21:20:15 - Aucun pour l'instant
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Includes system C
 #include <string.h>	// memset()
#include <sys/time.h>  // gettimeofday()
// Includes system C++
// A décommenter si besoin cout, cin, ...
// #include <iostream>
// Includes qt
// Includes application
// serial driver api include
#include <serial_driver_event.h>
#include <serial_driver_status.h>
//// protocol api include
#include <protocol_command_types.h>
#include <protocol_response_types.h>
//// domain include
#include "loader_serial_driver.h"
#include "loader_protocol.h"
#include "serial_driver_types.h"
#include "protocol_types.h"
#include "protocol_types_utils.h"

#include "CWavePort.h"
#include "CWNExceptions.h"
//#include "serial_driver_status.h" // pour OK_SERIALDRIVER_SUCCESS,
//#include "loader_serial_driver.h"
//#include "loader_protocol.h"
// protocol api include

//#include "protocol_types.h"
//#include "protocol_types_utils.h"

#define _DEBUG

/// Espace de noms pour les classes de la librairie WAVENIS
namespace WAVENIS {

/// Constructeur 1 de la classe
/// \arg le port série associé est le port par défaut de la classe SerialPort. Il sera ouvert lors de l'appel à la méthode Open().
/// \arg l'adresse radio est forcée à 0 et devra être lue dans le composant avant d'être utilisée.
/// \param Conf :  l'objet de configuration du protocole WAVENIS
/// \param Stream : un flux pour la sortie des messages de traçage.
/// \arg Si Stream=NULL : aucune trace n'est crée
/// \arg Le flux de sortie peut être un fichier régulier de type texte, mais aussi un périphérique comme
/// un terminal (/dev/pts/1), la sortie standard (stdout) ou la sortie erreur (stderr).
/// \see getRadio_address
/// \see CConfig
CWavePort::CWavePort(CConfig& Conf, FILE* Stream) {
	// ouvrir le port série associé
	//this->pS = new SerialPort();    // le port par défaut de la classe est /dev/ttyM0
	//this->Flush();
	// proprety ReadTimeout
	this->ReadTimeout.setContainer(this);
	ReadTimeout.setter(&CWavePort::set_ReadTimeout);
	ReadTimeout.getter(&CWavePort::get_ReadTimeout);
	// le flux de traçage
	this->Stream = Stream;
	// l'adresse radio n'est pas connue. Elle est forcée à 0.
	this->m_radio_address = new CAdrRadio("000000000000");
	this->m_Nb_Polling_Route = 0;
	// les constantes de la trame WAVENIS
	this->PDUe[0] = SYNC;
	this->PDUe[1] = STX;
	// les paramètres lus dans la config
	this->RetryRec = Conf.MaxRetryREC;
	this->RetryRes = Conf.MaxRetryRES;
	this->RetryReq = Conf.MaxRetryREQ;
	this->RetryTO = Conf.MaxRetryTO;
	this->ReadTimeout = Conf.ReadTimeout;
	m_driverId = "csrs232driver";
	m_protocolId = "cswaveportprotocol";
}

/// Constructeur 2 de la classe  void Erreur( std::string Msg, Byte Trame[] );   // en cas d'erreur
/// \arg le port série associé est le port par défaut de la classe SerialPort. Il sera ouvert lors de l'appel à la méthode Open().
/// \param Conf :  l'objet de configuration du protocole WAVENIS
/// \param AdrRadio : l'adresse Radio du waveport sur 6 octets.
/// \param Stream : un flux pour la sortie des messages de traçage.
/// \arg Si Stream=NULL : aucune trace n'est crée
/// \arg Le flux de sortie peut être un fichier régulier de type texte, mais aussi un périphérique comme
/// un terminal (/dev/pts/1), la sortie standard (stdout) ou la sortie erreur (stderr).
/// \see Open
CWavePort::CWavePort(CConfig& Conf, CAdrRadio* AdrRadio, FILE* Stream) {
	// ouvrir le port série associé
	//this->pS = new SerialPort();    // le port par défaut de la classe est /dev/ttyM0
	//this->Flush();
	// property ReadTimeout
	this->ReadTimeout.setContainer(this);
	ReadTimeout.setter(&CWavePort::set_ReadTimeout);
	ReadTimeout.getter(&CWavePort::get_ReadTimeout);
	// le flux de traçage
	this->Stream = Stream;
	// l'adresse radio
	this->m_radio_address = new CAdrRadio(AdrRadio->ToString());
	this->m_Nb_Polling_Route = 0;
	// les constantes de la trame WAVENIS
	this->PDUe[0] = SYNC;
	this->PDUe[1] = STX;
	// les paramètres lus dans la config
	this->RetryRec = Conf.MaxRetryREC;
	this->RetryRes = Conf.MaxRetryRES;
	this->RetryReq = Conf.MaxRetryREQ;
	this->RetryTO = Conf.MaxRetryTO;
	this->ReadTimeout = Conf.ReadTimeout; // met à jour le timeout dans le pilote série
}

/// Constructeur 3 de la classe
/// \param Conf :  l'objet de configuration du protocole WAVENIS
/// \param PortName : le nom du port série associé. Le port est ouvert lors de l'appel à CWavePort::Open();
/// \param AdrRadio : l'adresse Radio du waveport sur 6 octets.
/// \param Stream : un flux pour la sortie des messages de traçage.
/// \arg Si Stream=NULL : aucune trace n'est crée
/// \arg Le flux de sortie peut être un fichier régulier de type texte, mais aussi un périphérique comme
/// un terminal (/dev/pts/1), la sortie standard (stdout) ou la sortie erreur (stderr).
/// \see Open
CWavePort::CWavePort(CConfig& Conf, string PortName, CAdrRadio* AdrRadio,
		FILE* Stream) {
	// ouvrir le port série associé
	this->PortName = PortName;
	//this->pS = new SerialPort(this->PortName);
	//this->Flush();
	// proprety ReadTimeout
	ReadTimeout.setContainer(this);
	ReadTimeout.setter(&CWavePort::set_ReadTimeout);
	ReadTimeout.getter(&CWavePort::get_ReadTimeout);
	// le flux de traçage
	this->Stream = Stream;
	// l'adresse radio
	this->m_radio_address = new CAdrRadio(AdrRadio->ToString());
	this->m_Nb_Polling_Route = 0;
	// les constantes de la trame WAVENIS
	this->PDUe[0] = SYNC;
	this->PDUe[1] = STX;
	// les paramètres lus dans la config
	this->RetryRec = Conf.MaxRetryREC;
	this->RetryRes = Conf.MaxRetryRES;
	this->RetryReq = Conf.MaxRetryREQ;
	this->RetryTO = Conf.MaxRetryTO;
	this->ReadTimeout = Conf.ReadTimeout;
}

/// Constructeur 4 de la classe
/// \param Conf :  l'objet de configuration du protocole WAVENIS
/// \param PortName : le nom du port série associé. Le port est ouvert lors de l'appel à CWavePort::Open();
/// \arg l'adresse radio est forcée à 0 et devra être lue dans le composant avant d'être utilisée.
/// \param Stream : un flux pour la sortie des messages de traçage.
/// \arg Si Stream=NULL : aucune trace n'est crée
/// \arg Le flux de sortie peut être un fichier régulier de type texte, mais aussi un périphérique comme
/// un terminal (/dev/pts/1), la sortie standard (stdout) ou la sortie erreur (stderr).
/// \see getRadio_address
CWavePort::CWavePort(CConfig& Conf, string PortName, FILE* Stream) {
	// ouvrir le port série associé
	this->PortName = PortName;
	//this->pS = new SerialPort(this->PortName);
	//this->Flush();
	// proprety ReadTimeout
	this->ReadTimeout.setContainer(this);
	ReadTimeout.setter(&CWavePort::set_ReadTimeout);
	ReadTimeout.getter(&CWavePort::get_ReadTimeout);
	// le flux de traçage
	this->Stream = Stream;
	// l'adresse radio n'est pas connue. Elle est forcée à 0.
	this->m_radio_address = new CAdrRadio("000000000000");
	this->m_Nb_Polling_Route = 0;
	// les constantes de la trame WAVENIS
	this->PDUe[0] = SYNC;
	this->PDUe[1] = STX;
	// les paramètres lus dans la config
	this->RetryRec = Conf.MaxRetryREC;
	this->RetryRes = Conf.MaxRetryRES;
	this->RetryReq = Conf.MaxRetryREQ;
	this->RetryTO = Conf.MaxRetryTO;
	this->ReadTimeout = Conf.ReadTimeout;
}

CWavePort::~CWavePort() {
	//delete pS;
}

/// Fixer le flux de sortie pour le traçage du fonctionnement.
/// \par cette méthode est utilisée lorsque le flux n'est pas défini à la création de l'objet Waveport ou pour changer le flux de sortie durant l'exécution.
/// \par La fermeture du flux est toujours à la charge de l'appelant.
/// \param f : le flux de sortie
/// \arg Le flux de sortie peut être un fichier régulier de type texte, mais aussi un périphérique comme
/// un terminal (/dev/pts/1), la sortie standard (stdout) ou la sortie erreur (stderr).
void CWavePort::setFlux_trace(FILE* f) {
	this->Stream = f;
}

/// Fixer l'adresse radio du module associé.
/// \par Cette méthode permet de fixer l'adresse radio du module après la création de l'objet lorsque celle-ci n'est pas connue à la création.
/// \par L'adresse radio est obtenue par getRadio_address().
/// \param AdrRadio :  l'adresse radio du module
/// \see getRadio_address
void CWavePort::setRadio_address(CAdrRadio* AdrRadio) {
	*this->m_radio_address = getRadio_address();
}

/// Fixer le délai d'attente dans les lectures sur le port série. Par défaut les lectures sont bloquantes.
/// \param ReadTimeout : le délai d'attente en multiples de 100ms (0.1s).
/// \par ReadTimeout = 0 : les lectures sont bloquantes (attente infinie).
/// \arg ReadTimeout=5   -> délai d'attente 500ms.
void CWavePort::set_ReadTimeout(int ReadTimeout) {
	//this->pS->ReadTimeout = ReadTimeout;
	this->m_ReadTimeout = ReadTimeout;
}

/// Retrouver le timeout en lecture
/// Le délai d'attente est en multiples de 100ms (0.1s).
int CWavePort::get_ReadTimeout() const {
	//return this->pS->ReadTimeout;
	return 5;
}

/// Ouvrir l'accès au WavePort associé. le port série est ouvert.
/// \par Gestion des exceptions:
/// \arg : SPException : erreur d'ouverture du port série associé
/// \see SPException
//void CWavePort::Open ( void ) {
//  try {
//    pS->Open();
//  }
//  catch ( SPException& e) {
//    _MR_THROW(e);
//  }
//}

/// Fermer l'accès au WavePort associé. Le port série associé est fermé.
/// \par Gestion des exceptions:
/// \arg : SPException : erreur de fermeture du port série associé
/// \see SPException
//void CWavePort::Close ( void ) {
//  try {
//    this->pS->Close();
//  }
//  catch ( SPException& e) {
//    _MR_THROW(e);
//  }
//}

/// Vider les buffer du port série associé
/// Le vidage n'impose pas que le port soit ouvert.
//void CWavePort::Flush() {
//  this->pS->Flush();
//}
//CWavePort::~CWavePort() {
//  delete pS;
//}

/////////////////////////////////////////////////////////////////////
// Méthodes privées
/////////////////////////////////////////////////////////////////////

/// Pour tracer l'activité du pilote.
/// \par La sortie s'effectue sur le flux Stream défini lors de la construction de l'objet.
/// \arg Si Stream=NULL : aucune trace n'est crée
/// \arg Le flux de sortie peut être un fichier régulier de type texte, mais aussi un périphérique comme
/// un terminal (/dev/pts/1) ou la sortie standard (stdout) ou lma sortie erreur (stderr).
/// \param Msg : un message préalable à afficher.
/// \param Trame : la trame reçue ou émise.
/// \arg Si le pointeur est nul, l'affichage sera ignoré.
/// \see CWavePort::CWavePort
void CWavePort::Tracer(std::string Msg, Byte Trame[]) {
	if (Stream) {                 // un périphérique est passé
		if (Trame != NULL) {
			fwrite(Msg.c_str(), Msg.length(), 1, Stream);
			int lt = Trame[2] + 3;
			string Fl = " --> ";
			fwrite(Fl.c_str(), Fl.length(), 1, Stream);
			Byte Hex[15];
			for (int i = 0; i < lt; i++) {
				sprintf((char*) Hex, "0x%0X ", (int) Trame[i]);
				string strHex((char*) Hex);
				fwrite(strHex.c_str(), strHex.length(), 1, Stream);
			}
		} else
			fwrite(Msg.c_str(), Msg.length(), 1, Stream);
		string lf = "\n";
		fwrite(lf.c_str(), lf.length(), 1, Stream);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// PARAMETRAGE DE CONTROLE DU WAVEPORT
///////////////////////////////////////////////////////////////////////////////////////////////////

/// Retrouver la version du firmware de waveport associé ainsi que le mode de transmission actuel.
/// \return La version du firmware du module associé.
/// \arg Le mode de transmission est mis à jour et accessible par la variable TrMode
/// \par Gestion des exceptions:
/// \arg : CWPCPCException : erreur lors d'un accès à un paramètre de contrôle
/// \arg CWPSIException : erreur en cas de status incorrect dans la trame RES.
/// \see CWPCPCException
/// \see CWPSIException
/// \see m_phyconfig
/// \see TRMODE
int CWavePort::getfirmware_version(void) {
	_MR_CMDPARAMCTRL(FIRMWARE_VERSION);
	m_phyconfig = (int) (PDUr[6] + (PDUr[5] << 8)); // information complémentaires lors de l'interrogation
	m_firmware_version = (int) (PDUr[8] + (PDUr[7] << 8));
	char Txt[10];
	sprintf(Txt, "%02d.%02d", PDUr[8], PDUr[7]);
	firmware_version.assign(Txt);
	return m_firmware_version;
}

/// retrouver le niveau RSSI local = signal reçu par le module local.
/// \return Le niveau RSSI sur un octet.
/// \arg 0x00 = 0%
/// \arg 0x2F = 100%
/// \arg Un niveau entre 92 et 95% est un signal saturé
/// \par Gestion des exceptions:
/// \arg : CWPCPCException : erreur lors d'un accès à un paramètre de contrôle
/// \arg CWPSIException : erreur en cas de status incorrect dans la trame RES.
/// \see CWPCPCException
/// \see CWPSIException
Byte CWavePort::getlocal_rssi(CAdrRadio* AdrRadioRemoteModule) {
	// Remplir la zone DATA adresse
	for (int i = 0; i < 6; i++)
		this->PDUe[i + INDDEBDATA] = AdrRadioRemoteModule->ToBytes()[i];
	_MR_CMDPARAMCTRL(READ_LOCAL_RSSI);
	this->m_local_rssi = PDUr[4];
	return m_local_rssi;
}

/// retrouver le niveau RSSI distant = signal reçu par le module distant.
/// \return Le niveau RSSI sur un octet.
/// \arg 0x00 = 0%
/// \arg 0x2F = 100%
/// \arg Un niveau entre 92 et 95% est un signal saturé
/// \par Gestion des exceptions:
/// \arg : CWPCPCException : erreur lors d'un accès à un paramètre de contrôle
/// \arg CWPSIException : erreur en cas de status incorrect dans la trame RES.
/// \see CWPCPCException
/// \see CWPSIException
Byte CWavePort::getremote_rssi(CAdrRadio* AdrRadioRemoteModule) {
	// Remplir la zone DATA adresse
	for (int i = 0; i < 6; i++)
		this->PDUe[i + INDDEBDATA] = AdrRadioRemoteModule->ToBytes()[i];
	_MR_CMDPARAMCTRL(READ_REMOTE_RSSI);
	m_remote_rssi = PDUr[4];
	return m_remote_rssi;
}

/// Fixer la vitesse de transmission dans le module Waveport. Le port série associé est réglé à cette vitesse.
/// les Vitesses sont dans BAUDRATES
/// \param uart_bdrate : la vitesse à appliquer
/// \see BAUDRATES
/// \par Gestion des exceptions:
/// \arg : CWPCPCException : erreur lors d'un accès à un paramètre de contrôle
/// \arg CWPSIException : erreur en cas de status incorrect dans la trame RES.
/// \see CWPCPCException
/// \see CWPSIException
//void CWavePort::setuart_bdrate(SerialPort::BaudRates uart_bdrate)  {
//  this->PDUe[4] = uart_bdrate;
//  m_uart_bdrate = uart_bdrate;
//  this->pS->BaudRate = uart_bdrate;   // régler la vitesse de transmission sur le port série associé
//  _MR_CMDPARAMCTRL(CHANGE_UART_BDRATE);
//}

/// Retrouver la version du firmware de waveport associé ainsi que le mode de transmission actuel.
/// \return La version du firmware du module associé.
/// \arg Le mode de transmission est mis à jour et accessible par la variable TrMode
/// \see TRMODE
/// \par Gestion des exceptions:
/// \arg : CWPCPCException : erreur lors d'un accès à un paramètre de contrôle
/// \arg CWPSIException : erreur en cas de status incorrect dans la trame RES.
/// \see CWPCPCException
/// \see CWPSIException
int CWavePort::getphyconfig(void) {
	_MR_CMDPARAMCTRL(READ_PHYCONFIG);
	m_phyconfig = (int) (PDUr[6] + (PDUr[5] << 8)); // information complémentaires lors de l'interrogation
	return m_phyconfig;
}

/// Fixer le mode de transmission série et Radio du waveport associé
/// \arg Les modes de transmission sont dans TRMODE
/// \see TRMODE
/// \par Gestion des exceptions:
/// \arg : CWPCPCException : erreur lors d'un accès à un paramètre de contrôle
/// \arg CWPSIException : erreur en cas de status incorrect dans la trame RES.
/// \see CWPCPCException
/// \see CWPSIException
void CWavePort::setphyconfig(int phyconfig) {
	this->PDUe[4] = phyconfig;
	this->PDUe[5] = phyconfig >> 8;
	_MR_CMDPARAMCTRL(SELECT_PHYCONFIG);
	this->m_phyconfig = phyconfig;
}

/// Obtenir le canal radio lorsque le mode FHSS est désactivé
/// \return Le canal Radio
/// \par Gestion des exceptions:
/// \arg : CWPCPCException : erreur lors d'un accès à un paramètre de contrôle
/// \arg CWPSIException : erreur en cas de status incorrect dans la trame RES.
/// \see CWPCPCException
/// \see CWPSIException
Byte CWavePort::getchannel(void) {
	_MR_CMDPARAMCTRL(READ_CHANNEL);
	return this->PDUr[5];
}

/// Fixer le canal de transmission lorsque le mode FHSS est désactivé
/// \par Gestion des exceptions:
/// \arg : CWPCPCException : erreur lors d'un accès à un paramètre de contrôle
/// \arg CWPSIException : erreur en cas de status incorrect dans la trame RES.
/// \see CWPCPCException
/// \see CWPSIException
void CWavePort::setchannel(Byte channel) {
	this->PDUe[4] = channel;
	_MR_CMDPARAMCTRL(SELECT_CHANNEL);
	this->m_channel = channel;
}

/// Retrouver la puissance de transmission radio.
/// \return La puissance de transmission radio
/// \arg Les puissances disponibles sont dans TXPOWER
/// \see TXPOWER
/// \par Gestion des exceptions:
/// \arg : CWPCPCException : erreur lors d'un accès à un paramètre de contrôle
/// \arg CWPSIException : erreur en cas de status incorrect dans la trame RES.
/// \see CWPCPCException
/// \see CWPSIException
Byte CWavePort::gettx_power(void) {
	_MR_CMDPARAMCTRL(READ_TX_POWER);
	this->m_tx_power = this->PDUr[4];
	return this->m_tx_power;
}

/// Fixer la puissance de transmission radio.
/// \arg Les puissances disponibles sont dans TXPOWER
/// \param tx_power : la puissance de transmission à utiliser
/// \see TXPOWER
/// \par Gestion des exceptions:
/// \arg : CWPCPCException : erreur lors d'un accès à un paramètre de contrôle
/// \arg CWPSIException : erreur en cas de status incorrect dans la trame RES.
/// \see CWPCPCException
/// \see CWPSIException
void CWavePort::settx_power(Byte tx_power) {
	this->PDUe[4] = tx_power;
	_MR_CMDPARAMCTRL(CHANGE_TX_POWER);
	this->m_tx_power = tx_power;
}

/// Retrouver l'état activé/désactivé du mode auto-correction RSSI
/// \see MODERSSI
/// \return L'état activé désactivé de l'auto-correction RSSI
Byte CWavePort::getautocorr_state(void) {
	_MR_CMDPARAMCTRL(READ_AUTOCORR_STATE);
	this->m_autocorr_state = PDUr[5];
	return this->m_autocorr_state;
}

/// Fixer l'état activé/désactivé du mode auto-correction RSSI
/// \see MODERSSI
/// \par Gestion des exceptions:
/// \arg : CWPCPCException : erreur lors d'un accès à un paramètre de contrôle
/// \arg CWPSIException : erreur en cas de status incorrect dans la trame RES.
/// \see CWPCPCException
/// \see CWPSIException
void CWavePort::setautocorr_state(Byte autocorr_state) {
	this->PDUe[4] = autocorr_state;
	_MR_CMDPARAMCTRL(WRITE_AUTOCORR_STATE);
	this->m_autocorr_state = autocorr_state;
}

/// Pour tester le transmetteur.
/// \par Les différents mode de test sont :
/// \arg Réception continue
/// \arg Transmission continue sans modulation
/// \arg Transmission continue avec modulation
/// \arg Standby
/// \par Pour sortir du mode standby il faut effectuer un RESET du transmetteur
/// \par Pour sorti des autres modes de test on peut :
/// \arg Effectuer un RESET du transmetteur
/// \arg Envoyer une trame avec un octet de données à 0. La méthode FinTestMode() a été créée dans ce but.
/// \see FinTestMode
/// \param Mode : le mode de test souhaité.
void CWavePort::TestMode(Byte Mode) {
	Byte PDU[] = { SYNC, STX, 0x05, MODE_TEST, 0x00, 0x00, 0x00, ETX };
	this->PDUe[4] = Mode;      // Appliquer le mode
	this->CRC(PDU, 8);         // Calculer et insérer le CRC16
	//this->pS->Write(PDU,0,8); // Emettre la PDU
	this->m_testmode = Mode;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// PARAMETRAGE FONCTIONNEL DU WAVEPORT
// Tous les paramètres sont READ WRITE sauf l'adresse Radio qui est fixée en usine.
// Créer une PDUe
//      REQ_WRITE_RADIO_PARAM         TAILLE VARIABLE = 8 + x     PDUe[4] = Numéro du paramètre
//                                                                Les données à écrire commencent à PDUe[5]
//      REQ_READ_RADIO_PARAM          TAILLE FIXE = 8             PDUe[4] = Numéro du paramètre
//                                                                Les données à lire commencent à PDUe[5]
//    Attendre PDUACK
//    Attendre PDU RES_
//      RES_WRITE_RADIO_PARAM         TAILLE FIXE = 8             PDUe[4] = STATUS
//      RES_READ_RADIO_PARAM          TAILLE VARIABLE = 8 + x     PDUe[4] = STATUS (OK/ERROR)
//                                                                Les données reçues commencent à PDUe[5]
//    Emettre PDUACK
// La méthode RequeteParam peut être utilisée directement
///////////////////////////////////////////////////////////////////////////////////////////////////

/// Retrouver le timeout durant lequel le port série n'est plus géré
/// \return La durée du timeout
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
Byte CWavePort::getRadio_user_timeout() {
	_MR_CMDREADPARAM(RADIO_USER_TIMEOUT);
	this->m_radio_user_timeout = this->PDUr[5];
	return this->m_radio_user_timeout;
}

/// Fixer le timeout durant lequel le port série n'est plus géré
/// \param radio_user_timeout : La durée du timeout
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setRadio_user_timeout(Byte radio_user_timeout) {
	_MR_CMDWRITEPARAM( RADIO_USER_TIMEOUT, &radio_user_timeout, 1);
	this->m_radio_user_timeout = radio_user_timeout;
}

/// Retrouver la période lors d'un polling des modules.
/// \arg La période est en multiple de 100ms
/// \arg la valeur par défaut est 0x0A soit 10*100ms = 1s
/// \return La période de polling
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
Byte CWavePort::getAwakening_period() {
	_MR_CMDREADPARAM(AWAKENING_PERIOD);
	this->m_awakening_period = this->PDUr[5];
	return m_awakening_period;
}

/// Fixer la période lors d'un polling des modules.
/// \arg La période est en multiple de 100ms
/// \param awakening_period : la période de polling souhaitée
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setAwakening_period(Byte awakening_period) {
	_MR_CMDWRITEPARAM( AWAKENING_PERIOD, &awakening_period, 1);
	this->m_awakening_period = awakening_period;
}

/// Retrouver le délai d'attente entre la réception d'une trame CSMA et une requête de diffudion REQ_SEND_BROADCAST
/// \arg Le délai est en multiple de 100ms
/// \arg la valeur par défaut est 0x3C soit 60*100ms = 6s
/// \return Le délai d'attente en nombre de fois 100ms.
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
Byte CWavePort::getBcst_reception_timeout() {
	_MR_CMDREADPARAM(BCST_RECEPTION_TIMEOUT);
	this->m_bcst_reception_timeout = this->PDUr[5];
	return m_bcst_reception_timeout;
}

/// Fixer le délai d'attente entre la réception d'une trame CSMA et une requête de diffudion REQ_SEND_BROADCAST
/// \arg La période est en multiple de 100ms
/// \arg la valeur par défaut est 0x3C soit 60*100ms = 6s
/// \param bcst_reception_timeout : le délai d'attente souhaité exprimé en multiple de 100ms.
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setBcst_reception_timeout(Byte bcst_reception_timeout) {
	_MR_CMDWRITEPARAM( BCST_RECEPTION_TIMEOUT, &bcst_reception_timeout, 1);
	this->m_bcst_reception_timeout = bcst_reception_timeout;
}

/// Retrouver le mode relatif à la gestion des erreurs de transmission.
/// \par Les valeurs possibles sont :
/// \arg 0 : mode état et erreur désactivé
/// \arg 1 : mode erreur de trame activé
/// \arg 2 : mode Etat de trame activé
/// \arg 3 : mode erreur et mode état activés simultanément
/// \arg la valeur par défaut est 0 : modes erreur et status désactivés
/// \return Le mode actuel
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
Byte CWavePort::getExchange_status() {
	_MR_CMDREADPARAM(EXCHANGE_STATUS);
	this->m_exchange_status = this->PDUr[5];
	return m_exchange_status;
}

/// Fixer le mode relatif à la gestion des erreurs de transmission.
/// \par Les valeurs possibles sont :
/// \arg 0 :  mode Etat/erreur désactivé
/// \arg 1 : mode erreur de trame activé
/// \arg 2 : mode status de trame activé
/// \arg 3 : mode erreur et mode status activés simultanément
/// \arg la valeur par défaut est 0 : modes erreur et status désactivés
/// \param exchange_status :  le mode souhaité
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setExchange_status(Byte exchange_status) {
	_MR_CMDWRITEPARAM( EXCHANGE_STATUS, &exchange_status, 1);
	this->m_exchange_status = exchange_status;
}

/// Retrouver le numéro de groupe lors d'un polling.
/// \arg la valeur par défaut est le groupe 0
/// \return Le numéro du groupe actuel
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
Byte CWavePort::getGroup_number() {
	_MR_CMDREADPARAM(GROUP_NUMBER);
	this->m_group_number = this->PDUr[5];
	return m_group_number;
}

/// Fixer le numéro de groupe lors d'un polling.
/// \arg la valeur par défaut est le groupe 0
/// \param group_number : le numéro du groupe souhaité
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setGroup_number(Byte group_number) {
	_MR_CMDWRITEPARAM( GROUP_NUMBER, &group_number, 1);
	this->m_group_number = group_number;
}

/// Retrouver la listes des adresses des modules interrogés en mode polling.
/// Le nombre de modules est dans size. Le nombre maximum de modules ets de 40.
/// La première adresse Radio est accessible par front()
/// La dernière adresses est accessible par back();
/// \par Pour parcourir la liste on utilise un itérateur LSTADRRADIO::iterator et begin() et end().
/// \return L'adresse de la liste interne des adresses des modules interrogés.
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
/// \see _MR_CMDREADPARAM
/// \see LSTADRRADIO
LSTADRRADIO CWavePort::getPolling_route() {
	this->m_polling_routes.clear();             // vider la liste actuelle
	_MR_CMDREADPARAM(POLLING_ROUTE);
	m_polling_route[0] = PDUr[5];            // le nombre de modules concernés
	for (int i = 0; i < PDUr[5]; i++) // remplir le tableau d'octets de la liste des adresses radio des modules concernés
		for (int j = 0; j < SIZEADRRADIO; j++)
			this->m_polling_route[1 + i + j] = PDUr[(i + 1) * 6 + j]; // première adresse Radio entre 6 et 11
//  for ( int i=0 ; i< PDUr[5]; i++ )       // lst.size est mis à jour
//    m_polling_routes.push_back(*new CAdrRadio(&PDUr[(i+1)*6])); // remplir la liste des adresses des modules concernés
	this->m_Nb_Polling_Route = PDUr[5]; // mettre à jour le nombre de noeuds concernés par le polling
	return m_polling_routes;
}

/// Fixer la liste des modules balayés lors d'un polling.Au maximum, 40 modules peuvent être balayés.
/// \param polling_routes : l'adresse de la liste des modules concernés par le balayage. Le nombre de modules est dans size.
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setPolling_route(LSTADRRADIO* polling_routes) {
	this->m_polling_routes.clear();             // vider la liste actuelle
	LSTADRRADIO::iterator it;
	// remplir la liste interne
	for (it = polling_routes->begin(); it != polling_routes->end(); it++)
		m_polling_routes.push_back(*it);
	m_polling_route[0] = polling_routes->size();
	// remplir le tableau interne d'octets
	int i = 0;
	for (it = polling_routes->begin(); it != polling_routes->end(); it++)
		for (int j = 0; j < SIZEADRRADIO; j++, i++)
			m_polling_route[(i * SIZEADRRADIO) + 1 + j] = it->ToBytes()[j];
	_MR_CMDWRITEPARAM( POLLING_ROUTE, m_polling_route,
			polling_routes->size()*SIZEADRRADIO+1);
	this->m_Nb_Polling_Route = polling_routes->size();
}

/// Retrouver le délai entre 2 transmissions lors d'un polling.
/// \arg Le délai est en multiple de 100ms
/// \arg la valeur par défaut est 0x0A soit 10*100ms = 1s
/// \return Le délai actuel
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
Byte CWavePort::getPolling_time() {
	_MR_CMDREADPARAM(POLLING_TIME);
	this->m_polling_time = this->PDUr[5];
	return m_polling_time;
}

/// Fixer le délai entre 2 transmissions lors d'un polling.
/// \arg Le délai est en multiple de 100ms
/// \arg la valeur par défaut est 0x0A soit 10*100ms = 1s
/// \return Le délai souhaité
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setPolling_time(Byte polling_time) {
	_MR_CMDWRITEPARAM( POLLING_ROUTE, &polling_time, 1);
	this->m_polling_time = polling_time;
}

/// Retrouver l'indicateur d'acquittement de trames actif/non actif.
/// \arg 0 : pas d'acquittement des trames
/// \arg 1 : acquittement obligatoire
/// \return L'état actuel de l'indicateur
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
Byte CWavePort::getRadio_acknowledge() {
	_MR_CMDREADPARAM(RADIO_ACKNOWLEDGE);
	this->m_radio_acknowledge = this->PDUr[5];
	return m_radio_acknowledge;
}

/// Fixer l'indicateur d'acquittement de trames actif/non actif.
/// \arg 0 : pas d'acquittement des trames
/// \arg 1 : acquittement obligatoire
/// \param radio_acknowledge : L'état souhaité de l'indicateur
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setRadio_acknowledge(Byte radio_acknowledge) {
	_MR_CMDWRITEPARAM( RADIO_ACKNOWLEDGE, &radio_acknowledge, 1);
	this->m_radio_acknowledge = radio_acknowledge;
}

/// Retrouver l'adresse radio du waveport
/// \return le tableau des 6 octets de l'adresse Radio
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
WAVENIS::CAdrRadio CWavePort::getRadio_address() {
	_MR_CMDREADPARAM(RADIO_ADDRESS);
	for (int i = 0; i < m_radio_address->Taille; i++)
		this->m_radio_address->ToBytes()[i] = PDUr[5 + i];
	return *m_radio_address;
}

/// Retrouver la listes des adresses des modules répéteurs en mode RELAY_ROUTE
/// Le nombre de modules répéteurs est dans size. Le nombre maximum de répéteurs est de 3.
/// La première adresse Radio est accessible par front()
/// La dernière adresses est accessible par back();
/// \par Pour parcourir la liste on utilise un itérateur LSTADRRADIO::iterator et begin() et end().
/// \return L'adresse de la liste interne des adresses des modules répéteurs.
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
/// \see _MR_CMDREADPARAM
/// \see LSTADRRADIO
LSTADRRADIO CWavePort::getRelay_route() {
	this->m_relay_routes.clear();             // vider la liste actuelle
	_MR_CMDREADPARAM(RELAY_ROUTE);
	m_relay_route[0] = PDUr[5];               // le nombre de modules relai
	for (int i = 0; i < PDUr[5]; i++) // remplir le tableau d'octets des adresses des modules relai
		for (int j = 0; j < SIZEADRRADIO; j++)
			this->m_relay_route[1 + i + j] = PDUr[(i + 1) * 6 + j]; // première adresse Radio entre 6 et 11
//  for ( int i=0 ; i< PDUr[5]; i++ )         // lst.size est mis à jour
//    m_relay_routes.push_back(*new CAdrRadio(&PDUr[(i+1)*6]));
	this->m_Nb_Relay_Route = PDUr[5]; // mettre à jour le nombre de noeuds concernés par le polling
	return m_relay_routes;
}

/// Fixer la liste des adresses des modules répéteurs en mode RELAY_ROUTE
/// \param relay_routes : liste des adresses des répéteurs en mode REALY
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
/// \see LSTADRRADIO
void CWavePort::setRelay_route(LSTADRRADIO* relay_routes) {
	this->m_relay_routes.clear();             // vider la liste actuelle
	LSTADRRADIO::iterator it;
	// remplir la liste interne
	for (it = relay_routes->begin(); it != relay_routes->end(); it++)
		m_relay_routes.push_back(*it);
	m_relay_route[0] = relay_routes->size();
	// remplir le tableau interne d'octets
	int i = 0;
	for (it = relay_routes->begin(); it != relay_routes->end(); it++, i++)
		for (int j = 0; j < SIZEADRRADIO; j++)
			m_relay_route[(i * SIZEADRRADIO) + 1 + j] = it->ToBytes()[j];
	_MR_CMDWRITEPARAM( RELAY_ROUTE, m_relay_route,
			relay_routes->size()*SIZEADRRADIO+1);
	this->m_Nb_Relay_Route = relay_routes->size();
}

/// Retrouver l'état de l'indicateur trames relayées / trames non relayées.
/// \arg 0 : pas de relayage (Mode RELAY non activé)
/// \arg 1 : relayage actif (Mode RELAY activé)
/// \arg : Le Mode RELAY
/// \par un module Waveport peut accéder à un module distant via une liste de modules qui feront fonction de répéteurs de commande.
/// Ce mode est appelé mode RELAY. La liste à fournir au waveport est la liste des adresses des modules répéteurs.
/// \par Dès qu'une liste de répéteurs est entrée dans le Waveport, le mode relayage est actif pour la prochaine commande radio.
/// \par LA COMMANDE RADIO ANNULLE LA LISTE DES ADRESSES DES MODULES REPETEURS : IL FAUT DONC RE-ENTRER CETTE LISTE A CHAQUE NOUVELLE COMMANDE RADIO pour maintenir le mode RELAY.
/// \par Elle n'effectue aucune configuration du mode RELAY. Seule la présence d'une liste de répéteurs déclenche le mode RELAY lors de la prochaine commande Radio.
/// \return l'état de l'indicateur
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
/// \see setRelay_route_status
Byte CWavePort::getRelay_route_status() {
	_MR_CMDREADPARAM(RELAY_ROUTE_STATUS);
	this->m_relay_route_status = this->PDUr[5];
	return m_relay_route_status;
}

/// \arg : Le Mode RELAY
/// \par un module Waveport peut accéder à un module distant via une liste de modules qui feront fonction de répéteurs de commande.
/// Ce mode est appelé mode RELAY. La liste à fournir au waveport est la liste des adresses des modules répéteurs.
/// \par Dès qu'une liste de répéteurs est entrée dans le Waveport, le mode relayage est actif pour la prochaine commande radio.
/// \par LA COMMANDE RADIO ANNULLE LA LISTE DES ADRESSES DES MODULES REPETEURS : IL FAUT DONC RE-ENTRER CETTE LISTE A CHAQUE NOUVELLE COMMANDE RADIO pour maintenir le mode RELAY.
/// Fixer l'état de l'indicateur trames relayées / trames non relayées.
/// \arg 0 : pas de relayage (Mode RELAY non activé)
/// \arg 1 : relayage actif (Mode RELAY activé)
/// \par Cette méthode n'est destinée qu'à assurer la compatibilité avec les firmware antérieurs.
/// \par Elle n'effectue aucune configuration du mode RELAY. Seule la présence d'une liste de répéteurs déclenche le mode RELAY lors de la prochaine commande Radio.
/// \param relay_route_status : état souhaité du mode RELAY
/// \see getRelay_route_status
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setRelay_route_status(Byte relay_route_status) {
	_MR_CMDWRITEPARAM( RELAY_ROUTE_STATUS, &relay_route_status, 1);
	this->m_relay_route_status = relay_route_status;
}

/// Retrouver l'état de l'indicateur "sélection automatique de RF".
/// \arg 0 : sélection automatique désactivé
/// \arg 1 : sélection automatique activée
/// \arg Par défaut le mode "sélection automatique" est activé.
/// \return l'état de l'indicateur Switch_mode_status
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
Byte CWavePort::getSwitch_mode_status() {
	_MR_CMDREADPARAM(SWITCH_MODE_STATUS);
	this->m_switch_mode_status = this->PDUr[5];
	return m_switch_mode_status;
}

/// Fixer l'état de l'indicateur "sélection automatique de RF".
/// \arg 0 : sélection automatique désactivé
/// \arg 1 : sélection automatique activée
/// \arg Par défaut le mode "sélection automatique" est activé.
/// \param switch_mode_status : l'état de l'indicateur Switch_mode_status souhaité
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setSwitch_mode_status(Byte switch_mode_status) {
	_MR_CMDWRITEPARAM( SWITCH_MODE_STATUS, &switch_mode_status, 1);
	this->m_switch_mode_status = switch_mode_status;
}

/// Retrouver la durée de réveil lorsque le mode wakeup long est utilisé.(voir Référence guide waveport.pdf p19 à p22)
/// La valeur sur 2 octets est stockée LSB d'abord dans la trame.
/// \arg 0 : la valeur est en multiple de 1ms
/// \arg 1 : La valeur doit être supérieure à la période de scrutation (polling) radio.
/// \arg La valeur par défaut est 1100 ms soit 0x044C
/// \arg La valeur mini est 20ms soit 0x0014 stockée par 0x14 0x00
/// \arg La valeur maxi est 10s soit 0x2710 stockée par 0x10 0x27
/// \return la valeur actuelle de la durée de réveil
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
int CWavePort::getWakeup_length() {
	_MR_CMDREADPARAM(WAKEUP_LENGTH);
	this->m_wakeup_length = (this->PDUr[5] + (this->PDUr[6] << 8));
	return m_wakeup_length;
}

/// Fixer la durée de réveil lorsque le mode wakeup long est utilisé.(voir Référence guide waveport.pdf p19 à p22)
/// La valeur sur 2 octets est stockée LSB d'abord dans la trame.
/// \arg 0 : la valeur est en multiple de 1ms
/// \arg 1 : La valeur doit être supérieure à la période de scrutation (polling) radio.
/// \arg La valeur par défaut est 1100 ms soit 0x044C
/// \arg La valeur mini est 20ms soit 0x0014 stockée par 0x14 0x00
/// \arg La valeur maxi est 10s soit 0x2710 stockée par 0x10 0x27
/// \param wakeup_length : la valeur souhaitée de la durée de réveil
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setWakeup_length(int wakeup_length) {
	Byte Data[2];
	Data[0] = (Byte) (wakeup_length & 0xFF);
	Data[1] = (Byte) ((wakeup_length >> 8) & 0xFF);
	_MR_CMDWRITEPARAM( WAKEUP_LENGTH, Data, 2);
	this->m_wakeup_length = wakeup_length;
}

/// Retrouver le mode de réveil actuel (wakeup mode)
/// \arg 0 : mode long wakeup
/// \arg 1 : mode short wakeup
/// \arg Par défaut le mode est long wakeup
/// \return le mode de réveil actuel
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
Byte CWavePort::getWakeup_type() {
	_MR_CMDREADPARAM(WAKEUP_TYPE);
	this->m_wakeup_type = this->PDUr[5];
	return m_wakeup_type;
}

/// Fixer le mode de réveil  (wakeup mode)
/// \arg 0 : mode long wakeup
/// \arg 1 : mode short wakeup
/// \arg Par défaut le mode est long wakeup
/// \param wakeup_type : le mode de réveil souhaité
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setWakeup_type(Byte wakeup_type) {
	_MR_CMDWRITEPARAM( WAKEUP_TYPE, &wakeup_type, 1);
	this->m_wakeup_type = wakeup_type;
}

/// Retrouver le numéro de groupe lors d'une transmission multicast.
/// \arg Par défaut aucun groupe n'est défini. Wavecard_multicast_group = 0xFF
/// \return le numéro de groupe actuel
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
Byte CWavePort::getWavecard_multicast_group() {
	_MR_CMDREADPARAM(WAVECARD_MULTICAST_GROUP);
	this->m_wavecard_multicast_group = this->PDUr[5];
	return m_wavecard_multicast_group;
}

/// Fixer le numéro de groupe lors d'une transmission multicast.
/// \arg Par défaut aucun groupe n'est défini. Wavecard_multicast_group = 0xFF
/// \param wavecard_multicast_group : le numéro de groupe souhaité
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setWavecard_multicast_group(Byte wavecard_multicast_group) {
	_MR_CMDWRITEPARAM( WAVECARD_MULTICAST_GROUP, &wavecard_multicast_group, 1);
	this->m_wavecard_multicast_group = wavecard_multicast_group;
}

/// Retrouver le numéro de groupe lors d'une scrutation (polling) des modules.
/// \arg Par défaut le groupe est de numéro 0
/// \return le numéro de groupe actuel
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
Byte CWavePort::getWavecard_polling_group() {
	_MR_CMDREADPARAM(WAVECARD_POLLING_GROUP);
	this->m_wavecard_polling_group = this->PDUr[5];
	return m_wavecard_polling_group;
}

/// Fixer le numéro de groupe lors d'une scrutation (polling) des modules.
/// \arg Par défaut le groupe est de numéro 0
/// \param wavecard_polling_group : le numéro de groupe souhaité
/// \par Gestion des exceptions:
/// \arg : CWPCWPException : erreur lors de l'écriture d'un paramètre fonctionnel.
/// \see CWPCWPException
void CWavePort::setWavecard_polling_group(Byte wavecard_polling_group) {
	_MR_CMDWRITEPARAM( WAVECARD_POLLING_GROUP, &wavecard_polling_group, 1);
	this->m_wavecard_polling_group = wavecard_polling_group;
}

/// Accès aux paramètres fonctionnels en lecture (commande READ_RADIO_PARAM)
/// \arg Construire la trame en vue d'effectuer la requête REQ_READ_RADIO_PARAM. La taille de la trame est constante et vaut SIZEREQ_READ_RADIO_PARAM.
/// \arg Effectuer la requête par appel de RequeteParam
/// \param NumParam : le numéro du paramètre pris dans la liste FONCTPARAM.
/// \par Les données sont dans PDUr en sortie de fonction
/// L'appelant peut alors tester le code CMDRES par rapport à la requête CMDREQ et gérer d'éventuelles erreurs.
/// \par Une exception est gérée :
/// \arg CWPCRPException : Erreur dans une commande de lecture de paramètre. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro du paramètre pour lequel la lecture a échoué
/// \see CWPCRPException
/// \see CMDPARAMFONCT
/// \see FONCTPARAM
/// \see SIZEREQCMDPARAMFONCT
/// \see SIZERESCMDPARAMFONCT
/// \par Gestion des exceptions:
/// \arg : CWPCRPException : erreur lors de la lecture d'un paramètre fonctionnel.
/// \see CWPCRPException
/// \arg : CWPCNSException : commande Non supportée par le module cible.
/// \see CWPRESException
/// \arg : CWPCRIException : commande de confirmation RES inconnue
/// \see CWPCRIException
void CWavePort::CmdReadParam(enum FONCTPARAM NumParam) {
	this->LongueurPDUe = SIZEREQ_READ_RADIO_PARAM;  // connu dans la doc WAVENIS
	this->PDUe[2] = this->LongueurPDUe - 3; // retrancher la partie fixe préfixe (2octets) et postfixe (1octet)
	this->PDUe[INDCMD] = REQ_READ_RADIO_PARAM;         // le CMD correspondant
	this->PDUe[4] = NumParam;                          // le numéro du paramètre
	CRC(this->PDUe, this->LongueurPDUe); // calculer le CRC et l'insérer dans la trame
	this->PDUe[this->LongueurPDUe - 1] = ETX;            // fin de trame
	try {
		this->Req_Res();
	} catch (CWPREQException& e) {
		_MR_THROW(CWPCRPException(e,NumParam));
	} catch (CWPRESException& e) {
		_MR_THROW(CWPCRPException(e,NumParam));
	}
//  catch (SPException& e ) {
//    _MR_THROW(CWPCRPException(e,NumParam) );
//  }
	if (PDUr[INDCMD] == WERROR) {                   // trame ERROR
		if (PDUr[4] == 0x01)
			_MR_THROW(CWPCNSException(NumParam));
	} else if (PDUr[INDCMD] != RES_READ_RADIO_PARAM) { // Commande RES_ incorrecte
		_MR_THROW(CWPCRIException(REQ_READ_RADIO_PARAM,PDUr[INDCMD]));
	}
}

/// Accès aux paramètres fonctionnels en écriture (commande READ_WRITE_PARAM)
/// \arg Construire la trame en vue d'effectuer la requête REQ_WRITE_RADIO_PARAM.
/// La taille de la trame vaut SIZEREQ_READ_RADIO_PARAM + la taille des données relatives à la requête d'écriture.
/// \arg Effectuer la requête par appel de RequeteParam
/// \param NumParam : le numéro du paramètre pris dans la liste FONCTPARAM.
/// \param Param :  le tableau des données relatives à la requête en écriture.
/// \param TailleParam :  la taille des données relatives à la requête d'écriture.
/// \par Les données sont dans PDUr en sortie de fonction
/// \par Gestion des exceptions:
/// \arg CWPCWPException : Erreur dans une commande d'écriture de paramètre. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req_Res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro du paramètre pour lequel l'écriture a échoué
/// \see CWPCWPException
/// \arg : CWPCNSException : commande Non supportée par le module cible.
/// \see CWPRESException
/// \arg : CWPCRIException : commande de confirmation RES inconnue
/// \see CWPCRIException
/// \see CMDPARAMFONCT
/// \see FONCTPARAM
/// \see SIZEREQCMDPARAMFONCT
/// \see SIZERESCMDPARAMFONCT
void CWavePort::CmdWriteParam(enum FONCTPARAM NumParam, Byte Param[],
		int TailleParam) {
	this->LongueurPDUe = SIZEREQ_WRITE_RADIO_PARAM + TailleParam; // connu dans la doc WAVENIS
	this->PDUe[2] = this->LongueurPDUe - 3; // retrancher la partie fixe préfixe (2octets) et postfixe (1octet)
	this->PDUe[INDCMD] = REQ_WRITE_RADIO_PARAM;        // le CMD correspondant
	this->PDUe[4] = NumParam;                          // le numéro du paramètre
	for (int i = 0; i < TailleParam; i++) {            // insérer les données
		PDUe[4 + 1 + i] = Param[i];
	}
	CRC(this->PDUe, this->LongueurPDUe); // calculer le CRC et l'insérer dans la trame
	this->PDUe[this->LongueurPDUe - 1] = ETX;            // fin de trame
	try {
		this->Req_Res();
	} catch (CWPREQException& e) {
		_MR_THROW(CWPCWPException(e,NumParam));
	} catch (CWPRESException& e) {
		_MR_THROW(CWPCWPException(e,NumParam));
	}
//  catch (SPException& e ) {
//    _MR_THROW(CWPCWPException(e,NumParam) );
//  }
	if (PDUr[INDCMD] == WERROR) {                   // trame ERROR
		if (PDUr[4] == 0x01)
			_MR_THROW(CWPCNSException(REQ_WRITE_RADIO_PARAM,NumParam));
	} else if (PDUr[INDCMD] != RES_WRITE_RADIO_PARAM) { // Commande RES_ incorrecte
		_MR_THROW(
				CWPCRIException(REQ_WRITE_RADIO_PARAM,PDUr[INDCMD], NumParam));
	}
}

/// Accès aux paramètres de contrôle :
/// \arg Construire la trame en vue d'effectuer une requête relative à un paramètre de contrôle
/// \arg Effectuer la requête par appel de Req_Res()
/// \param Cmd : la commande relative au paramètre de contrôle cible
/// \param treq : la taille de la trame de requête
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \see CMDPARAMCTRL
/// \see SIZEREQCMDPARAMCTRL
void CWavePort::CmdParamCtrl(enum CMDPARAMCTRL Cmd,
		enum SIZEREQCMDPARAMCTRL treq) {
	this->LongueurPDUe = treq;                      // connu dans la doc WAVENIS
	this->PDUe[2] = this->LongueurPDUe - 3; // retrancher la partie fixe préfixe (2octets) et postfixe (1octet)
	this->PDUe[INDCMD] = Cmd;                          // le CMD correspondant
	CRC(this->PDUe, this->LongueurPDUe); // calculer le CRC et l'insérer dans la trame
	this->PDUe[this->LongueurPDUe - 1] = ETX;            // fin de trame
	try {
		this->Req_Res();
	} catch (CWPREQException& e) {
		_MR_THROW(CWPCPCException(e,Cmd));
	} catch (CWPRESException& e) {
		_MR_THROW(CWPCPCException(e,Cmd));
	}
//  catch (SPException& e ) {
//    _MR_THROW(CWPCPCException(e,Cmd) );
//  }
	if (PDUr[INDCMD] == WERROR) {                   // trame ERROR
		if (PDUr[4] == 0x01)
			_MR_THROW(CWPCNSException(Cmd));
	} else if (PDUr[INDCMD] != (Cmd + 1)) {          // Commande RES_ incorrecte
		_MR_THROW(CWPCRIException(Cmd,PDUr[INDCMD]));
	}
}

/// Effectuer une commande radio à destination d'un module distant
/// \par Gestion des exceptions:
/// \arg CWPCNSException :  commande non supportée
/// \arg CWPCRIException :  commande de confirmation RES inconnue
/// \arg CWPCRException :  erreur dans la commande Radio
/// \param PDUmd :  l'unité de données à transmettre au module distant
/// \param TaillePDUmd : la taille de cette PDU
/// \see CWPCNSException
/// \see CWPCRException
/// \see CWPCRException
void CWavePort::CmdRadioMd(Byte PDUmd[], int TaillePDUmd) {
	CAdrRadio AdrRadio(PDUmd);
	try {
		cout << "CmdRadio" << endl;
		this->CmdRadio(REQ_SEND_FRAME, &AdrRadio, &PDUmd[SIZEADRRADIO],
				TaillePDUmd - SIZEADRRADIO);
	} catch (CWPCNSException& e) {
		_MR_THROW(e);
	} catch (CWPCRIException& e) {
		_MR_THROW(e);
	} catch (CWPCRException& e) {
		_MR_THROW(e);
	}
	// retourner la réponse du module

}

void CWavePort::CmdRadioMd(Payload payload, RadioAddress destinationAddress)
{
		try {
			cout << "CmdRadio" << endl;
			this->CmdRadio(payload,destinationAddress);
		} catch (CWPCNSException& e) {
			_MR_THROW(e);
		} catch (CWPCRIException& e) {
			_MR_THROW(e);
		} catch (CWPCRException& e) {
			_MR_THROW(e);
		}
}

/// Accès aux modules distants pour les commandes de service ou pour les commandes relatives aux échanges de trames avec les modules distants.
/// \par Construire la trame en vue d'effectuer une requête relative à une communication avec un module distant
/// \par Les données du module (Adresse Radio, type de la requête et Données spécifiques de la requête) sont insérées dans la trame à émettre dpuis la PDU du module. cette PDU est toujours de la taille maxi possible. Elle recevra en retour les données de la réponse du module.
/// \par Cette méthode est également utilisée pour les requêtes de service.
/// \par Effectuer la requête par de :
/// \arg Req_Res  : qui gère la transmission de la requête relative aux échanges radio ( commandes REQ_SEND_FRAME et RES_SEND_FRAME)
/// \arg Received : qui gère la réception de la réponse du module (commande RECEIVED_FRAME)
/// \param Cmd : la commande REQ_ à effectuer. Cette commande est prise dans CMDTRANSF ou CMDSERVICE.
/// \param AdresseRadio : l'adresse radio du module cible. En retour cette donnée contient l'adresse radio du transmetteur (en principe l'adresse radio du module cible sauf lorsqu'il y a eur relayage)
/// \param Datam : les données du module à transmettre lors de la requête. La taille de cette donnée est toujours suffisante pour contenir la réponse du module, soit 152 octets max)
/// \param TailleData : la taille des données Datam à insérer dans la PDU du waveport
/// \par Quatre exceptions sont gérées :
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \see Req_Res
/// \see Service
/// \see CMDTRANSRF
/// \see CMDSERVICE
/// \see REQTYPE
void CWavePort::CmdRadio(Byte Cmd, CAdrRadio* AdresseRadio, Byte Datam[],
		int TailleData) {
	/*
	 this->LongueurPDUe = 7 + TailleData + SIZEADRRADIO; // connu dans la doc WAVENIS
	 this->PDUe[2] = this->LongueurPDUe - 3; // retrancher la partie fixe préfixe (2octets) et postfixe (1octet)
	 this->PDUe[INDCMD] = Cmd;         // la commande relative aux échanges radio
	 for (int i = 0; i < SIZEADRRADIO; i++)
	 PDUe[i + INDDEBDATA] = AdresseRadio->ToBytes()[i];
	 for (int i = 0; i < TailleData; i++)
	 PDUe[i + 4 + SIZEADRRADIO] = Datam[i];
	 CRC(this->PDUe, this->LongueurPDUe); // calculer le CRC et l'insérer dans la trame
	 this->PDUe[this->LongueurPDUe - 1] = ETX;            // fin de trame
	 try {
	 this->Req_Res();
	 } catch (CWNException& e) {
	 _MR_THROW(CWPCRException(e,Cmd, AdresseRadio));
	 }
	 //  catch (SPException& e ) {
	 //    _MR_THROW(CWPCRException(e,Cmd,AdresseRadio) );
	 //  }
	 if (PDUr[INDCMD] == WERROR) {                   // trame ERROR
	 if (PDUr[4] == 0x01)
	 _MR_THROW(CWPCNSException(Cmd));
	 } else if (PDUr[INDCMD] != (Cmd + 1)) {          // Commande RES_ incorrecte
	 _MR_THROW(CWPCRIException(Cmd,PDUr[INDCMD]));
	 }
	 try {
	 this->Received();      // Effectuer la réception de la réponse du module
	 } catch (CWPRECException& e) {
	 _MR_THROW(CWPCRException(e, Cmd, AdresseRadio));
	 }
	 //  catch (SPException& e ) {
	 //    _MR_THROW(CWPCRException(e,Cmd, AdresseRadio) );
	 //  }
	 // transférer la réponse vers le module
	 for (int i = 0; i < SIZEADRRADIO; i++)
	 AdresseRadio->ToBytes()[i] = PDUr[i + INDDEBDATA];
	 int TailleRep = PDUr[2] - (4 + SIZEADRRADIO);
	 for (int i = 0; i < TailleRep; i++)
	 Datam[i] = PDUr[i + 4 + SIZEADRRADIO];
	 */
}

void CWavePort::CmdRadio(Payload payload, RadioAddress destinationAddress) {

	m_Command = create_radio_exchange_command(payload,destinationAddress);
	memset(&m_Response, 0, sizeof(m_Response));
	// Execute the application message
	m_protocolHandler.send_application_request(m_waveportInstance, &m_Command, &m_Response);
	cout << GetCurrentTimeString() << "process send request status: " << get_response_status_name(m_Response.response.status) << endl;

	char hexpayload[MAX_PAYLOAD_LEN * 2 + 1];
	const char* ra = ((RadioAddress*) &m_Response.command.dynamicStandardParams.nodeAddress)->addressValue;
	const char* pl = get_hex_payload(hexpayload, sizeof(hexpayload), (Payload*)&m_Response.command.command.payload);
	cout << GetCurrentTimeString() << "process send request response: " << ra << ";" << pl << endl;

}




/// Requête de service pour obtenir :
/// \arg le type du module interrogé à partir de son adresse radio.
/// \arg le RSSI level
/// \arg la période de réveil (wake-up period)
/// \param AdresseRadio :  l'adresse radio du module cible
/// \return le type du module interrogé.
/// gestion des exceptions
/// \arg CWPCSException : erreur lors d'une requête de service.
/// \see CWPCSException
Byte CWavePort::ServiceTypeModule(CAdrRadio* AdresseRadio) {
	try {
		Service(AdresseRadio, GET_TYPE, this->DataService, 0);
	} catch (CWPCSException& e) {
		_MR_THROW(e);
	}
	return this->DataService[1];
}

/// Requête de service pour obtenir :
/// \arg le firmware du module interrogé à partir de son adresse radio.
/// \par La réponse contient également :le mode de transmission
/// \arg le mode de transmission
/// \param AdresseRadio : l'adresse radio du module cible
/// \return le firmware du module interrogé.
/// gestion des exceptions
/// \arg CWPCSException : erreur lors d'une requête de service.
/// \see CWPCSException
int CWavePort::ServiceFirmware(CAdrRadio* AdresseRadio) {
	try {
		Service(AdresseRadio, GET_FW_VERSION, this->DataService, 0);
	} catch (CWPCSException& e) {
		_MR_THROW(e);
	}
	// XXXX Il s'agit d'un waveport ou d'un module distant
	// voir comment retourner les 2 valeurs dans le int struct PHYCONFIG ?
	this->m_phyconfig = this->DataService[3] + (this->DataService[2] << 8);
	/*
	 char Txt[80];
	 sprintf ( Txt,"phyconfig du module distant 0x%x", this->m_phyconfig );
	 Tracer ( Txt,NULL );
	 */
	return (this->DataService[5] + (this->DataService[4] << 8));
}

/// Effectuer une requête de service vers un module WavePort distant
/// Lorsqu'un module waveport reçoit une requête de service, il ne retourne aucune donnée à l'hôte auquel il est relié.
/// \param AdresseRadio : l'adresse radio du waveport cible.
/// \param ReqType : le type de requête de service demandée
/// \param Data : les données à transmettre lors de la requête autres que l'adresse radio.
/// \param Taille : la taille des données à transmettre lors de la requête
/// \arg Les données attendues sont :
/// - la version du firmware
/// - le mode de transmission avec ReqType = GET_FW_VERSION. Dans Response on a :
/// \arg byte 1 : le caractère 'V' ( 0x56)
/// \arg byte 2 : MSB default Radio Protocole
/// \arg byte 3 : LSB default Radio Protocole
/// \arg byte 4 : MSB Firmware version
/// \arg byte 5 : LSB Firmware version
/// - le type, le niveau RSSI et la période de réveil (wake-up period) ReqType = GET_TYPE. dans Response on a :
/// \arg byte 1 : module type
/// \arg byte 2 : RSSI level
/// \arg byte 3 : Wake-up Period
/// \arg byte 4 : module type
/// gestion des exceptions
/// \arg CWPCSException : erreur lors d'une requête de service.
/// \see CWPCSException
void CWavePort::Service(CAdrRadio* AdresseRadio, Byte ReqType, Byte Data[],
		int Taille) {
	Byte Datam[6];
	Datam[0] = ReqType;
	for (int i = 0; i < Taille; i++)
		Datam[i + 1] = Data[i];
	try {
		CmdRadio(REQ_SEND_SERVICE, AdresseRadio, Datam, Taille + 1);
	} catch (CWPCRException& e) {
		_MR_THROW(CWPCSException( e, AdresseRadio, (unsigned char)ReqType));
	} catch (CWPCNSException& e) {
		_MR_THROW(CWPCSException( e, AdresseRadio, ReqType));
	} catch (CWPCRIException& e) {
		_MR_THROW(CWPCSException( e, AdresseRadio, ReqType));
	}
	int TailleDataRep = PDUr[2] - (3 + SIZEADRRADIO);
	for (int i = 0; i < TailleDataRep; i++)
		Data[i] = Datam[i];
}

/// Traiter l'octet de status. Seules les méthodes concernées appellent cette méthode
/// L'octet de status lorsqu'il existe est toujours dans PDU[4]
/// \param CmdRes : la commande RES_ concernée
/// gestion des exceptions
/// \arg CWPSIException : Status incorrect dans une trame RES
/// \see CWPSIException
void CWavePort::Status(Byte CmdRes) {
	// Collision NOK SDK Coronis
//  if ( PDUr[4] == NOK ) {                          // Le status est incorrect dans la requête RES_
//    char Msg[20];
//    sprintf(Msg,"Requête RES_ 0x%02X",CmdRes);
//    Tracer ( Msg, NULL );
//    _MR_THROW(CWPSIException(CmdRes) );
//  }
}

/// Lire une trame.
/// \par La lecture se décompose en 2 phases :
/// \arg Lecture des 3 premiers octets : l'octet 3 contient la taille des données restant à lire.
/// \par Rappel structure de la trame WAVENIS
/// \par SYNC STX Length CMD    Data   CRCL   CRCM  ETX
/// \par          |_________Length octets________|
/// \par               |_________Length octets________|
/// \param PDU : emplacement des données de la trame. La taille doit être suffisante pour contenir les données reçues.
/// \par 2 tailles maxi de trames sont possibles :
/// \arg Les trames d'acquittement de bas niveau PDUa de taiulle 7 (ACK,NACK) ou 8 octets (ERROR).
/// \arg Les autres trames dont la taille maxi est 256 octets --> PDUr
/// \par En sortie de fonction la trame reçue est dans PDU.
/// gestion des exceptions
/// \arg SPException : Erreur sur le port série
/// \see SPException
//void CWavePort::LireTrame( Byte PDU[] ) {
//  RetryTO = MAXRETRYTO;                         // recommencer trois fois en cas de timeout
//  bool bFin = true;
//  while ( true ) {
//    try {
//      this->pS->Read( PDU,0,3);                 // Réception des trois premiers octets
//    }
//    catch ( SPTOException& e) {
//      if ( RetryTO ) {
//        RetryTO--;
//        this->Flush();                          // vider les buffer au cas où ...
//        bFin = false;
//        this->Tracer(e.ToString(),PDU);
//      }
//      else {
//        this->Flush();                          // vider les buffer au cas où ...
//        _MR_THROW(e);
//      }
//    }
//    catch ( SPException& e) {
//      this->Flush();                            // vider les buffer au cas où ...
//      _MR_THROW(e);                             // remonter l'erreur vers l'appelant
//    }
//    if ( bFin )
//      break;
//    bFin = true;
//  }
//  RetryTO = MAXRETRYTO;                         // recommencer trois fois en cas de timeout
//  int Taille = PDU[2];                          // récupérer la taille restant à lire
//  while ( true ) {
//    try {
//      this->pS->Read( PDU,3,Taille);            // Réception des Taille octets suivants (Offset de 3 dans PDU)
//    }
//    catch ( SPTOException& e) {
//      if ( RetryTO ) {
//        RetryTO--;
//        this->Flush();                            // vider les buffer au cas où ...
//        bFin = false;
//        this->Tracer(e.ToString(),PDU);
//      }
//      else {
//        this->Flush();                            // vider les buffer au cas où ...
//        _MR_THROW(e)
//      }
//    }
//    catch ( SPException& e) {
//      this->Flush();                            // vider les buffer au cas où ...
//      _MR_THROW(e);                             // remonter l'erreur vers l'appelant
//    }
//    if ( bFin ) {
//      break;
//    }
//    bFin = true;
//  }
//}

/// Effectuer une requête de paramétrage (fonctionnel ou de contrôle) en respectant les protocoles WAVENIS.
/// \par wavePort reference guide.pdf pages 8 et 9
/// \par Cette méthode constitue la première phase lors d'une requête en relation avec une communication radio avec un module distant.
/// \par La PDU a été préparé dans la méthode appelante.
/// \par La commande à effectuer peute être :
/// \arg Une commande spécifique pour les paramètres de contrôle prise dans CMDPARAMCTRL
/// \arg Les commandes READ_RADIO_PARAM ou WRITE_RADIO_PARM pour les paramètres fonctionnels.
/// \arg La REQ_SEND_FRAME pour la communication radio avec un module distant
/// La séquence à respecter est toujours :
/// \arg Recommencer Retry fois
/// \arg   Emission de la PDUe préparée avec le code CMD associé à l'opération souhaitée.
/// \arg   Réception et test de l'acquittement de bas niveau ackRS232
/// \arg FIN Recommencer
/// \arg Recommencer Retry fois
/// \arg   Réception de l'acquittement de haut niveau RES_SEND_FRAME qui constitue la réponse attendue.
/// \arg   Test de l'acquittement de haut niveau
/// \arg   Temporisation 1ms
/// \arg   envoi de l'acquittement RS232
/// \arg FIN Recommencer
/// \par Les résultats attendus sont dans PDUr
/// Gestion des exceptions
/// \arg CWPREQException :  le nombre d'essais maxi dans l'envoi de la requête est atteint
/// \see CWPREQException
/// \arg CWPRESException :  le nombre d'essais maxi dans la réception de la confirmation RES est atteint
/// \see CWPRESException
void CWavePort::Req_Res(void) {
//  this->RetryReq = MAXRETRYREQ;
//  this->RetryRes = MAXRETRYRES;
//  bool ErrLireTrame = false;
//  // Emettre la requête REQ_
//  while ( this->RetryReq ) {
//    try {
//      this->pS->Write( PDUe,0,this->LongueurPDUe);          // émettre la requête de firmware
//      Tracer("\nPDU émise",PDUe);                           // Tracer les E/S
//    }
//    catch ( SPException& e) {
//      cout << "Erreur WRITE PDU " << e.ToString() << endl;
//    }
//    try {
//      this->LireTrame(PDUa);                                // Lire une trame 3octets puis le reste
//    }
//    catch ( SPTOException& e ) {                            // LireTrame en timeout recommencer REQ
//      ErrLireTrame = true;                                  //      _MR_THROW(e);
//    }
//    catch ( SPException& e ) {
//      _MR_THROW(e);
//    }
//    if ( ! ErrLireTrame ) {
//      if ( ! this->TesterCRC( PDUa )                        // Tester le CRC
//          ||  PDUa[INDCMD] != ACK ) {                       // erreur de CRC ou cmd autre que ACK : problème
//        Tracer ( "Problème dans Req_Res. Acquittement incorrect\n",PDUa) ;
//        this->RetryReq--;                                   // on peut recommencer RetryReq fois
//        ErrLireTrame = false;
//      }
//      else break;                                           // ACKRS232 reçu
//    }
//    else {
//      Tracer ( "Problème dans LireTrame. Recommencer requête.\n",NULL) ;
//      this->RetryReq--;                                     // on peut recommencer RetryReq fois
//      ErrLireTrame = false;
//    }
//  }
//  if ( this->RetryReq == 0 ) {                              // le nombre maxi d'essai est atteint
//    _MR_THROW( CWPREQException() );                          // lancer l'exception
//  }
//  Tracer("Trame ACKRS232 reçue : ",PDUa);                   // tracer les E/S
//  // Recevoir la confirmation RES_
//  ErrLireTrame = false;
//  while ( this->RetryRes ) {
//    try {
//      this->LireTrame(PDUr);                                // Réception de la confirmation RES_
//    }
//    catch ( SPTOException& e ) {
//      ErrLireTrame = true;                                  //      _MR_THROW(e);
//    }
//    catch ( SPException& e ) {
//      _MR_THROW(e);
//    }
//    if ( ! ErrLireTrame ) {
//      if ( ! this->TesterCRC  ( PDUr ) ) {                  // Tester le CRC
//        this->PDUa[INDCMD] = NACK;                          // trame reçue non acquittée: on peut réutiliser la trame PDUa reçue dans la phase précédente.
//        this->pS->Write( PDUa,0,LONGUEURPDUNA);             // acquittement de bas niveau de la réponse RES_
//        this->Tracer( "Erreur réception RES_", PDUr);
//        this->RetryRes--;                                   // un essai en moins
//        ErrLireTrame = false;
//      }
//      else {                                                // Pas d'erreur trame acquitter RES
//        this->Tracer( "Trame RES_ reçue", PDUr);
//        this->PDUa[INDCMD] = ACK;                           // pas d'erreur CRC: tout c'est bien passé
//        Tempo();                                            // 1ms obligatoire
//        this->pS->Write( PDUa,0,LONGUEURPDUA);              // acquittement de bas niveau de la réponse
//        Tracer("Ack envoyée ",PDUa);
//        return;                                             // Tout c'est bien passé !
//      }
//    }
//    else {
//      Tracer ( "Problème dans LireTrame. Recommencer ACK.\n",NULL) ;
//      this->RetryRes--;                                     // on peut recommencer RetryReq fois
//      ErrLireTrame = false;
//    }
//  }
//  _MR_THROW(CWPRESException() );                          // lancer l'exception MaxRetryRES atteint
}

/// Recevoir la trame en réponse à une requête relative à une communication radio avec un module distant.
/// \par Voir "Reference Guide - Waveport.pdf" pages 37 à 64
/// La séquence à respecter est toujours :
/// \par Recommencer
/// \arg Reception de la trame de réponse RECEIVED_ (mode normal ou relayé)
/// \arg SI CRC incorrect OU CMD = RECEPTION_ERROR ALORS
/// - Tracer erreur CRC ou RECEPTION_ERROR
/// - Temporisation 1ms
/// - Emettre AckRS232 avec CMD=NACK
/// - Retry <-- Retry -1
/// \arg SINON
/// - Temporisation 1ms
/// - Emettre AckRS232 avec CMD=ACK
/// - Fin <-- vrai
/// \arg FINSI
/// \arg TQ Retry >0 ET Pas Fin
/// \par SI Retry = 0 ALORS
/// \arg Lancer une exception CWPRECException
/// \arg FINSI
/// \par Les résultats attendus sont dans PDUr
/// Gestion des exceptions
/// CWPRECException :  le nombre maxi d'essais d'une trame RECEIVED a été atteint.
/// \see CWPRECException
/// \see CMDTRANSRF
void CWavePort::Received(void) {
//  bool ErrLireTrame = false;
//  this->RetryRec = MAXRETRYREC;
//  while ( this->RetryRec ) {
//    try {
//      this->LireTrame( PDUr );                                // Lire la trame de réponse du module
//    }
//    catch ( SPTOException& e ) {
//      ErrLireTrame = true;                                    //      _MR_THROW(e);
//    }
//    catch ( SPException& e ) {
//      _MR_THROW(e);
//    }
//    if ( ! ErrLireTrame ) {
//      if ( ! this->TesterCRC( PDUr ) || PDUr[INDCMD] == RECEPTION_ERROR ) {  // Tester le CRC et le code CMD
//        this->PDUa[INDCMD] = NACK;                            // avant RECEIVED on a reçu une trame PDUa correcte ré utilisable
//        this->pS->Write( PDUa,0,LONGUEURPDUNA);               // acquittement de bas niveau de la réponse
//        ErrLireTrame = false;                                 // rétablir
//        this->RetryRec--;                                     // un essai en moins
//        Tracer ("Trame RECEIVED_ reçue ",PDUr );
//      }
//      else {
//        this->PDUa[INDCMD] = ACK;
//        Tracer ("Trame RECEIVED_ reçue ",PDUr );
//        Tempo();                                              // 1ms obligatoire
//        this->pS->Write( PDUa,0,LONGUEURPDUA);                // acquittement de bas niveau de la réponse
//        return ;
//      }
//    }
//    else {
//      Tracer ( "Problème dans LireTrame. Recommencer ACK.\n",NULL) ;
//      ErrLireTrame = false;
//      this->RetryRec--;                                       // on peut recommencer RetryReq fois
//    }
//  }
//  _MR_THROW(CWPRECException() );                              // lancer l'exception
}

/// Tester le CRC d'une trame donnée
/// \param Trame : La trame à tester
/// \return
/// \arg Vrai si le CRC dans la trame est égal au CRC calculé
/// \arg Faux sinon
bool CWavePort::TesterCRC(Byte Trame[]) {
	int lt = Trame[2] + 3;
	Byte Crc[2] = { Trame[lt - 3], Trame[lt - 2] };
	CRC(Trame, lt);
	if (Trame[lt - 3] == Crc[0] && Trame[lt - 2] == Crc[1]) {
		return true;
	}
	// Rétablir le contenu CRC d'origine dans la trame
	Trame[lt - 3] = Crc[0];
	Trame[lt - 2] = Crc[1];
	return false;
}

/// Calculer le CRC16 d'une trame dont on connaît la taille et insérer le CRC calculé dans la trame.
/// \param Trame : la trame à tester
/// \param lt: la longueur de la trame pour déterminer la zone de calcul du CRC
void CWavePort::CRC(Byte Trame[], int lt) {
	int Poly = 0x8408;
	unsigned int iCrc;
	int j, i_bits, carry;
	iCrc = 0;
	for (j = 2; j < lt - 3; j++) {
		iCrc = iCrc ^ Trame[j];
		for (i_bits = 0; i_bits < 8; i_bits++) {
			carry = iCrc & 1;
			iCrc = iCrc / 2;
			if (carry) {
				iCrc = iCrc ^ Poly;
			}
		}
	}
	// Insérer les octets FAIBLE et FORT du CRC dans la trame. octet FAIBLE en premier.
	Trame[lt - 3] = iCrc & 0xFF;
	Trame[lt - 2] = (iCrc >> 8);
}

FILE* CWavePort::getStream(void) {
	return this->Stream;
}

/*
 // XXXX A revoir car doc incohérente
 /// Pour mettre fin aux modes de test sauf le mode standby pour lequel il faut un reset du transmetteur
 /// \par Les différents mode de test sont :
 /// \arg Réception continue
 /// \arg Transmission continue sans modulation
 /// \arg Transmission continue avec modulation
 /// \arg Standby
 /// \par Pour sortir du mode standby il faut effectuer un RESET du transmetteur
 /// \par Pour sorti des autres modes de test on peut :
 /// \arg Effectuer un RESET du transmetteur
 /// \arg Envoyer une trame avec un octet de données à 0. La méthode FinTestMode() a été créée dans ce but.
 void CWavePort::FinTestMode( Byte Mode ) {
 Byte PDU[] = {0xff,0x02,0x05,0xB0,0x00,0x00,0x00,0x03};
 this->PDUe[4]= Mode;      // Appliquer le mode
 this->CRC(PDU,8);         // Calculer et insérer le CRC16
 this->pS->Write(PDU,0,8); // Emettre la PDU
 }
 */

bool CWavePort::SerialDriverAndProtocolLoad(void) {
	bool ret = false;
//	if (OK == load_serial_driver(m_driverId.c_str(), &m_driverHandler)
//			&& OK == load_protocol(m_protocolId.c_str(), &m_protocolHandler)) {
//		ret = true;
//	}
	if (OK == load_serial_driver("csrs232driver", &m_driverHandler)
			&& OK == load_protocol("cswaveportprotocol", &m_protocolHandler)) {
		ret = true;
	}
}

void CWavePort::SerialDriverAndProtocolFree(void) {
	free_protocol(&m_protocolHandler);
	free_serial_driver(&m_driverHandler);
}

bool CWavePort::SerialDriverAndProtocolInstanciation(void) {
	bool ret = false;
	m_rs232DriverInstance = m_driverHandler.create();
	m_waveportInstance = m_protocolHandler.create();
	if (m_rs232DriverInstance && m_waveportInstance) {
		ret = true;
	}
	return ret;
}

void CWavePort::SerialDriverAndProtocolSuppression(void) {
	m_protocolHandler.destroy(m_waveportInstance);
	m_driverHandler.destroy(m_rs232DriverInstance);
}

int CWavePort::ProtocolInit(void) {
	int status = OK_SERIALDRIVER_SUCCESS;
	//m_driverHandler.set_port_id(m_rs232DriverInstance, PortName.c_str());
	m_driverHandler.set_port_id(m_rs232DriverInstance, "/dev/ttyUSB0");
	m_protocolHandler.set_serial_driver_instance(m_waveportInstance,
			m_driverHandler.get_serial_driver_instance(m_rs232DriverInstance));
	status = m_protocolHandler.init(m_waveportInstance);
	return status;
}

int CWavePort::ProtocolOpen(void) {
	int status = m_protocolHandler.open(m_waveportInstance);
	return status;
}

void CWavePort::ProtocolClose(void) {
	m_protocolHandler.close(m_waveportInstance);
}

const char* CWavePort::GetCurrentTimeString()
{
    static char currTime[100] = "";
    currTime[0] = 0;

	struct timeval currentTime;
	gettimeofday( &currentTime, 0 );
	struct tm *tm = localtime(&currentTime.tv_sec);
	sprintf( currTime, "[%02d:%02d:%02d:%03d] ", tm->tm_hour, tm->tm_min, tm->tm_sec, (int)currentTime.tv_usec / 1000 );


    return currTime;
}

} // namespace WAVENIS

// Programme principal
// Si c'est la classe principale du projet, on place une fonction main()
// Dans ce cas, on peut supprimer les fichiers de tests unitaires
// ex :
// int main(int argc, char *argv[])
// {
//    // Construction de l'instance de la classe principale
//    cout << "Construction de l'instance de la classe principale CWavePort" << endl;
//    CWavePort *cwaveport = new CWavePort();
//    // Lancement de l'activité principale
//    return 0;
// }

