/**
 *  Copyright (C) 2012  plegal  (plegal@appert44.org)
 *  @file         CWavePort.h
 *  @brief        Pilote pour le transmetteur RS232 de WAVENIS
 *  @version      0.1
 *  @date         17 déc. 2012 21:20:15
 *
 *  @note         Voir la description detaillee explicite dans le fichier
 *                CWavePort.cpp
 *                C++ Google Style :
 *                http://google-styleguide.googlecode.com/svn/trunk/cppguide.xml
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef _CWAVEPORT_H
#define _CWAVEPORT_H

// Includes system C
//#include <stdint.h>  // definition des types int8_t, int16_t ...
#include <list>             // pour liste des adresses relais dans le mode relayage

// Includes system C++

// Includes qt

// Includes application
#include <protocol_command_types.h>
#include <protocol_response_types.h>
#include "CAdrRadio.h"
#include "CConfig.h"
// Constantes
// ex :
// const int kDaysInAWeek = 7;

// Enumerations
// ex :
// enum Couleur
// {
//     kBlue = 0,
//     kWhite,
//     kRed,
// };

// Structures
// ex:
// struct UrlTableProperties
// {
//  string name;
//  int numEntries;
// }
/// Type pour les listes d'adresses radio utilisées en mode POLLING ou en mode RELAY

// Declarations de classe avancees
// ex:
// class MaClasse;

/// Espace de noms pour les classes de la librairie WAVENIS
namespace WAVENIS {

#include "loader_serial_driver.h"
#include "loader_protocol.h"

class CConfig;
class CAdrRadio;

typedef list<CAdrRadio> LSTADRRADIO;

/** @brief Pilote pour le transmetteur RS232 de WAVENIS
 *  Description detaillee de la classe.
 */
/// Classe d'encapsulation des propriétés d'un module WAVEPORT: transmetteur Radio sur port série
/// \par Cette version gère :
/// \par les commandes d'application
/// \arg Dans le cadre de la configuration des paramètres
/// - Paramètres de contrôle : OUI
/// - Paramètres fonctionnels : OUI
/// \arg Dans le cadre de la communication par Radio avec les modules distants :
/// - Echanges point à point :
/// -# Exchange Frame Mode normal : OUI
/// -# Exchange Frame Mode relayé : NON
/// -# Message Mode normal : NON
/// -# Message Mode relayé : NON
/// - Echanges point à multipoint :
/// -# Polling Mode : NON
/// -# Broadcast Mode : NON
/// -# Multiframe Reception : NON
/// \par les commandes de contrôle : OUI
/// \par Les commandes de service : OUI
/// \par gestion des erreurs par exception
/// \see CWNException
class CWavePort {
public:

#define STX               2
#define ETX               3
#define SYNC              0xFF
#define MAXPOLLINGROUTE   40
#define MAXRELAYROUTES    3

// Dans le fichier de configuration lu par l'objet CConfig.
#define MAXRETRYREQ       3
#define MAXRETRYRES       3
#define MAXRETRYREC       3
#define MAXRETRYTO        3

	// index de la commande dans la trame
#define INDCMD        3
	// index du début des données dans la trame
#define INDDEBDATA    (INDCMD+1)

#define _MR_CMDPARAMCTRL(a) \
	    try{ \
	      CmdParamCtrl(REQ_##a,SIZEREQ_##a);\
	      this->Status(RES_##a); \
	    } \
	    catch( CWPCPCException& e) {\
	      _MR_THROW(e);\
	    } \
	    catch( CWPSIException& e) {\
	      _MR_THROW(e);\
	    } \
	    catch( CWPCNSException& e) {\
	      _MR_THROW(e);\
	    } \
	    catch( CWPCRIException& e) {\
	      _MR_THROW(e);\
	    }

#define _MR_CMDREADPARAM(a) \
	  try { \
	    CmdReadParam(a); \
	  } \
	  catch ( CWPCRPException& e ) { \
	    _MR_THROW(e);\
	  } \
	  catch ( CWPCNSException& e ) { \
	    _MR_THROW(e);\
	  } \
	  catch ( CWPCRIException& e ) { \
	    _MR_THROW(e);\
	  }

#define _MR_CMDWRITEPARAM(a,b,c) \
	  try { \
	    CmdWriteParam(a,b,c); \
	  } \
	  catch ( CWPCWPException& e ) { \
	    _MR_THROW(e);\
	  } \
	  catch ( CWPCNSException& e ) { \
	    _MR_THROW(e);\
	  } \
	  catch ( CWPCRIException& e ) { \
	    _MR_THROW(e);\
	  }

#define _MR_THROW(a) \
	    this->Tracer(a.ToString(),NULL); \
	    throw a;

	/// énumération pour les codes de contrôle
	enum CMDCTRL {
		ACK = 0x06, NACK = 0x15, WERROR = 0x00
	};

	// Collision avec SDK Coronis
	enum STATUS {
		OKC = 0x00, NOKC = 0x01, UPDERROR = 0xFF
	};

	/// Enumération des commandes de services
	enum CMDSERVICE {
		REQ_SEND_SERVICE = 0x80,
		RES_SEND_SERVICE = 0x81,
		SERVICE_RESPONSE = 0x82
	};

	/// Enumération pour les type de requête lors d'une requête de service
	enum REQTYPE {
		GET_TYPE = 0x20, GET_FW_VERSION = 0x28
	};

	/// Enumération pour les type de réponse lors d'une requête de service
	enum RESPONSETYPE {
		RESP_GET_TYPE = 0xA0, RESP_GET_FW_VERSION = 0xA8
	};

	/// Enumération des commandes de transmission de données par radio
	enum CMDTRANSRF {
		REQ_SEND_FRAME = 0x20,
		RES_SEND_FRAME = 0x21,
		REQ_SEND_MESSAGE = 0x22,
		REQ_SEND_BROADCAST_RESPONSE = 0x24,
		REQ_SEND_POLLING = 0x26,
		REQ_SEND_BROADCAST = 0x28,
		REQ_SEND_BROADCAST_MESSAGE = 0x2A,
		RECEIVED_FRAME = 0x30,
		RECEPTION_ERROR = 0x31,
		RECEIVED_FRAME_POLLING = 0x32,
		RECEIVED_BROADCAST_RESPONSE = 0x34,
		RECEIVED_FRAME_RELAYED = 0x35,
		RECEIVED_MULTIFRAME = 0x36,
		END_MESSAGE_EXCHANGE = 0x37,
		RECEIVED_BROADCAST_FRAME = 0x38
	};

	/// Enumération des commandes de paramétrage de contrôle
	enum CMDPARAMCTRL {
		REQ_CHANGE_UART_BDRATE = 0x42,
		RES_CHANGE_UART_BDRATE = 0x43,
		REQ_CHANGE_TX_POWER = 0x44,
		RES_CHANGE_TX_POWER = 0x45,
		REQ_WRITE_AUTOCORR_STATE = 0x46,
		RES_WRITE_AUTOCORR_STATE = 0x47,
		REQ_READ_TX_POWER = 0x54,
		RES_READ_TX_POWER = 0x55,
		REQ_READ_AUTOCORR_STATE = 0x5A,
		RES_READ_AUTOCORR_STATE = 0x5B,
		REQ_SELECT_CHANNEL = 0x60,
		RES_SELECT_CHANNEL = 0x61,
		REQ_READ_CHANNEL = 0x62,
		RES_READ_CHANNEL = 0x63,
		REQ_SELECT_PHYCONFIG = 0x64,
		RES_SELECT_PHYCONFIG = 0x65,
		REQ_READ_PHYCONFIG = 0x66,
		RES_READ_PHYCONFIG = 0x67,
		REQ_READ_REMOTE_RSSI = 0x68,
		RES_READ_REMOTE_RSSI = 0x69,
		REQ_READ_LOCAL_RSSI = 0x6A,
		RES_READ_LOCAL_RSSI = 0x6B,
		REQ_FIRMWARE_VERSION = 0xA0,
		RES_FIRMWARE_VERSION = 0xA1,
		MODE_TEST = 0xB0
	};

	/// énumération des tailles des réponses de commandes de paramétrage de contrôle
	enum SIZEREQCMDPARAMCTRL {
		SIZEREQ_CHANGE_UART_BDRATE = 8,
		SIZEREQ_CHANGE_TX_POWER = 8,
		SIZEREQ_WRITE_AUTOCORR_STATE = 8,
		SIZEREQ_READ_TX_POWER = 7,
		SIZEREQ_READ_AUTOCORR_STATE = 9,
		SIZEREQ_SELECT_CHANNEL = 8,
		SIZEREQ_READ_CHANNEL = 7,
		SIZEREQ_SELECT_PHYCONFIG = 9,
		SIZEREQ_READ_PHYCONFIG = 7,
		SIZEREQ_READ_REMOTE_RSSI = 13,
		SIZEREQ_READ_LOCAL_RSSI = 13,
		SIZEREQ_FIRMWARE_VERSION = 7,
		SIZEREQ_MODE_TEST = 0
	};

	/// Enumération des commandes de configuration des paramètres fonctionnels (internes- radio)
	enum FONCTPARAM {
		AWAKENING_PERIOD = 0x00,
		WAKEUP_TYPE = 0x01,
		WAKEUP_LENGTH = 0x02,
		WAVECARD_POLLING_GROUP = 0x03,
		RADIO_ACKNOWLEDGE = 0x04,
		RADIO_ADDRESS = 0x05,
		RELAY_ROUTE_STATUS = 0x06,
		RELAY_ROUTE = 0x07,
		POLLING_ROUTE = 0x08,
		GROUP_NUMBER = 0x09,
		POLLING_TIME = 0x0A,
		RADIO_USER_TIMEOUT = 0x0C,
		EXCHANGE_STATUS = 0x0E,
		SWITCH_MODE_STATUS = 0x10,
		WAVECARD_MULTICAST_GROUP = 0x16,
		BCST_RECEPTION_TIMEOUT = 0x17
	};

	enum CMDPARAMFONCT {
		REQ_WRITE_RADIO_PARAM = 0x40,
		RES_WRITE_RADIO_PARAM = 0x41,
		REQ_READ_RADIO_PARAM = 0x50,
		RES_READ_RADIO_PARAM = 0x51
	};

	enum SIZEREQCMDPARAMFONCT {
		SIZEREQ_READ_RADIO_PARAM = 8,
		SIZEREQ_WRITE_RADIO_PARAM = 8,
		SIZEREQ_WRITE_RADIO_PARAM_AWAKENING_PERIOD = 8 + 1,
		SIZEREQ_WRITE_RADIO_PARAM_WAKEUP_TYPE = 8 + 1,
		SIZEREQ_WRITE_RADIO_PARAM_WAKEUP_LENGTH = 8 + 2,
		SIZEREQ_WRITE_RADIO_PARAM_WAVECARD_POLLING_GROUP = 8 + 1,
		SIZEREQ_WRITE_RADIO_PARAM_RADIO_ACKNOWLEDGE = 8 + 1,
		SIZEREQ_WRITE_RADIO_PARAM_RADIO_ADDRESS = 8 + 6,
		SIZEREQ_WRITE_RADIO_PARAM_RELAY_ROUTE_STATUS = 8 + 1,
		SIZEREQ_WRITE_RADIO_PARAM_RELAY_ROUTE = 8 + 19, // de 1 à 19 = 1(nombre de module concernés) + 3(max)*6 octets d'adresse radio
		SIZEREQ_WRITE_RADIO_PARAM_POLLING_ROUTE = 8 + 241, // 1 à 241= 1(nombre de modules concernés) + 40(max)*6 octets d'adresse radio
		SIZEREQ_WRITE_RADIO_PARAM_GROUP_NUMBER = 8 + 1,
		SIZEREQ_WRITE_RADIO_PARAM_POLLING_TIME = 8 + 1,
		SIZEREQ_WRITE_RADIO_PARAM_RADIO_USER_TIMEOUT = 8 + 1,
		SIZEREQ_WRITE_RADIO_PARAM_EXCHANGE_STATUS = 8 + 1,
		SIZEREQ_WRITE_RADIO_PARAM_SWITCH_MODE_STATUS = 8 + 1,
		SIZEREQ_WRITE_RADIO_PARAM_WAVECARD_MULTICAST_GROUP = 8 + 1,
		SIZEREQ_WRITE_RADIO_PARAM_BCST_RECEPTION_TIMEOUT = 8 + 1
	};

	enum SIZERESCMDPARAMFONCT {
		SIZERES_WRITE_RADIO_PARAM = 8,
		SIZERES_READ_RADIO_PARAM_AWAKENING_PERIOD = 8 + 1,
		SIZERES_READ_RADIO_PARAM_WAKEUP_TYPE = 8 + 1,
		SIZERES_READ_RADIO_PARAM_WAKEUP_LENGTH = 8 + 2,
		SIZERES_READ_RADIO_PARAM_WAVECARD_POLLING_GROUP = 8 + 1,
		SIZERES_READ_RADIO_PARAM_RADIO_ACKNOWLEDGE = 8 + 1,
		SIZERES_READ_RADIO_PARAM_RADIO_ADDRESS = 8 + 6,
		SIZERES_READ_RADIO_PARAM_RELAY_ROUTE_STATUS = 8 + 1,
		SIZERES_READ_RADIO_PARAM_RELAY_ROUTE = 8 + 19,    // de 1 à 19 XXXX
		SIZERES_READ_RADIO_PARAM_POLLING_ROUTE = 8 + 241,      // 1 à 241 XXXX
		SIZERES_READ_RADIO_PARAM_GROUP_NUMBER = 8 + 1,
		SIZERES_READ_RADIO_PARAM_POLLING_TIME = 8 + 1,
		SIZERES_READ_RADIO_PARAM_RADIO_USER_TIMEOUT = 8 + 1,
		SIZERES_READ_RADIO_PARAM_EXCHANGE_STATUS = 8 + 1,
		SIZERES_READ_RADIO_PARAM_SWITCH_MODE_STATUS = 8 + 1,
		SIZERES_READ_RADIO_PARAM_WAVECARD_MULTICAST_GROUP = 8 + 1,
		SIZERES_READ_RADIO_PARAM_BCST_RECEPTION_TIMEOUT = 8 + 1
	};

	/// énumération des différentes configuration physique du module du point de vue Radio et du point de vue transmission série
	enum TRMODES {
		F433MHz_frequency_hopping_9600 = 0x00A1,
		F868MHz_single_channel_4800 = 0x0012,
		F868MHz_single_channel_4800_Alarm_Band = 0x0094,
		F868MHz_single_channel_9600_with_channel_selection = 0x00A2,
		F868MHz_frequency_hopping_9600 = 0x00A3,
		F868MHz_frequency_hopping_19200 = 0x00B3,
		F869MHz_500mW_Band = 0x00B6,
		F915MHz_frequency_hopping_19200 = 0x00B9
	};

	/// énumération des Longueurs des trames fixes
	enum SIZETRAMEFIXES {
		LONGUEURPDUA = 7, // longueur de la trame ACKRS232  0 octet de data =7
		LONGUEURPDUNA = 7, // longueur de la trame NACKRS232 0 octet de data = 7
		LONGUEURPDUERR = 8, // longueur de la trame ERROR 1 octet de data = 8
		LONGUEURPDURES = 8
	};

	enum TESTMODES {
		CONT_RECEPT = 0x00,
		CONT_TR_WOMOD = 0x01,
		CONT_TR_WMOD = 0x02,
		STANDBY = 0x03
	};

	// Constructeur sans adresse radio. Le port série par défaut est utilisé.
	CWavePort(CConfig& Conf, FILE* Stream = NULL);
	// Constructeur sans adresse radio. Le port série est passé en paramètre.
	CWavePort(CConfig& Conf, string PortName, FILE* Stream = NULL);
	// Constructeur avec adresse radio. Le port série par défaut est utilisé.
	CWavePort(CConfig& Conf, CAdrRadio* AdrRadio, FILE* Stream = NULL);
	// Constructeur avec adresse radio et port série.
	CWavePort(CConfig& Conf, string PortName, CAdrRadio* AdrRadio, FILE* Stream = NULL);
	virtual ~CWavePort();

	// Methodes publiques de la classe
	// ex : ReturnType NomMethode(Type);
	// ouvrir l'accès au waveport. le port série est ouvert.
//	void Open(void);
//	// Fermer l'accès au waveport. Le port série est fermé.
//	void Close(void);
//	// vider les buffers du port série associé
//	void Flush();

	bool SerialDriverAndProtocolLoad(void);
	void SerialDriverAndProtocolFree(void);
	bool SerialDriverAndProtocolInstanciation(void);
	void SerialDriverAndProtocolSuppression(void);
	int ProtocolInit(void);
	int ProtocolOpen(void);
	void ProtocolClose(void);
	const char* GetCurrentTimeString();

	// Modifier le flux de trace
	void setFlux_trace(FILE *);
	// Renseigner l'adresse radio du module
	void setRadio_address(CAdrRadio* AdrRadio);

	///////////////////////////////////////////////////////////////////////////////////////////////////
	// PARAMETRAGE DE CONTROLE DU WAVEPORT
	// Interrogation par les requêtes d'application
	///////////////////////////////////////////////////////////////////////////////////////////////////
	// Obtenir la version du firmware
	int getfirmware_version(void);
	// Obtenir le mode de transmission série et radio actuel
	int getphyconfig(void);
	// Fixer le mode de transmission série et radio
	void setphyconfig(int phyconfig);
	// Obtient l'indicateur de force du signal reçu par un module distant lors d'une émission vers ce module.
	Byte getremote_rssi(CAdrRadio* AdrRadioRemoteModule);
	// Obtenir l'indicateur de force du signal reçu lors de la réception d'un module distant.
	Byte getlocal_rssi(CAdrRadio* AdrRadioRemoteModule);
	// Fixer la vitesse de transmission. Les Vitesses sont dans BAUDRATES
	//void setuart_bdrate(SerialPort::BaudRates uart_bdrate);
	// Obtenir le canal radio lorsque le mode FHSS est désactivé
	Byte getchannel(void);
	// Fixer le canal radio lorsque le mode FHSS est désactivé
	void setchannel(Byte channel);
	// Obtenir la puissance de transmission. Les puissances disponibles sont dans TXPOWER
	Byte gettx_power(void);
	// Fixer la puissance de transmission. Les puissances disponibles sont dans TXPOWER
	void settx_power(Byte tx_power);
	// Obtenir l'état ACTIV/DESACTIVE du mode auto-correction RSSI
	Byte getautocorr_state();
	// Fixer l'état ACTIV/DESACTIVE du mode auto-correction RSSI
	void setautocorr_state(Byte autocorr_state);

	// Passer le waveport en mode test. 4 modes de test sont accessibles dans TESTMODES
	void TestMode(Byte Mode);

	///////////////////////////////////////////////////////////////////////////////////////////////////
	// PARAMETRAGE FONCTIONNEL DU WAVEPORT
	// Interrogation par les requêtes générales READ_RADIO_PARAM et/ou WRITE_RADIO_PARAM
	///////////////////////////////////////////////////////////////////////////////////////////////////

	// Retrouver le time out dans la transmission radio qui impose le temps d'inactivité sur la ligne série. Par défaut radio_user_timeout = 0x14 soit 20*100ms = 2 s.
	Byte getRadio_user_timeout();
	// Fixer le time out dans la transmission radio qui impose le temps d'inactivité sur la ligne série. La valeur est en multiple de 100ms. Par défaut radio_user_timeout = 0x14 soit 20*100ms = 2 s.
	void setRadio_user_timeout(Byte radio_user_timeout);
	Byte getAwakening_period();
	Byte getBcst_reception_timeout();
	Byte getExchange_status();
	Byte getGroup_number();
	LSTADRRADIO getPolling_route();
	Byte getPolling_time();
	Byte getRadio_acknowledge();
	CAdrRadio getRadio_address();
	LSTADRRADIO getRelay_route();
	Byte getRelay_route_status();
	Byte getSwitch_mode_status();
	int getWakeup_length();
	Byte getWakeup_type();
	Byte getWavecard_multicast_group();
	Byte getWavecard_polling_group();
	void setAwakening_period(Byte awakening_period);
	void setBcst_reception_timeout(Byte bcst_reception_timeout);
	void setChannel(Byte channel);
	void setExchange_status(Byte exchange_status);
	void setGroup_number(Byte group_number);
	void setPolling_route(LSTADRRADIO* polling_routes);
	void setPolling_time(Byte polling_time);
	void setRadio_acknowledge(Byte radio_acknowledge);
	void setRelay_route(LSTADRRADIO* relay_routes);
	void setRelay_route_status(Byte relay_route_status);
	void setSwitch_mode_status(Byte switch_mode_status);
	void setWakeup_length(int wakeup_length);
	void setWakeup_type(Byte wakeup_type);
	void setWavecard_multicast_group(Byte wavecard_multicast_group);
	void setWavecard_polling_group(Byte wavecard_polling_group);

	// implémentation d'une property pour ReadTimeout
	/// Obtient ou fixe la durée en 1/10 ième de seconde avant que le délai d'attente soit dépassé dans une opération de lecture.
	property<CWavePort, int, READ_WRITE> ReadTimeout;

	// Interrogation par les requêtes de service
	Byte ServiceTypeModule(CAdrRadio* AdresseRadio);
	int ServiceFirmware(CAdrRadio* AdresseRadio);

	// Utilisé par les modules distant pour communiquer avec l'hôte
	void CmdRadioMd(Byte PDUmd[], int TaillePDUmd);
	void CmdRadioMd(Payload payload, RadioAddress destinationAddress);

	// Retourner le flux de traçage pour les classes utilisatrices.
	FILE* getStream(void);

	// la version du firmware en chaîne
	string firmware_version;
	// Pour les associations :
	// Methodes publiques setter/getter (mutateurs/accesseurs) des attributs prives
	// ex :
	// void setNomAttribut(Type nomAttribut) { nomAttribut_ = nomAttribut; }
	// Type getNomAttribut(void) const { return nomAttribut_; }

	RequestResponse getResponse(void )  { return m_Response; };

protected:
	// Attributs proteges

	// Methode protegees

private:
	// Attributs prives
	// ex :
	// Type nomAttribut_;

	string m_driverId;
	string m_protocolId;

	SerialDriverLoader m_driverHandler;
	ProtocolLoader m_protocolHandler;

	void * m_rs232DriverInstance;
	void * m_waveportInstance;

	// Association avec un port série
	// SerialPort* pS;
	// délai d'attente dans les lectures
	int m_ReadTimeout;

	// délai d'attente dans les lectures
	int get_ReadTimeout() const;
	void set_ReadTimeout(int ReadTimeout);

	///////////////////////////////////////////////////////////////////////////////////////////////////
	// PARAMETRES DE CONTROLE DU WAVEPORT
	// accédés par des reqêtes spécifiques
	///////////////////////////////////////////////////////////////////////////////////////////////////
	int m_phyconfig;
	Byte m_channel;
	Byte m_tx_power;
	Byte m_autocorr_state;
	int m_firmware_version;
	Byte m_local_rssi;
	Byte m_remote_rssi;
	Byte m_uart_bdrate;

	// Etat du mode de test
	Byte m_testmode;

	///////////////////////////////////////////////////////////////////////////////////////////////////
	// PARAMETRES FONCTIONNELS DU WAVEPORT
	// accédé par les requêtes générales READ_RADIO_PARAM et/ou WRITE_RADIO_PARAM
	///////////////////////////////////////////////////////////////////////////////////////////////////

	Byte m_awakening_period;
	Byte m_wakeup_type;
	int m_wakeup_length;
	Byte m_wavecard_polling_group;
	Byte m_radio_acknowledge;
	CAdrRadio* m_radio_address;
	Byte m_relay_route_status;
	Byte m_relay_route[MAXRELAYROUTES * SIZEADRRADIO + 1];
	// liste des adresses relai dans le mode relayé
	LSTADRRADIO m_relay_routes;
	Byte m_polling_route[241];
	LSTADRRADIO m_polling_routes;
	Byte m_group_number;
	Byte m_polling_time;
	Byte m_radio_user_timeout;
	Byte m_exchange_status;
	Byte m_switch_mode_status;
	Byte m_wavecard_multicast_group;
	Byte m_bcst_reception_timeout;
	// Variables de configurations
	int m_Nb_Polling_Route;        // nombre de modules concernés par le polling
	int m_Nb_Relay_Route;        // nombre de modules concernés par le transfert

	///////////////////////////////////////////////////////////////////////////////////////////////////
	// NIVEAU INTERMEDIAIRE : CONSTRUIRE LA PDU A EMETTRE, EXPLOITER LA PDU RECUE
	///////////////////////////////////////////////////////////////////////////////////////////////////

	void CmdParamCtrl(enum CMDPARAMCTRL Cmd, enum SIZEREQCMDPARAMCTRL treq);
	void CmdReadParam(enum FONCTPARAM NumParam);
	void CmdWriteParam(enum FONCTPARAM NumParam, Byte Param[], int TailleParam);
	void CmdRadio(Byte Cmd, CAdrRadio* AdrRadio, Byte Data[], int TailleData);
	void CmdRadio(Payload payload, RadioAddress destinationAddress);
	// requête de services
	void Service(CAdrRadio* AdresseRadio, Byte ReqType, Byte Response[],
			int TailleR);

	///////////////////////////////////////////////////////////////////////////////////////////////////
	// BAS NIVEAU DANS L'ACCES AU WAVEPORT : EFFECTUER LES REQUETES
	///////////////////////////////////////////////////////////////////////////////////////////////////

	void LireTrame(Byte PDU[]);
	void Received(void);               // Séquence RECEIVED_
	void Req_Res(void);      // séquence REQ_ RES_

	///////////////////////////////////////////////////////////////////////////////////////////////////
	// VERIFICATION DE TRAMES
	///////////////////////////////////////////////////////////////////////////////////////////////////
	void CRC(Byte Trame[], int lt);       // Calculer le CRC16 d'une trame
	bool TesterCRC(Byte Trame[]);         // tester le CRC d'une trame reçue
	// Tester l'octet de status lorsqu'il existe
	void Status(Byte Cmd);

	///////////////////////////////////////////////////////////////////////////////////////////////////
	// TRACAGE DE TOUTE L'ACTIVITE DU WAVEPORT
	///////////////////////////////////////////////////////////////////////////////////////////////////
	void Tracer(std::string Msg, Byte Trame[]); // tracer l'actoivité du transmetteur

	// Temporisation entre RES_ et ACK  entre RECEIVED_ et ACK
	void Tempo(void) {
		usleep(1000);
	}

	FILE* Stream;          // flux pour le traçage
	string PortName;       // Nom du port série associé
	Byte PDUe[256];        // PDU pour l'émission
	Byte PDUr[256];        // PDU pour la réception RES_ ou RECEIVED_
	Byte PDUa[8]; // PDU d'acquittement RS232: ACK/NACK: 7 octets mais ERROR 8 octets
	Byte DataService[6]; // pour les données dans les requête de service. la taille maxi est 6 octets
	int RetryReq;          // Tentatives de ré-émission d'une requête  REQ_
	int RetryRes;          // tentatives de reprise d'une réception d'un RES_
	int RetryRec; // Tentatives de reprise d'une réception de RECEIVED_ lors de la réponse du module distant
	int RetryTO;           // Tentatives de relecture en cas de timeout
	int LongueurPDUe;      // longueur de la trame à émettre

	RadioExchangeCommand m_Command;
	RequestResponse m_Response;


//	void*	rs232DriverInstance;
//	void*	waveportInstance;

	// Methodes privees
};

// Methodes publiques inline
// ex :
// inline void CWavePort::maMethodeInline(Type valeur)
// {
//   
// }
// inline Type CWavePort::monAutreMethode_inline_(void)
// {
//   return 0;
// }

}// namespace WAVENIS

#endif  // _CWAVEPORT_H
