/*
 *  Copyright (C) 2012  plegal  (plegal@appert44.org)
 *  @file         CWavePortTest.cpp
 *  Classe        CWavePort
 *  @note         Implementation de la classe en charge des tests unitaires
 */

// Includes system C
// serial driver api include
#include <serial_driver_event.h>
#include <serial_driver_status.h>
// protocol api include
#include <protocol_command_types.h>
#include <protocol_response_types.h>
// domain include
#include "loader_serial_driver.h"
#include "loader_protocol.h"
#include "serial_driver_types.h"
#include "protocol_types.h"
#include "protocol_types_utils.h"

#include "utils.h"

// Includes system C++
#include <cppunit/CompilerOutputter.h>
#include <cppunit/XmlOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/TestResult.h>
#include <cppunit/TestResultCollector.h>
#include <cppunit/TestRunner.h>
#include <cppunit/BriefTestProgressListener.h>
#include <string>
#include <iostream>

using namespace std;

// Includes qt

// Includes application
#include "CWavePortTest.h"
#include "CConfig.h"
#include "CAdrRadio.h"

using namespace WAVENIS;
// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION(CWavePortTest);

// Exemple d'assertions possibles
// # CPPUNIT_ASSERT(condition)
// Assertions that a condition is true.
// # CPPUNIT_ASSERT_MESSAGE(message, condition)
// Assertion with a user specified message.
// # CPPUNIT_FAIL(message)
// Fails with the specified message
// # CPPUNIT_ASSERT_EQUAL(expected, actual)
// Asserts that two values are equals.
// # CPPUNIT_ASSERT_EQUAL_MESSAGE(message, expected, actual)
// Asserts that two values are equals, provides additional message on failure
// # CPPUNIT_ASSERT_DOUBLES_EQUAL(expected, actual, delta)

// setUp() to initialize the variables you will use for test
void CWavePortTest::setUp() {
}

// tearDown() to release any permanent resources you allocated in setUp()
void CWavePortTest::tearDown() {
}

// Suite des tests unitaires

void CWavePortTest::ConstructorTest() { 
	// Construction de l'instance de classe a tester
//	try {
		CConfig config("config.conf");
		CAdrRadio *adrRadio = new CAdrRadio("000000000000");
		CWavePort *cwaveport = new CWavePort(config, (string) "/dev/ttyUSB0",
				adrRadio, NULL);
		CPPUNIT_ASSERT(cwaveport != NULL);
		delete cwaveport;
		delete adrRadio;
//	} catch (...) {
//		printf("\nexception\n");
//	}

}

void CWavePortTest::SerialDriverAndProtocolTest() {
	// Construction de l'instance de classe a tester
	//try {
		cout << "load config.conf" << endl;
		CConfig config("config.conf");
		cout << "new adrRadio" << endl;
		CAdrRadio *adrRadio = new CAdrRadio("000000000000");
		cout << "new waveport" << endl;
		CWavePort *cwaveport = new CWavePort(config, (string) "/dev/ttyUSB0",
				adrRadio, NULL);
		CPPUNIT_ASSERT(cwaveport != NULL);
		cout << "SerialDriver and Protocol load" << endl;
		bool ret = cwaveport->SerialDriverAndProtocolLoad();
		CPPUNIT_ASSERT_MESSAGE("Serial Driver & Protocol Load", ret == true);
		cout << "SerialDriver and Protocol Instanciation" << endl;
		ret = cwaveport->SerialDriverAndProtocolInstanciation();
		CPPUNIT_ASSERT_MESSAGE("Serial Driver & Protocol Instanciation",
				ret == true);
		cout << "Protocol Init" << endl;
		int status = cwaveport->ProtocolInit();
		CPPUNIT_ASSERT_MESSAGE("Protocol Init", status == OK_PROTOCOL_SUCCESS);
		cout << "Protocol Open" << endl;
		status = cwaveport->ProtocolOpen();
		CPPUNIT_ASSERT_MESSAGE("Protocol Open", status == OK_PROTOCOL_SUCCESS);
		cout << "Protocol Close" << endl;
		cwaveport->ProtocolClose();
		cout << "SerialDriver and Protocol Suppression" << endl;
		cwaveport->SerialDriverAndProtocolSuppression();
		cout << "SerialDriver and Protocol Free" << endl;
		cwaveport->SerialDriverAndProtocolFree();
		cout << "delete cwaveport" << endl;
		delete cwaveport;
		cout << "delete adrRadio" << endl;
		delete adrRadio;
//	} catch (...) {
//		printf("\nexception\n");
//	}

}

// the main method
int main(int argc, char* argv[]) {
	// informs test-listener about testresults
	CPPUNIT_NS::TestResult testresult;

	// register listener for collecting the test-results
	CPPUNIT_NS::TestResultCollector collectedresults;
	testresult.addListener(&collectedresults);

	// register listener for per-test progress output
	CPPUNIT_NS::BriefTestProgressListener progress;
	testresult.addListener(&progress);

	// insert test-suite at test-runner by registry
	CPPUNIT_NS::TestRunner testrunner;
	testrunner.addTest(
			CPPUNIT_NS::TestFactoryRegistry::getRegistry().makeTest());
	testrunner.run(testresult);

	// output results in compiler-format
	CPPUNIT_NS::CompilerOutputter compileroutputter(&collectedresults,
			std::cerr);
	compileroutputter.write();

	// for hudson
	std::ofstream file("CWavePort-cppunit-report.xml");
	CPPUNIT_NS::XmlOutputter xmloutputter(&collectedresults, file);
	xmloutputter.write();
	file.close();

	// return 0 if tests were successful
	return collectedresults.wasSuccessful() ? 0 : 1;
}

