/**
 *  Copyright (C) 2012  plegal  (plegal@appert44.org)
 *  @file         CWaveTherm.cpp
 *  @brief        Pilote pour les éléments communs des périphériques capteurs thermiques
 *  @version      0.1
 *  @date         17 déc. 2012 21:08:44
 *
 *  Description detaillee du fichier CWaveTherm.cpp
 *  Fabrication   gcc (Ubuntu/Linaro 4.6.3-1ubuntu5) 4.6.3 
 *  @todo         Liste des choses restant a faire.
 *  @bug          17 déc. 2012 21:08:44 - Aucun pour l'instant
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Includes system C

// Includes system C++
// A décommenter si besoin cout, cin, ...
// #include <iostream>
// using namespace std;

// Includes qt

// serial driver api include
//#include <serial_driver_event.h>
//#include <serial_driver_status.h>
//// protocol api include
//#include <protocol_command_types.h>
//#include <protocol_response_types.h>
//// domain include
//#include "loader_serial_driver.h"
//#include "loader_protocol.h"
//#include "serial_driver_types.h"
#include "protocol_types.h"	// RadioAddress
//#include "protocol_types_utils.h"

// Includes application
#include "CWaveTherm.h"
#include "CWNExceptions.h"

/// Espace de noms pour les classes de la librairie WAVENIS
namespace WAVENIS {

/**
 * Constructeur
 */
/// Constructeur de la classe CWaveTherm
/// \param pwp : pointeur sur le module transmetteur de type WavePort.
/// \param AdresseRadio : adresse radio du module associé
/// \see CWavePort
CWaveTherm::CWaveTherm( CWavePort* pwp, CAdrRadio* AdresseRadio )
: CModuleFinal ( pwp, AdresseRadio ) {
// XXXX probleme  Init_Sensors();                     // Détecter le nombre effectif de capteurs connectés.
}


CWaveTherm::CWaveTherm( CWavePort* pwp, RadioAddress radioAddress )
: CModuleFinal ( pwp, radioAddress ) {
// XXXX probleme  Init_Sensors();                     // Détecter le nombre effectif de capteurs connectés.
}


/**
 * Destructeur
 */
CWaveTherm::~CWaveTherm()
{
}

// Methodes publiques
// ex :
/**
 * Exemple de description d'une methode
 *
 * @param un_parametre  Description d'un parametre
 * @return              Description du retour
 */
// ReturnType CWaveTherm::NomMethode(Type parametre)
// {
// }
/// Lire la table de détection des seuils en mode datalogging
/// \par Structure de la table. l'enregistrement pour un capteur est :
/// \arg Numéro du capteur (1 octet)
/// \arg Date de la détection (6 octet)
/// \arg Durée du dépassement (1 octet) : cette durée est exprimée en multiple de la période de mesure
/// \arg Valeur moyenne : valeur moyenne des mesures effectuée entre le début et la fin du dépassement. La taille de ce champ est fonction du format de la mesure. seul la classe dérivée en connaît la valeur.
/// \par La table de détection est une FIFO des 5 dernires enregistrements.
/// \par En sortie, la PDUe contient la table. La classe dérivée extraira les données car elle seule connaît la taille du champ "valeur moyenne".
/// \see DetectionSeuilsTable
void CWaveTherm::ReadDetectionSeuilsTable( void ){
  this->NbDonneesSpecifiques = 0;
  _MR_MF_CMDPARAMCTRL((enum CModuleFinal::CMDAPP)REQ_READ_THRESHOLD_DETECTION_TABLE);
}

/// Lire les mesures enregistrées dans la table en mode datalogging "time steps".
/// \par En sortie, la structure DataLoggingTable est renseignée comme si il y avait 2 capteurs actifs.
/// \par Il est à la charge de l'appelant de connaître le nombre de capteurs actifs et d'en déduire
/// le volume de données disponibles par capteurs.
/// \see DataLoggingTable
LPDATALOGGINGTABLE CWaveTherm::ReadDataLoggingTable(){
  this->NbDonneesSpecifiques = 0;
  _MR_MF_CMDPARAMCTRL((enum CModuleFinal::CMDAPP)REQ_READ_DATALOGGING_TABLE);
  // selon le nombre de capteurs détecté, il y a N ou N/2 mesures par capteurs
  // dans tous les cas la table est remplie comme si il y avait 2 capteurs actifs.
  // il est à la charge de l'appelant de connaître le nombre de capteurs actifs et d'en déduire
  // le volume de données disponibles.
  // remplir la table des mesures du capteur 1
  for ( int i=0; i< SIZEDLTABLE; i++ )
    this->DataLoggingTable.DataLogged[VOIE1][i] = PDUe[SIZEADRRADIO+1+1+1+i];
  // remplir la table des mesures du capteur 2
  for ( int i=0; i< SIZEDLTABLE; i++ )
    this->DataLoggingTable.DataLogged[VOIE2][i] = PDUe[SIZEADRRADIO+1+1+1+SIZEDLTABLE+i];
  // remplir la date du dernier échantillon
  for ( int i=0; i<SIZEDATE; i++ )
    this->DataLoggingTable.LastLogDate[i] = PDUe[SIZEADRRADIO+1+1+1+2*SIZEDLTABLE+i];
  this->m_Measurement_Period = PDUe[SIZEADRRADIO+1+1+1+2*SIZEDLTABLE+SIZEDATE];
  return &this->DataLoggingTable;
}

/// Détecter la présence des capteurs.
/// Selon le type de capteur DALLAS ou PT, des informations supplémentaires sont fournies
/// \see CDallas::Init_Sensors
/// \see CPT100::Init_Sensors
/// \see CPT1000::Init_Sensors
/// \return le nombre de capteurs détectés.
Byte CWaveTherm::Init_Sensors() {
  this->NbDonneesSpecifiques = 0;
  _MR_MF_CMDPARAMCTRL((enum CModuleFinal::CMDAPP)REQ_INIT_SENSORS);
  this->m_NCapteurs = this->PDUe[SIZEADRRADIO+1];
  return this->m_NCapteurs;
}

/// Fixer l'octet de status du module précisant la configuration du module.
/// Les éléments de configuration sont inscrits par un bit de cet octet.
/// \arg bit0 : End of battery life -> (1)= la fin de vie de la batterie est détectée  -- (0)= non détectée
/// \arg bit1 : sans signification
/// \arg bit2 : Nombre de capteurs détectés -> (0)= zéro ou un capteur détecté  -- (1)= 2 capteurs détectés
/// \arg bit3 : seuil haut du capteur1 -> (0)= non détecté -- (1)= détecté
/// \arg bit4 : seuil bas  du capteur1 -> (0)= non détecté -- (1)= détecté
/// \arg bit5 : seuil haut du capteur2 -> (0)= non détecté -- (1)= détecté
/// \arg bit6 : seuil bas  du capteur2 -> (0)= non détecté -- (1)= détecté
/// \arg bit7 : RAZ                    -> (0)= non détecté -- (1)= détecté
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
void CWaveTherm::setApplication_Status(Byte m_Application_Status) {
  CModuleFinal::setApplication_Status(m_Application_Status);
  this->m_Application_Status = m_Application_Status;
}

/// Retrouver l'octet de status du module précisant la configuration du module.
/// Les éléments de configuration sont inscrits par un bit de cet octet.
/// \arg bit0 : End of battery life -> (1)= la fin de vie de la batterie est détectée  -- (0)= non détectée
/// \arg bit1 : sans signification
/// \arg bit2 : Nombre de capteurs détectés -> (0)= zéro ou un capteur détecté  -- (1)= 2 capteurs détectés
/// \arg bit3 : seuil haut du capteur1 -> (0)= non détecté -- (1)= détecté
/// \arg bit4 : seuil bas  du capteur1 -> (0)= non détecté -- (1)= détecté
/// \arg bit5 : seuil haut du capteur2 -> (0)= non détecté -- (1)= détecté
/// \arg bit6 : seuil bas  du capteur2 -> (0)= non détecté -- (1)= détecté
/// \arg bit7 : RAZ                    -> (0)= non détecté -- (1)= détecté
/// \return Un octet contenant des informations de donfiguration inscrite dans chacun de ses 8 bits.
/// \par Gestion des exceptions:
/// \arg CWPCPCException : Erreur dans une commande de lecture ou d'écriture d'un paramètre de contrôle. Cette exception contient :
/// -# la nature de l'échec dans la séquence Req,res du protocole Wavenis (Nombre maxi d'essais REQ, nombre maxi d'essais RES)
/// -# le numéro de la commande pour laquelle la lecture ou l'écriture a échoué
/// \see CWPCPCException
/// \arg CWPCNSException : Erreur liée à une commande non supportée. Cette exception contient :
/// -# le numéro de la commande non supportée
/// \see CWPCNSException
/// \arg CWPCRIException : La commande RES retournée est incorrecte. Cette exception contient :
/// -# le numéro de la commande REQ émise
/// -# le numéro de la commande RES incorrecte reçue
/// \see CWPCRIException
/// \arg CWPCRException : erreur dans la commande Radio
/// \see CWPCRException
/// \see SetAlarm
Byte CWaveTherm::getApplication_Status() {
  m_Application_Status = CModuleFinal::getApplication_Status();
  return m_Application_Status;
}


/// Fixer le mode d'alarme du module.
/// \par
/// \par GESTION DES ALARMES
/// \par Le module peut déclencher des alarmes sur plusieurs conditions:
/// \arg Seuils de mesure haut ou bas atteint en mode datalogging
/// \arg seuil du compteur de durée de vie de la pile atteint
/// \arg Défaillance d'un capteur (seulement pour les PT100 et PT1000)
/// \par Format de l'octet de mode d'alarme :
/// \arg Bit0 : détection du seuil bas de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit1 : détection du seuil haut de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit2 : fin de vie de la batterie
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit3 : défaillance d'un capteur (seulement pour PT100 et PT1000)
/// -# 0 : désactivé
/// -# 1 : activé
/// \par La configuration actuelle des alarmes est disponible par lecture du status de l'application. GetApplication_Status.
/// \par CAS de la TRANSMISSION D'un SEUL TYPE D'ALARME
/// \par 1) Transmission d'une alarme par le module :
/// \arg émet la première alarme (CMDAPP REQ_ = 0x40) avec:
/// -# 1 octet = nature de l'alarme
/// \arg Bit0 : seuil bas de mesures détecté si bit=1
/// \arg Bit1 : seuil haut de mesures détecté si bit=1
/// \arg Bit2 : fin de vie de la batterie si bit=1
/// \arg Bit3 : défaillance d'un capteur détectée (seulement pour PT100 et PT1000) si bit=1
/// -# 6 octets = date de l'alarme
/// \arg en 1 : le jour
/// \arg en 2 : le mois
/// \arg en 3 : l'année dont la valeur complète est 2000+octet3
/// \arg en 4 : le jour de la semaine de 0 pour Dimanche à 6 pour samedi
/// \arg en 5 : l'heure
/// \arg en 6 ! la minute
/// -# n octets optionnels (dans le cas d'une alarme de type seuil atteint)
/// \arg 1 octet : numéro du capteur en défaut
/// \arg 2 octets : durée du défaut en multiples de la période de mesure
/// \arg 2 octets(DALLAS) ou 4 octets (PT100,PT1000) : valeur moyenne des mesures depuis la détection du seuil jusqu'au retour hosr seuil.
/// \par 2) attente un acquittement (CMDAPPACK RES = 0xC0) : l'envoi de l'alarme est recommencé Number_retries_alarm. Entre chaque ré-émission,
/// le module attend Delay_transmission_alarm.
/// \par CAS de la TRANSMISSION De PLUSIEURS TYPES D'ALARME
/// \arg Pour chaque alarme une alerte est transmise : un alarme ne contient qu'une seule cause d'alarme.
/// \arg Tant qu'une alarme n'a pas été acquittée, la suivante n'est pas émise.
/// \param AlarmConfig : le mode d'alarme souhaité
/// Note : Cette configuration peut aussi être fixée en modifiant le paramètre ALARM_CONFIGURATION par la méthode setAlarm_Configuration.
/// \par Seule la classe dérivée appelante connaît les modes d'alarme possibles.
/// \see setNumber_retries_alarm
/// \see getNumber_retries_alarm
/// \see getDelay_transmission_alarm
/// \see setDelay_transmission_alarm
void CWaveTherm::setAlarm( Byte AlarmConfig ) {
  this->NbDonneesSpecifiques = 1;
  int i = SIZEADRRADIO;
  this->PDUe[i++] = REQ_SET_ALARM;
  this->PDUe[i++] = AlarmConfig;
  _MR_MF_CMDPARAMCTRL(REQ_SET_ALARM);   // XXXX
  if ( this->PDUe[SIZEADRRADIO+1] == CWavePort::UPDERROR  )
    throw CWTSAException();                   //  Erreur à la configuration des alarmes
  this->m_Alarm_Configuration = AlarmConfig;
}

/// Retrouver le contenu de la zone de 32 octets des données utilisateurs. En principe cette zone est utilisée pour sauvegarder les coefficients de calibration des capteurs.
/// \return Un tableau de 32 octets contenant les données utilisateurs.
/// \param FirstData : premier octet à lire entre 0 et 31 seulement.
/// \param NbData : nombre d'octets à lire de 0 à 31 seulement.
Byte* CWaveTherm::getUser_data( int FirstData, int NbData ){
  this->NbDonneesSpecifiques = 3;   // 3 octets de données dans la requête MSB LSB first byte to read Nb bytes to read 1octet
  int i = SIZEADRRADIO;
  this->PDUe[i+++1] = (Byte)((FirstData>>8) & 0xFF);
  this->PDUe[i+++1] = (Byte)(FirstData & 0xFF);
  this->PDUe[i+1] = (Byte)(NbData & 0xFF);
  _MR_MF_CMDPARAMCTRL((CModuleFinal::CMDAPP)REQ_READ_USER_DATA);
  for ( int i=FirstData,j=0; i<FirstData+NbData; i++,j++ )
    this->m_User_data[i] = PDUe[SIZEADRRADIO+1+1+1+1+j];
  return m_User_data;
}
/// Enregistrer des données dans la zone de 32 octets des données utilisateurs. En principe cette zone est utilisée pour sauvegarder les coefficients de calibration des capteurs.
/// \param FirstData : premier octet à lire entre 0 et 31 seulement.
/// \param NbData : nombre d'octets à lire de 0 à 31 seulement.
/// \param Data :  les données à inscrire dans la zone de données utilisateurs
void CWaveTherm::setUser_data(  int FirstData, int NbData, Byte Data[]  ) {
  this->NbDonneesSpecifiques = 3+NbData ;     // 3 octets de données dans la requête MSB LSB first byte to read Nb bytes to read 1octet + NbData octets de données;   // 3 octets de données dans la requête MSB LSB first byte to read Nb bytes to read 1octet
  int i = SIZEADRRADIO;
  this->PDUe[i+++1] = (Byte)((FirstData>>8) & 0xFF);
  this->PDUe[i+++1] = (Byte)(FirstData & 0xFF);
  this->PDUe[i+++1] = (Byte)(NbData & 0xFF);
  for ( int j=0; j< NbData; j++ )
    this->PDUe[i+++1] = Data[j];
  _MR_MF_CMDPARAMCTRL((CModuleFinal::CMDAPP)REQ_WRITE_USER_DATA);
  if ( PDUe[SIZEADRRADIO+1+1+1+1] == 0x00 )
    throw CWTEUDException();            // exception Erreur User Data
}

/// Retrouver le nombre d'entrées de la table de stockage en mode datalogging.
/// \par Le datalogging
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la/les mesure(s) associée(s) au périphérique dans une mémoire permanente locale.
/// \arg datalogging standard :
/// -# DALLAS : 48 mesures pour 2 capteurs soit 24 mesures/capteur
/// -# PT100 et PT1000 : 24 mesures pour 2 capteurs soit 12 mesures/capteur
/// \arg datalogging étendu :
/// -# DALLAS : 4500 mesures pour 2 capteurs soit 2250 mesures/capteur
/// -# PT100 et PT1000 : 2000 mesures pour 2 capteurs soit 1000 mesures/capteur
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la mesure associée au périphérique dans une mémoire permanente locale.
/// \par Pour chaque mode de datalogging, trois sous-modes réglés par l'octet d'operating_mode sont possibles (CModuleFinal::setOperating_Mode) :
/// \arg 1) Le mode par tranche de temps exprimée en unité de temps (time steps): dans ce mode la sauvegarde se fait :
/// \par à partir d'une heure de démarrage (setStart_hour_datalogging) de 0 à 23h.
/// \par pour une tranche de temps exprimée en unité de temps (setMeasurement_Period).
/// -# La tranche de temps de la sauvegarde est fixée par les bits 2 à 7 du paramètre Measurement_Period soit de 1 à 63 fois l'unité de temps.
/// -# L'unité de temps correspond aux bits 0 et 1 du même paramètre Measurement_Period selon le tableau suivant :
/// \arg 00 : unité de temps = 1mn
/// \arg 01 : unité de temps = 5mn
/// \arg 10 : unité de temps = 15mn
/// \arg 11 : unité de temps = 30mn
/// Cela implique une durée de sauvegarde maximum de 63 fois 30 mn.
/// \arg 2) Le mode "une fois par semaine" (once a week)
/// \par Dans ce mode la sauvegarde se fait une fois par semaine
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour de la semaine fixé par setDay_week_month avec une valeur de 0 pour le dimanche à 6 pour le samedi.
/// \arg 3) Le mode "une fois par mois" (once a month)
/// \par Dans ce mode la sauvegarde se fait une fois par mois
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour du mois fixé par setDay_week_month avec une valeur allant de 0 à 28. Le changement du nombre de jours dans le mois n'est pas géré.
/// \see CModuleFinal::setOperating_Mode
/// \see setStart_hour_datalogging
/// \see setMeasurement_Period
/// \see setTime_Measurement
/// \return Le nombre d'entrées de la table de stockage en mode datalogging.
Byte CWaveTherm::getNumber_Datalogging_Loop() {
  _MR_MF_CMDREADPARAM((CModuleFinal::PARAM)NUMBER_DATALOGGING_LOOP
                       ,(CModuleFinal::SIZEREQPARAM)SIZEREQ_NUMBER_DATALOGGING_LOOP);
  this->m_Number_Datalogging_Loop = PDUe[SIZEADRRADIO+1+1+1+1];
  return m_Number_Datalogging_Loop;
}

/// Retrouver le compteur de durée de vie de la batterie. Chaque événement de consommation dans le module décrémente un compteur.
/// Lorsque ce compteur passe sous un seuil donné, un message "Fin de vie de la batterie" est signalé dans l'octet de application_status.
/// \return la valeur actuelle du compteur de vie de la batterie.
int CWaveTherm::getBatterylife_duration_counter() {
  _MR_MF_CMDREADPARAM((CModuleFinal::PARAM)BATTERYLIFE_DURATION_COUNTER
                       ,(CModuleFinal::SIZEREQPARAM)SIZEREQ_BATTERYLIFE_DURATION_COUNTER);
  this->m_Batterylife_duration_counter = (int)PDUe[SIZEADRRADIO+1+1+1+1] + ((int)PDUe[SIZEADRRADIO+1+1+1+1+1]<<8);
  return m_Batterylife_duration_counter;
}

/// Retrouver le jour de la semaine ou du mois où la température sera sauvegardée en mode DataLogging périodique "once a week" ou "once a month".
/// \arg si le mode est : une fois par semaine, ce nombre fixe le jour de la semaine. sa valeur est entre 0 pour dimanche à 6 pour samedi.
/// \arg si le mode est : une fois par mois , ce nombre fixe le jour du mois. sa valeur est entre 1 et 28.
/// \par Le datalogging
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la/les mesure(s) associée(s) au périphérique dans une mémoire permanente locale.
/// \arg datalogging standard :
/// -# DALLAS : 48 mesures pour 2 capteurs soit 24 mesures/capteur
/// -# PT100 et PT1000 : 24 mesures pour 2 capteurs soit 12 mesures/capteur
/// \arg datalogging étendu :
/// -# DALLAS : 4500 mesures pour 2 capteurs soit 2250 mesures/capteur
/// -# PT100 et PT1000 : 2000 mesures pour 2 capteurs soit 1000 mesures/capteur
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la mesure associée au périphérique dans une mémoire permanente locale.
/// \par Pour chaque mode de datalogging, trois sous-modes réglés par l'octet d'operating_mode sont possibles (CModuleFinal::setOperating_Mode) :
/// \arg 1) Le mode par tranche de temps exprimée en unité de temps (time steps): dans ce mode la sauvegarde se fait :
/// \par à partir d'une heure de démarrage (setStart_hour_datalogging) de 0 à 23h.
/// \par pour une tranche de temps exprimée en unité de temps (setMeasurement_Period).
/// -# La tranche de temps de la sauvegarde est fixée par les bits 2 à 7 du paramètre Measurement_Period soit de 1 à 63 fois l'unité de temps.
/// -# L'unité de temps correspond aux bits 0 et 1 du même paramètre Measurement_Period selon le tableau suivant :
/// \arg 00 : unité de temps = 1mn
/// \arg 01 : unité de temps = 5mn
/// \arg 10 : unité de temps = 15mn
/// \arg 11 : unité de temps = 30mn
/// Cela implique une durée de sauvegarde maximum de 63 fois 30 mn.
/// \arg 2) Le mode "une fois par semaine" (once a week)
/// \par Dans ce mode la sauvegarde se fait une fois par semaine
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour de la semaine fixé par setDay_week_month avec une valeur de 0 pour le dimanche à 6 pour le samedi.
/// \arg 3) Le mode "une fois par mois" (once a month)
/// \par Dans ce mode la sauvegarde se fait une fois par mois
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour du mois fixé par setDay_week_month avec une valeur allant de 0 à 28. Le changement du nombre de jours dans le mois n'est pas géré.
/// \see CModuleFinal::setOperating_Mode
/// \see setStart_hour_datalogging
/// \see setMeasurement_Period
/// \see setTime_Measurement
/// \return la valeur actuelle du jour d'enregistrement périodique dans la semaine ou le mois.
Byte CWaveTherm::getDay_week_month() {
  _MR_MF_CMDREADPARAM((CModuleFinal::PARAM)DAY_WEEK_MONTH
                       ,(CModuleFinal::SIZEREQPARAM)SIZEREQ_DAY_WEEK_MONTH);
  this->m_Number_Datalogging_Loop = PDUe[SIZEADRRADIO+1+1+1+1];
  return m_Day_week_month;
}

/// Retrouver le délai entre 2 re-transmission d'alarme exprimé en multiples de la période de mesure.
/// \par
/// \par GESTION DES ALARMES
/// \par Le module peut déclencher des alarmes sur plusieurs conditions:
/// \arg Seuils de mesure haut ou bas atteint en mode datalogging
/// \arg seuil du compteur de durée de vie de la pile atteint
/// \arg Défaillance d'un capteur (seulement pour les PT100 et PT1000)
/// \par Format de l'octet de mode d'alarme :
/// \arg Bit0 : détection du seuil bas de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit1 : détection du seuil haut de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit2 : fin de vie de la batterie
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit3 : défaillance d'un capteur (seulement pour PT100 et PT1000)
/// -# 0 : désactivé
/// -# 1 : activé
/// \par La configuration actuelle des alarmes est disponible par lecture du status de l'application. GetApplication_Status.
/// \par CAS de la TRANSMISSION D'un SEUL TYPE D'ALARME
/// \par 1) Transmission d'une alarme par le module :
/// \arg émet la première alarme (CMDAPP REQ_ = 0x40) avec:
/// -# 1 octet = nature de l'alarme
/// \arg Bit0 : seuil bas de mesures détecté si bit=1
/// \arg Bit1 : seuil haut de mesures détecté si bit=1
/// \arg Bit2 : fin de vie de la batterie si bit=1
/// \arg Bit3 : défaillance d'un capteur détectée (seulement pour PT100 et PT1000) si bit=1
/// -# 6 octets = date de l'alarme
/// \arg en 1 : le jour
/// \arg en 2 : le mois
/// \arg en 3 : l'année dont la valeur complète est 2000+octet3
/// \arg en 4 : le jour de la semaine de 0 pour Dimanche à 6 pour samedi
/// \arg en 5 : l'heure
/// \arg en 6 ! la minute
/// -# n octets optionnels (dans le cas d'une alarme de type seuil atteint)
/// \arg 1 octet : numéro du capteur en défaut
/// \arg 2 octets : durée du défaut en multiples de la période de mesure
/// \arg 2 octets(DALLAS) ou 4 octets (PT100,PT1000) : valeur moyenne des mesures depuis la détection du seuil jusqu'au retour hosr seuil.
/// \par 2) attente un acquittement (CMDAPPACK RES = 0xC0) : l'envoi de l'alarme est recommencé Number_retries_alarm. Entre chaque ré-émission,
/// le module attend Delay_transmission_alarm.
/// \par CAS de la TRANSMISSION De PLUSIEURS TYPES D'ALARME
/// \arg Pour chaque alarme une alerte est transmise : un alarme ne contient qu'une seule cause d'alarme.
/// \arg Tant qu'une alarme n'a pas été acquittée, la suivante n'est pas émise.
/// Note : Cette configuration peut aussi être fixée en modifiant le paramètre ALARM_CONFIGURATION par la méthode setAlarm_Configuration.
/// \par Seule la classe dérivée appelante connaît les modes d'alarme possibles.
/// \see setNumber_retries_alarm
/// \see getNumber_retries_alarm
/// \see getDelay_transmission_alarm
/// \return la valeur actuelle du délai.
Byte CWaveTherm::getDelay_transmission_alarm() {
  _MR_MF_CMDREADPARAM((CModuleFinal::PARAM)DELAY_TRANSMISSION_ALARM
                       ,(CModuleFinal::SIZEREQPARAM)SIZEREQ_DELAY_TRANSMISSION_ALARM);
  this->m_Number_Datalogging_Loop = PDUe[SIZEADRRADIO+1+1+1+1];
  return m_Delay_transmission_alarm;
}

/// Retrouver la période de mesure lorsque le mode datalogging est de type times step.
/// \par Le mode opératoire (operating Mode) est un octet qui précise pour un WaveTherm
/// \arg bit0 : sans signification
/// \arg bit1 : Mode d'arrêt du datalogging:
/// -# 0 : permanent loop. Le tableau des mesures stockées est traité comme une FOIFO circulaire.
/// Lorsque la fin du tableau est atteinte, le stockage continue en début de tableau en écrasant les valeurs précédemment stockées.
/// -# 1 : stop memory full. Lorsque la fin du tableau de stockage est atteinte, le datalogging s'interrompt. aucune mesure n'est écrasée.
/// \arg bit2-bit3 : Datalogging
/// -# 00 : désactivé
/// -# 01 : time steps
/// -# 10 : once a week
/// -# 11 : once a month
/// \arg bit4 : high threshold detection - seuil haut de détection
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg bit5 : low threshold detection - seuil bas de détection
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg bit6 : mode de détection des seuils :
/// -# 0 : successive mode
/// -# 1 : cumulative mode
/// \arg bit7 : sans signification
/// \see CModuleFinal::getOperating_Mode
/// \par Le datalogging
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la/les mesure(s) associée(s) au périphérique dans une mémoire permanente locale.
/// \arg datalogging standard :
/// -# DALLAS : 48 mesures pour 2 capteurs soit 24 mesures/capteur
/// -# PT100 et PT1000 : 24 mesures pour 2 capteurs soit 12 mesures/capteur
/// \arg datalogging étendu :
/// -# DALLAS : 4500 mesures pour 2 capteurs soit 2250 mesures/capteur
/// -# PT100 et PT1000 : 2000 mesures pour 2 capteurs soit 1000 mesures/capteur
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la mesure associée au périphérique dans une mémoire permanente locale.
/// \par Pour chaque mode de datalogging, trois sous-modes réglés par l'octet d'operating_mode sont possibles (CModuleFinal::setOperating_Mode) :
/// \arg 1) Le mode par tranche de temps exprimée en unité de temps (time steps): dans ce mode la sauvegarde se fait :
/// \par à partir d'une heure de démarrage (setStart_hour_datalogging) de 0 à 23h.
/// \par pour une tranche de temps exprimée en unité de temps (setMeasurement_Period).
/// -# La tranche de temps de la sauvegarde est fixée par les bits 2 à 7 du paramètre Measurement_Period soit de 1 à 63 fois l'unité de temps.
/// -# L'unité de temps correspond aux bits 0 et 1 du même paramètre Measurement_Period selon le tableau suivant :
/// \arg 00 : unité de temps = 1mn
/// \arg 01 : unité de temps = 5mn
/// \arg 10 : unité de temps = 15mn
/// \arg 11 : unité de temps = 30mn
/// Cela implique une durée de sauvegarde maximum de 63 fois 30 mn.
/// \arg 2) Le mode "une fois par semaine" (once a week)
/// \par Dans ce mode la sauvegarde se fait une fois par semaine
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour de la semaine fixé par setDay_week_month avec une valeur de 0 pour le dimanche à 6 pour le samedi.
/// \arg 3) Le mode "une fois par mois" (once a month)
/// \par Dans ce mode la sauvegarde se fait une fois par mois
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour du mois fixé par setDay_week_month avec une valeur allant de 0 à 28. Le changement du nombre de jours dans le mois n'est pas géré.
/// \see CModuleFinal::setOperating_Mode
/// \see setStart_hour_datalogging
/// \see setMeasurement_Period
/// \see setTime_Measurement
/// \return la valeur de la période de mesure.
Byte CWaveTherm::getMeasurement_Period() {
  _MR_MF_CMDREADPARAM((CModuleFinal::PARAM)MEASUREMENT_PERIOD,(CModuleFinal::SIZEREQPARAM)SIZEREQ_MEASUREMENT_PERIOD);
  this->m_Number_Datalogging_Loop = PDUe[SIZEADRRADIO+1+1+1+1];
  return m_Measurement_Period;
}

/// Retrouver le nombre de tentative d'émission d'une alarme
/// \par
/// \par GESTION DES ALARMES
/// \par Le module peut déclencher des alarmes sur plusieurs conditions:
/// \arg Seuils de mesure haut ou bas atteint en mode datalogging
/// \arg seuil du compteur de durée de vie de la pile atteint
/// \arg Défaillance d'un capteur (seulement pour les PT100 et PT1000)
/// \par Format de l'octet de mode d'alarme :
/// \arg Bit0 : détection du seuil bas de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit1 : détection du seuil haut de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit2 : fin de vie de la batterie
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit3 : défaillance d'un capteur (seulement pour PT100 et PT1000)
/// -# 0 : désactivé
/// -# 1 : activé
/// \par La configuration actuelle des alarmes est disponible par lecture du status de l'application. GetApplication_Status.
/// \par CAS de la TRANSMISSION D'un SEUL TYPE D'ALARME
/// \par 1) Transmission d'une alarme par le module :
/// \arg émet la première alarme (CMDAPP REQ_ = 0x40) avec:
/// -# 1 octet = nature de l'alarme
/// \arg Bit0 : seuil bas de mesures détecté si bit=1
/// \arg Bit1 : seuil haut de mesures détecté si bit=1
/// \arg Bit2 : fin de vie de la batterie si bit=1
/// \arg Bit3 : défaillance d'un capteur détectée (seulement pour PT100 et PT1000) si bit=1
/// -# 6 octets = date de l'alarme
/// \arg en 1 : le jour
/// \arg en 2 : le mois
/// \arg en 3 : l'année dont la valeur complète est 2000+octet3
/// \arg en 4 : le jour de la semaine de 0 pour Dimanche à 6 pour samedi
/// \arg en 5 : l'heure
/// \arg en 6 ! la minute
/// -# n octets optionnels (dans le cas d'une alarme de type seuil atteint)
/// \arg 1 octet : numéro du capteur en défaut
/// \arg 2 octets : durée du défaut en multiples de la période de mesure
/// \arg 2 octets(DALLAS) ou 4 octets (PT100,PT1000) : valeur moyenne des mesures depuis la détection du seuil jusqu'au retour hosr seuil.
/// \par 2) attente un acquittement (CMDAPPACK RES = 0xC0) : l'envoi de l'alarme est recommencé Number_retries_alarm. Entre chaque ré-émission,
/// le module attend Delay_transmission_alarm.
/// \par CAS de la TRANSMISSION De PLUSIEURS TYPES D'ALARME
/// \arg Pour chaque alarme une alerte est transmise : un alarme ne contient qu'une seule cause d'alarme.
/// \arg Tant qu'une alarme n'a pas été acquittée, la suivante n'est pas émise.
/// Note : Cette configuration peut aussi être fixée en modifiant le paramètre ALARM_CONFIGURATION par la méthode setAlarm_Configuration.
/// \par Seule la classe dérivée appelante connaît les modes d'alarme possibles.
/// \see setNumber_retries_alarm
/// \see getNumber_retries_alarm
/// \see getDelay_transmission_alarm
/// \return le nombre actuel de tentatives
Byte CWaveTherm::getNumber_retries_alarm() {
  _MR_MF_CMDREADPARAM((CModuleFinal::PARAM)NUMBER_RETRIES_ALARM
                       ,(CModuleFinal::SIZEREQPARAM)SIZEREQ_NUMBER_RETRIES_ALARM);
  this->m_Number_Datalogging_Loop = PDUe[SIZEADRRADIO+1+1+1+1];
  return m_Number_retries_alarm;
}

/// Retrouver le nombre de données stockées actuellement lorsque le mode "extended datalogging" est activé.
/// \par Le datalogging
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la/les mesure(s) associée(s) au périphérique dans une mémoire permanente locale.
/// \arg datalogging standard :
/// -# DALLAS : 48 mesures pour 2 capteurs soit 24 mesures/capteur
/// -# PT100 et PT1000 : 24 mesures pour 2 capteurs soit 12 mesures/capteur
/// \arg datalogging étendu :
/// -# DALLAS : 4500 mesures pour 2 capteurs soit 2250 mesures/capteur
/// -# PT100 et PT1000 : 2000 mesures pour 2 capteurs soit 1000 mesures/capteur
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la mesure associée au périphérique dans une mémoire permanente locale.
/// \par Pour chaque mode de datalogging, trois sous-modes réglés par l'octet d'operating_mode sont possibles (CModuleFinal::setOperating_Mode) :
/// \arg 1) Le mode par tranche de temps exprimée en unité de temps (time steps): dans ce mode la sauvegarde se fait :
/// \par à partir d'une heure de démarrage (setStart_hour_datalogging) de 0 à 23h.
/// \par pour une tranche de temps exprimée en unité de temps (setMeasurement_Period).
/// -# La tranche de temps de la sauvegarde est fixée par les bits 2 à 7 du paramètre Measurement_Period soit de 1 à 63 fois l'unité de temps.
/// -# L'unité de temps correspond aux bits 0 et 1 du même paramètre Measurement_Period selon le tableau suivant :
/// \arg 00 : unité de temps = 1mn
/// \arg 01 : unité de temps = 5mn
/// \arg 10 : unité de temps = 15mn
/// \arg 11 : unité de temps = 30mn
/// Cela implique une durée de sauvegarde maximum de 63 fois 30 mn.
/// \arg 2) Le mode "une fois par semaine" (once a week)
/// \par Dans ce mode la sauvegarde se fait une fois par semaine
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour de la semaine fixé par setDay_week_month avec une valeur de 0 pour le dimanche à 6 pour le samedi.
/// \arg 3) Le mode "une fois par mois" (once a month)
/// \par Dans ce mode la sauvegarde se fait une fois par mois
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour du mois fixé par setDay_week_month avec une valeur allant de 0 à 28. Le changement du nombre de jours dans le mois n'est pas géré.
/// \see CModuleFinal::setOperating_Mode
/// \see setStart_hour_datalogging
/// \see setMeasurement_Period
/// \see setTime_Measurement
/// \return Le nombre de mesures actuellement stockées.
int CWaveTherm::getNumber_values_stored() {
  _MR_MF_CMDREADPARAM((CModuleFinal::PARAM)NUMBER_VALUES_STORED
                       ,(CModuleFinal::SIZEREQPARAM)SIZEREQ_NUMBER_VALUES_STORED);
  this->m_Number_values_stored = (int)PDUe[SIZEADRRADIO+1+1+1+1] + ((int)PDUe[SIZEADRRADIO+1+1+1+1+1]<<8);
  return m_Number_values_stored;
}

/// Retrouver l'heure de démarrage de la sauvegarde périodique de mesures lorsque le mode datalogging "time steps" est activé.
/// \par Le datalogging
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la/les mesure(s) associée(s) au périphérique dans une mémoire permanente locale.
/// \arg datalogging standard :
/// -# DALLAS : 48 mesures pour 2 capteurs soit 24 mesures/capteur
/// -# PT100 et PT1000 : 24 mesures pour 2 capteurs soit 12 mesures/capteur
/// \arg datalogging étendu :
/// -# DALLAS : 4500 mesures pour 2 capteurs soit 2250 mesures/capteur
/// -# PT100 et PT1000 : 2000 mesures pour 2 capteurs soit 1000 mesures/capteur
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la mesure associée au périphérique dans une mémoire permanente locale.
/// \par Pour chaque mode de datalogging, trois sous-modes réglés par l'octet d'operating_mode sont possibles (CModuleFinal::setOperating_Mode) :
/// \arg 1) Le mode par tranche de temps exprimée en unité de temps (time steps): dans ce mode la sauvegarde se fait :
/// \par à partir d'une heure de démarrage (setStart_hour_datalogging) de 0 à 23h.
/// \par pour une tranche de temps exprimée en unité de temps (setMeasurement_Period).
/// -# La tranche de temps de la sauvegarde est fixée par les bits 2 à 7 du paramètre Measurement_Period soit de 1 à 63 fois l'unité de temps.
/// -# L'unité de temps correspond aux bits 0 et 1 du même paramètre Measurement_Period selon le tableau suivant :
/// \arg 00 : unité de temps = 1mn
/// \arg 01 : unité de temps = 5mn
/// \arg 10 : unité de temps = 15mn
/// \arg 11 : unité de temps = 30mn
/// Cela implique une durée de sauvegarde maximum de 63 fois 30 mn.
/// \arg 2) Le mode "une fois par semaine" (once a week)
/// \par Dans ce mode la sauvegarde se fait une fois par semaine
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour de la semaine fixé par setDay_week_month avec une valeur de 0 pour le dimanche à 6 pour le samedi.
/// \arg 3) Le mode "une fois par mois" (once a month)
/// \par Dans ce mode la sauvegarde se fait une fois par mois
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour du mois fixé par setDay_week_month avec une valeur allant de 0 à 28. Le changement du nombre de jours dans le mois n'est pas géré.
/// \see CModuleFinal::setOperating_Mode
/// \see setStart_hour_datalogging
/// \see setMeasurement_Period
/// \see setTime_Measurement
/// \return L'heure entre 0 et 23h de démarrage de la sauvegarde en mode datalogging "once a week" ou "once a month".
Byte CWaveTherm::getStart_hour_datalogging() {
  _MR_MF_CMDREADPARAM((CModuleFinal::PARAM)START_HOUR_DATALOGGING
                       ,(CModuleFinal::SIZEREQPARAM)SIZEREQ_START_HOUR_DATALOGGING);
  this->m_Number_Datalogging_Loop = PDUe[SIZEADRRADIO+1+1+1+1];
  return m_Start_hour_datalogging;
}

/// Retrouver l'heure de la sauvegarde des mesures dans le mode Datalogging une fois par semaine ou une fois par mois.
/// \par Le datalogging
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la/les mesure(s) associée(s) au périphérique dans une mémoire permanente locale.
/// \arg datalogging standard :
/// -# DALLAS : 48 mesures pour 2 capteurs soit 24 mesures/capteur
/// -# PT100 et PT1000 : 24 mesures pour 2 capteurs soit 12 mesures/capteur
/// \arg datalogging étendu :
/// -# DALLAS : 4500 mesures pour 2 capteurs soit 2250 mesures/capteur
/// -# PT100 et PT1000 : 2000 mesures pour 2 capteurs soit 1000 mesures/capteur
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la mesure associée au périphérique dans une mémoire permanente locale.
/// \par Pour chaque mode de datalogging, trois sous-modes réglés par l'octet d'operating_mode sont possibles (CModuleFinal::setOperating_Mode) :
/// \arg 1) Le mode par tranche de temps exprimée en unité de temps (time steps): dans ce mode la sauvegarde se fait :
/// \par à partir d'une heure de démarrage (setStart_hour_datalogging) de 0 à 23h.
/// \par pour une tranche de temps exprimée en unité de temps (setMeasurement_Period).
/// -# La tranche de temps de la sauvegarde est fixée par les bits 2 à 7 du paramètre Measurement_Period soit de 1 à 63 fois l'unité de temps.
/// -# L'unité de temps correspond aux bits 0 et 1 du même paramètre Measurement_Period selon le tableau suivant :
/// \arg 00 : unité de temps = 1mn
/// \arg 01 : unité de temps = 5mn
/// \arg 10 : unité de temps = 15mn
/// \arg 11 : unité de temps = 30mn
/// Cela implique une durée de sauvegarde maximum de 63 fois 30 mn.
/// \arg 2) Le mode "une fois par semaine" (once a week)
/// \par Dans ce mode la sauvegarde se fait une fois par semaine
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour de la semaine fixé par setDay_week_month avec une valeur de 0 pour le dimanche à 6 pour le samedi.
/// \arg 3) Le mode "une fois par mois" (once a month)
/// \par Dans ce mode la sauvegarde se fait une fois par mois
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour du mois fixé par setDay_week_month avec une valeur allant de 0 à 28. Le changement du nombre de jours dans le mois n'est pas géré.
/// \see CModuleFinal::setOperating_Mode
/// \see setStart_hour_datalogging
/// \see setMeasurement_Period
/// \see setTime_Measurement
/// \return L'heure de la sauvegarde des mesures dans le mode Datalogging une fois par semaine ou une fois par mois.
Byte CWaveTherm::getTime_measurement() {
  _MR_MF_CMDREADPARAM((CModuleFinal::PARAM)TIME_MEASUREMENT
                       ,(CModuleFinal::SIZEREQPARAM)SIZEREQ_TIME_MEASUREMENT);
  this->m_Number_Datalogging_Loop = PDUe[SIZEADRRADIO+1+1+1+1];
  return m_Time_measurement;
}

/// Fixer le mode d'alarme du module.
/// \par
/// \par GESTION DES ALARMES
/// \par Le module peut déclencher des alarmes sur plusieurs conditions:
/// \arg Seuils de mesure haut ou bas atteint en mode datalogging
/// \arg seuil du compteur de durée de vie de la pile atteint
/// \arg Défaillance d'un capteur (seulement pour les PT100 et PT1000)
/// \par Format de l'octet de mode d'alarme :
/// \arg Bit0 : détection du seuil bas de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit1 : détection du seuil haut de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit2 : fin de vie de la batterie
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit3 : défaillance d'un capteur (seulement pour PT100 et PT1000)
/// -# 0 : désactivé
/// -# 1 : activé
/// \par La configuration actuelle des alarmes est disponible par lecture du status de l'application. GetApplication_Status.
/// \param m_Alarm_Configuration : le mode d'alarme souhaité
/// Note : Cette configuration peut aussi être fixée en appelant la méthode SetAlarm.
/// \see SetAlarm
/// \see getApplication_Status
/// \par Seule la classe dérivée appelante connaît les modes d'alarme possibles.
void CWaveTherm::setAlarm_Configuration(Byte m_Alarm_Configuration) {
  Byte Data[] = {m_Alarm_Configuration};
  _MR_WT_CMDWRITEPARAM(ALARM_CONFIGURATION,SIZEREQ_ALARM_CONFIGURATION,Data);
  this->m_Alarm_Configuration = m_Alarm_Configuration;
}

/// Fixer le jour de la semaine ou du mois pour la sauvegarde périodique (datalogging).
/// \par Le datalogging
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la/les mesure(s) associée(s) au périphérique dans une mémoire permanente locale.
/// \arg datalogging standard :
/// -# DALLAS : 48 mesures pour 2 capteurs soit 24 mesures/capteur
/// -# PT100 et PT1000 : 24 mesures pour 2 capteurs soit 12 mesures/capteur
/// \arg datalogging étendu :
/// -# DALLAS : 4500 mesures pour 2 capteurs soit 2250 mesures/capteur
/// -# PT100 et PT1000 : 2000 mesures pour 2 capteurs soit 1000 mesures/capteur
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la mesure associée au périphérique dans une mémoire permanente locale.
/// \par Pour chaque mode de datalogging, trois sous-modes réglés par l'octet d'operating_mode sont possibles (CModuleFinal::setOperating_Mode) :
/// \arg 1) Le mode par tranche de temps exprimée en unité de temps (time steps): dans ce mode la sauvegarde se fait :
/// \par à partir d'une heure de démarrage (setStart_hour_datalogging) de 0 à 23h.
/// \par pour une tranche de temps exprimée en unité de temps (setMeasurement_Period).
/// -# La tranche de temps de la sauvegarde est fixée par les bits 2 à 7 du paramètre Measurement_Period soit de 1 à 63 fois l'unité de temps.
/// -# L'unité de temps correspond aux bits 0 et 1 du même paramètre Measurement_Period selon le tableau suivant :
/// \arg 00 : unité de temps = 1mn
/// \arg 01 : unité de temps = 5mn
/// \arg 10 : unité de temps = 15mn
/// \arg 11 : unité de temps = 30mn
/// Cela implique une durée de sauvegarde maximum de 63 fois 30 mn.
/// \arg 2) Le mode "une fois par semaine" (once a week)
/// \par Dans ce mode la sauvegarde se fait une fois par semaine
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour de la semaine fixé par setDay_week_month avec une valeur de 0 pour le dimanche à 6 pour le samedi.
/// \arg 3) Le mode "une fois par mois" (once a month)
/// \par Dans ce mode la sauvegarde se fait une fois par mois
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour du mois fixé par setDay_week_month avec une valeur allant de 0 à 28. Le changement du nombre de jours dans le mois n'est pas géré.
/// \see CModuleFinal::setOperating_Mode
/// \see setStart_hour_datalogging
/// \see setMeasurement_Period
/// \see setTime_Measurement
/// \param m_Day_week_month : le jour de la semaine ou du mois pour la sauvegarde périodique.
void CWaveTherm::setDay_week_month(Byte m_Day_week_month) {
  Byte Data[] = {m_Day_week_month};
  _MR_WT_CMDWRITEPARAM(DAY_WEEK_MONTH,SIZEREQ_DAY_WEEK_MONTH,Data);
  this->m_Day_week_month = m_Day_week_month;
}

/// Fixer le délai entre 2 re-transmission d'alarme exprimé en multiples de la période de mesure.
/// \par
/// \par GESTION DES ALARMES
/// \par Le module peut déclencher des alarmes sur plusieurs conditions:
/// \arg Seuils de mesure haut ou bas atteint en mode datalogging
/// \arg seuil du compteur de durée de vie de la pile atteint
/// \arg Défaillance d'un capteur (seulement pour les PT100 et PT1000)
/// \par Format de l'octet de mode d'alarme :
/// \arg Bit0 : détection du seuil bas de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit1 : détection du seuil haut de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit2 : fin de vie de la batterie
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit3 : défaillance d'un capteur (seulement pour PT100 et PT1000)
/// -# 0 : désactivé
/// -# 1 : activé
/// \par La configuration actuelle des alarmes est disponible par lecture du status de l'application. GetApplication_Status.
/// \par CAS de la TRANSMISSION D'un SEUL TYPE D'ALARME
/// \par 1) Transmission d'une alarme par le module :
/// \arg émet la première alarme (CMDAPP REQ_ = 0x40) avec:
/// -# 1 octet = nature de l'alarme
/// \arg Bit0 : seuil bas de mesures détecté si bit=1
/// \arg Bit1 : seuil haut de mesures détecté si bit=1
/// \arg Bit2 : fin de vie de la batterie si bit=1
/// \arg Bit3 : défaillance d'un capteur détectée (seulement pour PT100 et PT1000) si bit=1
/// -# 6 octets = date de l'alarme
/// \arg en 1 : le jour
/// \arg en 2 : le mois
/// \arg en 3 : l'année dont la valeur complète est 2000+octet3
/// \arg en 4 : le jour de la semaine de 0 pour Dimanche à 6 pour samedi
/// \arg en 5 : l'heure
/// \arg en 6 ! la minute
/// -# n octets optionnels (dans le cas d'une alarme de type seuil atteint)
/// \arg 1 octet : numéro du capteur en défaut
/// \arg 2 octets : durée du défaut en multiples de la période de mesure
/// \arg 2 octets(DALLAS) ou 4 octets (PT100,PT1000) : valeur moyenne des mesures depuis la détection du seuil jusqu'au retour hosr seuil.
/// \par 2) attente un acquittement (CMDAPPACK RES = 0xC0) : l'envoi de l'alarme est recommencé Number_retries_alarm. Entre chaque ré-émission,
/// le module attend Delay_transmission_alarm.
/// \par CAS de la TRANSMISSION De PLUSIEURS TYPES D'ALARME
/// \arg Pour chaque alarme une alerte est transmise : un alarme ne contient qu'une seule cause d'alarme.
/// \arg Tant qu'une alarme n'a pas été acquittée, la suivante n'est pas émise.
/// Note : Cette configuration peut aussi être fixée en modifiant le paramètre ALARM_CONFIGURATION par la méthode setAlarm_Configuration.
/// \par Seule la classe dérivée appelante connaît les modes d'alarme possibles.
/// \see setNumber_retries_alarm
/// \see getNumber_retries_alarm
/// \see getDelay_transmission_alarm
/// \param m_Delay_transmission_alarm : délai entre 2 re-transmission d'alarme exprimé en multiples de la période de mesure.
void CWaveTherm::setDelay_transmission_alarm(Byte m_Delay_transmission_alarm) {
  Byte Data[] = {m_Delay_transmission_alarm};
  _MR_WT_CMDWRITEPARAM(DELAY_TRANSMISSION_ALARM,SIZEREQ_DELAY_TRANSMISSION_ALARM,Data);
  this->m_Delay_transmission_alarm = m_Delay_transmission_alarm;
}

/// Fixer la période de mesure dans le mode datalogging.
/// \par Le datalogging
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la/les mesure(s) associée(s) au périphérique dans une mémoire permanente locale.
/// \arg datalogging standard :
/// -# DALLAS : 48 mesures pour 2 capteurs soit 24 mesures/capteur
/// -# PT100 et PT1000 : 24 mesures pour 2 capteurs soit 12 mesures/capteur
/// \arg datalogging étendu :
/// -# DALLAS : 4500 mesures pour 2 capteurs soit 2250 mesures/capteur
/// -# PT100 et PT1000 : 2000 mesures pour 2 capteurs soit 1000 mesures/capteur
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la mesure associée au périphérique dans une mémoire permanente locale.
/// \par Pour chaque mode de datalogging, trois sous-modes réglés par l'octet d'operating_mode sont possibles (CModuleFinal::setOperating_Mode) :
/// \arg 1) Le mode par tranche de temps exprimée en unité de temps (time steps): dans ce mode la sauvegarde se fait :
/// \par à partir d'une heure de démarrage (setStart_hour_datalogging) de 0 à 23h.
/// \par pour une tranche de temps exprimée en unité de temps (setMeasurement_Period).
/// -# La tranche de temps de la sauvegarde est fixée par les bits 2 à 7 du paramètre Measurement_Period soit de 1 à 63 fois l'unité de temps.
/// -# L'unité de temps correspond aux bits 0 et 1 du même paramètre Measurement_Period selon le tableau suivant :
/// \arg 00 : unité de temps = 1mn
/// \arg 01 : unité de temps = 5mn
/// \arg 10 : unité de temps = 15mn
/// \arg 11 : unité de temps = 30mn
/// Cela implique une durée de sauvegarde maximum de 63 fois 30 mn.
/// \arg 2) Le mode "une fois par semaine" (once a week)
/// \par Dans ce mode la sauvegarde se fait une fois par semaine
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour de la semaine fixé par setDay_week_month avec une valeur de 0 pour le dimanche à 6 pour le samedi.
/// \arg 3) Le mode "une fois par mois" (once a month)
/// \par Dans ce mode la sauvegarde se fait une fois par mois
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour du mois fixé par setDay_week_month avec une valeur allant de 0 à 28. Le changement du nombre de jours dans le mois n'est pas géré.
/// \see CModuleFinal::setOperating_Mode
/// \see setStart_hour_datalogging
/// \see setMeasurement_Period
/// \see setTime_Measurement
/// \param m_Measurement_Period : la période de mesure souhaitée dans le mode datalogging.
void CWaveTherm::setMeasurement_Period(Byte m_Measurement_Period) {
  Byte Data[] = {m_Measurement_Period};
  _MR_WT_CMDWRITEPARAM(MEASUREMENT_PERIOD,SIZEREQ_MEASUREMENT_PERIOD,Data);
  this->m_Measurement_Period = m_Measurement_Period;
}

/// Fixer le nombre de tentatives lors de l'émission d'une alarme non acquittée
/// \par
/// \par GESTION DES ALARMES
/// \par Le module peut déclencher des alarmes sur plusieurs conditions:
/// \arg Seuils de mesure haut ou bas atteint en mode datalogging
/// \arg seuil du compteur de durée de vie de la pile atteint
/// \arg Défaillance d'un capteur (seulement pour les PT100 et PT1000)
/// \par Format de l'octet de mode d'alarme :
/// \arg Bit0 : détection du seuil bas de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit1 : détection du seuil haut de mesures en mode datalogging
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit2 : fin de vie de la batterie
/// -# 0 : désactivé
/// -# 1 : activé
/// \arg Bit3 : défaillance d'un capteur (seulement pour PT100 et PT1000)
/// -# 0 : désactivé
/// -# 1 : activé
/// \par La configuration actuelle des alarmes est disponible par lecture du status de l'application. GetApplication_Status.
/// \par CAS de la TRANSMISSION D'un SEUL TYPE D'ALARME
/// \par 1) Transmission d'une alarme par le module :
/// \arg émet la première alarme (CMDAPP REQ_ = 0x40) avec:
/// -# 1 octet = nature de l'alarme
/// \arg Bit0 : seuil bas de mesures détecté si bit=1
/// \arg Bit1 : seuil haut de mesures détecté si bit=1
/// \arg Bit2 : fin de vie de la batterie si bit=1
/// \arg Bit3 : défaillance d'un capteur détectée (seulement pour PT100 et PT1000) si bit=1
/// -# 6 octets = date de l'alarme
/// \arg en 1 : le jour
/// \arg en 2 : le mois
/// \arg en 3 : l'année dont la valeur complète est 2000+octet3
/// \arg en 4 : le jour de la semaine de 0 pour Dimanche à 6 pour samedi
/// \arg en 5 : l'heure
/// \arg en 6 ! la minute
/// -# n octets optionnels (dans le cas d'une alarme de type seuil atteint)
/// \arg 1 octet : numéro du capteur en défaut
/// \arg 2 octets : durée du défaut en multiples de la période de mesure
/// \arg 2 octets(DALLAS) ou 4 octets (PT100,PT1000) : valeur moyenne des mesures depuis la détection du seuil jusqu'au retour hosr seuil.
/// \par 2) attente un acquittement (CMDAPPACK RES = 0xC0) : l'envoi de l'alarme est recommencé Number_retries_alarm. Entre chaque ré-émission,
/// le module attend Delay_transmission_alarm.
/// \par CAS de la TRANSMISSION De PLUSIEURS TYPES D'ALARME
/// \arg Pour chaque alarme une alerte est transmise : un alarme ne contient qu'une seule cause d'alarme.
/// \arg Tant qu'une alarme n'a pas été acquittée, la suivante n'est pas émise.
/// Note : Cette configuration peut aussi être fixée en modifiant le paramètre ALARM_CONFIGURATION par la méthode setAlarm_Configuration.
/// \par Seule la classe dérivée appelante connaît les modes d'alarme possibles.
/// \see setNumber_retries_alarm
/// \see getNumber_retries_alarm
/// \see getDelay_transmission_alarm
/// \param m_Number_retries_alarm : le nombre de tentatives lors de l'émission d'une alarme non acquittée
void CWaveTherm::setNumber_retries_alarm(Byte m_Number_retries_alarm) {
  Byte Data[] = {m_Number_retries_alarm};
  _MR_WT_CMDWRITEPARAM(NUMBER_RETRIES_ALARM,SIZEREQ_NUMBER_RETRIES_ALARM,Data);
  this->m_Number_retries_alarm = m_Number_retries_alarm;
}

/// Fixer l'heure de démarrage de la sauvegarde périodique des mesures dans le mode datalogging "Time steps".
/// \par Le datalogging
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la/les mesure(s) associée(s) au périphérique dans une mémoire permanente locale.
/// \arg datalogging standard :
/// -# DALLAS : 48 mesures pour 2 capteurs soit 24 mesures/capteur
/// -# PT100 et PT1000 : 24 mesures pour 2 capteurs soit 12 mesures/capteur
/// \arg datalogging étendu :
/// -# DALLAS : 4500 mesures pour 2 capteurs soit 2250 mesures/capteur
/// -# PT100 et PT1000 : 2000 mesures pour 2 capteurs soit 1000 mesures/capteur
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la mesure associée au périphérique dans une mémoire permanente locale.
/// \par Pour chaque mode de datalogging, trois sous-modes réglés par l'octet d'operating_mode sont possibles (CModuleFinal::setOperating_Mode) :
/// \arg 1) Le mode par tranche de temps exprimée en unité de temps (time steps): dans ce mode la sauvegarde se fait :
/// \par à partir d'une heure de démarrage (setStart_hour_datalogging) de 0 à 23h.
/// \par pour une tranche de temps exprimée en unité de temps (setMeasurement_Period).
/// -# La tranche de temps de la sauvegarde est fixée par les bits 2 à 7 du paramètre Measurement_Period soit de 1 à 63 fois l'unité de temps.
/// -# L'unité de temps correspond aux bits 0 et 1 du même paramètre Measurement_Period selon le tableau suivant :
/// \arg 00 : unité de temps = 1mn
/// \arg 01 : unité de temps = 5mn
/// \arg 10 : unité de temps = 15mn
/// \arg 11 : unité de temps = 30mn
/// Cela implique une durée de sauvegarde maximum de 63 fois 30 mn.
/// \arg 2) Le mode "une fois par semaine" (once a week)
/// \par Dans ce mode la sauvegarde se fait une fois par semaine
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour de la semaine fixé par setDay_week_month avec une valeur de 0 pour le dimanche à 6 pour le samedi.
/// \arg 3) Le mode "une fois par mois" (once a month)
/// \par Dans ce mode la sauvegarde se fait une fois par mois
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour du mois fixé par setDay_week_month avec une valeur allant de 0 à 28. Le changement du nombre de jours dans le mois n'est pas géré.
/// \see CModuleFinal::setOperating_Mode
/// \see setStart_hour_datalogging
/// \see setMeasurement_Period
/// \see setTime_Measurement
/// \param m_Start_hour_datalogging :  l'heure de démarrage de la sauvegarde périodique des mesures dans le mode datalogging "Time steps".
void CWaveTherm::setStart_hour_datalogging(Byte m_Start_hour_datalogging) {
  Byte Data[] = {m_Start_hour_datalogging};
  _MR_WT_CMDWRITEPARAM(START_HOUR_DATALOGGING,SIZEREQ_START_HOUR_DATALOGGING,Data);
  this->m_Start_hour_datalogging = m_Start_hour_datalogging;
}

/// Fixer l'heure de la sauvegarde automatique dans le mode datalogging "once a week" ou "once a month"
/// \par Le datalogging
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la/les mesure(s) associée(s) au périphérique dans une mémoire permanente locale.
/// \arg datalogging standard :
/// -# DALLAS : 48 mesures pour 2 capteurs soit 24 mesures/capteur
/// -# PT100 et PT1000 : 24 mesures pour 2 capteurs soit 12 mesures/capteur
/// \arg datalogging étendu :
/// -# DALLAS : 4500 mesures pour 2 capteurs soit 2250 mesures/capteur
/// -# PT100 et PT1000 : 2000 mesures pour 2 capteurs soit 1000 mesures/capteur
/// \par Le datalogging consiste pour un module à sauvegarder périodiquement la mesure associée au périphérique dans une mémoire permanente locale.
/// \par Pour chaque mode de datalogging, trois sous-modes réglés par l'octet d'operating_mode sont possibles (CModuleFinal::setOperating_Mode) :
/// \arg 1) Le mode par tranche de temps exprimée en unité de temps (time steps): dans ce mode la sauvegarde se fait :
/// \par à partir d'une heure de démarrage (setStart_hour_datalogging) de 0 à 23h.
/// \par pour une tranche de temps exprimée en unité de temps (setMeasurement_Period).
/// -# La tranche de temps de la sauvegarde est fixée par les bits 2 à 7 du paramètre Measurement_Period soit de 1 à 63 fois l'unité de temps.
/// -# L'unité de temps correspond aux bits 0 et 1 du même paramètre Measurement_Period selon le tableau suivant :
/// \arg 00 : unité de temps = 1mn
/// \arg 01 : unité de temps = 5mn
/// \arg 10 : unité de temps = 15mn
/// \arg 11 : unité de temps = 30mn
/// Cela implique une durée de sauvegarde maximum de 63 fois 30 mn.
/// \arg 2) Le mode "une fois par semaine" (once a week)
/// \par Dans ce mode la sauvegarde se fait une fois par semaine
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour de la semaine fixé par setDay_week_month avec une valeur de 0 pour le dimanche à 6 pour le samedi.
/// \arg 3) Le mode "une fois par mois" (once a month)
/// \par Dans ce mode la sauvegarde se fait une fois par mois
/// \arg à une heure fixée par setTime_Measurement allant de 0 à 23h.
/// \arg un jour du mois fixé par setDay_week_month avec une valeur allant de 0 à 28. Le changement du nombre de jours dans le mois n'est pas géré.
/// \see CModuleFinal::setOperating_Mode
/// \see setStart_hour_datalogging
/// \see setMeasurement_Period
/// \see setTime_Measurement
/// \param m_Time_measurement : l'heure de la sauvegarde automatique dans le mode datalogging "once a week" ou "once a month"
void CWaveTherm::setTime_measurement(Byte m_Time_measurement) {
  Byte Data[] = {m_Time_measurement};
  _MR_WT_CMDWRITEPARAM(TIME_MEASUREMENT,SIZEREQ_TIME_MEASUREMENT,Data);
  this->m_Time_measurement = m_Time_measurement;
}

// Methodes protegees

// Methodes privees

} // namespace WAVENIS

// Programme principal
// Si c'est la classe principale du projet, on place une fonction main()
// Dans ce cas, on peut supprimer les fichiers de tests unitaires
// ex :
// int main(int argc, char *argv[])
// {
//    // Construction de l'instance de la classe principale
//    cout << "Construction de l'instance de la classe principale CWaveTherm" << endl;
//    CWaveTherm *cwavetherm = new CWaveTherm();
//    // Lancement de l'activité principale
//    return 0;
// }

