///////////////////////////////////////////////////////////////////////////////////////
// Projet   	  : classe Property
// Auteur   	  : MR
// Date   		  : Février 2010
// Version  	  : v2
// Description  : Template pour la classe Property qui permettra d'implémenter des
// 				        properties pour les attributs de la classe CAdrRadio
/////////////////////////////////////////////////////////////////////////////////////////
#ifndef PROPERTY_H_
#define PROPERTY_H_

#include <assert.h>
#include <stdlib.h> // pour NULL

#define READ_ONLY 1
#define WRITE_ONLY 2
#define READ_WRITE 3

// macros pour l'initialisation des objets Property : get et set de la classe CAdrRadio
#define _MR_GET_SET(a) \
  a.setContainer(this); \
  a.setter(&CAdrRadio::set##a);\
  a.getter(&CAdrRadio::get##a);

// macro pour l'initialisation des objets Property : get seulement de la classe CAdrRadio
#define _MR_GET(a) \
  a.setContainer(this); \
  a.getter(&CAdrRadio::get##a);

// macro pour l'initialisation des objets Property : set seulement de la classe CAdrRadio
#define _MR_SET(a) \
  a.setContainer(this); \
  a.setter(&CAdrRadio::set##a);

/// Classe template pour les property d'attributs
template <typename Container, typename ValueType, int nPropType>
class property {
public:
property() {
  m_cObject = NULL;
  Set = NULL;
  Get = NULL;
}
//-- This to set a pointer to the class that contain the
//   property --
void setContainer(Container* cObject) {
  m_cObject = cObject;
}
//-- Set the set member function that will change the value --
void setter(void (Container::*pSet)(ValueType value)) {
  if((nPropType == WRITE_ONLY) || (nPropType == READ_WRITE))
    Set = pSet;
  else
    Set = NULL;
}

//-- Set the get member function that will retrieve the value --
void getter(ValueType (Container::*pGet)()const) {
  if((nPropType == READ_ONLY) || (nPropType == READ_WRITE))
    Get = pGet;
  else
    Get = NULL;
}

//-- Overload the '=' sign to set the value using the set
//   member --
ValueType operator =( const ValueType& value) {
  assert(m_cObject != NULL);
  assert(Set != NULL);
  (m_cObject->*Set)(value);
  return value;
}

//-- To make possible to cast the property class to the
//   internal type --
operator ValueType() {
  assert(m_cObject != NULL);
  assert(Get != NULL);
  return (m_cObject->*Get)();
}

private:
  Container* m_cObject;  //-- Pointer to the module that
                         //   contains the property --
  void (Container::*Set)(ValueType value);
                         //-- Pointer to set member function --
  ValueType (Container::*Get)()const;
                         //-- Pointer to get member function --
};


#endif /* PROPERTY_H_ */
