/**
 *  Copyright (C) 2013  plegal  (plegal@appert44.org)
 *  @file         SensorsManager.cpp
 *  @brief        Gestionnaire des capteurs
 *  @version      0.1
 *  @date         13 févr. 2013 09:44:39
 *
 *  Description detaillee du fichier SensorsManager.cpp
 *  Fabrication   gcc (Ubuntu/Linaro 4.6.3-1ubuntu5) 4.6.3 
 *  @todo         Liste des choses restant a faire.
 *  @bug          13 févr. 2013 09:44:39 - Aucun pour l'instant
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Includes system C
// Includes system C++
// A décommenter si besoin cout, cin, ...
// #include <iostream>
// using namespace std;
#include <vector>
#include <iostream>

#include <boost/msm/back/state_machine.hpp>
#include <boost/msm/front/euml/euml.hpp>
#include <boost/asio.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include "boost/date_time/gregorian/gregorian.hpp"
#include <boost/thread/thread.hpp>
#include <boost/thread/xtime.hpp>

using namespace std;
using namespace boost::msm::front::euml;
namespace msm = boost::msm;
using namespace boost::posix_time;
using namespace boost::gregorian;


// entry/exit/action/guard logging functors
#include "logging_functors.h"

// Includes qt

// Includes application

#include "protocol_types.h"
#include "CConfig.h"
#include "CAdrRadio.h"
#include "CWavePort.h"
#include "CDallas.h"
#include "HTTPClient.h"

using namespace WAVENIS;

#define SLEEPING_PERIOD 300 

CAdrRadio *adrRadioPort;
RadioAddress dallasRadioAddress;
CWavePort *cwaveport;
CDallas *cdallas;
HTTPClient *httpclient;
string measureServerName;
string measureServerPort;
string measureURL;

bool job1 = false;
bool job2 = false;
bool end_asked = false;

const std::locale date_fmt(std::locale::classic(), new boost::gregorian::date_facet("%Y-%m-%d"));


// events
BOOST_MSM_EUML_EVENT(server_off)
BOOST_MSM_EUML_EVENT(server_on)
BOOST_MSM_EUML_EVENT(tick)
BOOST_MSM_EUML_EVENT(init_ok)
BOOST_MSM_EUML_EVENT(init_ko)

BOOST_MSM_EUML_ACTION(Init_Entry) {
	template<class Event, class FSM, class STATE>
	void operator()(Event const&, FSM& fsm, STATE&) {

		cout << "entering: Init" << endl;
//#define _LOCAL_
#ifdef _LOCAL_
		measureServerName = "localhost";
		measureServerPort = "8000";
		measureURL = "/measure";
#else
		measureServerName = "www.skillsit.fr";
		measureServerPort = "80";
		measureURL = "/ewtswavenis/measure";
#endif
		cout << "load config.conf" << endl;
		CConfig config("config.conf");

		cout << "new adrRadio" << endl;
		adrRadioPort = new CAdrRadio("000000000000");

		cout << "new adrRadioDallas" << endl;

		dallasRadioAddress = (RadioAddress ) { "10194C300559" };

		cout << "new waveport" << endl;
		cwaveport = new CWavePort(config, (string) "/dev/ttyUSB0", adrRadioPort,
				NULL);

		cout << "new cdallas" << endl;
		cdallas = new CDallas(cwaveport, dallasRadioAddress);

		cout << "SerialDriver and Protocol load" << endl;
		bool status = cwaveport->SerialDriverAndProtocolLoad();

		if (true == status) {
			cout << "SerialDriver and Protocol Instanciation" << endl;
			status = cwaveport->SerialDriverAndProtocolInstanciation();
			if (true == status) {
				cout << "Protocol Init" << endl;
				int protocol_status = cwaveport->ProtocolInit();
				if (OK_PROTOCOL_SUCCESS == protocol_status) {
					cout << "Protocol Open" << endl;
					int open_status = cwaveport->ProtocolOpen();
					if (OK_PROTOCOL_SUCCESS == open_status) {
						fsm.process_event(init_ok);
					}
				}
				else
				{
					cout << "prococol init problem" << endl;
					fsm.process_event(init_ko);
				}
			}
			else
			{
				cout << "serial driver and prococol instanciation problem" << endl;
				fsm.process_event(init_ko);
			}
		}
		else
		{
			cout << "serial driver and prococol load problem" << endl;
			fsm.process_event(init_ko);
		}
	}
};

BOOST_MSM_EUML_ACTION(Terminate_Entry) {
	template<class Event, class FSM, class STATE>
	void operator()(Event const&, FSM& fsm, STATE&) {
		cout << "Exit" << endl;
		end_asked = true;
	}
};

BOOST_MSM_EUML_ACTION(Offline_Entry) {
	template<class Event, class FSM, class STATE>
	void operator()(Event const&, FSM& fsm, STATE&) {
		cout << "Offline Entry" << endl;
		try {
			boost::asio::io_service io_service;
			httpclient = new HTTPClient(io_service);
			httpclient->GET("www.skillsit.fr", "80", "/");
			io_service.run();
			fsm.process_event(server_on);
		} catch (http_client_error & x) {
			if (int const * mi = boost::get_error_info<http_client_info>(x))
				std::cerr << "HTTPClient info: " << *mi;

			fsm.process_event(server_off);
		}
		delete httpclient;
	}
};

// Concrete FSM implementation
// The list of FSM states
// state not needing any entry or exit
BOOST_MSM_EUML_STATE((Init_Entry, no_action, attributes_ << no_attributes_),
		Init)

BOOST_MSM_EUML_STATE((Offline_Entry, no_action, attributes_ << no_attributes_),
		Offline)
BOOST_MSM_EUML_STATE((), Online)
BOOST_MSM_EUML_TERMINATE_STATE((Terminate_Entry, no_action, attributes_ << no_attributes_),
		Terminate)

BOOST_MSM_EUML_ACTION(stop_measure) {
	template<class Fsm, class Evt, class SourceState, class TargetState>
	void operator()(Evt const&, Fsm&, SourceState&, TargetState&) {
		cout << "stm::stop_measure" << endl;
		job1 = false;
	}
};

BOOST_MSM_EUML_ACTION(resume_measure) {
	template<class FSM, class EVT, class SourceState, class TargetState>
	void operator()(EVT const&, FSM&, SourceState&, TargetState&) {
		cout << "stm::resume_measure" << endl;
		job1 = true;
	}
};

BOOST_MSM_EUML_ACTION(process_measure) {
	template<class FSM, class EVT, class SourceState, class TargetState>
	void operator()(EVT const&, FSM& fsm, SourceState&, TargetState&) {
		cout << "stm::process_measure" << endl;

		int temperatureBrute = cdallas->TemperatureBrute(1);
		printf("temperature brute : %4x \n",temperatureBrute);

		double temperature = cdallas->Temperature(temperatureBrute);
		printf("temperature : %f \n",temperature);

		std::ostringstream oss;
		oss << temperature;

		// string snow will contain date
		std::ostringstream snow;
		// formatting date
		snow.imbue(date_fmt);

		// date
		ptime now = second_clock::local_time(); // current *LOCAL TIMEZONE* time/date
		date today = now.date();
		snow << today << "%20" << now.time_of_day().hours() << ":" << now.time_of_day().minutes() << ":" << now.time_of_day().seconds();
		cout << snow.str() << endl;

		try {
			boost::asio::io_service io_service;
			httpclient = new HTTPClient(io_service);

			//httpclient->POST("www.skillsit.fr", "80", "/ewtswavenis/measure","time=2013-02-17%2015:13&sensor_type=wavetherm&value="+oss.str());
			httpclient->POST(measureServerName,
					measureServerPort,
					measureURL,
					"time="+snow.str()+
					"&sensor_sn="+string(dallasRadioAddress.addressValue)+
					"&sensor_type=wavetherm&value="+oss.str());
			io_service.run();
		} catch (http_client_error & x) {
			if (int const * mi = boost::get_error_info<http_client_info>(x))
				std::cerr << "HTTPClient info: " << *mi;
			fsm.process_event(server_off);
		}
		delete httpclient;

	}
};

// replaces the old transition table
BOOST_MSM_EUML_TRANSITION_TABLE(
		( 	Offline == Init + init_ok,
			Offline == Online + server_off / stop_measure(),
			//  +------------------------------------------------------------------------------+
			Online == Offline + server_on / resume_measure(),
			Online == Online + tick / process_measure(),
			//  +------------------------------------------------------------------------------+
			Terminate == Init + init_ko
		), transition_table)

// create a state machine "on the fly"
BOOST_MSM_EUML_DECLARE_STATE_MACHINE(( transition_table,//STT
		init_ << Init,// Init State
		no_action,// Entry
		no_action,// Exit
		attributes_ << no_attributes_,// Attributes
		configure_ << no_configure_,// configuration
		Log_No_Transition// no_transition handler
		), stm_)
//fsm name

// choice of back-end
typedef msm::back::state_machine<stm_> stm;

stm m_stm;

void thread_alarm()
{
	while (!end_asked)
    	{
    		boost::xtime xt;
    		boost::xtime_get(&xt, boost::TIME_UTC);
    		xt.sec += SLEEPING_PERIOD;
    		boost::thread::sleep(xt);
    		std::cout << "alarm sounded..." << std::endl;
    		if (true == job1) {
    			std::cout << "job1..." << std::endl;
    			m_stm.process_event(tick);
    		}

    		if (true == job2)
    		{
    			std::cout << "job2..." << std::endl;
    		}
    	}
}

//
// Testing utilities.
//
static char const* const state_names[] = { "Init", "Online", "Offline", "Terminate" };

// Methodes publiques de la classe
// ex : ReturnType NomMethode(Type);

void stmstate(stm const& s) {
	std::cout << " -> " << state_names[s.current_state()[0]] << std::endl;
}

// Programme principal
// Si c'est la classe principale du projet, on place une fonction main()
// Dans ce cas, on peut supprimer les fichiers de tests unitaires
// ex :
int main(int argc, char *argv[]) {

    end_asked = false;
    job1 = false;
    job2 = false;

	boost::asio::io_service io;

	m_stm.start();
	stmstate(m_stm);

    boost::thread thrd(thread_alarm);

	while (! end_asked)
	{
	  boost::asio::deadline_timer t(io, boost::posix_time::seconds(5));
	  t.wait();
	  stmstate(m_stm);
	}
	thrd.join();

	return 0;
}

