/**
 *  Copyright (C) 2013  plegal  (plegal@appert44.org)
 *  @file         SensorsManager.h
 *  @brief        Gestionnaire des capteurs
 *  @version      0.1
 *  @date         13 févr. 2013 09:44:39
 *
 *  @note         Voir la description detaillee explicite dans le fichier
 *                SensorsManager.cpp
 *                C++ Google Style :
 *                http://google-styleguide.googlecode.com/svn/trunk/cppguide.xml
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef _SENSORSMANAGER_H
#define _SENSORSMANAGER_H

// Includes system C
//#include <stdint.h>  // definition des types int8_t, int16_t ...

// Includes system C++
#include <vector>
#include <iostream>

#include <boost/msm/back/state_machine.hpp>
#include <boost/msm/front/euml/euml.hpp>

using namespace std;
using namespace boost::msm::front::euml;
namespace msm = boost::msm;

// entry/exit/action/guard logging functors
#include "logging_functors.h"
// Includes qt

// Includes application

// Constantes
// ex :
// const int kDaysInAWeek = 7;

// Enumerations
// ex :
// enum Couleur
// {
//     kBlue = 0,
//     kWhite,
//     kRed,
// };

// Structures
// ex:
// struct UrlTableProperties
// {
//  string name;
//  int numEntries;
// }

// Declarations de classe avancees
// ex:
// class MaClasse;

// events
BOOST_MSM_EUML_EVENT(server_off)
BOOST_MSM_EUML_EVENT(server_on)
BOOST_MSM_EUML_EVENT(tick)
//BOOST_MSM_EUML_EVENT(stop_measure)
//BOOST_MSM_EUML_EVENT(resume_measure)

// Concrete FSM implementation
// The list of FSM states
// state not needing any entry or exit
BOOST_MSM_EUML_STATE((),Offline)
BOOST_MSM_EUML_STATE((),Online)

struct stop_measure
{
    template <class FSM,class EVT,class SourceState,class TargetState>
    void operator()(EVT const& ,FSM&,SourceState& ,TargetState& )
    {
        cout << "stm::stop_measure" << endl;
    }
};

struct resume_measure
{
    template <class FSM,class EVT,class SourceState,class TargetState>
    void operator()(EVT const& ,FSM&,SourceState& ,TargetState& )
    {
        cout << "stm::resume_measure" << endl;
    }
};

struct process_measure
{
    template <class FSM,class EVT,class SourceState,class TargetState>
    void operator()(EVT const& ,FSM&,SourceState& ,TargetState& )
    {
        cout << "stm::process_measure" << endl;
    }
};

// replaces the old transition table
BOOST_MSM_EUML_TRANSITION_TABLE((
      Offline   == Online  + server_off        / stop_measure() ,
      //  +------------------------------------------------------------------------------+
      Online    == Offline + server_on         / resume_measure(),
      Online	== Online  + tick              / process_measure()
      //  +------------------------------------------------------------------------------+
     ),transition_table)

// create a state machine "on the fly"
BOOST_MSM_EUML_DECLARE_STATE_MACHINE(( transition_table,//STT
		init_ << Offline,// Init State
		no_action,// Entry
		no_action,// Exit
		attributes_ << no_attributes_,// Attributes
		configure_ << no_configure_,// configuration
		Log_No_Transition// no_transition handler
		), stm_)
//fsm name

// choice of back-end
typedef msm::back::state_machine<stm_> stm;

//
// Testing utilities.
//
static char const* const state_names[] = { "Online", "Offline" };


/** @brief Gestionnaire des capteurs
 *  Description detaillee de la classe.
 */
namespace WAVENIS {
class SensorsManager {

public:
	/**
	 * Constructeur
	 */
	SensorsManager();
	/**
	 * Destructeur
	 */
	~SensorsManager();

	/**
	 * Meta State Machine
	 */




	stm m_stm;

	// Methodes publiques de la classe
	// ex : ReturnType NomMethode(Type);

	void stmstate(stm const& s)
	{
	    std::cout << " -> " << state_names[s.current_state()[0]] << std::endl;
	}


	void stop_measure(void) {
		cout << "sensorsmanager::stop_measure" << endl;
	}
	// Pour les associations :
	// Methodes publiques setter/getter (mutateurs/accesseurs) des attributs prives
	// ex :
	// void setNomAttribut(Type nomAttribut) { nomAttribut_ = nomAttribut; }
	// Type getNomAttribut(void) const { return nomAttribut_; }


protected:
	// Attributs proteges

	// Methode protegees

private:
	// Attributs prives
	// ex :
	// Type nomAttribut_;

	// Methodes privees
};
}
// Methodes publiques inline
// ex :
// inline void SensorsManager::maMethodeInline(Type valeur)
// {
//
// }
// inline Type SensorsManager::monAutreMethode_inline_(void)
// {
//   return 0;
// }

#endif  // _SENSORSMANAGER_H
