/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

#include "loader_protocol.h"
#include "utils.h"

#include <i_protocol_service.h>

void free_protocol( ProtocolLoader* protocol ) {
	FREE_LIBRARY( protocol->protocolHandle );
	memset( protocol, 0, sizeof(*protocol) );
}

int load_protocol( const char* protocolId, ProtocolLoader* protocol ) {
	int iRet = OK;

	if ( 0 != strcmp( protocolId, protocol->protocolId ) ) {
		if ( protocol->protocolHandle ) {
			free_protocol( protocol );
		}
		char protocolLibName[MAX_PATH] = "";
#ifdef _WINDOWS
		snprintf( protocolLibName, sizeof(protocolLibName) - 1, "%s.dll", protocolId );
#else
		snprintf( protocolLibName,sizeof(protocolLibName) - 1, "./lib%s.so", protocolId );
#endif
		if ( (protocol->protocolHandle = LOAD_LIBRARY(protocolLibName)) != NULL ) {
			// load version and check it
			if((protocol->get_version  = (protocol_get_version_t)LOAD_FUNCTION(protocol->protocolHandle, FN_PROTOCOL_GET_VERSION)) == NULL ){

				//LOG_ERROR(g_Logger, "load_protocol: Failed to load API's", 0);
				free_protocol( protocol );
				//return (int)GET_ERROR();
				return -1;
			}
			int expected_version = ((PROTOCOL_API_VERSION << 16) & 0xFF0000);
			int protocol_version = protocol->get_version() & 0xFF0000; // API version is important

			// check version
			if (protocol_version != expected_version)  {
				//LOG_ERROR(g_Logger, "load_protocol: Protocol version mismatch.", 0);
				free_protocol( protocol );
				return -1;	// wrong version
			}
			if ( ((protocol->init = (protocol_init_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_INIT)) != NULL) &&
					((protocol->init_from_file = (protocol_init_from_file_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_INIT_FROM_FILE)) != NULL) &&
					((protocol->create = (protocol_create_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_CREATE)) != NULL) &&
					((protocol->destroy = (protocol_destroy_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_DESTROY)) != NULL ) &&
					((protocol->open = (protocol_open_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_OPEN)) != NULL) &&
					((protocol->close = (protocol_close_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_CLOSE)) != NULL) &&
					((protocol->is_open = (protocol_is_open_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_IS_OPEN)) != NULL) &&
					((protocol->get_serial_driver_id = (protocol_get_serial_driver_id_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_GET_SERIAL_DRIVER_ID)) != NULL) &&
					((protocol->set_serial_driver_id = (protocol_set_serial_driver_id_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_SET_SERIAL_DRIVER_ID)) != NULL) &&
					((protocol->set_serial_driver_instance = (protocol_set_serial_driver_instance_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_SET_SERIAL_DRIVER_INSTANCE)) != NULL) &&
					((protocol->add_message_listener = (protocol_add_message_listener_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_ADD_MESSAGE_LISTENER)) != NULL) &&
					((protocol->remove_message_listener = (protocol_remove_message_listener_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_REMOVE_MESSAGE_LISTENER)) != NULL) &&
					((protocol->add_spy_message_listener = (protocol_add_spy_message_listener_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_ADD_SPY_MESSAGE_LISTENER)) != NULL) &&
					((protocol->remove_spy_message_listener = (protocol_remove_spy_message_listener_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_REMOVE_SPY_MESSAGE_LISTENER)) != NULL) &&
					((protocol->send_transparent_message = (protocol_send_transparent_message_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_SEND_TRANSPARENT_MESSAGE)) != NULL) &&
					((protocol->send_application_message = (protocol_send_application_message_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_SEND_APPLICATION_MESSAGE)) != NULL) &&
					((protocol->send_application_broadcast_message = (protocol_send_application_broadcast_message_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_SEND_APPLICATION_BROADCAST_MESSAGE)) != NULL) &&
					((protocol->send_application_request = (protocol_send_application_request_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_SEND_APPLICATION_REQUEST)) != NULL) &&
					((protocol->send_application_broadcast_request = (protocol_send_application_broadcast_request_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_SEND_APPLICATION_BROADCAST_REQUEST)) != NULL) &&
					((protocol->send_application_response = (protocol_send_application_response_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_SEND_APPLICATION_RESPONSE)) != NULL) &&
					((protocol->write_radio_params = (protocol_write_radio_params_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_WRITE_RADIO_PARAMS)) != NULL) &&
					((protocol->read_dynamic_expert_params = (protocol_read_dynamic_expert_params_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_READ_DYNAMIC_EXPERT_PARAMS)) != NULL) &&
					((protocol->read_static_standard_params = (protocol_read_static_standard_params_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_READ_STATIC_STANDARD_PARAMS)) != NULL) &&
					((protocol->read_static_expert_params = (protocol_read_static_expert_params_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_READ_STATIC_EXPERT_PARAMS)) != NULL) &&
					((protocol->request_local_node_service = (protocol_request_local_node_service_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_REQUEST_LOCAL_NODE_SERVICE)) != NULL) &&
					((protocol->send_service_message = (protocol_send_service_message_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_SEND_SERVICE_MESSAGE)) != NULL) &&
					((protocol->send_service_request = (protocol_send_service_request_t)LOAD_FUNCTION(protocol->protocolHandle,FN_PROTOCOL_SEND_SERVICE_REQUEST)) != NULL)
					) {
				strncpy( protocol->protocolId, protocolId, min( MAX_PATH, strlen(protocolId) ) );
				iRet = OK;
			} else { // LOAD_FUNCTION(s)
				free_protocol( protocol );
				//iRet =  (int)GET_ERROR();
				iRet = -1;
			}
		} else // LOAD_LIBRARY;
			iRet = -1;
			//iRet =  (int)GET_ERROR();
	}
	return iRet;
}
