/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
#include "loader_serial_driver.h"
#include "utils.h"

#include <serial_driver_types.h>
#include <i_serial_driver_service.h>
#include <i_serial_frame.h>


void free_serial_driver( SerialDriverLoader* serialDriver ) {
	if (serialDriver->serialDriverHandle){
		FREE_LIBRARY( serialDriver->serialDriverHandle );
	}
	memset( serialDriver, 0, sizeof(*serialDriver) );
}

int load_serial_driver( const char* serialDriverId, SerialDriverLoader* serialDriver ) {
	int iRet = OK;
	//LOG_DEBUG(g_Logger, "load_serial_driver: Start loading driver ... [1]", 0);
	if ( 0 != strcmp( serialDriverId, serialDriver->serialDriverId ) ) {
		if ( serialDriver->serialDriverHandle ) {
			free_serial_driver( serialDriver );
		}
		char serialDriverLibName[MAX_PATH] = "";
#ifdef _WINDOWS
		snprintf( serialDriverLibName, sizeof(serialDriverLibName) - 1, "%s.dll", serialDriverId );
#else
		snprintf( serialDriverLibName, sizeof(serialDriverLibName) - 1, "./lib%s.so", serialDriverId );
#endif
		//LOG_DEBUG(g_Logger, "load_serial_driver: Loading all exported APIs from : %s", serialDriverId);
		if ( (serialDriver->serialDriverHandle = LOAD_LIBRARY(serialDriverLibName)) != NULL ) {
			// load version and check it
			//LOG_DEBUG(g_Logger, "load_serial_driver: Loading version check API", 0);
			if((serialDriver->get_version  = (driver_get_version_t)LOAD_FUNCTION(serialDriver->serialDriverHandle, FN_DRIVER_GET_VERSION)) == NULL ){

				//LOG_ERROR(g_Logger, "load_serial_driver: Failed to load get version API!", 0);
				free_serial_driver( serialDriver );
				//return (int)GET_ERROR();
				return -1;
			}
			int expected_version = ( (SERIAL_DRIVER_API_VERSION << 16) & 0xFF0000);
			int driver_version = serialDriver->get_version() & 0xFF0000;

			// check version
			if (driver_version != expected_version)  {

				//LOG_ERROR(g_Logger, "load_serial_driver: Serial driver version mismatch.", 0);
				free_serial_driver( serialDriver );
				return -1;	// wrong version
			}
			if ( ((serialDriver->init =	(driver_init_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_INIT)) != NULL ) &&
					((serialDriver->init_from_file = (driver_init_from_file_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_INIT_FROM_FILE)) != NULL ) &&
					((serialDriver->create = (driver_create_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_CREATE)) != NULL ) &&
					((serialDriver->destroy = (driver_destroy_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_DESTROY)) != NULL ) &&
					((serialDriver->open = (driver_open_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_OPEN)) != NULL ) &&
					((serialDriver->close = (driver_close_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_CLOSE)) != NULL ) &&
					((serialDriver->is_open = (driver_is_open_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_IS_OPEN)) != NULL ) &&
					((serialDriver->get_port_id = (driver_get_port_id_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_GET_PORT_ID)) != NULL ) &&
					((serialDriver->set_port_id = (driver_set_port_id_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_SET_PORT_ID)) != NULL ) &&
					((serialDriver->set_port_params = (driver_set_port_params_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_SET_PORT_PARAMS)) != NULL ) &&
					((serialDriver->set_listening_period_param = (driver_set_listening_period_param_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_SET_LISTENING_PERIOD_PARAM)) != NULL ) &&
					((serialDriver->set_flow_control_mode_params = (driver_set_flow_control_mode_params_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_SET_FLOW_CONTROL_MODE_PARAMS)) != NULL ) &&
					((serialDriver->send = (driver_send_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_SEND)) != NULL ) &&
					((serialDriver->add_listener = (driver_add_listener_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_ADD_LISTENER)) != NULL ) &&
					((serialDriver->remove_listener = (driver_remove_listener_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_REMOVE_LISTENER)) != NULL ) &&
					((serialDriver->add_spy_listener = (driver_add_spy_listener_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_ADD_SPY_LISTENER)) != NULL ) &&
					((serialDriver->get_serial_driver_instance = (driver_get_serial_driver_instance_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_GET_SERIAL_DRIVER_INSTANCE)) != NULL ) &&
					((serialDriver->remove_spy_listener = (driver_remove_spy_listener_t)LOAD_FUNCTION(serialDriver->serialDriverHandle,FN_DRIVER_REMOVE_SPY_LISTENER) ) ) ) {
				strncpy( serialDriver->serialDriverId, serialDriverId, min( MAX_PATH, strlen(serialDriverId) ) );
				iRet = OK;
			} else { // LOAD_FUNCTION(s)
				//LOG_ERROR(g_Logger, "load_serial_driver: Failed to load API's", 0);
				free_serial_driver( serialDriver );
				//iRet = (int)GET_ERROR();
				iRet = -1;
			}

		} else{ // LOAD_LIBRARY;
			//LOG_ERROR(g_Logger, "load_serial_driver: Failed to load library", 0);
			//iRet = (int)GET_ERROR();
			iRet = -1;
		}
	}
	return iRet;
}
