/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
#include "serial_driver_types_utils.h"
#include "utils.h"

SerialExchangeData create_serial_exchange_data( SerialFrame frame ) {
	SerialExchangeData exchangeData;
	memcpy(exchangeData.serialFrame, &frame, sizeof(SerialFrame));
	((SerialFrame*)&exchangeData.serialFrame)->len = frame.len;
	return exchangeData;
}

SerialExchangeData create_serial_exchange_data_from_string( const char* hex_string ) {
	unsigned int i = 0;
	SerialFrame concreteFrame;
	char* hex_ptr = (char *)hex_string;
	concreteFrame.len =strlen(hex_string)>>1;

	if (concreteFrame.len > MAX_SERIALFRAME_LEN)
		concreteFrame.len = MAX_SERIALFRAME_LEN;

	for  ( i= 0; i<concreteFrame.len; i++ ){
		concreteFrame.frame[i] =two_chars_to_byte(hex_ptr);
		hex_ptr += 2;
	}

	return(create_serial_exchange_data (concreteFrame));
}

void set_serial_frame_value(pISerialFrame dest, const SerialFrame *src) {
	if (!dest || !src) return;
	memcpy(dest, src, sizeof(SerialFrame));
}


const char* get_hex_serialframe(char *frameString, const int frameStringSize, const SerialFrame* serialFrame ) {
	//static char frameString[MAX_SERIALFRAME_LEN*2+1] = "";

	// validate input parameters
	if (!frameString || (frameStringSize <= 0)) {
		return frameString;
	}

	frameString[0] = 0;
	if (serialFrame == 0) {
		return frameString;
	}

	sd_buffer_to_hex(frameString, serialFrame->frame, serialFrame->len, /*sizeof(frameString)*/ frameStringSize);
	return frameString;
}

const char* get_hex_serialexchangeddata(char *frameString, const int frameStringSize, const pSerialExchangeData serialExchangeData ) {
	SerialFrame* sf = (SerialFrame*)&serialExchangeData->serialFrame;
	return get_hex_serialframe(frameString, frameStringSize, sf);
}

const char* get_sending_status_name( SerialDriverStatus ss ) {
	static char sendingStatus[100] = "";
	const char* sendingStatusCode = "";
	sendingStatus[0] = 0;
	switch( ss ) {
	case OK_SERIALDRIVER_SUCCESS:                     sendingStatusCode = "OK_SERIALDRIVER_SUCCESS"; break;
	case KO_SENDING_FAILURE:                          sendingStatusCode = "KO_SENDING_FAILURE"; break;
	case KO_SERIALFRAME_REQUIRED:                     sendingStatusCode = "KO_SERIALFRAME_REQUIRED"; break;
	case KO_SERIALDRIVER_NOT_OPEN:                    sendingStatusCode = "KO_SERIALDRIVER_NOT_OPEN"; break;
	case KO_SERIALDRIVER_ALREADY_IN_USE:              sendingStatusCode = "KO_SERIALDRIVER_ALREADY_IN_USE"; break;
	case KO_SERIALDRIVER_ALREADY_OPEN:                sendingStatusCode = "KO_SERIALDRIVER_ALREADY_OPEN"; break;
	case KO_SERIALDRIVER_LISTENER_ALREADY_REGISTERED: sendingStatusCode = "KO_SERIALDRIVER_LISTENER_ALREADY_REGISTERED"; break;
	case KO_SERIALDRIVER_LISTENER_NOT_REGISTERED:     sendingStatusCode = "KO_SERIALDRIVER_LISTENER_NOT_REGISTERED"; break;
	case KO_SERIALDRIVER_MAX_SPY_LISTENER_EXCEED:     sendingStatusCode = "KO_SERIALDRIVER_MAX_SPY_LISTENER_EXCEED"; break;
	case KO_SERIALPORT_DISCONNECTED:                  sendingStatusCode = "KO_SERIALPORT_DISCONNECTED"; break;
	case KO_SERIALPORT_UNEXPECTED_ERROR:              sendingStatusCode = "KO_SERIALPORT_UNEXPECTED_ERROR"; break;
	case KO_SERIALPORT_NOT_AVAILABLE:                 sendingStatusCode = "KO_SERIALPORT_NOT_AVAILABLE"; break;
	case KO_PARAM_SERIALPORTID_NOT_EXIST:             sendingStatusCode = "KO_PARAM_SERIALPORTID_NOT_EXIST"; break;
	case KO_PARAM_BAUDRATE_NOT_VALID:                 sendingStatusCode = "KO_PARAM_BAUDRATE_NOT_VALID"; break;
	case KO_PARAM_DATABIT_NOT_VALID:                  sendingStatusCode = "KO_PARAM_DATABIT_NOT_VALID"; break;
	case KO_PARAM_STOPBIT_NOT_VALID:                  sendingStatusCode = "KO_PARAM_STOPBIT_NOT_VALID"; break;
	case KO_PARAM_PARITY_NOT_VALID:                   sendingStatusCode = "KO_PARAM_PARITY_NOT_VALID"; break;
	case KO_PARAM_FLOWCONTROLIN_NOT_VALID:            sendingStatusCode = "KO_PARAM_FLOWCONTROLIN_NOT_VALID"; break;
	case KO_PARAM_FLOWCONTROLOUT_NOT_VALID:           sendingStatusCode = "KO_PARAM_FLOWCONTROLOUT_NOT_VALID"; break;
	case KO_PARAM_LISTENINGPERIOD_NOT_VALID:          sendingStatusCode = "KO_PARAM_LISTENINGPERIOD_NOT_VALID"; break;
	case KO_PARAM_NOT_VALID:                          sendingStatusCode = "KO_PARAM_NOT_VALID"; break;
	default:                                          sendingStatusCode = "KO_SERIALPORT_UNEXPECTED_ERROR"; break;
	}
	strcpy( sendingStatus, sendingStatusCode );
	return sendingStatus;
}
