/*
 * utils.h
 *
 *  Created on: 31.10.2012
 *      Author: vmacari
 */

#ifndef _UTILS_H_
#define _UTILS_H_

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>
#include <ctype.h>

#ifdef _WINDOWS
#ifdef WINVER
#undef WINVER
#endif

#define WINVER 0x0502	// to include functions available in verssions upper
						// to winnt and win95
#include <windows.h>
#include <io.h>
#define SLEEP(x) Sleep(x);
#define DELTA_EPOCH_IN_MICROSECS  11644473600000000UL
struct timezone {
  int tz_minuteswest; /* of Greenwich */
  int tz_dsttime;     /* type of dst correction to apply */
};
#else
#include <stdarg.h>
#include <dlfcn.h>
#include <sys/time.h>
#define SLEEP(x) { struct timeval tv; tv.tv_sec = 0; tv.tv_usec = x * 1000; select(0, NULL, NULL, NULL, &tv); }
#endif //_WINDOWS

#include <generic_types.h>
#include <generic_constants.h>

unsigned int calc_crc( const byte* frame, int len );

/**
 * 	Function will determine path of the library/executable that is currently loaded
 */
_bool get_module_path( const void* moduleHandle, char* path, int pathLen );

/**
 * 	get difference in millisenconds between two passed timeval structures
 */
long millisecondtimediff( struct timeval tvEarly, struct timeval tvLater );

#ifdef _WINDOWS
/**
 * 	Return current time of the day using windows system functions
 */
int gettimeofday(struct timeval *tv, struct timezone *tz);
#endif // _WINDOWS

/**
 * 	Check if passed string is composed from valid hexadecimal characters
 */
_bool is_hex( const char* hexStr );

void sd_buffer_to_hex(char *string, const byte *data, const int size, const int max_string_size);

#endif /* _UTILS_H_ */
