/*
 * jni_utils.c
 *
 *  Created on: 02.11.2012
 *      Author: vmacari
 */
#ifdef _JAVA_WRAPPER
#include "jni_utils.h"

#include <serial_driver_event.h>
#include <loader_logger.h>
#include <JNI/domain_jni.h>

#include "protocol_waveport_enum_converssion.h"
#include "serial_driver_types_utils.h"
#include "protocol_types_utils.h"

#include <loader_serial_driver.h>
#include <loader_protocol.h>

extern void jni_notify_spies_command( const ProtocolInfo* waveProtocol, jobject command, _bool sendEvent );



/**
 * 		Throw an exception
 */
void throw_exception_message( JNIEnv *env, const char* errorMessage, char *errorDescription ) {

	if (!errorMessage || !strlen(errorMessage)) {
		return;
	}

	LOG_DEBUG( g_Logger, "throw_exception_message: Trying to throw an EXCEPTION: %s", errorMessage );
	jclass g_excCls = (*env)->FindClass(env,
			"com/coronis/sdk/protocol/api/ProtocolException");

	if (!g_excCls) return;

	LOG_DEBUG( g_Logger, "throw_exception_message: Try to get error code related field name from java: %s", errorMessage + 3);
	jfieldID id = (*env)->GetStaticFieldID(env, g_excCls, errorMessage + 3, "Ljava/lang/String;");
	LOG_DEBUG( g_Logger, "throw_exception_message: Check search result ...", 0);

	if (id != NULL) {

		LOG_DEBUG( g_Logger, "throw_exception_message: Id found, getting field object ... ", 0);

		jstring jstr = (*env)->GetStaticObjectField(env, g_excCls, id);
		if (jstr != NULL) {
			LOG_DEBUG( g_Logger, "throw_exception_message: Got field object, now getting message", 0);

			const char* msg = (*env)->GetStringUTFChars(env, jstr, NULL);
			if (msg != 0) {

				LOG_DEBUG( g_Logger, "throw_exception_message: Message got, fill in error description", 0);
				LOG_DEBUG( g_Logger, "throw_exception_message: msg: >>%s<<", msg);
				LOG_DEBUG( g_Logger, "throw_exception_message: description: >>%s<<", errorDescription);
				LOG_DEBUG( g_Logger, "throw_exception_message: errorMessage: >>%s<<", errorMessage);

				char fullMessage[150] = "";
				if (errorDescription && strlen(errorDescription))
					sprintf( fullMessage, "%s - %s", strlen(msg) > 0 ? msg : (errorMessage + 3), errorDescription);
				else
					sprintf( fullMessage, "%s", strlen(msg) > 0 ? msg : (errorMessage + 3));

				LOG_DEBUG( g_Logger, "throw_exception EXCEPTION: %s", fullMessage);
				(*env)->ThrowNew(env, g_excCls, fullMessage);
				(*env)->ReleaseStringUTFChars(env, jstr, msg);
			}
		} else {
			LOG_DEBUG( g_Logger, "throw_exception_message: Field object not found", 0);

		}
	} else {
		LOG_DEBUG( g_Logger, "throw_exception_message: Static field ID not found for field %s", errorMessage + 3);
		LOG_DEBUG( g_Logger, "throw_exception_message: Clean Java exception NoSuchFieldError which occured in this case", 0);
		(*env)->ExceptionClear( env );
	}

	LOG_DEBUG( g_Logger, "throw_exception_message: Delete local ref", 0);

	(*env)->DeleteLocalRef(env, g_excCls);
	LOG_DEBUG( g_Logger, "throw_exception_message: Exiting.", 0);
}

void throw_exception_id(JNIEnv *env, int errorCode) {

	if (errorCode != OK_PROTOCOL_SUCCESS) {
		LOG_DEBUG( g_Logger, "throw_exception EXCEPTION: %d", errorCode );
		const char* fieldName = get_response_status_name(errorCode);
		throw_exception_message(env, fieldName, NULL);
	}
}

 void throw_exception( JNIEnv *env, CommandResponse* cr ) {
	 if ( cr->status != OK_PROTOCOL_SUCCESS ) {
		 throw_exception_message( env, get_response_status_name( cr->status ),  cr->statusDescription);
	 }
 }

 /*
  * 	All throw exceptions should call a function that will check if given
  * 	exception is expected.
  *
  *
  * 		Catch an SerialDriverException and throw ProtocolException
  * 		The function checks if an exception has been thrown in
  * 		JNI environment, and if so, it will be catch and thrown
  * 		as a Protocol exception.
  * 		Basically,  here is performed SerialDriver to Protocol exception
  * 		conversion.
  *
  * 		Rethroing an exception should check if the exception is defined in
  * 		java part.
  *
  * 		@param overriden_error_code - the error code to override the error
  * 		from a lower layer
  */
 _bool catch_and_throw_exception( JNIEnv *env, int overriden_error_code) {

	 jthrowable exception = (*env)->ExceptionOccurred(env);

	 if ( !exception ){
		 LOG_DEBUG( g_Logger, "catch_and_throw_exception: Ok, not exceptions", 0);
		 return _false;
	 }

	 LOG_DEBUG( g_Logger, "catch_and_throw_exception: An exception occurred, get it's description", 0);
	 if (overriden_error_code == -1) {
		 // an SerialDriverException occurred
		 (*env)->ExceptionDescribe( env );
		 (*env)->ExceptionClear( env );
		 jclass classID = (*env)->GetObjectClass( env, exception);
		 if ( classID == NULL ) return _true;

		 jfieldID fieldID = (*env)->GetFieldID( env, classID, "exceptionValue", "Ljava/lang/String;");

		 if ( fieldID != NULL ) {
			 jstring jstr = (*env)->GetObjectField( env, exception, fieldID );
			 if ( jstr != NULL ) {
				 const char* str = (*env)->GetStringUTFChars( env, jstr, NULL);

				 if ( str != NULL) {
					 LOG_DEBUG( g_Logger, "Exception occur: %s", str);
					 // throw_exception_message( env, str );
					 throw_exception_message( env, str, 0 );
					 (*env)->ReleaseStringUTFChars( env, jstr, str);
				 }
				 (*env)->DeleteLocalRef( env, jstr);
			 }
		 }

		 (*env)->DeleteLocalRef( env, classID);
	 } else {
		 // rethrow exception with a new description code
		 throw_exception_id(env, overriden_error_code );
	 }
	 return _true;
 }

 jobject create_int_object( JNIEnv *env, jint value ) {
 	jobject intValue = NULL;
 	jclass intClassID = (*env)->FindClass( env, "java/lang/Integer");
 	if ( intClassID != NULL ) {
 		jmethodID methodID = (*env)->GetMethodID( env, intClassID, "<init>", "(I)V");
 		if ( methodID != NULL ) {
 			intValue = (*env)->NewObject( env, intClassID, methodID, value );
 		}
 		(*env)->DeleteLocalRef( env, intClassID );
 	}
 	return intValue;
 }

 jobject create_float_object( JNIEnv *env, jfloat value ) {
 	jobject floatValue = NULL;
 	jclass floatClassID = (*env)->FindClass( env, "java/lang/Float");
 	if ( floatClassID != NULL ) {
 		jmethodID methodID = (*env)->GetMethodID( env, floatClassID, "<init>", "(F)V");
 		if ( methodID != NULL ) {
 			floatValue = (*env)->NewObject( env, floatClassID, methodID, value );
 		}
 		(*env)->DeleteLocalRef( env, floatClassID );
 	}
 	return floatValue;
 }

 jobject create_boolean_object( JNIEnv *env, jboolean value ) {
 	jobject intValue = NULL;
 	jclass classID = (*env)->FindClass( env, "java/lang/Boolean");
 	if ( classID != NULL ) {
 		jmethodID methodID = (*env)->GetMethodID( env, classID, "<init>", "(Z)V");
 		if ( methodID != NULL ) {
 			intValue = (*env)->NewObject( env, classID, methodID, value );
 		}
 		(*env)->DeleteLocalRef( env, classID );
 	}
 	return intValue;
 }

 jobject create_serial_frame_object( JNIEnv *env, jclass classID, const char* value ) {
 	jobject serialFrame = NULL;

 	LOG_DEBUG(g_Logger, "create_serial_frame_object: 1 - for hex string %s", value);
 	jstring jstr = (*env)->NewStringUTF( env, value );
 	LOG_DEBUG(g_Logger, "create_serial_frame_object: 2", 0);
 	if ( jstr ) {
 		LOG_DEBUG(g_Logger, "create_serial_frame_object: 3", 0);
 		if ( classID ) {
 			LOG_DEBUG(g_Logger, "create_serial_frame_object: 4", 0);
 			jmethodID methodID = (*env)->GetMethodID( env, classID, "<init>", "(Ljava/lang/String;)V");
 			LOG_DEBUG(g_Logger, "create_serial_frame_object: 5", 0);
 			if ( methodID != NULL ) {
 				LOG_DEBUG(g_Logger, "create_serial_frame_object: 6", 0);
 				serialFrame = (*env)->NewObject( env, classID, methodID, jstr );
 				LOG_DEBUG(g_Logger, "create_serial_frame_object: 7", 0);
 			}
 			LOG_DEBUG(g_Logger, "create_serial_frame_object: 8", 0);
 		}
 		LOG_DEBUG(g_Logger, "create_serial_frame_object: 9", 0);
 		(*env)->DeleteLocalRef( env, jstr );
 		LOG_DEBUG(g_Logger, "create_serial_frame_object: 10", 0);
 	}
 	LOG_DEBUG(g_Logger, "create_serial_frame_object: 11", 0);

 	return serialFrame;
 }

 jobject create_payload_object( JNIEnv *env, jclass classID, const char* value ) {
 	jobject payload = NULL;
 	jstring jstr = (*env)->NewStringUTF( env, value );
 	if ( jstr ) {
 		if ( classID ) {
 			jmethodID methodID = (*env)->GetMethodID( env, classID, "<init>", "(Ljava/lang/String;)V");
 			if ( methodID != NULL ) {
 				payload = (*env)->NewObject( env, classID, methodID, jstr );
 			}
 		}
 		(*env)->DeleteLocalRef( env, jstr );
 	}
 	return payload;
 }

 jobject create_response_status( JNIEnv *env, ResponseStatus rs ) {

 	jobject responseStatus = NULL;
 	jclass classResponseStatusID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/ResponseStatus");

 	if ((*env)->ExceptionCheck(env)) {
 		LOG_DEBUG(g_Logger, "jni create_response_status: Java exception occurred 1", 0);
 	}

 	if ( classResponseStatusID != NULL ) {
 		const char* fieldIDName = get_response_status_name( rs );
 		LOG_DEBUG( g_Logger, "jni create_response_status: %d - %s", rs, fieldIDName);
 		jfieldID id = (*env)->GetStaticFieldID( env, classResponseStatusID, fieldIDName, "Lcom/coronis/sdk/protocol/api/ResponseStatus;");
 		if ((*env)->ExceptionCheck(env)) {
 			LOG_DEBUG(g_Logger, "jni create_response_status: Java exception occurred when trying to get error description from java. Please add"
 					"following error code in com.coronis.sdk.protocol.api.ResponseStatus : %s", fieldIDName);
 		}

 		if ( id != NULL )
 			responseStatus = (*env)->GetStaticObjectField( env, classResponseStatusID, id);
 		(*env)->DeleteLocalRef( env, classResponseStatusID );
 	}

 	return responseStatus;
 }

 jobject create_node_service_id( JNIEnv *env, NodeServiceId nodeServiceId ) {
 	jobject jNodeServiceId = NULL;
 	jclass classNodeServiceIdID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/NodeServiceId");
 	if ( classNodeServiceIdID != NULL ) {
 		const char* fieldIDName = NULL;
 		switch( nodeServiceId ) {
 		case STACK_SERVICE:  				   fieldIDName = "STACK_SERVICE";  break;
 		case SDP_SERVICE:          			   fieldIDName = "SDP_SERVICE";          break;
 		case FIRMWARE_UPGRADE_SERVICE:         fieldIDName = "FIRMWARE_UPGRADE_SERVICE";         break;
 		case NODE_SERVICE_ID_UNDEFINED:
 		default:
 			fieldIDName = "UNDEFINED"; break;
 		}
 		jfieldID id = (*env)->GetStaticFieldID( env, classNodeServiceIdID, fieldIDName, "Lcom/coronis/sdk/protocol/api/NodeServiceId;");
 		if ( id != NULL )
 			jNodeServiceId = (*env)->GetStaticObjectField( env, classNodeServiceIdID, id);
 		(*env)->DeleteLocalRef( env, classNodeServiceIdID );
 	}
 	return jNodeServiceId;
 }

 /**
  * Function used to create JNI CommandResponse object from pointer to C CommandResponse object.
  * @param env - a JNIEnv instance.
  * @param cr - a pointer to C CommandResponse object.
  * @return JNI CommandResponse object if success and NULL in conversion fails.
  */
 jobject create_command_response( JNIEnv *env, jclass classSerialFrame, CommandResponse* cr ) {
 	jobject commandResponse = NULL;

 	LOG_DEBUG(g_Logger, "create_command_response: Entered ", 0);
 	if ((*env)->ExceptionCheck(env)) {
 		LOG_ERROR(g_Logger, "create_command_response: Java exception occurred 2", 0);
 	}

 	jobject statusDescription = strlen(cr->statusDescription) ? (*env)->NewStringUTF( env, cr->statusDescription ): NULL;
 	jobject responseStatus = create_response_status( env, cr->status );


 	if ( responseStatus != NULL ) {
 		jclass classCommandResponseID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/CommandResponse");
 		if ( classCommandResponseID != NULL ) {

 			const char* methodName = statusDescription ? "(Lcom/coronis/sdk/protocol/api/ResponseStatus;Ljava/lang/String;)V" : "(Lcom/coronis/sdk/protocol/api/ResponseStatus;)V";

 			jmethodID methodID = (*env)->GetMethodID( env, classCommandResponseID, "<init>", methodName);// get constructor

 			if ( methodID != NULL )
 				commandResponse = (*env)->NewObject( env, classCommandResponseID, methodID, responseStatus, statusDescription);

 			// Set serial frame in response command
 			if (commandResponse && classSerialFrame && cr->serialFrame) {
 				methodID = (*env)->GetMethodID( env, classCommandResponseID, "setSerialFrame", "(Lcom/coronis/sdk/serialdriver/api/ISerialFrame;)V");
 				if ( methodID != NULL ) {

 					char frameString[MAX_SERIALFRAME_LEN*2+1] = "";
 					const char *hex_frame = get_hex_serialframe(frameString, sizeof(frameString), (SerialFrame*)&cr->serialFrame);
 					jobject jSerialFrame = create_serial_frame_object( env, classSerialFrame,
 							hex_frame);

 					if ( jSerialFrame != NULL ) {
 						(*env)->CallVoidMethod( env, commandResponse, methodID, jSerialFrame );
 						(*env)->DeleteLocalRef( env, jSerialFrame );
 					}
 				}
 			}
 			(*env)->DeleteLocalRef( env, classCommandResponseID);
 		}

 		(*env)->DeleteLocalRef( env, responseStatus);
 	}

 	if ( statusDescription )
 		(*env)->DeleteLocalRef( env, statusDescription);

 	return commandResponse;
 }

 jobject create_local_node_service_response( JNIEnv *env, ResponseStatus rs, jobject localNodeServiceCommand ) {
 	jobject commandResponse = NULL;
 	jobject responseStatus = create_response_status( env, rs );
 	if ( responseStatus != NULL ) {
 		jclass classCommandResponseID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/LocalNodeServiceResponse");
 		if ( classCommandResponseID != NULL ) {
 			jmethodID methodID = (*env)->GetMethodID( env, classCommandResponseID, "<init>", "(Lcom/coronis/sdk/protocol/api/ResponseStatus;Lcom/coronis/sdk/protocol/api/LocalNodeServiceCommand;)V");// get constructor
 			if ( methodID != NULL )
 				commandResponse = (*env)->NewObject( env, classCommandResponseID, methodID, responseStatus, localNodeServiceCommand);
 			(*env)->DeleteLocalRef( env, classCommandResponseID);
 		}
 		(*env)->DeleteLocalRef( env, responseStatus);
 	}

 	if ( localNodeServiceCommand != NULL )
 		(*env)->DeleteLocalRef( env, localNodeServiceCommand);

 	return commandResponse;
 }

 jobject create_radio_settings_response( JNIEnv *env, ResponseStatus rs, jobject radioSettingsCommand ) {
 	jobject commandResponse = NULL;
 	jobject responseStatus = create_response_status( env, rs );
 	if ( responseStatus != NULL ) {
 		jclass classCommandResponseID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/RadioSettingsResponse");
 		if ( classCommandResponseID != NULL ) {
 			jmethodID methodID = (*env)->GetMethodID( env, classCommandResponseID, "<init>", "(Lcom/coronis/sdk/protocol/api/ResponseStatus;Lcom/coronis/sdk/protocol/api/RadioSettingsCommand;)V");// get constructor
 			if ( methodID != NULL ) {
 				commandResponse = (*env)->NewObject( env, classCommandResponseID, methodID, responseStatus, radioSettingsCommand);
 			}
 			(*env)->DeleteLocalRef( env, classCommandResponseID);
 		}
 		(*env)->DeleteLocalRef( env, responseStatus);
 	}

 	if ( radioSettingsCommand != NULL )
 		(*env)->DeleteLocalRef( env, radioSettingsCommand);

 	return commandResponse;
 }

 /**
  * Function used to create JNI RequestResponse object from pointer to C CommandResponse object
  * and JNI RadioExchangeCommand object.
  * @param env - a JNIEnv instance.
  * @param cr - a pointer to C CommandResponse object.
  * @param radioExchangeCommand - a pointer to JNI RadioExchangeCommand object.
  * @return JNI CommandResponse object if success and NULL in conversion fails.
  */
 jobject create_request_response( JNIEnv *env,
		 jclass classSerialFrame, const CommandResponse* cr,
		 jobject radioExchangeCommand) {
 	jobject requestResponse = NULL;
 	jobject statusDescription = strlen(cr->statusDescription) ? (*env)->NewStringUTF( env, cr->statusDescription ): NULL;
 	jobject responseStatus = create_response_status( env, cr->status );

 	LOG_DEBUG(g_Logger,"create_request_response: Start processing", 0);
 	if ( responseStatus != NULL ) {
 		jclass classID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/RequestResponse");
 		if ( classID != NULL ) {//
 			jmethodID methodID = NULL;

 			if ( statusDescription != NULL )
 				methodID = (*env)->GetMethodID( env, classID, "<init>", "(Lcom/coronis/sdk/protocol/api/ResponseStatus;Ljava/lang/String;Lcom/coronis/sdk/protocol/api/RadioExchangeCommand;)V" );// get constructor
 			else
 				methodID = (*env)->GetMethodID( env, classID, "<init>", "(Lcom/coronis/sdk/protocol/api/ResponseStatus;Lcom/coronis/sdk/protocol/api/RadioExchangeCommand;)V" );// get constructor


 			if ((*env)->ExceptionCheck(env)) {
 				LOG_ERROR(g_Logger,"create_request_response: Exception occured 1", 0);
 			}

 			LOG_DEBUG(g_Logger,"create_request_response: Constructor obtained", 0);
 			if ( methodID != NULL ) {
 				if ( statusDescription != NULL )
 					requestResponse = (*env)->NewObject( env, classID, methodID, responseStatus, statusDescription, radioExchangeCommand);
 				else
 					requestResponse = (*env)->NewObject( env, classID, methodID, responseStatus, radioExchangeCommand);
 			}

 			if ((*env)->ExceptionCheck(env)) {
 				LOG_ERROR(g_Logger,"create_request_response: Exception occurred 2", 0);
 			}

 			// set serial frame
 			LOG_DEBUG(g_Logger,"create_request_response: Try to set serial frame", 0);

 			if (cr->serialFrame || (((SerialFrame*)cr->serialFrame)->len > 0)) {
 				LOG_DEBUG(g_Logger,"create_request_response: Serial frame not set!", 0);

 			}


 			if (requestResponse && classSerialFrame && cr->serialFrame && (((SerialFrame*)&cr->serialFrame)->len > 0)) {
 				methodID = (*env)->GetMethodID( env, classID, "setSerialFrame", "(Lcom/coronis/sdk/serialdriver/api/ISerialFrame;)V");
 				if ((*env)->ExceptionCheck(env)) {
 					LOG_ERROR(g_Logger,"create_request_response: Exception occurred 3", 0);
 				}

 				if ( methodID != NULL ) {

 					if ((*env)->ExceptionCheck(env)) {
 						LOG_ERROR(g_Logger,"create_request_response: Exception occurred 4", 0);
 					}

 					char frameString[MAX_SERIALFRAME_LEN*2+1] = "";
 					jobject jSerialFrame = create_serial_frame_object( env, classSerialFrame,
 							get_hex_serialframe(frameString, sizeof(frameString), (SerialFrame*)&cr->serialFrame) );
 					if ( jSerialFrame != NULL ) {

 						if ((*env)->ExceptionCheck(env)) {
 							LOG_ERROR(g_Logger,"create_request_response: Exception occurred 5", 0);
 						}

 						(*env)->CallVoidMethod( env, requestResponse, methodID, jSerialFrame );
 						(*env)->DeleteLocalRef( env, jSerialFrame );

 						if ((*env)->ExceptionCheck(env)) {
 							LOG_ERROR(g_Logger,"create_request_response: Exception occurred 6", 0);
 						}

 					}

 					if ((*env)->ExceptionCheck(env)) {
 						LOG_ERROR(g_Logger,"create_request_response: Exception occurred 7", 0);
 					}

 				}
 			}

 			if ((*env)->ExceptionCheck(env)) {
 				LOG_ERROR(g_Logger,"create_request_response: Exception occurred 4", 0);
 			}

 			(*env)->DeleteLocalRef( env, classID);
 		}
 		(*env)->DeleteLocalRef( env, responseStatus);
 	}
 	if ( statusDescription != NULL )
 		(*env)->DeleteLocalRef( env, statusDescription);
 	if ( radioExchangeCommand != NULL )
 		(*env)->DeleteLocalRef( env, radioExchangeCommand);
 	return requestResponse;
 }

 jobject create_multicast_request_response( JNIEnv *env,
		 jclass classPayload, jclass classRadioExchangeCommand,
		 ProtocolInfo* waveProtocol, MulticastRequestResponse* response, jobject jDynamicExpertParams ) {
 	jobject multicastRequestResponse = NULL;

 	jobject responseStatus = create_response_status( env, response->response.status );

 	LOG_DEBUG(g_Logger,"create_multicast_request_response: Entered ...", 0);
 	if ( responseStatus != NULL ) {
 		jobject list = NULL;
 		jclass classID = (*env)->FindClass( env, "java/util/ArrayList");
 		LOG_DEBUG(g_Logger,"create_multicast_request_response: 1", 0);
 		if ( classID != NULL ) {
 			LOG_DEBUG(g_Logger,"create_multicast_request_response: 2", 0);
 			jmethodID methodID = (*env)->GetMethodID( env, classID, "<init>", "()V" );// get constructor
 			if ( methodID != NULL )
 				list = (*env)->NewObject( env, classID, methodID);

 			if (list == NULL) {
 				LOG_ERROR(g_Logger,"create_multicast_request_response: List is NULL", 0);
 			}

 			LOG_DEBUG(g_Logger,"create_radio_exchange_command_object: response->commandsLen is : %d", response->commandsLen);

 			if ( list != NULL && response->commandsLen > 0 ) {
 				LOG_DEBUG(g_Logger,"create_multicast_request_response: 3", 0);
 				methodID = (*env)->GetMethodID( env, classID, "add", "(Ljava/lang/Object;)Z");
 				if ( methodID != NULL ) {
 					LOG_DEBUG(g_Logger,"create_multicast_request_response: 4", 0);
 					int i;
 					LOG_DEBUG(g_Logger,"create_multicast_request_response: Setup command for each received broadcast response.", 0);
 					for ( i = 0; i < response->commandsLen; ++i ) {

 						WavePortResponses responseId = -1;
 						if (((SerialFrame*)&response->response.serialFrame)->len > 0) {
 							responseId = (WavePortResponses)((SerialFrame*)&response->response.serialFrame)->frame[0];
 						}

 						jobject dynamicStandardParams = create_dynamic_standard_params_object( env, &response->commands[i].dynamicStandardParams, 0 );
 						jobject command = create_radio_exchange_command_object( env,
 								classPayload,
 								classRadioExchangeCommand,
 								&(response->commands[i]),
 								dynamicStandardParams, jDynamicExpertParams, responseId);

 						if ( command != NULL ) {
 							if ( response->response.status == OK_PROTOCOL_SUCCESS )
 								jni_notify_spies_command( waveProtocol, command, _false );
 							(*env)->CallBooleanMethod( env, list, methodID, command);
 							(*env)->DeleteLocalRef( env, command);
 						}
 						if ( dynamicStandardParams != NULL )
 							(*env)->DeleteLocalRef( env, dynamicStandardParams);
 					}
 				}
 			}
 			(*env)->DeleteLocalRef( env, classID);
 		}

 		if ( list != NULL ) {
 			classID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/MulticastRequestResponse");
 			if ( classID != NULL ) {
 				jmethodID methodID = (*env)->GetMethodID( env, classID, "<init>", "(Lcom/coronis/sdk/protocol/api/ResponseStatus;Ljava/util/List;)V" );// get constructor
 				if ( methodID != NULL ) {
 					multicastRequestResponse = (*env)->NewObject( env, classID, methodID, responseStatus, list);
 					LOG_DEBUG( g_Logger, "create_multicast_request_response: %X", multicastRequestResponse);
 				}
 				(*env)->DeleteLocalRef( env, classID);
 			}
 			(*env)->DeleteLocalRef( env, list);
 		}
 		(*env)->DeleteLocalRef( env, responseStatus);
 	}

 	if ( jDynamicExpertParams != NULL )
 		(*env)->DeleteLocalRef( env, jDynamicExpertParams);

 	LOG_DEBUG(g_Logger,"create_multicast_request_response: Exit", 0);

 	return multicastRequestResponse;
 }

 jobject create_radio_settings_command( JNIEnv *env,
		jclass classPayload,
 		jobject dynamicStandardParams,
 		jobject dynamicExpertParams,
 		jobject staticStandardParams,
 		jobject staticExpertParams,
 		const char* payloadValue ) {
 	jobject cmd = NULL;
 	char methodIDName[1024] = "()V";
 	sprintf( methodIDName, "(%s%s%s%s)V",
 			"Lcom/coronis/sdk/protocol/api/DynamicStandardParams;",
 			"Lcom/coronis/sdk/protocol/api/DynamicExpertParams;",
 			"Lcom/coronis/sdk/protocol/api/StaticStandardParams;",
 			"Lcom/coronis/sdk/protocol/api/StaticExpertParams;" );
 	jclass classID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/RadioSettingsCommand");
 	if ( classID != NULL ) {
 		jmethodID methodID = (*env)->GetMethodID( env, classID, "<init>", methodIDName);
 		if ( methodID != NULL ) {

 			LOG_DEBUG(g_Logger,"create_radio_settings_command: Call constructor of RadioSettingsCommand with following parameters NULL/NotNULL"
 					" dynamicStandardParams %s, dynamicExpertParams %s, staticStandardParams %s, staticExpertParams %s",
 					(dynamicStandardParams == NULL ?  "NULL" : "Not NULL"),
 					(dynamicExpertParams == NULL ?  "NULL" : "Not NULL"),
 					(staticStandardParams == NULL ?  "NULL" : "Not NULL"),
 					(staticExpertParams == NULL ?  "NULL" : "Not NULL")
 			);

 			LOG_DEBUG( g_Logger, "create_radio_settings_command: Create command", 0);

 			cmd = (*env)->NewObject( env, classID, methodID, dynamicStandardParams, dynamicExpertParams, staticStandardParams, staticExpertParams );
 			if ( cmd != NULL && payloadValue != NULL && classPayload != NULL) {

 				LOG_DEBUG( g_Logger, "create_radio_settings_command: Create method id", 0);
 				methodID = (*env)->GetMethodID( env, classID, "setPayload", "(Lcom/coronis/sdk/protocol/api/IPayload;)V");
 				LOG_DEBUG( g_Logger, "create_radio_settings_command: Method id created, check value ", 0);
 				if ( methodID != NULL ) {

 					LOG_DEBUG( g_Logger, "create_radio_settings_command: Method id is not null, create payload object ", 0);
 					jobject jPayload = create_payload_object (env, classPayload, payloadValue );
 					LOG_DEBUG( g_Logger, "create_radio_settings_command: Payload created, invoke it ", 0);

 					if ( jPayload != NULL ) {
 						(*env)->CallVoidMethod( env, cmd, methodID, jPayload );
 						(*env)->DeleteLocalRef( env, jPayload );
 					}

 					LOG_DEBUG( g_Logger, "create_radio_settings_command: Payload set invoked, check exception", 0);
 					if ((*env)->ExceptionOccurred(env)) {
 						LOG_ERROR( g_Logger, "create_radio_settings_command: Exception occured when payload is beign set", 0);
 					}

 				}
 			}
 		}
 		(*env)->DeleteLocalRef( env, classID );
 	}

 	return cmd;
 }


 jobject create_dynamic_standard_params_object( JNIEnv *env, DynamicStandardParams* dynamicStandardParams, WavePortResponses response) {

 	jobject nodeAddress = NULL;
 	jobject routePath = NULL;
 	jobject jDynamicStandardParams = NULL;

 	jclass classID  = NULL;
 	jmethodID methodID = NULL;


 	LOG_DEBUG(g_Logger, "create_dynamic_standard_params_object: Save node address :  %s",  ((RadioAddress*)&dynamicStandardParams->nodeAddress)->addressValue);
 	jobject jstr = (*env)->NewStringUTF( env, ((RadioAddress*)&dynamicStandardParams->nodeAddress)->addressValue );

 	classID = (*env)->FindClass( env, "com/coronis/sdk/domain/RadioAddress");
 	if ( classID != NULL ) {
 		methodID = (*env)->GetMethodID( env, classID, "<init>", "(Ljava/lang/String;)V");
 		if ( methodID != NULL && jstr != NULL )
 			nodeAddress = (*env)->NewObject( env, classID, methodID, jstr );
 	}

 	// create route path (an array of node addresses)
 	if (response == RECEIVED_FRAME_RELAYED && classID != NULL && methodID != NULL) {
 		// get elements count
 		int i, nodesCount = 0;
 		for (i = 0; i < MAX_ROUTER_PATH; i ++) {

 			if (strlen( ((RadioAddress*)&dynamicStandardParams->routerPath[i])->addressValue) <= 0)
 				break;

 			nodesCount ++;
 		}

 		// create array
 		jobject jNodeAddressObj = NULL;
 		jobject jNodeAddress = (*env)->NewStringUTF( env, "");
 		routePath = (*env)->NewObjectArray(env, nodesCount,
 				(*env)->FindClass( env, "com/coronis/sdk/domain/RadioAddress"), jNodeAddress);

 		if (jNodeAddress != NULL)
 			(*env)->DeleteLocalRef( env, jNodeAddress);

 		// create and set each array element
 		for (i = 0; i < nodesCount; i ++) {

 			LOG_DEBUG(g_Logger, "create_dynamic_standard_params_object: Set route address :  %s",
 					((RadioAddress*)&dynamicStandardParams->routerPath[i])->addressValue);

 			jNodeAddress 	= (*env)->NewStringUTF( env, ((RadioAddress*)&dynamicStandardParams->routerPath[i])->addressValue);
 			jNodeAddressObj = (*env)->NewObject( env, classID, methodID, jNodeAddress);

 			if (jNodeAddress == NULL)
 				LOG_ERROR(g_Logger, "create_dynamic_standard_params_object: Node address string is null", 0);

 			if (jNodeAddressObj == NULL)
 				LOG_ERROR(g_Logger, "create_dynamic_standard_params_object: Node address object is null", 0);

 			(*env)->SetObjectArrayElement(env, routePath, i,  jNodeAddressObj);

 			if (jNodeAddressObj != NULL)
 				(*env)->DeleteLocalRef( env, jNodeAddressObj);

 			if (jNodeAddress != NULL)
 				(*env)->DeleteLocalRef( env, jNodeAddress);
 		}
 	}

 	if (classID != NULL)
 		(*env)->DeleteLocalRef( env, classID );

 	classID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/DynamicStandardParams");
 	if ( classID != NULL ) {
 		jmethodID methodID = (*env)->GetMethodID( env, classID, "<init>",
 				routePath != NULL ?  "(Lcom/coronis/sdk/protocol/api/IRadioAddress;[Lcom/coronis/sdk/protocol/api/IRadioAddress;)V" :
 						"(Lcom/coronis/sdk/protocol/api/IRadioAddress;)V"
 		);

 		if ( methodID != NULL && nodeAddress != NULL ) {

 			if (routePath != NULL)
 				jDynamicStandardParams = (*env)->NewObject( env, classID, methodID, nodeAddress, routePath);
 			else
 				jDynamicStandardParams = (*env)->NewObject( env, classID, methodID, nodeAddress );
 		}
 		(*env)->DeleteLocalRef( env, classID );
 	}

 	if ( jstr != NULL )
 		(*env)->DeleteLocalRef( env, jstr );
 	if ( nodeAddress != NULL )
 		(*env)->DeleteLocalRef( env, nodeAddress );
 	if (routePath != NULL)
 		(*env)->DeleteLocalRef( env, routePath );

 	return jDynamicStandardParams;
 }


 jobject create_dynamic_expert_params( JNIEnv *env, DynamicExpertParams* dynamicExpertParams ) {
 	jobject params 							= NULL;
 	jobject transmissionPowerLevel 			= NULL;
 	jobject broadcastSessionMaxResponsesNb 	= NULL;
 	jobject broadcastSessionTimeout 		= NULL;
 	jobject responseTimeout 				= NULL;
 	jobject transmissionRadioChannel 		= NULL;
 	jobject wakeupLength 					= NULL;
 	jobject receptionMode 					= NULL;

 	LOG_DEBUG(g_Logger, "create_dynamic_expert_params: entered!", 0);
 	if (dynamicExpertParams != NULL) {
 		if ( dynamicExpertParams->isSet & FLAG_TRANSMISSION_POWER_LEVEL )
 			transmissionPowerLevel = create_float_object( env, dynamicExpertParams->transmissionPowerLevel);
 		if ( dynamicExpertParams->isSet & FLAG_BROADCAST_SESSION_MAX_RESPONSES_NB )
 			broadcastSessionMaxResponsesNb = create_int_object( env, dynamicExpertParams->broadcastSessionMaxResponsesNb);
 		if ( dynamicExpertParams->isSet & FLAG_BROADCAST_SESSION_TIMEOUT )
 			broadcastSessionTimeout = create_int_object( env, dynamicExpertParams->broadcastSessionTimeout);
 		if ( dynamicExpertParams->isSet & FLAG_RESPONSE_TIMEOUT )
 			responseTimeout = create_int_object( env, dynamicExpertParams->responseTimeout);
 		if ( dynamicExpertParams->isSet & FLAG_TRANSMISSION_RADIO_CHANNEL )
 			transmissionRadioChannel = create_radiochannel_enum_object( env, dynamicExpertParams->transmissionRadioChannel);
 		if ( dynamicExpertParams->isSet & FLAG_WAKEUP_LENGTH )
 			wakeupLength = create_int_object( env, dynamicExpertParams->wakeupLength);
 		if ( dynamicExpertParams->isSet & FLAG_RECEPTION_MODE ) {
 			LOG_DEBUG(g_Logger, "create_dynamic_expert_params: Reception mode flag is set to %d", dynamicExpertParams->receptionMode);
 			receptionMode = create_reception_mode_enum_object( env, dynamicExpertParams->receptionMode);
 			if (receptionMode !=  NULL) {
 				LOG_DEBUG(g_Logger, "create_dynamic_expert_params: Reception mode enum was created.", 0);
 			}
 		}
 	}

 	jclass classID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/DynamicExpertParams");
 	if ( classID != NULL ) {
 		jmethodID methodID = (*env)->GetMethodID( env, classID, "<init>", "()V");
 		if ( methodID != NULL ) {
 			params = (*env)->NewObject( env, classID, methodID );
 			if ( params != NULL ) {

 				methodID = (*env)->GetMethodID( env, classID, "setWakeupLength", "(Ljava/lang/Integer;)V");
 				if ( methodID != NULL && wakeupLength != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, wakeupLength);

 				methodID = (*env)->GetMethodID( env, classID, "setTransmissionRadioChannel", "(Lcom/coronis/sdk/protocol/api/ConstantParams$RadioChannel;)V");
 				if ( methodID != NULL && transmissionRadioChannel != NULL ) {
 					LOG_DEBUG(g_Logger, "create_dynamic_expert_params: Call of setTransmissionRadioChannel", 0);
 					(*env)->CallVoidMethod( env, params, methodID, transmissionRadioChannel);
 				}

 				methodID = (*env)->GetMethodID( env, classID, "setTransmissionPowerLevel", "(Ljava/lang/Float;)V");
 				if ( methodID != NULL && transmissionPowerLevel != NULL ) {
 					LOG_DEBUG(g_Logger, "create_dynamic_expert_params: Call of setTransmissionPowerLevel %d", dynamicExpertParams->transmissionPowerLevel);
 					(*env)->CallVoidMethod( env, params, methodID, transmissionPowerLevel);
 				}

 				methodID = (*env)->GetMethodID( env, classID, "setResponseTimeout", "(Ljava/lang/Integer;)V");
 				if ( methodID != NULL && responseTimeout != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, responseTimeout);

 				methodID = (*env)->GetMethodID( env, classID, "setBroadcastSessionTimeout", "(Ljava/lang/Integer;)V");
 				if ( methodID != NULL && broadcastSessionTimeout != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, broadcastSessionTimeout);

 				methodID = (*env)->GetMethodID( env, classID, "setBroadcastSessionMaxResponsesNb", "(Ljava/lang/Integer;)V");
 				if ( methodID != NULL && broadcastSessionMaxResponsesNb != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, broadcastSessionMaxResponsesNb);

 				methodID = (*env)->GetMethodID( env, classID, "setReceptionMode", "(Lcom/coronis/sdk/protocol/api/ConstantParams$ReceptionMode;)V");
 				if ( methodID != NULL && receptionMode != NULL ) {
 					LOG_DEBUG(g_Logger, "create_dynamic_expert_params: call setReceptionMode of dynamicExpertPram object", 0);
 					(*env)->CallVoidMethod( env, params, methodID, receptionMode);
 				}
 			}
 		}
 		(*env)->DeleteLocalRef( env, classID );
 	}

 	if ( transmissionPowerLevel != NULL )
 		(*env)->DeleteLocalRef( env, transmissionPowerLevel );
 	if ( broadcastSessionMaxResponsesNb != NULL )
 		(*env)->DeleteLocalRef( env, broadcastSessionMaxResponsesNb );
 	if ( broadcastSessionTimeout != NULL )
 		(*env)->DeleteLocalRef( env, broadcastSessionTimeout );
 	if ( responseTimeout != NULL )
 		(*env)->DeleteLocalRef( env, responseTimeout );
 	if ( transmissionRadioChannel != NULL )
 		(*env)->DeleteLocalRef( env, transmissionRadioChannel );
 	if ( wakeupLength != NULL )
 		(*env)->DeleteLocalRef( env, wakeupLength );
 	if ( receptionMode != NULL )
 		(*env)->DeleteLocalRef( env, receptionMode );
 	return params;
 }

 jobject create_static_standard_params( JNIEnv *env, StaticStandardParams* staticStandardParams ) {
 	jobject params = NULL;
 	jobject radioAckActivation = NULL;
 	jobject firstScanActivityPeriod = NULL;
 	jobject switchingModeOnRadioAddressActivation = NULL;
 	jobject receptionGroupId = NULL;
 	jobject firstScanEnableActivation = NULL;

 	if ( staticStandardParams->isSet & FLAG_RADIO_ACK_ACTIVATION )
 		radioAckActivation = create_boolean_object( env, staticStandardParams->radioAckActivation);
 	if ( staticStandardParams->isSet & FLAG_ACTIVITY_PERIOD_ON_FIRST_SCAN )
 		firstScanActivityPeriod = create_int_object( env, staticStandardParams->firstScanActivityPeriod);
 	if ( staticStandardParams->isSet & FLAG_SWITCHING_MODE_ON_RADIO_ADDRESS_ACTIVATION )
 		switchingModeOnRadioAddressActivation = create_boolean_object( env, staticStandardParams->switchingModeOnRadioAddressActivation);
 	if ( staticStandardParams->isSet & FLAG_RECEPTION_GROUPID )
 		receptionGroupId = create_int_object( env, staticStandardParams->receptionGroupId);
 	if ( staticStandardParams->isSet & FLAG_FIRST_SCAN_ENABLE_ACTIVATION )
 		firstScanEnableActivation = create_boolean_object( env, staticStandardParams->firstScanEnableActivation);

 	jclass classID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/StaticStandardParams");
 	if ( classID != NULL ) {
 		jmethodID methodID = (*env)->GetMethodID( env, classID, "<init>", "()V");
 		if ( methodID != NULL ) {
 			params = (*env)->NewObject( env, classID, methodID );
 			if ( params != NULL ) {

 				methodID = (*env)->GetMethodID( env, classID, "setRadioAckActivation", "(Ljava/lang/Boolean;)V");
 				if ( methodID != NULL && radioAckActivation != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, radioAckActivation);

 				methodID = (*env)->GetMethodID( env, classID, "setFirstScanActivityPeriod", "(Ljava/lang/Integer;)V");
 				if ( methodID != NULL && firstScanActivityPeriod != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, firstScanActivityPeriod);

 				methodID = (*env)->GetMethodID( env, classID, "setSwitchingModeOnRadioAddressActivation", "(Ljava/lang/Boolean;)V");
 				if ( methodID != NULL && switchingModeOnRadioAddressActivation != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, switchingModeOnRadioAddressActivation);

 				methodID = (*env)->GetMethodID( env, classID, "setReceptionGroupId", "(Ljava/lang/Integer;)V");
 				if ( methodID != NULL && receptionGroupId != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, receptionGroupId);

 				methodID = (*env)->GetMethodID( env, classID, "setFirstScanEnableActivation", "(Ljava/lang/Boolean;)V");
 				if ( methodID != NULL && firstScanEnableActivation != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, firstScanEnableActivation);
 			}
 		}
 		(*env)->DeleteLocalRef( env, classID );
 	}

 	if ( radioAckActivation != NULL )
 		(*env)->DeleteLocalRef( env, radioAckActivation );
 	if ( firstScanActivityPeriod != NULL )
 		(*env)->DeleteLocalRef( env, firstScanActivityPeriod );
 	if ( switchingModeOnRadioAddressActivation != NULL )
 		(*env)->DeleteLocalRef( env, switchingModeOnRadioAddressActivation );
 	if ( receptionGroupId != NULL )
 		(*env)->DeleteLocalRef( env, receptionGroupId );
 	if ( firstScanEnableActivation != NULL )
 		(*env)->DeleteLocalRef( env, firstScanEnableActivation );

 	return params;
 }


 jobject create_static_expert_params( JNIEnv *env, StaticExpertParams* staticExpertParams ) {
 	jobject params = NULL;
 	jobject secondScanEnableActivation = NULL;
 	jobject radioCRCActivation = NULL;
 	jobject secondScanActivityPeriod = NULL;
 	jobject firstScanRadioChannel = NULL;
 	jobject firstScanPhysicalConfig = NULL;
 	jobject secondScanPhysicalConfig = NULL;
 	jobject complexScanEnableActivation = NULL;
 	jobject firstScanSensitivityReductionFactor = NULL;
 	jobject secondScanRadioChannel = NULL;
 	jobject secondScanSensitivityReductionFactor = NULL;

 	if ( staticExpertParams->isSet & FLAG_SECOND_SCAN_ENABLE_ACTIVATION )
 		secondScanEnableActivation = create_boolean_object( env, staticExpertParams->secondScanEnableActivation);
 	if ( staticExpertParams->isSet & FLAG_RADIO_CRC_ACTIVATION )
 		radioCRCActivation = create_boolean_object( env, staticExpertParams->radioCRCActivation);
 	if ( staticExpertParams->isSet & FLAG_ACTIVITY_PERIOD_ON_SECOND_SCAN )
 		secondScanActivityPeriod = create_int_object( env, staticExpertParams->secondScanActivityPeriod);
 	if ( staticExpertParams->isSet & FLAG_RADIO_CHANNEL_ON_FIRST_SCAN )
 		firstScanRadioChannel = create_radiochannel_enum_object( env, staticExpertParams->firstScanRadioChannel);

 	LOG_DEBUG(g_Logger, "create_static_expert_params: start conversion", 0);
 	if ( staticExpertParams->isSet & FLAG_PHYSICAL_CONFIG_ON_FIRST_SCAN )
 		firstScanPhysicalConfig = create_physical_layer_object( env, staticExpertParams->firstScanPhysicalConfig);
 	if ( staticExpertParams->isSet & FLAG_PHYSICAL_CONFIG_ON_SECOND_SCAN )
 		secondScanPhysicalConfig = create_physical_layer_object( env, staticExpertParams->secondScanPhysicalConfig);
 	if ( staticExpertParams->isSet & FLAG_COMPLEX_SCAN_ENABLE_ACTIVATION )
 		complexScanEnableActivation = create_boolean_object( env, staticExpertParams->complexScanEnableActivation );

 	LOG_DEBUG(g_Logger, "create_static_expert_params: analize FLAG_SENSITIVITY_REDUCTION_FACTOR_ON_FIRST_SCAN", 0);
 	if ( staticExpertParams->isSet & FLAG_SENSITIVITY_REDUCTION_FACTOR_ON_FIRST_SCAN ) {

 		LOG_DEBUG(g_Logger, "create_static_expert_params: Create in object for firstScanSensitivityReductionFactor", 0);
 		firstScanSensitivityReductionFactor = create_int_object( env, staticExpertParams->firstScanSensitivityReductionFactor);
 		if (firstScanSensitivityReductionFactor != NULL){
 			LOG_DEBUG(g_Logger, "create_static_expert_params: Object for firstScanSensitivityReductionFactor has been created", 0);
 		}
 	}

 	if ( staticExpertParams->isSet & FLAG_RADIO_CHANNEL_ON_SECOND_SCAN )
 		secondScanRadioChannel = create_radiochannel_enum_object( env, staticExpertParams->secondScanRadioChannel);
 	if ( staticExpertParams->isSet & FLAG_SENSITIVITY_REDUCTION_FACTOR_ON_SECOND_SCAN )
 		secondScanSensitivityReductionFactor = create_int_object( env, staticExpertParams->secondScanSensitivityReductionFactor);

 	jclass classID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/StaticExpertParams");
 	if ( classID != NULL ) {
 		jmethodID methodID = (*env)->GetMethodID( env, classID, "<init>", "()V");
 		if ( methodID != NULL ) {
 			params = (*env)->NewObject( env, classID, methodID );
 			if ( params != NULL ) {

 				methodID = (*env)->GetMethodID( env, classID, "setSecondScanEnableActivation", "(Ljava/lang/Boolean;)V");
 				if ( methodID != NULL && secondScanEnableActivation != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, secondScanEnableActivation);

 				methodID = (*env)->GetMethodID( env, classID, "setRadioCRCActivation", "(Ljava/lang/Boolean;)V");
 				if ( methodID != NULL && radioCRCActivation != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, radioCRCActivation);

 				methodID = (*env)->GetMethodID( env, classID, "setSecondScanActivityPeriod", "(Ljava/lang/Integer;)V");
 				if ( methodID != NULL && secondScanActivityPeriod != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, secondScanActivityPeriod);

 				methodID = (*env)->GetMethodID( env, classID, "setFirstScanRadioChannel", "(Lcom/coronis/sdk/protocol/api/ConstantParams$RadioChannel;)V");
 				if ( methodID != NULL && firstScanRadioChannel != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, firstScanRadioChannel);

 				methodID = (*env)->GetMethodID( env, classID, "setFirstScanPhysicalConfig", "(Lcom/coronis/sdk/protocol/api/ConstantParams$PhysicalLayer;)V");
 				if ( methodID != NULL && firstScanPhysicalConfig != NULL ){
 					LOG_DEBUG(g_Logger, "create_static_expert_params: Calling method setFirstScanPhysicalConfig", 0);
 					(*env)->CallVoidMethod( env, params, methodID, firstScanPhysicalConfig);
 				}

 				methodID = (*env)->GetMethodID( env, classID, "setSecondScanPhysicalConfig", "(Lcom/coronis/sdk/protocol/api/ConstantParams$PhysicalLayer;)V");
 				if ( methodID != NULL && secondScanPhysicalConfig != NULL ) {
 					LOG_DEBUG(g_Logger, "create_static_expert_params: Calling method setSecondScanPhysicalConfig", 0);
 					(*env)->CallVoidMethod( env, params, methodID, secondScanPhysicalConfig);
 				}

 				methodID = (*env)->GetMethodID( env, classID, "setComplexScanEnableActivation", "(Ljava/lang/Boolean;)V");
 				if ( methodID != NULL && complexScanEnableActivation != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, complexScanEnableActivation);

 				methodID = (*env)->GetMethodID( env, classID, "setFirstScanSensitivityReductionFactor", "(Ljava/lang/Integer;)V");
 				if ( methodID != NULL && firstScanSensitivityReductionFactor != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, firstScanSensitivityReductionFactor);

 				methodID = (*env)->GetMethodID( env, classID, "setSecondScanRadioChannel", "(Lcom/coronis/sdk/protocol/api/ConstantParams$RadioChannel;)V");
 				if ( methodID != NULL && secondScanRadioChannel != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, secondScanRadioChannel);

 				methodID = (*env)->GetMethodID( env, classID, "setSecondScanSensitivityReductionFactor", "(Ljava/lang/Integer;)V");
 				if ( methodID != NULL && secondScanSensitivityReductionFactor != NULL )
 					(*env)->CallVoidMethod( env, params, methodID, secondScanSensitivityReductionFactor);
 			}
 		}
 		(*env)->DeleteLocalRef( env, classID );
 	}

 	if ( secondScanEnableActivation != NULL )
 		(*env)->DeleteLocalRef( env, secondScanEnableActivation );
 	if ( radioCRCActivation != NULL )
 		(*env)->DeleteLocalRef( env, radioCRCActivation );
 	if ( secondScanActivityPeriod != NULL )
 		(*env)->DeleteLocalRef( env, secondScanActivityPeriod );
 	if ( firstScanRadioChannel != NULL )
 		(*env)->DeleteLocalRef( env, firstScanRadioChannel );
 	if ( firstScanPhysicalConfig != NULL )
 		(*env)->DeleteLocalRef( env, firstScanPhysicalConfig );
 	if ( secondScanPhysicalConfig != NULL )
 		(*env)->DeleteLocalRef( env, secondScanPhysicalConfig );
 	if ( complexScanEnableActivation != NULL )
 		(*env)->DeleteLocalRef( env, complexScanEnableActivation );
 	if ( firstScanSensitivityReductionFactor != NULL )
 		(*env)->DeleteLocalRef( env, firstScanSensitivityReductionFactor );
 	if ( secondScanRadioChannel != NULL )
 		(*env)->DeleteLocalRef( env, secondScanRadioChannel );
 	if ( secondScanSensitivityReductionFactor != NULL )
 		(*env)->DeleteLocalRef( env, secondScanSensitivityReductionFactor );

 	return params;
 }

 jobject create_local_node_service_command_object( JNIEnv *env,
		 jclass classPayload,
		 NodeServiceId serviceId,
		 const SerialFrame* payload ) {
 	jobject localNodeServiceCommand = NULL;
 	char data[6*2 + 1] = "";
 	sprintf( data, "%02X%02X%02X%02X%02X%02X",
 			payload->frame[0],
 			payload->frame[1],
 			payload->frame[2],
 			payload->frame[3],
 			payload->frame[4],
 			payload->frame[5]);

 	jobject jPayload = create_serial_frame_object( env, classPayload, data );
 	if ( jPayload != NULL ) {
 		jobject nodeServiceId = create_node_service_id( env, serviceId);
 		if ( nodeServiceId != NULL ) {
 			jclass classLocalNodeServiceCommandID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/LocalNodeServiceCommand");
 			if ( classLocalNodeServiceCommandID != NULL ) {
 				jmethodID methodID = (*env)->GetMethodID( env, classLocalNodeServiceCommandID, "<init>", "(Lcom/coronis/sdk/protocol/api/NodeServiceId;Lcom/coronis/sdk/protocol/api/IPayload;)V");// get constructor
 				if ( methodID != NULL ) {
 					localNodeServiceCommand = (*env)->NewObject( env, classLocalNodeServiceCommandID, methodID, nodeServiceId, jPayload);
 				}
 				(*env)->DeleteLocalRef( env, classLocalNodeServiceCommandID);
 			}
 			(*env)->DeleteLocalRef( env, nodeServiceId );
 		}
 		(*env)->DeleteLocalRef( env, jPayload );
 	}
 	return localNodeServiceCommand;
 }

 jobject create_local_node_service_command_from_payload_object( JNIEnv *env,
		 jclass classPayload,
		 NodeServiceId serviceId, const Payload* payload ) {
 	jobject localNodeServiceCommand = NULL;
 	char data[6*2 + 1] = "";
 	sprintf( data, "%02X%02X%02X%02X%02X%02X",
 			payload->frame[0],
 			payload->frame[1],
 			payload->frame[2],
 			payload->frame[3],
 			payload->frame[4],
 			payload->frame[5]);

 	jobject jPayload = create_serial_frame_object( env, classPayload, data );
 	if ( jPayload != NULL ) {
 		jobject nodeServiceId = create_node_service_id( env, serviceId);
 		if ( nodeServiceId != NULL ) {
 			jclass classLocalNodeServiceCommandID = (*env)->FindClass( env, "com/coronis/sdk/protocol/api/LocalNodeServiceCommand");
 			if ( classLocalNodeServiceCommandID != NULL ) {
 				jmethodID methodID = (*env)->GetMethodID( env, classLocalNodeServiceCommandID, "<init>", "(Lcom/coronis/sdk/protocol/api/NodeServiceId;Lcom/coronis/sdk/protocol/api/IPayload;)V");// get constructor
 				if ( methodID != NULL ) {
 					localNodeServiceCommand = (*env)->NewObject( env, classLocalNodeServiceCommandID, methodID, nodeServiceId, jPayload);
 				}
 				(*env)->DeleteLocalRef( env, classLocalNodeServiceCommandID);
 			}
 			(*env)->DeleteLocalRef( env, nodeServiceId );
 		}
 		(*env)->DeleteLocalRef( env, jPayload );
 	}
 	return localNodeServiceCommand;
 }

 jobject create_radio_exchange_command_object( JNIEnv *env,
		 jclass classPayload,
		 jclass classRadioExchangeCommand,
		 RadioExchangeCommand* reCommand,
 		jobject dynamicStandardParams, jobject dynamicExpertParams, WavePortResponses response) {

 	jobject radioExchangeCommand = NULL;
 	jobject dynamicStandardParamsLocal = NULL;
 	_bool dynamicStandardParamsLocalCreated = _false;
 	jobject dynamicExpertParamsLocal = NULL;

 	//jobject jSerialFrame = create_serial_frame_object( env, g_classSerialFrame,
 	//		get_hex_payload( (Payload*) &reCommand->command.payload));
 	jobject jPayload = NULL;
 	char payloadString[MAX_SERIALFRAME_LEN*2+1] = "";
 	const char *payload = get_hex_payload(payloadString, sizeof(payloadString), (Payload*)&reCommand->command.payload);

 	if (reCommand != NULL) {
 		LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: Received frame of length %d",
 				((Payload*)&reCommand->command.payload)->len);

 	}

 	if (strlen(payload) > 0) {
 		LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: Received payload %s", payload);
 		jPayload = create_payload_object( env, classPayload, payload );
 	} else
 		LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: Payload is not set", 0);

 	if (jPayload != NULL) {
 		LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: Payload jObject has been created!", 0);
 	}

 	LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: Entered : 0x%02x", response);

 	if ( classRadioExchangeCommand ) {
 		if (strlen(((RadioAddress*)&reCommand->dynamicStandardParams.nodeAddress)->addressValue) ) {

 			LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: Check if dynamicStandardParams should be created for response : 0x%02x",
 					response);

 			if (response == RECEIVED_FRAME || response == RECEIVED_FRAME_RELAYED
 					|| response == RECEIVED_BROADCAST_FRAME) {

 				LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: Create dynamicStandardParams", 0);
 				dynamicStandardParamsLocal = create_dynamic_standard_params_object( env, &reCommand->dynamicStandardParams, response );
 				dynamicStandardParamsLocalCreated = _true;
 			}
 			else if (dynamicStandardParams != NULL)
 				dynamicStandardParamsLocal = dynamicStandardParams;
 			else
 				dynamicStandardParamsLocal = NULL;
 		}

 		char data[150] = "";
 		sprintf( data, "(Lcom/coronis/sdk/protocol/api/IPayload;%s)V",
 				dynamicStandardParamsLocal != NULL ? "Lcom/coronis/sdk/protocol/api/DynamicStandardParams;" : "");

 		LOG_DEBUG(g_Logger, "create_radio_exchange_command_object:  prepare constructor params: %s ", data);

 		jmethodID methodID = (*env)->GetMethodID( env, classRadioExchangeCommand, "<init>", data);// get constructor
 		if ( methodID != NULL ) {
 			radioExchangeCommand = (*env)->NewObject( env, classRadioExchangeCommand, methodID, jPayload, dynamicStandardParamsLocal);
 		}

 		if (radioExchangeCommand) {
 			LOG_DEBUG(g_Logger, "create_radio_exchange_command_object:  radioExchangeCommand has been created", 0);

 		}
 //		if ( jSerialFrame && radioExchangeCommand ) {
 //			jmethodID methodID = (*env)->GetMethodID( env, g_classRadioExchangeCommand, "setSerialFrame", "(Lcom/coronis/sdk/serialdriver/api/ISerialFrame;)V");
 //			if ( methodID != NULL )
 //				(*env)->CallVoidMethod( env, radioExchangeCommand, methodID, jSerialFrame );
 //		}
 	}

 	(*env)->DeleteLocalRef( env, jPayload );

 	//if ( jSerialFrame )
 	//	(*env)->DeleteLocalRef( env, jSerialFrame );

 	if ( dynamicStandardParamsLocal != NULL && dynamicStandardParamsLocalCreated )
 		(*env)->DeleteLocalRef( env, dynamicStandardParamsLocal );

 	//--------------------------------------------------------------------------
  	if (reCommand->dynamicExpertParams.isSet & FLAG_RECEPTION_MODE /*&& dynamicExpertParams == NULL*/) {
 		LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: dynamicExpertParams is NOT set, create one to pass", 0);
 		dynamicExpertParamsLocal = create_dynamic_expert_params(env, &reCommand->dynamicExpertParams);
 	}

 	//--------------------------------------------------------------------------
 	// update reception mode for command in dynamic expert params
 	if (dynamicExpertParamsLocal != NULL && radioExchangeCommand != NULL) {

 		LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: Set the reception mode to command response", 0);
 		jmethodID methodID = (*env)->GetMethodID( env, classRadioExchangeCommand, "setDynamicExpertParams",
 				"(Lcom/coronis/sdk/protocol/api/DynamicExpertParams;)V");

 		if ( methodID != NULL ) {
 			LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: set dynamic expert parameter value", 0);
 			(*env)->CallVoidMethod( env, radioExchangeCommand, methodID, dynamicExpertParamsLocal);
 			LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: setDynamicExpertParams method has been called", 0);
 		}
 	}

 	if (dynamicExpertParamsLocal)
 		(*env)->DeleteLocalRef( env, dynamicExpertParamsLocal );

 	LOG_DEBUG(g_Logger, "create_radio_exchange_command_object: Exit", 0);
 	return radioExchangeCommand;
 }

 _bool get_int_field_from_object( JNIEnv *env, jobject object, jclass classID, const char* fieldName, jint* value ) {
 	_bool bRet = _false;
 	/************************* START handling of object->fieldName *************************/
 	jfieldID fieldID = (*env)->GetFieldID( env, classID, fieldName, "Ljava/lang/Integer;");
 	if ( fieldID != NULL ) {
 		jobject fieldValue = (*env)->GetObjectField( env, object, fieldID );
 		if ( fieldValue != NULL ) {
 			jclass classIntegerID = (*env)->GetObjectClass( env, fieldValue);
 			if ( classIntegerID != NULL ) {
 				jmethodID methodID = (*env)->GetMethodID( env, classIntegerID, "intValue", "()I");
 				if ( methodID != NULL ) {
 					*value = (*env)->CallIntMethod( env, fieldValue, methodID );
 					bRet = _true;
 					LOG_DEBUG( g_Logger, "get_int_field_from_object->%s: %d", fieldName, *value);
 				}
 				(*env)->DeleteLocalRef( env, classIntegerID);
 			}
 			(*env)->DeleteLocalRef( env, fieldValue);
 		}
 	}/************************* END handling of object->fieldName *************************/
 	return bRet;
 }


 _bool get_float_field_from_object( JNIEnv *env, jobject object, jclass classID, const char* fieldName, jfloat* value ) {
 	_bool bRet = _false;

 	/************************* START handling of object->fieldName *************************/
 	LOG_DEBUG( g_Logger, "get_float_field_from_object->%s started", fieldName);
 	jfieldID fieldID = (*env)->GetFieldID( env, classID, fieldName, "Ljava/lang/Float;");

 	if ( fieldID != NULL ) {
 		LOG_DEBUG( g_Logger, "get_float_field_from_object: Got field ID", 0);
 		jobject fieldValue = (*env)->GetObjectField( env, object, fieldID );
 		if ( fieldValue != NULL ) {
 			LOG_DEBUG( g_Logger, "get_float_field_from_object: Got field Value", 0);
 			jclass classFloatID = (*env)->GetObjectClass( env, fieldValue);
 			if ( classFloatID != NULL ) {
 				LOG_DEBUG( g_Logger, "get_float_field_from_object: Got class float ID", 0);
 				jmethodID methodID = (*env)->GetMethodID( env, classFloatID, "floatValue", "()F");
 				if ( methodID != NULL ) {
 					LOG_DEBUG( g_Logger, "get_float_field_from_object: Got method ID", 0);
 					*value = (*env)->CallFloatMethod( env, fieldValue, methodID );
 					bRet = _true;
 					LOG_DEBUG( g_Logger, "get_float_field_from_object->%s: %f", fieldName, *value);
 				}
 				(*env)->DeleteLocalRef( env, classFloatID);
 			}
 			(*env)->DeleteLocalRef( env, fieldValue);
 		}
 	}

 	/************************* END handling of object->fieldName *************************/
 	return bRet;
 }

 _bool get_boolean_field_from_object( JNIEnv *env, jobject object, jclass classID, const char* fieldName, jboolean* value ) {
 	_bool bRet = _false;
 	/************************* START handling of object->fieldName *************************/
 	jfieldID fieldID = (*env)->GetFieldID( env, classID, fieldName, "Ljava/lang/Boolean;");
 	if ( fieldID != NULL ) {
 		jobject fieldValue = (*env)->GetObjectField( env, object, fieldID );
 		if ( fieldValue != NULL ) {
 			jclass classIntegerID = (*env)->GetObjectClass( env, fieldValue);
 			if ( classIntegerID != NULL ) {
 				jmethodID methodID = (*env)->GetMethodID( env, classIntegerID, "booleanValue", "()Z");
 				if ( methodID != NULL ) {
 					*value = (*env)->CallBooleanMethod( env, fieldValue, methodID );
 					bRet = _true;
 					LOG_DEBUG( g_Logger, "get_boolean_field_from_object->%s: %d", fieldName, *value);
 				}
 				(*env)->DeleteLocalRef( env, classIntegerID);
 			}
 			(*env)->DeleteLocalRef( env, fieldValue);
 		}
 	}/************************* END handling of object->fieldName *************************/
 	return bRet;
 }

 NodeServiceId get_node_service_id( JNIEnv *env, jclass classID, jobject command ) {
 	NodeServiceId nodeServiceId = NODE_SERVICE_ID_UNDEFINED;
 	/*+++++++++++++++++++++++++++++++++++++++ START handling of type +++++++++++++++++++++++++++++++++++++++*/
 	jfieldID fieldID = (*env)->GetFieldID( env, classID, "serviceId", "Lcom/coronis/sdk/protocol/api/NodeServiceId;");
 	if ( fieldID != NULL ) {
 		jobject type = (*env)->GetObjectField( env, command, fieldID );
 		if ( type != NULL ) {
 			jclass classCommandTypeID = (*env)->GetObjectClass( env, type);
 			if ( classCommandTypeID != NULL ) {
 				jmethodID methodID = (*env)->GetMethodID( env, classCommandTypeID, "ordinal", "()I");
 				if ( methodID != NULL ) {
 					nodeServiceId = (*env)->CallIntMethod( env, type, methodID );
 				}
 				(*env)->DeleteLocalRef( env, classCommandTypeID);
 			}
 			(*env)->DeleteLocalRef( env, type);
 		}
 	}
 	/*+++++++++++++++++++++++++++++++++++++++ END handling of type +++++++++++++++++++++++++++++++++++++++*/
 	return nodeServiceId;
 }

 int get_dynamic_standard_params( JNIEnv *env, jclass classID, jobject command, DynamicStandardParams* dynamicStandardParams ) {

 	_bool atLeastOneParamIsValid = _false;
 	int iRet = OK_PROTOCOL_SUCCESS;
 	memset( dynamicStandardParams, 0, sizeof(*dynamicStandardParams));
 	/*+++++++++++++++++++++++++++++++++++++++ START handling of dynamicStandardParams +++++++++++++++++++++++++++++++++++++++*/

 	LOG_DEBUG(g_Logger, "get_dynamic_standard_params: Started.", 0);
 	jfieldID fieldID = (*env)->GetFieldID( env, classID, "dynamicStandardParams", "Lcom/coronis/sdk/protocol/api/DynamicStandardParams;");
 	if ( fieldID != NULL ) {
 		LOG_DEBUG(g_Logger, "get_dynamic_standard_params: Got class ID.", 0);
 		jobject jDynamicStandardParams = (*env)->GetObjectField( env, command, fieldID );
 		if ( jDynamicStandardParams != NULL ) {
 			LOG_DEBUG(g_Logger, "get_dynamic_standard_params: Got class object from command", 0);
 			jclass classDynamicStandardParamsID = (*env)->GetObjectClass( env, jDynamicStandardParams);
 			if ( classDynamicStandardParamsID != NULL ) {
 				LOG_DEBUG(g_Logger, "get_dynamic_standard_params: Got class ID from command", 0);
 				/************************* START handling of dynamicStandardParams->nodeAddress *************************/
 				iRet = KO_NODE_ADDRESS_REQUIRED;
 				fieldID = (*env)->GetFieldID( env, classDynamicStandardParamsID, "nodeAddress", "Lcom/coronis/sdk/protocol/api/IRadioAddress;");
 				if ( fieldID != NULL ) {
 					LOG_DEBUG(g_Logger, "get_dynamic_standard_params: Got node address.", 0);
 					jobject nodeAddress = (*env)->GetObjectField( env, jDynamicStandardParams, fieldID );
 					if ( nodeAddress != NULL ) {
 						jclass classIRadioAddressID = (*env)->GetObjectClass( env, nodeAddress);
 						if ( classIRadioAddressID != NULL ) {
 							jmethodID methodID = (*env)->GetMethodID( env, classIRadioAddressID, "toString", "()Ljava/lang/String;");
 							if ( methodID != NULL ) {
 								jobject jstr = (*env)->CallObjectMethod( env, nodeAddress, methodID );
 								if ( jstr != NULL ) {
 									const char* str = (*env)->GetStringUTFChars( env, jstr, NULL);
 									if ( str != NULL) {
 										iRet = KO_PARAMETER_NODE_ADDRESS_NOT_VALID;
 										if ( strlen(str) == (MAX_RADIOADDRESS_LEN - 1) ) {
 											if ( is_hex( str ) ) {
 												strcpy( ((RadioAddress*)&dynamicStandardParams->nodeAddress)->addressValue, str );
 												iRet = OK_PROTOCOL_SUCCESS;
 												atLeastOneParamIsValid = _true;
 											}
 										}
 										LOG_DEBUG( g_Logger, "get_dynamic_standard_params nodeAddress: %s", str);
 										(*env)->ReleaseStringUTFChars( env, jstr, str);
 									}
 									(*env)->DeleteLocalRef( env, jstr);
 								}
 							}
 							(*env)->DeleteLocalRef( env, classIRadioAddressID);
 						}

 						(*env)->DeleteLocalRef( env, nodeAddress);
 					}
 				}/************************* END handling of dynamicStandardParams->nodeAddress *************************/

 				/************************* START handling of dynamicStandardParams->routerPath *************************/
 				fieldID = (*env)->GetFieldID( env, classDynamicStandardParamsID, "routerPath", "[Lcom/coronis/sdk/protocol/api/IRadioAddress;");
 				if ( ( iRet == OK_PROTOCOL_SUCCESS ) && fieldID != NULL ) {
 					jobjectArray routerPath = (*env)->GetObjectField( env, jDynamicStandardParams, fieldID );
 					if ( routerPath != NULL ) {
 						jint len = min( ARRAYSIZE(dynamicStandardParams->routerPath), (*env)->GetArrayLength( env, routerPath) );
 						if ( len > 0 ) {
 							jint i = 0;
 							for( ; i < len; ++i ) {
 								jobject nodeAddress = (*env)->GetObjectArrayElement( env, routerPath, i);
 								if ( nodeAddress != NULL ) {
 									jclass classIRadioAddressID = (*env)->GetObjectClass( env, nodeAddress);
 									if ( classIRadioAddressID != NULL ) {
 										jmethodID methodID = (*env)->GetMethodID( env, classIRadioAddressID, "toString", "()Ljava/lang/String;");
 										if ( methodID != NULL ) {
 											jobject jstr = (*env)->CallObjectMethod( env, nodeAddress, methodID );
 											if ( jstr != NULL ) {
 												const char* str = (*env)->GetStringUTFChars( env, jstr, NULL);
 												if ( str != NULL) {
 													iRet = KO_PARAMETER_RELAY_ROUTE_NOT_VALID;
 													if ( strlen(str) == (MAX_RADIOADDRESS_LEN-1) ) {
 														if ( is_hex( str ) ) {
 															strcpy( ((RadioAddress*)&dynamicStandardParams->routerPath[i])->addressValue, str );
 															iRet = OK_PROTOCOL_SUCCESS;
 															atLeastOneParamIsValid = _true;
 														}
 													}
 													LOG_DEBUG( g_Logger, "get_dynamic_standard_params routerPath[%d]: %s", i, str);
 													(*env)->ReleaseStringUTFChars( env, jstr, str);
 												}
 												(*env)->DeleteLocalRef( env, jstr);
 											}
 										}
 										(*env)->DeleteLocalRef( env, classIRadioAddressID);
 									}
 									(*env)->DeleteLocalRef( env, nodeAddress);
 								}
 								if ( iRet != OK_PROTOCOL_SUCCESS )
 									break;
 							}
 						}
 						(*env)->DeleteLocalRef( env, routerPath);
 					}
 				}/************************* END handling of dynamicStandardParams->routerPath *************************/

 				/************************* START handling of dynamicStandardParams->groupId *************************/
 				jint nGroupId = 0;
 				dynamicStandardParams->groupId = -1;
 				if ( ( iRet == OK_PROTOCOL_SUCCESS ) && get_int_field_from_object( env, jDynamicStandardParams, classDynamicStandardParamsID, "groupId", &nGroupId) ) {
 					if ( nGroupId < 0 || nGroupId > 0xFF )
 						iRet = KO_PARAMETER_GROUP_NUMBER_NOT_VALID;
 					else {
 						dynamicStandardParams->groupId = nGroupId;
 						atLeastOneParamIsValid = _true;
 					}
 				}
 				/************************* END handling of dynamicStandardParams->groupId *************************/

 				(*env)->DeleteLocalRef( env, classDynamicStandardParamsID);
 			}

 			(*env)->DeleteLocalRef( env, jDynamicStandardParams);
 		}
 	}

 	/*+++++++++++++++++++++++++++++++++++++++ END handling of dynamicStandardParams +++++++++++++++++++++++++++++++++++++++*/
 	if ((iRet == OK_PROTOCOL_SUCCESS) && (atLeastOneParamIsValid == _false) ) {
 		LOG_DEBUG(g_Logger, "get_dynamic_standard_params: there is not dynamic parameter specified", 0);
 		iRet = KO_PROTOCOL_NO_VALID_PARAMETER;
 	}

 	return iRet;
 }

 int get_dynamic_expert_params( JNIEnv *env, jclass classID, jobject command, DynamicExpertParams* dynamicExpertParams ) {

 	int iRet = OK_PROTOCOL_SUCCESS;
 	memset( dynamicExpertParams, 0, sizeof(*dynamicExpertParams));

 	LOG_DEBUG(g_Logger," get_dynamic_expert_params: Entering method", 0);

 	/*+++++++++++++++++++++++++++++++++++++++ START handling of dynamicExpertParams +++++++++++++++++++++++++++++++++++++++*/
 	jfieldID fieldID = (*env)->GetFieldID( env, classID, "dynamicExpertParams", "Lcom/coronis/sdk/protocol/api/DynamicExpertParams;");
 	if ( fieldID != NULL ) {
 		jobject jDynamicExpertParams = (*env)->GetObjectField( env, command, fieldID );
 		if ( jDynamicExpertParams != NULL ) {
 			jclass classDynamicExpertParamsID = (*env)->GetObjectClass( env, jDynamicExpertParams);
 			if ( classDynamicExpertParamsID != NULL ) {

 				/************************* START handling of dynamicExpertParams->wakeupLenght *************************/
 				LOG_DEBUG(g_Logger," get_dynamic_expert_params: Start handling of dynamicExpertParams wakeupLength (status %d)", iRet);

 				jint nWakeupLength = 0;
 				if ( ( iRet == OK_PROTOCOL_SUCCESS ) && get_int_field_from_object( env, jDynamicExpertParams, classDynamicExpertParamsID, "wakeupLength", &nWakeupLength) ) {
 					if ( nWakeupLength < WAKEUP_LENGTH_MIN || nWakeupLength > WAKEUP_LENGTH_MAX )
 						iRet = KO_PARAMETER_WAKEUP_LENGTH_NOT_VALID;
 					else {
 						dynamicExpertParams->wakeupLength = nWakeupLength;
 						dynamicExpertParams->isSet |= FLAG_WAKEUP_LENGTH;
 					}
 				}

 				LOG_DEBUG(g_Logger," get_dynamic_expert_params: End handling of dynamicExpertParams wakeupLength with code: %d", iRet);
 				/************************* END handling of dynamicExpertParams->wakeupLenght *************************/

 				/************************* START handling of dynamicExpertParams->responseTimeout *************************/
 				jint responseTimeout = 0;
 				if ( ( iRet == OK_PROTOCOL_SUCCESS ) && get_int_field_from_object( env, jDynamicExpertParams, classDynamicExpertParamsID, "responseTimeout", &responseTimeout) ) {
 					if ((responseTimeout < RADIO_USER_TIMEOUT_MIN) || (responseTimeout > RADIO_USER_TIMEOUT_MAX))
 						iRet = KO_PARAMETER_RESPONSE_TIMEOUT_NOT_VALID;
 					else if ((responseTimeout % 100) != 0)
 						iRet = KO_PARAMETER_RESPONSE_TIMEOUT_VALUE_NOT_SUPPORTED;
 					else {
 						dynamicExpertParams->responseTimeout = responseTimeout;
 						dynamicExpertParams->isSet |= FLAG_RESPONSE_TIMEOUT;
 					}
 				}
 				/************************* END handling of dynamicExpertParams->responseTimeout *************************/

 				/************************* START handling of dynamicExpertParams->broadcastSessionTimeout *************************/
 				jint broadcastSessionTimeout = 0;
 				if ( ( iRet == OK_PROTOCOL_SUCCESS ) && get_int_field_from_object( env, jDynamicExpertParams, classDynamicExpertParamsID, "broadcastSessionTimeout", &broadcastSessionTimeout) ) {
 					if ( broadcastSessionTimeout < BCST_RECEPTION_TIMEOUT_MIN ||
 							broadcastSessionTimeout > BCST_RECEPTION_TIMEOUT_MAX ||
 							((broadcastSessionTimeout % 100) != 0) )
 						iRet = KO_PARAMETER_BROADCAST_SESSION_TIMEOUT_NOT_VALID;
 					else {
 						dynamicExpertParams->broadcastSessionTimeout = broadcastSessionTimeout;
 						dynamicExpertParams->isSet |= FLAG_BROADCAST_SESSION_TIMEOUT;
 					}
 				}
 				/************************* END handling of dynamicExpertParams->broadcastSessionTimeout *************************/

 				/************************* START handling of dynamicExpertParams->broadcastSessionMaxResponsesNb *************************/
 				jint broadcastSessionMaxResponsesNb = 0;
 				if ( ( iRet == OK_PROTOCOL_SUCCESS ) && get_int_field_from_object( env, jDynamicExpertParams, classDynamicExpertParamsID, "broadcastSessionMaxResponsesNb", &broadcastSessionMaxResponsesNb) ) {
 					if ( broadcastSessionMaxResponsesNb <= 0 ||
 							broadcastSessionMaxResponsesNb >= 0xFF )
 						iRet = KO_PARAMETER_BROADCAST_SESSION_MAX_RESPONSES_NB_NOT_VALID;
 					else {
 						dynamicExpertParams->broadcastSessionMaxResponsesNb = broadcastSessionMaxResponsesNb;
 						dynamicExpertParams->isSet |= FLAG_BROADCAST_SESSION_MAX_RESPONSES_NB;
 					}
 				}
 				/************************* END handling of dynamicExpertParams->broadcastSessionMaxResponsesNb *************************/

 				/************************* START handling of dynamicExpertParams->transmissionRadioChannel *************************/
 				RadioChannel transmissionRadioChannel = 0;
 				if ( ( iRet == OK_PROTOCOL_SUCCESS ) && get_radio_channel_enum_field_from_object( env, jDynamicExpertParams, classDynamicExpertParamsID, "transmissionRadioChannel",
 						&transmissionRadioChannel, &iRet, KO_PARAMETER_TRANSMISSION_RADIO_CHANNEL_NOT_VALID) ) {
 						dynamicExpertParams->transmissionRadioChannel = transmissionRadioChannel;
 						dynamicExpertParams->isSet |= FLAG_TRANSMISSION_RADIO_CHANNEL;
 				}
 				/************************* END handling of dynamicExpertParams->transmissionRadioChannel *************************/

 				/************************* START handling of dynamicExpertParams->transmissionPowerLevel *************************/
 				jfloat transmissionPowerLevel = 0;
 				if ( ( iRet == OK_PROTOCOL_SUCCESS ) && get_float_field_from_object( env, jDynamicExpertParams, classDynamicExpertParamsID, "transmissionPowerLevel", &transmissionPowerLevel) ) {
 					dynamicExpertParams->transmissionPowerLevel = transmissionPowerLevel;
 					dynamicExpertParams->isSet |= FLAG_TRANSMISSION_POWER_LEVEL;

 				}
 				/************************* END handling of dynamicExpertParams->transmissionPowerLevel *************************/

 				/************************* START handling of dynamicExpertParams->receptionMode *************************/
 				LOG_DEBUG(g_Logger," get_dynamic_expert_params: START handling of dynamicExpertParams->receptionMode", 0);
 				ReceptionMode receptionMode = 0;
 				if ( ( iRet == OK_PROTOCOL_SUCCESS ) && get_reception_mode_enum_field_from_object( env, jDynamicExpertParams, classDynamicExpertParamsID, &receptionMode, &iRet) ) {
 						dynamicExpertParams->receptionMode = receptionMode;
 						dynamicExpertParams->isSet |= FLAG_RECEPTION_MODE;
 						LOG_ERROR(g_Logger," get_dynamic_expert_params: Got dynamicExpertParams->receptionMode = %d", dynamicExpertParams->receptionMode);
 				}

 				LOG_DEBUG(g_Logger," get_dynamic_expert_params: END handling of dynamicExpertParams->receptionMode", 0);

 				/************************* END handling of dynamicExpertParams->receptionMode *************************/

 				(*env)->DeleteLocalRef( env, classDynamicExpertParamsID);
 			}

 			(*env)->DeleteLocalRef( env, jDynamicExpertParams);
 		}
 	}
 	/*+++++++++++++++++++++++++++++++++++++++ END handling of dynamicExpertParams +++++++++++++++++++++++++++++++++++++++*/

 	LOG_DEBUG(g_Logger," get_dynamic_expert_params: END Exiting with ret code %d", iRet);
 	return iRet;
 }

 _bool is_command_response_ok( JNIEnv *env, jobject commandResponse, _bool raiseException ) {
 	_bool bRet = _false;
 	jclass classID = (*env)->GetObjectClass( env, commandResponse);
 	if ( classID != NULL ) {
 		jmethodID methodID = (*env)->GetMethodID( env, classID, "toString", "()Ljava/lang/String;");
 		if ( methodID != NULL ) {
 			jstring jstr = (*env)->CallObjectMethod( env, commandResponse, methodID );
 			if ( jstr != NULL ) {
 				const char* str = (*env)->GetStringUTFChars( env, jstr, NULL);
 				if ( str != NULL) {
 					LOG_DEBUG( g_Logger, "is_command_response_ok: %s", str);
 					bRet = strcmp( "OK;", str) == 0 ? _true : _false;
 					if ( raiseException && !bRet )
 						throw_exception_message( env, str, 0 );
 					(*env)->ReleaseStringUTFChars( env, jstr, str);
 				}
 				(*env)->DeleteLocalRef( env, jstr );
 			}
 		}
 		(*env)->DeleteLocalRef( env, classID );
 	}
 	return bRet;
 }

 ResponseStatus get_response_status( JNIEnv *env, jobject commandResponse ) {
 	ResponseStatus responseStatus = KO_UNEXPECTED_ERROR;
 	jclass classID = (*env)->GetObjectClass( env, commandResponse);
 	if ( classID != NULL ) {
 		jmethodID methodID = (*env)->GetMethodID( env, classID, "getStatus", "()Lcom/coronis/sdk/protocol/api/ResponseStatus;");
 		if ( methodID != NULL ) {
 			jobject jResponseStatus = (*env)->CallObjectMethod( env, commandResponse, methodID );
 			if ( jResponseStatus != NULL ) {
 				jclass classResponseStatusID = (*env)->GetObjectClass( env, jResponseStatus);
 				if ( classResponseStatusID != NULL ) {
 					methodID = (*env)->GetMethodID( env, classResponseStatusID, "ordinal", "()I");
 					if ( methodID != NULL )
 						responseStatus = (*env)->CallIntMethod( env, jResponseStatus, methodID );
 					(*env)->DeleteLocalRef( env, classResponseStatusID);
 				}
 				(*env)->DeleteLocalRef( env, jResponseStatus );
 			}
 		}
 		(*env)->DeleteLocalRef( env, classID );
 	}
 	return responseStatus;
 }

#endif // #_JAVA_WRAPPER



