/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
#ifndef LOADER_PROTOCOL_H_
#define LOADER_PROTOCOL_H_

#include <generic_constants.h>
#include <loader.h>
#include <i_protocol_export_types.h>

/**
 * 	Protocol loaded API list that forms the protocol descriptor.
  */
typedef struct tagProtocolLoader {
	char  protocolId[MAX_PATH];
	void* protocolHandle;

	protocol_create_t                             create;
	protocol_destroy_t                            destroy;
	protocol_init_t                               init;
	protocol_init_from_file_t                     init_from_file;
	protocol_open_t                               open;
	protocol_close_t                              close;
	protocol_is_open_t                            is_open;
	protocol_get_serial_driver_id_t               get_serial_driver_id;
	protocol_set_serial_driver_id_t               set_serial_driver_id;
	protocol_set_serial_driver_instance_t         set_serial_driver_instance;
	protocol_add_message_listener_t               add_message_listener;
	protocol_remove_message_listener_t            remove_message_listener;
	protocol_add_spy_message_listener_t           add_spy_message_listener;
	protocol_remove_spy_message_listener_t        remove_spy_message_listener;
	protocol_send_transparent_message_t           send_transparent_message;
	protocol_send_application_message_t           send_application_message;
	protocol_send_application_broadcast_message_t send_application_broadcast_message;
	protocol_send_application_request_t           send_application_request;
	protocol_send_application_broadcast_request_t send_application_broadcast_request;
	protocol_send_application_response_t          send_application_response;
	protocol_write_radio_params_t                 write_radio_params;
	protocol_read_dynamic_expert_params_t         read_dynamic_expert_params;
	protocol_read_static_standard_params_t        read_static_standard_params;
	protocol_read_static_expert_params_t          read_static_expert_params;
	protocol_request_local_node_service_t         request_local_node_service;
	protocol_send_service_request_t               send_service_request;
	protocol_send_service_message_t               send_service_message;
	protocol_get_version_t          		      get_version;

} ProtocolLoader;

#ifdef __cplusplus
extern "C" {
#endif

/**
 * 	Release protocol library from memory and reset protocol descriptor
 * @param protocol - the protocol descriptor to be released.
 */
void free_protocol( ProtocolLoader* protocol );

/**
 * 	Load protocol API list from specified library. The loader performs
 * 	loading of exported library functions by name and assign them to structure
 * 	members <b>protocol</b> which are pointers to function types.
 * 	 The protocol is considered to be succesfully loaded if all exported
 * 	functions were loader.
 *
 * @param protocolName - the protocol name (without extension and library prefix)
 * @param protocol - a pointer to protocol descriptor.
 * @return - operation code <b>OK</b> if library succesfully loaded.
 */
int load_protocol( const char* protocolName, ProtocolLoader* protocol );

#ifdef __cplusplus
}
#endif

#endif /* LOADER_PROTOCOL_H_ */
