/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

// serial driver api include
#include <serial_driver_event.h>
#include <serial_driver_status.h>
// coronis domain includes
#include "loader_serial_driver.h"
#include "serial_driver_types.h"
#include "serial_driver_types_utils.h"
#include "utils.h"

SerialDriverLoader driverHandler;
void*              rs232DriverInstance;

#ifdef _WINDOWS
char portId[256] = "COM4";
#else
char portId[256] = "/dev/ttyUSB0";
#endif //_WINDOWS
SerialExchangeData serialExchangedData;

int receiveFrameListener( void* param, const pSerialExchangeData i_frame );
int serialDriverEventListener( void* param, int notifyCode );
const char* getCurrentTimeString();

int main(void) {
	printf( "%s csrs232driver example BEGIN. \n", getCurrentTimeString() );
    if ( OK == load_serial_driver( "csrs232driver", &driverHandler) )
    {
    	// 1. SerialDriver instantiation
        rs232DriverInstance = driverHandler.create();
        if ( rs232DriverInstance )
        {
            printf( "%s csrs232driver creation success. \n",
                getCurrentTimeString() );
            int status = OK_SERIALDRIVER_SUCCESS;

            // 2. SerialDriver initialization
            status = driverHandler.init( rs232DriverInstance );
            printf( "%s csrs232driver init status: %i \n",
                getCurrentTimeString(), status );
            driverHandler.set_port_id( rs232DriverInstance, portId);

            // 3. Registration of listener to receive spontenous frame
            driverHandler.add_listener( rs232DriverInstance,
                receiveFrameListener, serialDriverEventListener, 0 );

            // 4. Open the serial driver connection
            status = driverHandler.open( rs232DriverInstance );
            printf( "%s csrs232driver open status: %i \n",
                getCurrentTimeString(), status );

            // 5. Frame to send declaration (use the coronis serial frame declared in domain)
            serialExchangedData = create_serial_exchange_data ((SerialFrame){ {0x50, 0x05}, 2 });

            // 6. Send of frame
            status = driverHandler.send( rs232DriverInstance, &serialExchangedData );
            char hexframe[MAX_SERIALFRAME_LEN*2+1];
            printf( "%s csrs232driver send status for frame '%s': %s \n",
                getCurrentTimeString(),
                get_hex_serialexchangeddata(hexframe, sizeof(hexframe), &serialExchangedData),
                get_sending_status_name(status));

            // 7. Close the serial driver connection
            driverHandler.close( rs232DriverInstance );

            // 8. Serial driver instance suppression
            driverHandler.destroy( rs232DriverInstance );
        }
        free_serial_driver( &driverHandler );
    } else {
    	printf( "%s csrs232driver loading failure. \n", getCurrentTimeString() );
    }
    printf( "%s csrs232driver example END. \n", getCurrentTimeString() );
    return 0;
}

int receiveFrameListener( void* param, const pSerialExchangeData serialData )
{
    char hexframe[MAX_SERIALFRAME_LEN*2+1];
    printf( "%s receiveFrame: %s, \n",
    		getCurrentTimeString(),
    		get_hex_serialexchangeddata(hexframe, sizeof(hexframe), serialData) );
    return 0;
}

int serialDriverEventListener( void* param, int notifyCode )
{
    return 0;
}

const char* getCurrentTimeString()
{
	static char currTime[100] = "";
	currTime[0] = 0;

#ifdef _WINDOWS
	SYSTEMTIME st = {0};
	GetLocalTime(&st);
	sprintf( currTime, "[%02d:%02d:%02d:%03d]", st.wHour, st.wMinute, st.wSecond, st.wMilliseconds );
#else
	struct timeval currentTime;
	gettimeofday( &currentTime, 0 );
	struct tm *tm = localtime(&currentTime.tv_sec);
	sprintf( currTime, "[%02d:%02d:%02d:%03d] ", tm->tm_hour, tm->tm_min, tm->tm_sec, (int)currentTime.tv_usec / 1000 );
#endif

	return currTime;
}
