/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
#ifndef LOGGER_H_
#define LOGGER_H_

//#ifdef _WINDOWS
//#include <winsock2.h>
//#endif //_WINDOWS

#include <pthread.h>
#include <generic_constants.h>

#ifdef  _WINDOWS
#define CORONIS_API __declspec(dllexport)
#else
#define CORONIS_API extern
#endif//_WINDOWS

/**
 * 	Log level constants
 */
typedef enum tagLogLevel {
	LL_DEBUG,//!< LL_DEBUG
	LL_INFO, //!< LL_INFO
	LL_WARN, //!< LL_WARN
	LL_ERROR,//!< LL_ERROR
	LL_FATAL //!< LL_FATAL
} LogLevel;

/**
 * 	Log descriptor structure
 */
typedef struct tagLogInfo {
	char     logfile[MAX_PATH];
	int      fd;
	LogLevel level;
	unsigned long offset;
	unsigned long currentSize;
	unsigned long maxFileSize;

} LogInfo;

/**
 * 	maximum buffer size used to preformat log messages (because of variable arguments list)
 */
#define LOG_MAX_BUFFER_SIZE 5 * 1024


extern void* g_ModuleHandle;

/**
 * 	Logger configuration information. The structure is filled in with data
 * 	by property file loader.
 */
extern LogInfo linfo;

/**
 * 	Mutex used to ensure that logger is thread safe.
 */
extern pthread_mutex_t mutexLogUsage;

/**
 * 	Initialize logger( mutex, variables and associated structures).
 * 	Call initialization from properties file).
 */
CORONIS_API void init_log( );

/**
 * 	Load logger configuration from logger file. Loaded information is stored
 * 	in <b>linfo</b> structure.
 * @param propertyFile - property file name
 */
CORONIS_API void init_log_from_property_file( const char* propertyFile );

/**
 * 	Write debug messages to a file. Function is thread safe. A global buffer <b>logBuffer</b>
 * 	is used to format variable arguments list.
 */
CORONIS_API void debug( const char* format, ... );

/**
 * 	Write info messages to a file. Function is thread safe. A global buffer <b>logBuffer</b>
 * 	is used to format variable arguments list.
 */
CORONIS_API void info( const char* format, ... );

/**
 * 	Write warning messages to a file. Function is thread safe. A global buffer <b>logBuffer</b>
 * 	is used to format variable arguments list.
 */
CORONIS_API void warn( const char* format, ... );

/**
 * 	Write error messages to a file. Function is thread safe. A global buffer <b>logBuffer</b>
 * 	is used to format variable arguments list.
 */
CORONIS_API void error( const char* format, ... );

/**
 * 	Write fatal messages to a file. Function is thread safe. A global buffer <b>logBuffer</b>
 * 	is used to format variable arguments list.
 */
CORONIS_API void fatal( const char* format, ... );

#endif /* LOGGER_H_ */
