/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
#include <generic_types.h>

#include "property_reader.h"
#include "utils.h"

#include <stdlib.h>
#include <loader_logger.h>


int read_properties_file(void * moduleHandle, char *data_buffer, int data_buffer_size, const char *file_name) {
	int fReadCount = -1;

	// check parameters
	if (!data_buffer || !file_name || data_buffer_size <= 0) {
		return fReadCount; // invalid parameters
	}


	// init buffer
	memset(data_buffer, 0, data_buffer_size);

	// compose full file path
	char path[MAX_PATH+200] = "";

	if ( get_module_path( moduleHandle, path, ARRAYSIZE(path) ) )
#ifdef _WINDOWS
		SetDllDirectoryA( path );
#else
	strcpy( path, "./");
#endif

	// try to access file without path (from current folder), otherwise in the same
	/// folder as module

	if ( access( file_name, 0 ) == -1 )
		strncat( path, file_name, sizeof(path) - 1 );
	else
		strncpy( path, file_name, sizeof(path) - 1 );


	// open file
	int fd = open( file_name, O_RDONLY, 0);
	if ( fd < 0 ){
		return fReadCount;
	}

	// read properties in one single buffer
	fReadCount = read( fd, data_buffer, data_buffer_size );
	close( fd );

	if (fReadCount > 0) data_buffer[fReadCount -1] = 0;
	return fReadCount;
}

void get_property_value( const char* propertyString, const char* propertyName, const char* propertyDefaultValue, PropertyType type, void* param ) {
	const char* buff = get_property( propertyString, propertyName );
	if ( !buff || ( buff && (strlen(buff) == 0) ) ) {
		LOG_WARN( g_Logger, "Fail to read %s parameter from property file, it has been initialized with default value: %s", propertyName, propertyDefaultValue );
		buff = propertyDefaultValue;
	}
	switch( type ) {
	case PT_BOOL:
		*((_bool*)param) = strcmp( buff, "false" );
		break;
	case PT_HEX: {
		char *stopstring;
		*((unsigned int*)param) = (byte)strtol( buff, &stopstring, 16 );
	    }
	    break;
	case PT_UINT:
		*((unsigned int*)param) = atoi(buff);
		break;
	case PT_FLOAT: {
		char *stopstring;
		*((float*)param) = strtof(buff, &stopstring);
		}
		break;
	case PT_STRING:
	default:
		strcpy( (char*)param, buff );
		break;
	}
}

const char* get_property( const char* propertyString, const char* propertyName ) {
	static char property[MAX_PROPERTY_LENGTH] = "";
	memset( property, 0, sizeof(property) );
	if ( propertyString && propertyName	) {
		const char* propertyValue = strstr( propertyString, propertyName);
		// check if property is not commented ( # someproperty.PropertyName=somevalue )
		const char* buff = propertyValue;
		while ( buff && buff != propertyString && buff[0] != '\n' ) {
			if ( buff[0] == '#' ) {
				propertyValue = strstr( propertyValue + strlen(propertyName), propertyName);
				if ( propertyValue )
					buff = propertyValue;
				else
					break;// property was commented and no other property!
			}
			--buff;
		}
		if ( propertyValue ) {
			propertyValue += strlen(propertyName); // search for value
			propertyValue = strchr( propertyValue, '=');
			if ( propertyValue ) {
				++propertyValue;
				while( propertyValue[0] == ' ' ) // skip all space characters
					++propertyValue;
				strncpy( property, propertyValue, MAX_PROPERTY_LENGTH-1 );
				char* eol = strchr( property, '\n');
				if ( eol != NULL ) {
					if ( eol[-1] == '\r' )
						eol[-1] = 0;
					eol[0] = 0;
				}
				return property;
			}
		}
	}

	return 0;
}
