/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
// serial driver api include
#include <serial_driver_event.h>
#include <serial_driver_status.h>
// protocol api include
#include <protocol_command_types.h>
#include <protocol_response_types.h>
// domain include
#include "loader_serial_driver.h"
#include "loader_protocol.h"
#include "serial_driver_types.h"
#include "protocol_types.h"
#include "protocol_types_utils.h"

#include "utils.h"

SerialDriverLoader driverHandler;
ProtocolLoader protocolHandler;
void*              rs232DriverInstance;
void*              waveportInstance;

char driverId2[256] = "csrs232driver";
char protocolId2[256] = "cswaveportprotocol";

Payload payloadStackService_READ_RADIOADDRESS2 = (Payload){ {0x50, 0x05}, 2 };

int process_local_node_service2(char *portId);
void scan_for_radio_devices(char *portId);

char *get_port_id (int index) {
	static char port_buffer [100];
#ifdef _WINDOWS
	sprintf(port_buffer, "COM%d", index);
#else
	sprintf(port_buffer, "/dev/ttyUSB%d", index);
#endif // _WINDOWS
	return port_buffer;
}

int main2(void) {

	int i;
	int  cnt  = 0;
	char *detectedPortId = 0;

	printf("Detect radio devices on all connected waveports.\n");
	printf("--------------------------------\n");
    if ( OK != load_serial_driver( driverId2, &driverHandler) ||
    		OK != load_protocol( protocolId2, &protocolHandler)) {

    	printf("ERROR Failed to load driver or protocol.\n");
    	return -1;
    }
	// SerialDriver instantiation
	rs232DriverInstance = driverHandler.create();

	// Protocol instantiation
	waveportInstance = protocolHandler.create();
	if ( !rs232DriverInstance || !waveportInstance) {
    	printf("ERROR Failed to initialize  driver or protocol.\n");
    	return -2;
	}

	// detect port id
	for( i = 0; i < 20; i ++ ) {
		char * portId = get_port_id (i);
		// set port Id to use
		driverHandler.set_port_id( rs232DriverInstance, portId);
		SerialDriverStatus status = OK_SERIALDRIVER_SUCCESS;
		// Protocol initialization (use the define serialDriver)
		protocolHandler.set_serial_driver_instance(
				waveportInstance,
				driverHandler.get_serial_driver_instance(
						rs232DriverInstance));

		status = protocolHandler.init( waveportInstance );
		status = protocolHandler.open( waveportInstance );

		if(process_local_node_service2(portId) != 0)
		{
			detectedPortId = portId;
			// call blocking scan function
			scan_for_radio_devices(detectedPortId);
			SLEEP(6);
		}
		protocolHandler.close( waveportInstance );
	}

	protocolHandler.destroy( waveportInstance );


	printf( "\nDone, scanned on %d waveport(s) \n", cnt);
	free_protocol( &protocolHandler );

	return 0;
}

int process_local_node_service2(char *portId) {
	// Prepare the request to send (throw 1 relay)
	LocalNodeServiceCommand command = create_local_node_service_command(STACK_SERVICE, payloadStackService_READ_RADIOADDRESS2);
	LocalNodeServiceResponse rsp;

	memset( &rsp, 0, sizeof(rsp) );

	// Execute the application message
	protocolHandler.request_local_node_service( waveportInstance, &command, &rsp );
	char hexpayload[MAX_PAYLOAD_LEN*2+1];

	if (rsp.response.status == OK_PROTOCOL_SUCCESS) {
		printf( "Connected on %s: waveport id  %s;\n", portId,
						get_hex_payload(hexpayload, sizeof(hexpayload),
								(Payload*)&rsp.command.command.payload));
		return 1;
	}

	return 0;
}

int listener( void* param, const RadioExchangeCommand* receiveCommand ) {

	printf("Detected on port %s: remote device address %s", (char*) param,
			((RadioAddress*)&receiveCommand->dynamicStandardParams.nodeAddress)->addressValue);

	return 0;
}


int listener_event( void* param, int eventCode ) {
	return 0;
}

void scan_for_radio_devices(char *portId) {
	printf("Scanning on port %s\n", portId);

	//int index = 0;
	protocolHandler.add_message_listener( waveportInstance, listener, listener_event, portId );

	RadioExchangeCommand command;
	MulticastRequestResponse rsp;
	memset( &command, 0, sizeof(command) );
	memset( &rsp, 0, sizeof(rsp) );
	command.dynamicStandardParams.groupId = -1;
	((Payload*)&command.command.payload)->frame[0] = 0x20;
	((Payload*)&command.command.payload)->len = 1;
	protocolHandler.send_application_broadcast_request( waveportInstance, &command, &rsp );
}
