/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

/** @file i_serial_driver_listener_types.h
 * @brief This interface represents listeners of a i_serial_driver_service implementation
 * To be notified of event process by a SerialDriver instance,
 * program has to implement this interface and to registered to the SerialDriver instance.
 *
 * Note: For a SerialDriver instance, only one listener could be registered
 */
#ifndef I_SERIAL_DRIVER_LISTENER_TYPES_H_
#define I_SERIAL_DRIVER_LISTENER_TYPES_H_

#include <i_serial_frame.h>

#ifdef __cplusplus
extern "C" {  
#endif

/**
 * Called method when a serial frame is received on a SerialDriver instance.
 * @param param A user define data setted during the registration
 * @param frame The sent/received Serial Exchange Data structure pointer
 *
 * Note: For a SerialDriver instance, only one listener could be registered
 * If many listener are needed, serial_driver_spy_t interface must be used.
 */
typedef int (* serial_driver_listener_t)           ( void* param, const pSerialExchangeData frame );

/**
 * Called method when a serial driver event occurs on a SerialDriver instance.
 * @param param A user define data setted during the registration
 * @param notifyCode The occured serial_driver_event
 *        IGNORED_FRAME_RECEIVED, SERIALPORT_DISCONNECTION, etc.
 */
typedef int (* serial_driver_listener_notify_t)    ( void* param, int notifyCode );

/**
 * Called method when a valid serial frame has been received or
 * has been sent successfully
 * via a SerialDriver instance.
 * @param param A user define data setted during the registration
 * @param spyIndex The index of the Spy listener
 * @param frame The sent/received Serial Exchange Data structure pointer
 *
 * Note: These interface is not supposed to be used in a production context,
 * but is useful for Debug context.
 */
typedef int (* serial_driver_spy_t)                ( void* param, int spyIndex, const pSerialExchangeData frame );


#ifdef __cplusplus
}
#endif

#endif /* I_SERIAL_DRIVER_LISTENER_TYPES_H_ */
