/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

#include "serial_utils.h"

/****************************************************************************************************
 *                        HELPER SECTION - OTHER FUNCTIONS DEFINITION
 ****************************************************************************************************/

int driver_verify_baudrate( int baudRate ) {
	int iRet = KO_PARAM_BAUDRATE_NOT_VALID;
	switch (baudRate) {
	case CBR_4800:
	case CBR_9600:
	case CBR_14400:
	case CBR_19200:
	case CBR_38400:
	case CBR_57600:
	case CBR_115200:
		iRet = OK_SERIALDRIVER_SUCCESS;
		break;
	}
	return iRet;
}

int driver_verify_databit( int dataBit ) {
	int iRet = KO_PARAM_DATABIT_NOT_VALID;
	if ( dataBit == DATABITS_EIGHT )
		iRet = OK_SERIALDRIVER_SUCCESS;
	return iRet;
}

int driver_verify_stopbit( int stopBit ) {
	int iRet = KO_PARAM_STOPBIT_NOT_VALID;
	switch( stopBit ) {
	case ONESTOPBIT:
	case TWOSTOPBITS:
	iRet = OK_SERIALDRIVER_SUCCESS;
	break;
	}
	return iRet;
}

int driver_verify_parity( int parity ) {
	int iRet = KO_PARAM_PARITY_NOT_VALID;
	switch( parity ) {
	case NOPARITY:
	case ODDPARITY:
	case EVENPARITY:
	iRet = OK_SERIALDRIVER_SUCCESS;
	break;
	}
	return iRet;
}

int driver_verify_flowcontrolmode_in( int flowControlModeIn ) {
	int iRet = OK_SERIALDRIVER_SUCCESS;
	if ( flowControlModeIn != FLOWCONTROL_NONE ) {
		int mask = 0xFFFFFFF0;
		if ( mask & flowControlModeIn )
			iRet =  KO_PARAM_FLOWCONTROLIN_NOT_VALID;
		else if ( ( mask | (FLOWCONTROL_RTSCTS_OUT | FLOWCONTROL_XONXOFF_OUT) ) & flowControlModeIn )
			iRet =  KO_PARAM_FLOWCONTROLIN_NOT_VALID;
	}
	return iRet;
}

int driver_verify_flowcontrolmode_out( int flowControlModeOut ) {
	int iRet = OK_SERIALDRIVER_SUCCESS; ;
	if ( flowControlModeOut != FLOWCONTROL_NONE ) {
		int mask = 0xFFFFFFF0;
		if ( mask & flowControlModeOut )
			iRet =  KO_PARAM_FLOWCONTROLOUT_NOT_VALID;
		else if ( ( mask | (FLOWCONTROL_RTSCTS_IN | FLOWCONTROL_XONXOFF_IN) ) & flowControlModeOut )
			iRet =  KO_PARAM_FLOWCONTROLOUT_NOT_VALID;
	}
	return iRet;
}


