/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
#ifndef LOADER_SERIAL_DRIVER_H_
#define LOADER_SERIAL_DRIVER_H_

#include <generic_constants.h>
#include "loader.h"

#include <i_serial_driver_export_types.h>

/**
 * 	Serial driver descriptor. Contains the API list loaded from exported library.
 */
typedef struct tagSerialDriverLoader {
	char  serialDriverId[MAX_PATH];
	void* serialDriverHandle;

	driver_create_t                       create;
	driver_destroy_t                      destroy;
	driver_init_t                         init;
	driver_init_from_file_t               init_from_file;
	driver_open_t                         open;
	driver_close_t                        close;
	driver_is_open_t                      is_open;
	driver_get_port_id_t                  get_port_id;
	driver_set_port_id_t                  set_port_id;
	driver_set_port_params_t              set_port_params;
	driver_set_listening_period_param_t   set_listening_period_param;
	driver_set_flow_control_mode_params_t set_flow_control_mode_params;
	driver_send_t                         send;
	driver_add_listener_t                 add_listener;
	driver_remove_listener_t              remove_listener;
	driver_add_spy_listener_t             add_spy_listener;
	driver_remove_spy_listener_t          remove_spy_listener;
	driver_get_version_t          		  get_version;
	driver_get_serial_driver_instance_t   get_serial_driver_instance;

} SerialDriverLoader;


#ifdef __cplusplus
extern "C" {
#endif

/**
 * 	Release serial driver library from memory and reset serial driver descriptor
 * @param serialDriver - the serial driver descriptor to be released.
 */
void 	free_serial_driver( SerialDriverLoader* serialDriver );

/**
 * 	Load serial driver API list from specified library. The loader performs
 * 	loading of exported library functions by name and assign them to structure
 * 	members <b>serialDriver</b> which are pointers to function types.
 * 	 The serial driver is considered to be successfully loaded if all exported
 * 	functions were loader.
 *
 * @param serialDriverName - the serial driver name (without extension and library prefix)
 * @param serialDriver - a pointer to serial driver descriptor.
 * @return - operation code <b>OK</b> if library successfully loaded.
 */
int 	load_serial_driver( const char* serialDriverName, SerialDriverLoader* serialDriver );

#ifdef __cplusplus
}
#endif



#endif /* LOADER_SERIAL_DRIVER_H_ */
