/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

/**
 * @file protocol_response_types.h
 * @brief Definition of response structures used to return the Response information
 * after the process of a Command by a Protocol service
 */
#ifndef PROTOCOL_RESPONSE_TYPES_H_
#define PROTOCOL_RESPONSE_TYPES_H_

#include <protocol_command_types.h>



/**
 * Parent structure used to return the Response information
 * after the process of a Command by a Protocol service
 */
typedef struct tagICommandResponse {

	/**
	 * Status of the process command
	 * status must not be <code>empty</code>
	 */
	ResponseStatus status;

	/**
	 * Literal description of the status
	 * useful to add details to ResponseStatus
	 */
	char statusDescription[MAX_STATUS_DESCRIPTION_LEN];

	/**
	 * 	Response's serial frame
	 */
	pISerialFrame	serialFrame;

} CommandResponse;

/**
 * Structure used to return the response information
 * after the process of a radio request command by a Protocol service
 * Only the response to UNICAST request are considered here.
 */
typedef struct tagIRequestResponse {
	/**
	 * Inherited parent CommandResponse
	 */
	CommandResponse response;

	/**
	 * Received radio command in response to the request
	 * Can be <code>empty</code> if no response has been received
	 */
	RadioExchangeCommand command;
} RequestResponse;

/**
 * Structure used to return the response information
 * after the process of a radio request command by a Protocol service
 * Only the response to MULTICAST or BROADCAST request are considered here.
 */
typedef struct tagIMulticastRequestResponse {
	/**
	 * Inherited parent CommandResponse
	 */
	CommandResponse response;

	/**
	 * List of received radio command in response to the request
	 * Can be <code>empty</code> if no response has been received
	 */
	RadioExchangeCommand commands[MAX_MULTICAST_REQUEST_COMMANDS_LEN];

	int commandsLen;
} MulticastRequestResponse;

/**
 * API of Structure used to return response information of a setting radio request
 */
typedef struct tagIRadioSettingsResponse {
	/**
	 * Inherited parent CommandResponse
	 */
	CommandResponse response;

	/**
	 * Radio settings in response to the command
	 * Can be <code>empty</code> if no response has been received
	 */
	RadioSettingsCommand command;
} RadioSettingsResponse;

/**
 * Structure used to return the response information
 * after the process IProtocolService.requestLocalNodeService
 */
typedef struct tagILocalNodeServiceResponse {
	/**
	 * Inherited parent CommandResponse
	 */
	CommandResponse response;

	/**
	 * local service response return by process
	 */
	LocalNodeServiceCommand command;
} LocalNodeServiceResponse;

#endif /* PROTOCOL_RESPONSE_TYPES_H_ */
