/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

/**
 * @file serial_port_constants.h
 * @brief Define all constants values for serial port generic parameters,
 * which could be used for connect a SerialDriver instance
 */
#ifndef _SERIAL_PORT_CONSTANTS_H_
#define _SERIAL_PORT_CONSTANTS_H_

/**
 * Baud rates at which the communication device operates
 */
#ifndef CBR_4800
#define CBR_4800            4800
#endif
#ifndef CBR_9600
#define CBR_9600            9600
#endif
#ifndef CBR_14400
#define CBR_14400           14400
#endif
#ifndef CBR_19200
#define CBR_19200           19200
#endif
#ifndef CBR_38400
#define CBR_38400           38400
#endif
#ifndef CBR_57600
#define CBR_57600           57600
#endif
#ifndef CBR_115200
#define CBR_115200          115200
#endif

/**
 * Data bits at which the communication device operates
 */
#define DATABITS_FIVE           5
#define DATABITS_SIX            6
#define DATABITS_SEVEN          7
#define DATABITS_EIGHT          8

/**
 * Stop bit used for the communication
 */
#ifndef ONESTOPBIT
#define ONESTOPBIT          0
#endif
#ifndef ONE5STOPBITS
#define ONE5STOPBITS        1
#endif
#ifndef TWOSTOPBITS
#define TWOSTOPBITS         2
#endif


/**
 * Parity at which the communication device operates
 */
#ifndef NOPARITY
#define NOPARITY            0
#endif
#ifndef ODDPARITY
#define ODDPARITY           1
#endif
#ifndef EVENPARITY
#define EVENPARITY          2
#endif
#ifndef MARKPARITY
#define MARKPARITY          3
#endif
#ifndef SPACEPARITY
#define SPACEPARITY         4
#endif

/**
 * Flow control used for the communication
 */
#define FLOWCONTROL_NONE         0
#define FLOWCONTROL_RTSCTS_IN    1
#define FLOWCONTROL_RTSCTS_OUT   2
#define FLOWCONTROL_XONXOFF_IN   4
#define FLOWCONTROL_XONXOFF_OUT  8

#endif//_SERIAL_PORT_CONSTANTS_H_
