/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

/**
 * @file protocol_types.h
 * @brief Declaration of protocol types that are used for
 * Coronis implementation of Protocol implementation
 */
#ifndef PROTOCOL_TYPES_H_
#define PROTOCOL_TYPES_H_

#include <generic_types.h>


#define CORONIS_SDK_PROTOCOL_CONSTANTS_V_1_0

/**
 * Length used for buffering the payload information
 * over Coronis Protocol implementation
 * 2560 bytes for data storage + 4 for length
 */
#define PAYLOAD_BUFFER_LEN  2564
/**
 * Maximum length of payload that can be used over
 * Coronis Protocol implementation
 */
#define MAX_PAYLOAD_LEN  (PAYLOAD_BUFFER_LEN - 4)

/**
 * Length used for buffering the radioAddress information
 * over Coronis Protocol implementations (Waveport, WavenisTransport)
 * 12 bytes + 1 byte for null terminated C string
 */
#define RADIOADDRESS_BUFFER_LEN  13
/**
 * Length of RadioAddress that can be used over
 * Coronis Protocol implementations (Waveport, WavenisTransport)
 */
#define MAX_RADIOADDRESS_LEN  13

/**
 * Structure used to store payload information
 * for Coronis Protocol implementations usage (Waveport, WavenisTransport).
 * 		frame: data that contains the payload in byte[] format
 * 		len: length of the data that contains in the payload value
 */
typedef struct tagIPayload {
	byte         frame[PAYLOAD_BUFFER_LEN - 4];
	unsigned int len;
} Payload;

/**
 * This interface describes radio address methods available to manipulate RadioAddress content.
 * Address is stored as hex string
 * 		addressValue: data that contains the radio address value
 */
typedef struct tagIRadioAddress {
	char addressValue[RADIOADDRESS_BUFFER_LEN];
} RadioAddress;

/****************************************************************************************************/
/*
 * 	Will dump to log file the content of passed payload
 * 	@param payload - poinetr to payload strucre
 */
void protocol_dump_payload (Payload *payload);

_bool is_node_address_valid(const char *node_address);

#endif /* PROTOCOL_TYPES_H_ */
