/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

#ifndef LOADER_LOGGER_H_
#define LOADER_LOGGER_H_

#define DOMAIN_NAME "csdomain"

#include <generic_constants.h>
#include <i_logger_export_types.h>


/**
 * 	Logger descriptor structure
 */
typedef struct tagLoggerLoader {
	char  loggerName[MAX_PATH];
	void* loggerHandle;

	// init_log_t init_log;
	init_log_from_property_file_t init_log_from_property_file;
	debug_t    debug;
	info_t     info;
	warn_t     warn;
	error_t_    error;
	fatal_t    fatal;
} LoggerLoader;

#ifdef __cplusplus
extern "C" {
#endif

/**
 * 	Unload dynamic library from memory associated with logger descriptot,
 * 	and reset this descriptor.
 * @param logger
 */
void free_logger( LoggerLoader* logger );

/**
 * 	Load exported API set for logger from library specified in loggerName parameter.
 *
 * @param loggerName the logger library name (without extension and "lib" prefix)
 * @param logger - the logger descriptor
 * @return OK if logger has been loaded successfully
 */
int load_logger( const char* loggerName, LoggerLoader* logger );

/**
 * 	Global variable that holds the logger descriptor
 */
extern LoggerLoader g_Logger;

/**
 * 	Log DEBUG macros, used to write debug log messages
 */
#define LOG_DEBUG( domain, format, ... ) { domain.debug ? domain.debug( format, __VA_ARGS__) : 0; }

/**
 * 	Log INFO macros, used to write info log messages
 */
#define LOG_INFO( domain, format, ... ) { domain.info ? domain.info( format, __VA_ARGS__) : 0; }

/**
 * 	Log WARN macros, used to write warning log messages
 */
#define LOG_WARN( domain, format, ... ) { domain.warn ? domain.warn( format, __VA_ARGS__) : 0; }

/**
 * 	Log ERROR macros, used to write error log messages
 */
#define LOG_ERROR( domain, format, ... ) { domain.error ? domain.error( format, __VA_ARGS__) : 0; }

/**
 * 	Log FATAL macros, used to write fatal log messages
 */
#define LOG_FATAL( domain, format, ... ) { domain.fatal ? domain.fatal( format, __VA_ARGS__) : 0; }

#ifdef __cplusplus
}
#endif

#endif /* LOADER_LOGGER_H_ */
