/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
#ifndef PROPERTY_READER_H_
#define PROPERTY_READER_H_

/**
 *	Property file reader. The property file is organized into a key=value  manier.
 *	The property file supports commments.
 */

/**
 * 	Properties fields type
 */
typedef enum tagPropertyType{
	PT_BOOL,  //!< PT_BOOL
	PT_UINT,  //!< PT_UINT
	PT_HEX,   //!< PT_HEX
	PT_STRING,//!< PT_STRING
	PT_FLOAT  //!< PT_FLOAT
} PropertyType;

/**
 * 	read_properties_file - reads the data from a text file to passed buffer. The data read is of text type,
 * 	and the buffer length is adjusted by setting the last read byte to zero.
 *
 * 	@param  data_buffer - buffer where to store data
 * 	@param data_buffer_size - the maximal count of bytes to read
 * 	@param  file_name - the file name (without path)
 * 	@return the number of read bytes
 */
int read_properties_file(void * moduleHandle, char *data_buffer, int data_buffer_size, const char *file_name);

/**
 * 		Read from property file a specific property value by name
 * @param propertyString - the all properties in one single string
 * @param propertyName - the property key name
 * @param propertyDefaultValue - the default value in case if requered property
 * 		  could not be loaded for some reason.
 * @param type - the type of property to read
 * @param param - the pointer of the variable where to store loaded value
 */
void get_property_value( const char* propertyString, const char* propertyName, const char* propertyDefaultValue, PropertyType type, void* param );

/**
 * 	Get property value as a string
 * @param propertyString - all properties as a single string
 * @param propertyName - the name of the requested property.
 * @return a pointer to property string.
 */
const char* get_property( const char* propertyString, const char* propertyName );

#endif /* PROPERTY_READER_H_ */
