/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
#ifdef _JAVA_WRAPPER
#include "../../include/logger.h"

#include "com_coronis_sdk_domain_framework_Logger.h"

typedef void (*message_function_t)( const char* format, ...);

void write_log_message(JNIEnv *env, jobject this, jstring message, message_function_t message_function ) {
	const char* str = (*env)->GetStringUTFChars(env, message, NULL);
	if ( str != NULL) {
		message_function( str );
		(*env)->ReleaseStringUTFChars(env, message, str);
	}
}

JNIEXPORT void JNICALL JNIFUNCTION(initNative)(JNIEnv *env, jobject this ) {
	init_log( );
}

JNIEXPORT void JNICALL JNIFUNCTION(init)(JNIEnv *env, jobject this, jobject url) {
	jclass clazz = (*env)->GetObjectClass( env, url);
	if ( clazz != NULL ) {
		jmethodID methodID = (*env)->GetMethodID( env, clazz, "getPath", "()Ljava/lang/String;");
		if ( methodID != NULL ) {
			jstring jstr = (jobject)(*env)->CallObjectMethod( env, url, methodID );
			if ( jstr != NULL ) {
				const char* str = (*env)->GetStringUTFChars(env, jstr, NULL);
				if ( str != NULL) {
					init_log_from_property_file( str[0] == '/' ? str+1 : str );
					(*env)->ReleaseStringUTFChars(env, jstr, str);
				}
				(*env)->DeleteLocalRef(env, jstr);
			}
		}
		(*env)->DeleteLocalRef(env, clazz);
	}
}

JNIEXPORT void JNICALL JNIFUNCTION(debug)(JNIEnv *env, jobject this, jstring message) {
	write_log_message( env, this, message, debug );
}

JNIEXPORT void JNICALL JNIFUNCTION(error)(JNIEnv *env, jobject this, jstring message) {
	write_log_message( env, this, message, error );
}

JNIEXPORT void JNICALL JNIFUNCTION(fatal)(JNIEnv *env, jobject this, jstring message) {
	write_log_message( env, this, message, fatal );
}

JNIEXPORT void JNICALL JNIFUNCTION(info)(JNIEnv *env, jobject this, jstring message) {
	write_log_message( env, this, message, info );
}

JNIEXPORT void JNICALL JNIFUNCTION(warn)(JNIEnv *env, jobject this, jstring message) {
	write_log_message( env, this, message, warn );
}

/* caching the class references, or method and field IDs
 * definition of this function*/
void initialize_global_class_references( JavaVM *jvm, jint jniVersion ) {
}

/* uncaching the class references, or method and field IDs
* definition of this function*/
void uninitialize_global_class_references( JavaVM *jvm, jint jniVersion ) {
}
#endif //_JAVA_WRAPPER
