/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

/** @file i_protocol_export_types.h
 * @brief This interface define the type that must be used
 * to create a dynamic instance of a Protocol implementation
 * To use a protocol implementation in accordance with the API.
 * For example, the Coronis domain module use this definition
 * inside the Protocol loading feature.
 */
#ifndef I_PROTOCOL_EXPORT_TYPES_H_
#define I_PROTOCOL_EXPORT_TYPES_H_

#include <i_protocol_types.h>
#include <protocol_command_types.h>

// Name of method that must be called.
#define FN_PROTOCOL_CREATE                             "protocol_create"
#define FN_PROTOCOL_DESTROY                            "protocol_destroy"
#define FN_PROTOCOL_INIT                               "protocol_init"
#define FN_PROTOCOL_INIT_FROM_FILE                     "protocol_init_from_file"
#define FN_PROTOCOL_OPEN                               "protocol_open"                              
#define FN_PROTOCOL_CLOSE                              "protocol_close"                             
#define FN_PROTOCOL_IS_OPEN                            "protocol_is_open"                           
#define FN_PROTOCOL_GET_SERIAL_DRIVER_ID               "protocol_get_serial_driver_id"              
#define FN_PROTOCOL_SET_SERIAL_DRIVER_ID               "protocol_set_serial_driver_id"              
#define FN_PROTOCOL_SET_SERIAL_DRIVER_INSTANCE         "protocol_set_serial_driver_instance"
#define FN_PROTOCOL_ADD_MESSAGE_LISTENER               "protocol_add_message_listener"
#define FN_PROTOCOL_REMOVE_MESSAGE_LISTENER            "protocol_remove_message_listener"
#define FN_PROTOCOL_ADD_SPY_MESSAGE_LISTENER           "protocol_add_spy_message_listener"
#define FN_PROTOCOL_REMOVE_SPY_MESSAGE_LISTENER        "protocol_remove_spy_message_listener"
#define FN_PROTOCOL_SEND_TRANSPARENT_MESSAGE           "protocol_send_transparent_message"          
#define FN_PROTOCOL_SEND_APPLICATION_MESSAGE           "protocol_send_application_message"          
#define FN_PROTOCOL_SEND_APPLICATION_BROADCAST_MESSAGE "protocol_send_application_broadcast_message"
#define FN_PROTOCOL_SEND_APPLICATION_REQUEST           "protocol_send_application_request"          
#define FN_PROTOCOL_SEND_APPLICATION_BROADCAST_REQUEST "protocol_send_application_broadcast_request"
#define FN_PROTOCOL_SEND_APPLICATION_RESPONSE          "protocol_send_application_response"         
#define FN_PROTOCOL_WRITE_RADIO_PARAMS                 "protocol_write_radio_params"                
#define FN_PROTOCOL_READ_RADIO_STANDARD_PARAMS         "protocol_read_radio_standard_params"        
#define FN_PROTOCOL_READ_RADIO_EXPERT_PARAMS           "protocol_read_radio_expert_params"          
#define FN_PROTOCOL_READ_RADIO_CONFIGURATION_PARAMS    "protocol_read_radio_configuration_params"
#define FN_PROTOCOL_READ_DYNAMIC_EXPERT_PARAMS         "protocol_read_dynamic_expert_params"
#define FN_PROTOCOL_READ_STATIC_STANDARD_PARAMS        "protocol_read_static_standard_params"
#define FN_PROTOCOL_READ_STATIC_EXPERT_PARAMS          "protocol_read_static_expert_params"
#define FN_PROTOCOL_REQUEST_LOCAL_NODE_SERVICE         "protocol_request_local_node_service"
#define FN_PROTOCOL_SEND_SERVICE_REQUEST               "protocol_send_service_request"
#define FN_PROTOCOL_SEND_SERVICE_MESSAGE               "protocol_send_service_message"
#define FN_PROTOCOL_GET_VERSION               		   "protocol_get_version"


#ifdef __cplusplus
extern "C" {  
#endif

// Function type definition for service offers by the Protocol API.
typedef void* (* protocol_create_t)                             ();
typedef int   (* protocol_destroy_t)                            ( const void* protocol );
typedef int   (* protocol_init_t)                               ( const void* protocol );
typedef int   (* protocol_init_from_file_t )                    ( const void* protocol, const char* file );
typedef int   (* protocol_open_t)                               ( const void* protocol );
typedef int   (* protocol_close_t)                              ( const void* protocol );
typedef int   (* protocol_is_open_t)                            ( const void* protocol );
typedef int   (* protocol_get_serial_driver_id_t)               ( const void* protocol, char* serialDriverId, int idLen );
typedef int   (* protocol_set_serial_driver_id_t)               ( const void* protocol, const char* serialDriverId);
typedef int   (* protocol_set_serial_driver_instance_t)         ( const void* protocol, const void* serialDriver);
typedef int   (* protocol_add_message_listener_t)               ( const void* protocol, int (*receiveRadioExchangeCommand)( void* param, const RadioExchangeCommand *receiveCommand ),
		                                                                        int (*receiveEvent)( void* param, int eventCode ),
		                                                                        void* param);
typedef int   (* protocol_remove_message_listener_t)            ( const void* protocol );
typedef int   (* protocol_add_spy_message_listener_t)           ( const void* protocol, int (*receivedRadioExchangeCommand)( void* param, int spyIndex, const RadioExchangeCommand *command ),
		                                                                        int (*sentRadioExchangeCommand)( void* param, int spyIndex, const RadioExchangeCommand *command ),
		                                                                        void* param,
		                                                                        int* spyIndex );
typedef int   (* protocol_remove_spy_message_listener_t)        ( const void* protocol, int spyIndex );
typedef int   (* protocol_send_transparent_message_t)           ( const void* protocol, const void* frame, void* response );
typedef int   (* protocol_send_application_message_t)           ( const void* protocol, const void* command, void* response );
typedef int   (* protocol_send_application_broadcast_message_t) ( const void* protocol, const void* command, void* response );
typedef int   (* protocol_send_application_request_t)           ( const void* protocol, const void* command, void* response );
typedef int   (* protocol_send_application_broadcast_request_t) ( const void* protocol, const void* command, void* response );
typedef int   (* protocol_send_application_response_t)          ( const void* protocol, const void* command, void* response );
typedef int   (* protocol_write_radio_params_t)                 ( const void* protocol, const void* command, void* response );
typedef int   (* protocol_read_dynamic_expert_params_t)         ( const void* protocol, void* response);
typedef int   (* protocol_read_static_standard_params_t)        ( const void* protocol, void* response);
typedef int   (* protocol_read_static_expert_params_t)          ( const void* protocol,	void* response);
typedef int   (* protocol_request_local_node_service_t)         ( const void* protocol, const void* command, void* response );
typedef int   (* protocol_send_service_request_t)               ( const void* protocol, const NodeServiceId serviceId, const void* command, void* response );
typedef int   (* protocol_send_service_message_t)               ( const void* protocol, const NodeServiceId serviceId, const void* command, void* response );
typedef int   (* protocol_get_version_t)                        ( );

#ifdef __cplusplus
}
#endif

#endif /* I_PROTOCOL_EXPORT_TYPES_H_ */
