/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

/**
 * @file protocol_constants.h
 * @brief Define all constants values for protocol generic radio parameters,
 * which could be used to configure the connected Radio modem
 */
#ifndef PROTOCOL_CONSTANTS_H_
#define PROTOCOL_CONSTANTS_H_

#define MAX_ROUTER_PATH  3
#define MAX_BYTE_ID_LEN  3
#define MAX_STATUS_DESCRIPTION_LEN 256
#define MAX_MULTICAST_REQUEST_COMMANDS_LEN 10
#define MAX_REQ_SEND_FRAME_BYTES_LEN 152

#define MAX_FREQUENCY_HOPPING_CHANNEL_TABLE_LENGTH       132

#define MAX_TRANSMISSION_MODE_LEN  13

#define MAX_PHYSICAL_CONFIG_LEN  20

#define FLAG_SERIAL_DRIVER_ID                            0x0000000001
#define FLAG_CONFIGURATION_TIMEOUT                       0x0000000002
#define FLAG_LISTENING_STATUS                            0x0000000004
#define FLAG_WRITE_DEFAULT_RADIO_SETTINGS                0x0000000008
#define FLAG_RESPONSE_TIMEOUT                            0x0000000010
#define FLAG_TRANSMISSION_RADIO_CHANNEL                  0x0000000020
#define FLAG_WAKEUP_LENGTH                               0x0000000040
#define FLAG_BROADCAST_SESSION_MAX_RESPONSES_NB          0x0000000080
#define FLAG_BROADCAST_SESSION_TIMEOUT                   0x0000000100
#define FLAG_TRANSMISSION_POWER_LEVEL                    0x0000000200
#define FLAG_RADIO_ACK_ACTIVATION                        0x0000000400
#define FLAG_FIRST_SCAN_ENABLE_ACTIVATION                0x0000000800
#define FLAG_SWITCHING_MODE_ON_RADIO_ADDRESS_ACTIVATION  0x0000001000
#define FLAG_ACTIVITY_PERIOD_ON_FIRST_SCAN               0x0000002000
#define FLAG_PHYSICAL_CONFIG_ON_FIRST_SCAN               0x0000004000
#define FLAG_RADIO_CHANNEL_ON_FIRST_SCAN                 0x0000008000
#define FLAG_RECEPTION_GROUPID                           0x0000010000
#define FLAG_SECOND_SCAN_ENABLE_ACTIVATION               0x0000020000
#define FLAG_COMPLEX_SCAN_ENABLE_ACTIVATION              0x0000040000
#define FLAG_RADIO_CRC_ACTIVATION                        0x0000080000
#define FLAG_CWUP_TRANSMISSION_ACTIVATION                0x0000100000
#define FLAG_CWUP_AUTOSELECTION_ACTIVATION               0x0000200000
#define FLAG_CWUP_REQUEST_ONLY_ACTIVATION                0x0000400000
#define FLAG_CWUP_ONLY_ACTIVATION                        0x0000800000
#define FLAG_ACTIVITY_PERIOD_ON_SECOND_SCAN              0x0001000000
#define FLAG_PHYSICAL_CONFIG_ON_SECOND_SCAN              0x0002000000
#define FLAG_RADIO_CHANNEL_ON_SECOND_SCAN                0x0004000000
#define FLAG_SENSITIVITY_REDUCTION_FACTOR_ON_FIRST_SCAN  0x0008000000
#define FLAG_SENSITIVITY_REDUCTION_FACTOR_ON_SECOND_SCAN 0x0010000000
#define FLAG_NB_SCAN_ON                                  0x0020000000
#define FLAG_NB_SCAN_OFF                                 0x0040000000
#define FLAG_FREQUENCY_HOPPING_CHANNEL_TABLE             0x0080000000
#define FLAG_PROTOCOL_LOG_PROP_FILE_NAME			     0x0100000000
#define FLAG_CONFIGURATION_RETRY_NUMBER					 0x0200000000
#define	FLAG_RECEPTION_MODE								 0x0400000000

/**
 * Enumeration of supported Physical layers
 */
typedef enum tagPhysicalLayer {
	MONO_CHANNEL_868MHZ_9600BD,			// 0x00A2
	FHSS_868MHZ_9600BD,					// 0x00A3
	MONO_CHANNEL_868MHZ_500mW_19200BD	// 0x00B6
}PhysicalLayer;

/**
 * Enumeration of supported radio reception modes
 */
typedef enum tagReceptionMode {
	RECEPTION_MODE_UNDEFINED,
	RECEPTION_MODE_UNICAST,
	RECEPTION_MODE_MULTICAST,
	RECEPTION_MODE_BROADCAST
}ReceptionMode;

/**
 * Enumeration of supported Radio channel
 */
typedef enum tagRadioChannel {
	RADIO_CHANNEL_0 = 0,
	RADIO_CHANNEL_1 = 1,
	RADIO_CHANNEL_2 = 2,
	RADIO_CHANNEL_3 = 3,
	RADIO_CHANNEL_4 = 4,
	RADIO_CHANNEL_5 = 5,
	RADIO_CHANNEL_6 = 6,
	RADIO_CHANNEL_7 = 7,
	RADIO_CHANNEL_8 = 8,
	RADIO_CHANNEL_9 = 9,
	RADIO_CHANNEL_10 = 10,
	RADIO_CHANNEL_11 = 11,
	RADIO_CHANNEL_12 = 12,
	RADIO_CHANNEL_13 = 13,
	RADIO_CHANNEL_14 = 14,
	RADIO_CHANNEL_15 = 15,
	RADIO_CHANNEL_16 = 16,
	RADIO_CHANNEL_17 = 17,
	RADIO_CHANNEL_18 = 18,
	RADIO_CHANNEL_19 = 19,
	RADIO_CHANNEL_20 = 20,
	RADIO_CHANNEL_21 = 21
} RadioChannel;

/**
 * Enumeration of response status that could be return
 * via the IProtocolService implementation
 */
typedef enum tagResponseStatus {
	OK_PROTOCOL_SUCCESS,
	KO_PAYLOAD_REQUIRED,
	KO_NODE_ADDRESS_REQUIRED,
	KO_PROTOCOL_NOT_OPEN,// protocol is not opened
	KO_PROTOCOL_ALREADY_OPEN,// protocol already open
	KO_PROTOCOL_ALREADY_IN_USE,
	KO_PROTOCOL_WRITING_ERROR,
	KO_PROTOCOL_NO_VALID_PARAMETER,// A one of command parameters is absent or wrong
	KO_PROTOCOL_CONFIGURATION_TIMEOUT,// Error wile waiting for frame, timeout expired
	KO_PROTOCOL_UNEXPECTED_FRAME,// An unexpected frame received
	KO_PROTOCOL_PARAM_UPDATE_ERROR,// 10. An error occurred while writing command ( CMD_STATUS_KO(01) received! )
	KO_PROTOCOL_PARAM_READ_ERROR,// An error occurred while reading command ( CMD_STATUS_KO(01) received! )
	KO_PROTOCOL_NODE_SERVICE_ID_REQUIRED,
	KO_PROTOCOL_NODE_SERVICE_NOT_SUPPORTED,
	KO_PROTOCOL_MAX_SPY_LISTENERS_EXCEEDED,
	KO_PROTOCOL_SETTING_ERROR,
	KO_PROTOCOL_PROCESS_FAILURE ,
	KO_SERIALDRIVER_SENDING_FAILURE,// An error occurred in serial driver while sending command
	KO_SERIALDRIVER_NOT_AVAILABLE,// An error occurred while try to connect to serial_port
	KO_SERIALDRIVER_ALREADY_OPENED,// Serial Driver instance already open
	KO_SERIALDRIVER_ID_NOT_EXIST,//Serial Driver ID not exist
	KO_SERIALDRIVER_PORT_NOT_OPENED,// 20. Serial Port is not opened
	KO_SERIALDRIVER_NOT_INITIALIZED,// Serial Driver is not initialized
	KO_PARAM_WRITING_FAILURE,
	KO_PARAMETER_SERIAL_DRIVER_ID_NOT_VALID,// Serial Driver instance not loaded
	KO_PARAMETER_NODE_ADDRESS_NOT_VALID,
	KO_PARAMETER_RELAY_ROUTE_NOT_VALID,
	KO_PARAMETER_GROUP_NUMBER_NOT_VALID,
	KO_PARAMETER_WAKEUP_LENGTH_NOT_VALID, // 27
	KO_PARAMETER_RESPONSE_TIMEOUT_NOT_VALID,
	KO_PARAMETER_BROADCAST_SESSION_TIMEOUT_NOT_VALID,
	KO_PARAMETER_BROADCAST_SESSION_MAX_RESPONSES_NB_NOT_VALID, // 30
	KO_PARAMETER_TRANSMISSION_RADIO_CHANNEL_NOT_VALID,
	KO_PARAMETER_TRANSMISSION_POWER_LEVEL_NOT_VALID,
	KO_PARAMETER_RADIO_ACK_ACTIVATION_NOT_VALID,
	KO_PARAMETER_FIRST_SCAN_ACTIVITY_PERIOD_NOT_VALID,
	KO_PARAMETER_SWITCHING_MODE_ON_RADIO_ADDRESS_ACTIVATION_NOT_VALID, // 35
	KO_PARAMETER_RECEPTION_GROUP_ID_NOT_VALID,
	KO_PARAMETER_FIRST_SCAN_ENABLE_ACTIVATION_NOT_VALID,
	KO_PARAMETER_SECOND_SCAN_ENABLE_ACTIVATION_NOT_VALID,
	KO_PARAMETER_RADIO_CRC_ACTIVATION_NOT_VALID,
	KO_PARAMETER_FIRST_SCAN_PHYSICAL_CONFIG_NOT_VALID, //40
	KO_PARAMETER_SECOND_SCAN_PHYSICAL_CONFIG_NOT_VALID,
	KO_PARAMETER_NODE_ADDRESS_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_RELAY_ROUTE_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_GROUP_NUMBER_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_WAKEUP_LENGTH_VALUE_NOT_SUPPORTED, // 45
	KO_PARAMETER_RESPONSE_TIMEOUT_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_BROADCAST_SESSION_TIMEOUT_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_BROADCAST_SESSION_MAX_RESPONSES_NB_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_TRANSMISSION_RADIO_CHANNEL_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_TRANSMISSION_POWER_LEVEL_VALUE_NOT_SUPPORTED, // 50
	KO_PARAMETER_RADIO_ACK_ACTIVATION_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_FIRST_SCAN_ACTIVITY_PERIOD_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_SWITCHING_MODE_ON_RADIO_ADDRESS_ACTIVATION_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_RECEPTION_GROUP_ID_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_FIRST_SCAN_ENABLE_ACTIVATION_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_SECOND_SCAN_ENABLE_ACTIVATION_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_RADIO_CRC_ACTIVATION_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_FIRST_SCAN_PHYSICAL_CONFIG_VALUE_NOT_SUPPORTED,
	KO_PARAMETER_SECOND_SCAN_PHYSICAL_CONFIG_VALUE_NOT_SUPPORTED,
	KO_RADIO_TRANSMISSION_FAILURE, // 60
	KO_RADIO_TRANSMISSION_FAILURE_WITH_DESTINATION_NODE,
	KO_RADIO_TRANSMISSION_FAILURE_WITH_FIRST_RELAY,
	KO_RADIO_TRANSMISSION_FAILURE_WITH_SECOND_RELAY,
	KO_RADIO_TRANSMISSION_FAILURE_WITH_THIRD_RELAY,
	KO_PROTOCOL_NODE_LOCAL_SERVICE_FAILURE,
	KO_PROTOCOL_NO_RESPONSE_SESSION,
	KO_SERIALDRIVER_ALREADY_LOADED,
	KO_PROTOCOL_NOT_INITIALIZED,
	KO_PAYLOAD_NOT_VALID,
	KO_PARAMETER_RECEPTION_MODE_NOT_VALID,
	KO_NODE_ADDRESS_NOT_VALID,
	KO_UNEXPECTED_ERROR

} ResponseStatus;

/**
 * Enumeration that list all type of service that could be request
 * to a local node (serial driver link)
 */
typedef enum tagNodeServiceId {
	NODE_SERVICE_ID_UNDEFINED,
	STACK_SERVICE,
	SDP_SERVICE,
	FIRMWARE_UPGRADE_SERVICE
} NodeServiceId;

#ifndef _CORONIS_BOOLEAN_
#define _CORONIS_BOOLEAN_
typedef enum tag_bool {
	_false = 0,
	_true = !_false
} _bool;
#endif //_CORONIS_BOOLEAN_

#ifndef _CORONIS_UINT_64_
#define _CORONIS_UINT_64_
#ifdef _WINDOWS
//#define int64 __int64
typedef unsigned long long uint64;
#else
//#define int64  long long int
typedef unsigned long long int uint64;
#endif
#endif // _CORONIS_UINT_64_

#endif /* PROTOCOL_CONSTANTS_H_ */
