/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
#include <loader_logger.h>
#include <loader.h>

#include "utils.h"

LoggerLoader g_Logger;

void free_logger( LoggerLoader* logger ) {
	FREE_LIBRARY( logger->loggerHandle );
	memset( logger, 0, sizeof(*logger) );
}

int load_logger( const char* loggerName, LoggerLoader* logger ) {
	int iRet = OK;
	if ( 0 != strcmp( loggerName, logger->loggerName ) ) {
		if ( logger->loggerHandle ) {
			free_logger( logger );
		}
		char loggerLibName[MAX_PATH] = "";
#ifdef _WINDOWS
		snprintf( loggerLibName, sizeof(loggerLibName) - 1, "%s.dll", loggerName );
#else
		if (strlen(loggerName) > 1 && loggerName[0] == '.' &&  loggerName[1] == '/') {
			snprintf( loggerLibName, sizeof(loggerLibName) - 1, "./lib%s.so", (loggerName + 2) );
		} else {
			snprintf( loggerLibName, sizeof(loggerLibName) - 1, "./lib%s.so", loggerName );
		}
#endif
		//printf("load_logger :: loading logger from %s ... \n", loggerLibName);
		if ( (logger->loggerHandle = LOAD_LIBRARY(loggerLibName)) != NULL ) {

			//printf("load_logger :: Ok,  load logger functions ... \n");

			if ( ((logger->init_log_from_property_file = (init_log_from_property_file_t)LOAD_FUNCTION(logger->loggerHandle,FN_INIT_LOG_FROM_PROPR_FILE)) != NULL  ) &&
					((logger->debug = (debug_t)LOAD_FUNCTION(logger->loggerHandle,FN_DEBUG)) != NULL  ) &&
					((logger->info = (info_t)LOAD_FUNCTION(logger->loggerHandle,FN_INFO)) != NULL  ) &&
					((logger->warn = (warn_t)LOAD_FUNCTION(logger->loggerHandle,FN_WARN)) != NULL  ) &&
					((logger->error = (error_t_)LOAD_FUNCTION(logger->loggerHandle,FN_ERROR)) != NULL  ) &&
					((logger->fatal = (fatal_t)LOAD_FUNCTION(logger->loggerHandle,FN_FATAL)) != NULL  )
					) {

				// printf("load_logger :: Ok, logger loaded \n");
				strncpy( logger->loggerName, loggerName, min( MAX_PATH, strlen(loggerName) ) );
				iRet = OK;
			} else { // LOAD_FUNCTION(s)

				#ifndef _WINDOWS
				//printf("load_logger :: Failed to load functions %s\n", GET_ERROR());
				#endif

				free_logger( logger );
				iRet = (int)GET_ERROR();
			}
		} else // LOAD_LIBRARY;
			#ifndef _WINDOWS
			//printf("load_logger :: Failed to load library  %s\n", GET_ERROR());
			#endif
			iRet = (int)GET_ERROR();
	}
	return iRet;
}
