/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
#include "utils.h"

#include <time.h>
#include <ctype.h>

unsigned int calc_crc( const byte* frame, int len ) {
	int poly = 0x8408;
	unsigned int crc = 0;
	int carry = 0;
	int j = 0;
	int i_bits = 0;
	for ( ; j < len; ++j ) {
		crc = crc ^ frame[j] ;
		for ( i_bits = 0; i_bits < 8 ; ++i_bits ) {
			carry = crc & 1 ;
			crc = crc / 2 ;
			if ( carry )
				crc = crc ^ poly;
		}
	}
	return crc;
}

#ifdef _WINDOWS
int gettimeofday(struct timeval *tv, struct timezone *tz)
{
	// Define a structure to receive the current Windows filetime
	FILETIME ft;

	// Initialize the present time to 0 and the timezone to UTC
	unsigned __int64 tmpres = 0;
	static int tzflag = 0;

	if (NULL != tv)
	{
		GetSystemTimeAsFileTime(&ft);

		// The GetSystemTimeAsFileTime returns the number of 100 nanosecond
		// intervals since Jan 1, 1601 in a structure. Copy the high bits to
		// the 64 bit tmpres, shift it left by 32 then or in the low 32 bits.
		tmpres |= ft.dwHighDateTime;
		tmpres <<= 32;
		tmpres |= ft.dwLowDateTime;

		// Convert to microseconds by dividing by 10
		tmpres /= 10;

		// The Unix epoch starts on Jan 1 1970.  Need to subtract the difference
		// in seconds from Jan 1 1601.
		tmpres -= DELTA_EPOCH_IN_MICROSECS;

		// Finally change microseconds to seconds and place in the seconds value.
		// The modulus picks up the microseconds.
		tv->tv_sec = (long)(tmpres / 1000000UL);
		tv->tv_usec = (long)(tmpres % 1000000UL);
	}

	if (NULL != tz)
	{
		if (!tzflag)
		{
			_tzset();
			tzflag++;
		}

		// Adjust for the timezone west of Greenwich
		tz->tz_minuteswest = _timezone / 60;
		tz->tz_dsttime = _daylight;
	}

	return 0;
}
/*__________________________________________________________________________________________________*/

#endif /* _WINDOWS */


_bool get_module_path( const void* moduleHandle, char* path, int pathLen ) {
	_bool bRet = _false;

	if ( path && pathLen > 0 ) {
#ifdef _WINDOWS
#ifdef __cplusplus
	    if( GetModuleFileNameA( (HINSTANCE)moduleHandle, path, pathLen ) ) {
#else
	    if( GetModuleFileNameA( (HANDLE)moduleHandle, path, pathLen ) ) {
#endif
	    	char* buff = strrchr( path, '\\' );
	    	if ( buff )
	    		buff[1] = 0;
	    	bRet = _true;
	    }
#else
	    strcpy( path, "./");
	    bRet = _true;
#endif//_WINDOWS
	}
	return bRet;
}

long millisecondtimediff( struct timeval tvEarly, struct timeval tvLater ) {
	uint64 later = ((uint64)tvLater.tv_sec*1000000+tvLater.tv_usec);
	uint64 early = ((uint64)tvEarly.tv_sec*1000000+tvEarly.tv_usec);
	long diff = ((later - early) / 1000);
	return diff;
}

_bool is_hex( const char* hexStr ) {
	_bool bRet = _false;
	size_t len = 0;
	if ( hexStr && ( len = strlen( hexStr ) ) ) {
		char ch = 0;
		size_t j = 0;
		bRet = _true;
		for ( j = 0; j < len; ++j ) {
			ch = toupper( hexStr[j] );
			if ( ( ch < '0' || ch > '9') && ( ch < 'A' || ch > 'F') ) {
				bRet = _false;
				break;
			}
		}
	}
	return bRet;
}

void sd_buffer_to_hex(char *string, const byte *data, const int size, const int max_string_size) {
	static char hex_chars [] = {'0', '1', '2', '3', '4', '5',
			'6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
	};

	if (string == 0 || max_string_size <= 0) return;
	string[0] = 0;
	int i;

	for(i = 0; i < size;  i ++) {

		if ((i * 2 + 1) >= max_string_size) break;

		string[i*2] = hex_chars[(data[i] >> 4) & 0x0F];
		string[i*2 + 1] = hex_chars[data[i] & 0x0F];
	}

	if ((i * 2 + 1) >= max_string_size)
		string[(i - 1)* 2 + 1] = 0;
	else
		string[i*2] = 0;
}
