/*
 * jni_utils.h
 *
 *  Created on: 02.11.2012
 *      Author: vmacari
 */

#ifndef _JNI_UTILS_H_
#define _JNI_UTILS_H_

#ifdef _JAVA_WRAPPER

#include "../protocol.h"
#include "protocol_types.h"
#include <jni.h>

JNIEnv* get_jni_env();

_bool catch_and_throw_exception( JNIEnv *env, int overriden_error_code);

void throw_exception( JNIEnv *env, CommandResponse* cr );

void throw_exception_id(JNIEnv *env, int errorCode);

void throw_exception_message( JNIEnv *env, const char* errorMessage,
				char *errorDescription );

jobject create_int_object( JNIEnv *env, jint value );

jobject create_float_object( JNIEnv *env, jfloat value );

jobject create_boolean_object( JNIEnv *env, jboolean value );

jobject create_serial_frame_object( JNIEnv *env, jclass classID,
				const char* value ) ;

jobject create_payload_object( JNIEnv *env, jclass classID, const char* value );

jobject create_response_status( JNIEnv *env, ResponseStatus rs );

jobject create_node_service_id( JNIEnv *env, NodeServiceId nodeServiceId );

jobject create_command_response( JNIEnv *env, jclass classSerialFrame,
				CommandResponse* cr );

jobject create_local_node_service_response( JNIEnv *env, ResponseStatus rs,
				jobject localNodeServiceCommand );

jobject create_radio_settings_response( JNIEnv *env, ResponseStatus rs,
				jobject radioSettingsCommand );

jobject create_request_response( JNIEnv *env, jclass classSerialFrame,
				const CommandResponse* cr, jobject radioExchangeCommand);

jobject create_multicast_request_response( JNIEnv *env,
				jclass classPayload, jclass classRadioExchangeCommand,
				ProtocolInfo* waveProtocol, MulticastRequestResponse* response,
				jobject jDynamicExpertParams );

jobject create_radio_settings_command( JNIEnv *env,
				jclass classPayload,
				jobject dynamicStandardParams, jobject dynamicExpertParams,
				jobject staticStandardParams, jobject staticExpertParams,
				const char* payloadValue );

jobject create_dynamic_standard_params_object( JNIEnv *env,
				DynamicStandardParams* dynamicStandardParams, WavePortResponses response);

jobject create_dynamic_expert_params( JNIEnv *env,
				DynamicExpertParams* dynamicExpertParams );

jobject create_static_standard_params( JNIEnv *env,
				StaticStandardParams* staticStandardParams );

jobject create_static_expert_params( JNIEnv *env,
				StaticExpertParams* staticExpertParams );

jobject create_local_node_service_command_object( JNIEnv *env,
				jclass classPayload,
				NodeServiceId serviceId, const SerialFrame* payload );

jobject create_local_node_service_command_from_payload_object( JNIEnv *env,
				jclass classPayload, NodeServiceId serviceId,
				const Payload* payload );

jobject create_radio_exchange_command_object( JNIEnv *env,
				jclass classPayload, jclass classRadioExchangeCommand,
				RadioExchangeCommand* reCommand, jobject dynamicStandardParams,
				jobject dynamicExpertParams, WavePortResponses response);

_bool get_int_field_from_object( JNIEnv *env, jobject object, jclass classID,
				const char* fieldName, jint* value );

_bool get_float_field_from_object( JNIEnv *env, jobject object, jclass classID,
				const char* fieldName, jfloat* value );

_bool get_boolean_field_from_object( JNIEnv *env, jobject object, jclass classID,
				const char* fieldName, jboolean* value );

NodeServiceId get_node_service_id( JNIEnv *env, jclass classID, jobject command );

int get_dynamic_standard_params( JNIEnv *env, jclass classID, jobject command,
				DynamicStandardParams* dynamicStandardParams );

int get_dynamic_expert_params( JNIEnv *env, jclass classID, jobject command,
				DynamicExpertParams* dynamicExpertParams );


_bool is_command_response_ok( JNIEnv *env, jobject commandResponse,
				_bool raiseException );

ResponseStatus get_response_status( JNIEnv *env, jobject commandResponse );
#endif // _JAVA_WRAPPER
#endif /* _JNI_UTILS_H_ */
