/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

#ifndef PROTOCOL_WAVEPORT_ENUM_CONVERSSION_H_
#define PROTOCOL_WAVEPORT_ENUM_CONVERSSION_H_

#include "generic_types.h"
#include <JNI/domain_jni.h>
#include "../protocol.h"

/**
 *		Function extracts enumeration value of radio channel from Java object
 *		@param [IN] env - jvm jni env.
 *		@param [IN] object - the java object that contains this enumeration
 *		@param [IN] classID - java object class ID
 *		@param [IN] enumItemName - the name of enumeration item from passed object
 *		@param [OUT] enumItemValue - enumeration C value
 *		@param [OUT] iRet - extraction return code
 *		@param [IN] iRetCode - the code to be placed in iRet in case if extraction fails
 *					This code is provided because this function is used for different
 *					enumeration of the same type.
 *		@return - true of extraction was ok
 */
_bool get_radio_channel_enum_field_from_object( JNIEnv * env, jobject object,
		jclass classID, char *enumItemName, RadioChannel *enumItemValue, int *iRet,
		const int iRetCode);
/**
 *		Function extracts enumeration value of reception mode from Java object
 *		@param [IN] env - jvm jni env.
 *		@param [IN] object - the java object that contains this enumeration
 *		@param [IN] classID - java object class ID
 *		@param [OUT] receptionMode - enumeration C value
 *		@param [OUT] iRet - extraction return code
 *		@return - true of extraction was ok
 */
_bool get_reception_mode_enum_field_from_object( JNIEnv * env, jobject object,
									jclass classID, ReceptionMode *receptionMode, int *iRet );
/**
 *		Function created a Java enumeration object and sets it's value according to
 *		passed C value.
 *
 *		@param [IN] env - jvm jni env.
 *		@param [IN] enumValue - the C enumeration value
 *		@return  - java object that represents the enumeration with set value.
 *				  NULL is returned if enumeration could not be created.
 */
jobject create_radiochannel_enum_object ( JNIEnv *env, RadioChannel enumValue );

/**
 *		Function created a Java enumeration object and sets it's value according to
 *		passed C value.
 *
 *		@param [IN] env - jvm jni env.
 *		@param [IN] enumValue - the C enumeration value
 *		@return  - java object that represents the enumeration with set value.
 *				  NULL is returned if enumeration could not be created.
 */
jobject create_reception_mode_enum_object ( JNIEnv *env, ReceptionMode enumValue );

/**
 *		Function created a Java enumeration object and sets it's value according to
 *		passed C value.
 *
 *		@param [IN] env - jvm jni env.
 *		@param [IN] enumValue - the C enumeration value
 *		@return  - java object that represents the enumeration with set value.
 *				  NULL is returned if enumeration could not be created.
 */

jobject create_physical_layer_object ( JNIEnv *env, PhysicalLayer enumValue );


/**
 *		Function extracts enumeration value of reception mode from Java object
 *		@param [IN] env - jvm jni env.
 *		@param [IN] object - the java object that contains this enumeration
 *		@param [IN] classID - java object class ID
 *		@param [OUT] receptionMode - enumeration C value
 *		@param [OUT] iRet - extraction return code
 *		@return - true of extraction was ok
 */
_bool get_physical_config_field_from_object( JNIEnv * env, jobject object,
		jclass classID, char *enumItemName, PhysicalLayer *enumItemValue, int *iRet,
		const int iRetCode);

#endif /* PROTOCOL_WAVEPORT_ENUM_CONVERSSION_H_ */
