/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

#include "protocol_properties.h"

char g_loger_protocol_prop_file_name[1024];

void get_all_properties( const char* propertyString, ProtocolParams* params ) {
	if ( !(params->isSet & FLAG_SERIAL_DRIVER_ID ) ) {
		get_property_value( propertyString, WAVEPORT_PROTOCOL_SERIAL_DRIVER_ID, "", PT_STRING, params->serialDriverId );
		params->isSet |= (strlen(params->serialDriverId) > 0) ? FLAG_SERIAL_DRIVER_ID: 0;
	}

	get_property_value( propertyString, WAVEPORT_PROTOCOL_CONFIGURATION_TIMEOUT, "500", PT_UINT, &params->configurationTimeout );
	params->isSet |= FLAG_CONFIGURATION_TIMEOUT;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_CONFIGURATION_RETRY_NUMBER, "1", PT_UINT, &params->configurationRetryNumber );
	params->isSet |= FLAG_CONFIGURATION_RETRY_NUMBER;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_LISTENING_STATUS, "true", PT_BOOL, &params->listeningStatus );
	params->isSet |= FLAG_LISTENING_STATUS;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_WRITE_DEFAULT_RADIO_SETTINGS, "false", PT_BOOL, &params->writeDefaultRadioSettings );
	params->isSet |= FLAG_WRITE_DEFAULT_RADIO_SETTINGS;

	/******************************************* Initialize dynamic expert parameters ***********************************************/
	get_property_value( propertyString, WAVEPORT_PROTOCOL_RESPONSE_TIMEOUT, "2000", PT_UINT, &params->dynamicExpertParams.responseTimeout );
	params->dynamicExpertParams.isSet |= FLAG_RESPONSE_TIMEOUT;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_RADIO_CHANNEL, "0", PT_UINT, &params->dynamicExpertParams.transmissionRadioChannel );
	params->dynamicExpertParams.isSet |= FLAG_TRANSMISSION_RADIO_CHANNEL;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_WAKEUP_LENGTH, "1100", PT_UINT, &params->dynamicExpertParams.wakeupLength );
	params->dynamicExpertParams.isSet |= FLAG_WAKEUP_LENGTH;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_BROADCAST_SESSION_MAX_RESPONSES_NB, "20", PT_UINT, &params->dynamicExpertParams.broadcastSessionMaxResponsesNb );
	params->dynamicExpertParams.isSet |= FLAG_BROADCAST_SESSION_MAX_RESPONSES_NB;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_BROADCAST_SESSION_TIMEOUT, "6000", PT_UINT, &params->dynamicExpertParams.broadcastSessionTimeout );
	params->dynamicExpertParams.isSet |= FLAG_BROADCAST_SESSION_TIMEOUT;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_TRANSMISSION_POWER_LEVEL, "14", PT_FLOAT, &params->dynamicExpertParams.transmissionPowerLevel );
	params->dynamicExpertParams.isSet |= FLAG_TRANSMISSION_POWER_LEVEL;
	/*+++++++++++++++++++++++++++++++++++++++ Finish to initialize dynamic expert parameters +++++++++++++++++++++++++++++++++++++++*/


	/******************************************* Initialize static standard parameters ***********************************************/
	get_property_value( propertyString, WAVEPORT_PROTOCOL_RADIO_ACK_ACTIVATION, "true", PT_BOOL, &params->staticStandardParams.radioAckActivation );
	params->staticStandardParams.isSet |= FLAG_RADIO_ACK_ACTIVATION;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_SWITCHING_MODE_ON_RADIO_ADDRESS_ACTIVATION, "false", PT_BOOL, &params->staticStandardParams.switchingModeOnRadioAddressActivation );
	params->staticStandardParams.isSet |= FLAG_SWITCHING_MODE_ON_RADIO_ADDRESS_ACTIVATION;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_RECEPTION_GROUPID, "55", PT_UINT, &params->staticStandardParams.receptionGroupId );
	params->staticStandardParams.isSet |= FLAG_RECEPTION_GROUPID;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_ACTIVITY_PERIOD_ON_FIRST_SCAN, "1000", PT_UINT, &params->staticStandardParams.firstScanActivityPeriod );
	params->staticStandardParams.isSet |= FLAG_ACTIVITY_PERIOD_ON_FIRST_SCAN;
	/*+++++++++++++++++++++++++++++++++++++++ Finish to initialize static standard parameters +++++++++++++++++++++++++++++++++++++++*/


	/******************************************* Initialize static expert parameters ***********************************************/
	get_property_value( propertyString, WAVEPORT_PROTOCOL_PHYSICAL_CONFIG_ON_FIRST_SCAN, "A3", PT_HEX, &params->staticExpertParams.firstScanPhysicalConfig );
	params->staticExpertParams.isSet |= FLAG_PHYSICAL_CONFIG_ON_FIRST_SCAN;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_SECOND_SCAN_ENABLE_ACTIVATION, "false", PT_BOOL, &params->staticExpertParams.secondScanEnableActivation );
	params->staticExpertParams.isSet |= FLAG_SECOND_SCAN_ENABLE_ACTIVATION;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_RADIO_CRC_ACTIVATION, "false", PT_BOOL, &params->staticExpertParams.radioCRCActivation );
	params->staticExpertParams.isSet |= FLAG_RADIO_CRC_ACTIVATION;

	get_property_value( propertyString, WAVEPORT_PROTOCOL_PHYSICAL_CONFIGON_SECOND_SCAN, "B6", PT_HEX, &params->staticExpertParams.secondScanPhysicalConfig );
	params->staticExpertParams.isSet |= FLAG_PHYSICAL_CONFIG_ON_SECOND_SCAN;

	get_property_value( propertyString,
			WAVEPORT_PROTOCOL_LOG_PROP_FILE_NAME,
			"logger.properties",
			PT_STRING,
			g_loger_protocol_prop_file_name );
	params->staticExpertParams.isSet |= FLAG_PROTOCOL_LOG_PROP_FILE_NAME;
	/*+++++++++++++++++++++++++++++++++++++++ Finish to initialize static expert parameters +++++++++++++++++++++++++++++++++++++++*/
}
