/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

#ifndef PROTOCOL_PROPERTIES_H_
#define PROTOCOL_PROPERTIES_H_

#include <i_protocol_service.h>
#include <property_reader.h>
#include "protocol_common.h"

#define WAVEPORT_PROTOCOL_PROPERTIES_FILE             "cswaveportProtocol.properties"

#define WAVEPORT_PROTOCOL_SERIAL_DRIVER_ID                            "protocol.serialDriverId"
#define WAVEPORT_PROTOCOL_CONFIGURATION_TIMEOUT                       "waveportProtocol.configurationTimeout"
#define WAVEPORT_PROTOCOL_CONFIGURATION_RETRY_NUMBER                  "waveportProtocol.configurationRetryNumber"
#define WAVEPORT_PROTOCOL_LISTENING_STATUS                            "protocol.listeningStatus"
#define WAVEPORT_PROTOCOL_WRITE_DEFAULT_RADIO_SETTINGS                "waveportProtocol.writeDefaultRadioSettings"
#define WAVEPORT_PROTOCOL_RESPONSE_TIMEOUT                            "waveportProtocol.responseTimeout"
#define WAVEPORT_PROTOCOL_RADIO_CHANNEL                               "waveportProtocol.radioChannel"
#define WAVEPORT_PROTOCOL_WAKEUP_LENGTH                               "waveportProtocol.wakeupLength"
#define WAVEPORT_PROTOCOL_BROADCAST_SESSION_MAX_RESPONSES_NB          "waveportProtocol.broadcastSessionMaxResponsesNb"
#define WAVEPORT_PROTOCOL_BROADCAST_SESSION_TIMEOUT                   "waveportProtocol.broadcastSessionTimeout"
#define WAVEPORT_PROTOCOL_TRANSMISSION_POWER_LEVEL                    "waveportProtocol.transmissionPowerLevel"
#define WAVEPORT_PROTOCOL_RADIO_ACK_ACTIVATION                        "waveportProtocol.radioAckActivation"
#define WAVEPORT_PROTOCOL_FIRST_SCAN_ENABLE_ACTIVATION                "waveportProtocol.firstScanEnableActivation"
#define WAVEPORT_PROTOCOL_SWITCHING_MODE_ON_RADIO_ADDRESS_ACTIVATION  "waveportProtocol.switchingModeOnRadioAddressActivation"
#define WAVEPORT_PROTOCOL_ACTIVITY_PERIOD_ON_FIRST_SCAN               "waveportProtocol.activityPeriodOnFirstScan"
#define WAVEPORT_PROTOCOL_PHYSICAL_CONFIG_ON_FIRST_SCAN               "waveportProtocol.physicalConfigOnFirstScan"
#define WAVEPORT_PROTOCOL_RADIO_CHANNEL_ON_FIRST_SCAN                 "waveportProtocol.radioChannelOnFirstScan"
#define WAVEPORT_PROTOCOL_RECEPTION_GROUPID                           "waveportProtocol.receptionGroupId"
#define WAVEPORT_PROTOCOL_SECOND_SCAN_ENABLE_ACTIVATION               "waveportProtocol.secondScanEnableActivation"
#define WAVEPORT_PROTOCOL_COMPLEX_SCAN_ENABLE_ACTIVATION              "waveportProtocol.complexScanEnableActivation"
#define WAVEPORT_PROTOCOL_RADIO_CRC_ACTIVATION                        "waveportProtocol.radioCRCActivation"
#define WAVEPORT_PROTOCOL_CWUP_TRANSMISSION_ACTIVATION                "waveportProtocol.cwupTransmissionActivation"
#define WAVEPORT_PROTOCOL_CWUP_AUTOSELECTION_ACTIVATION               "waveportProtocol.cwupAutoSelectionActivation"
#define WAVEPORT_PROTOCOL_CWUP_REQUEST_ONLY_ACTIVATION                "waveportProtocol.cwupRequestOnlyActivation"
#define WAVEPORT_PROTOCOL_CWUP_ONLY_ACTIVATION                        "waveportProtocol.cwupOnlyActivation"
#define WAVEPORT_PROTOCOL_ACTIVITY_PERIOD_ON_SECOND_SCAN              "waveportProtocol.activityPeriodOnSecondScan"
#define WAVEPORT_PROTOCOL_PHYSICAL_CONFIGON_SECOND_SCAN               "waveportProtocol.physicalConfigOnSecondScan"
#define WAVEPORT_PROTOCOL_RADIO_CHANNEL_ON_SECOND_SCAN                "waveportProtocol.radioChannelOnSecondScan"
#define WAVEPORT_PROTOCOL_SENSITIVITY_REDUCTION_FACTOR_ON_FIRST_SCAN  "waveportProtocol.sensitivityReductionFactorOnFirstScan"
#define WAVEPORT_PROTOCOL_SENSITIVITY_REDUCTION_FACTOR_ON_SECOND_SCAN "waveportProtocol.sensitivityReductionFactorOnSecondScan"
#define WAVEPORT_PROTOCOL_NB_SCAN_ON                                  "waveportProtocol.nbScanOn"
#define WAVEPORT_PROTOCOL_NB_SCAN_OFF                                 "waveportProtocol.nbScanOff"
#define WAVEPORT_PROTOCOL_FREQUENCY_HOPPING_CHANNEL_TABLE             "waveportProtocol.frequencyHoppingChannelTable"
#define WAVEPORT_PROTOCOL_LOG_PROP_FILE_NAME					      "waveportProtocol.loggerFileNameProperties"


typedef struct tagProtocolParams {
	/* mask of all protocol params
	 * bit      bit     Parameter
	 * position number
	 * B1       0       serialDriverId
	 * B2       1       configurationTimeout
	 * B3       2       listeningStatus
	 * B4       3       writeDefaultRadioSettings
	 * ...  ....        RFU (reserved for future use)
	 * B64      7       RFU
	 * */
	uint64 isSet;

	/* waveport protocol general settings */

	// Id of serial driver
	char serialDriverId[MAX_PATH];
	// configuration timeout in ms (default 500ms)
	unsigned int configurationTimeout;

	// Number of retry to write a waveport configuration parameter (default 1)
	unsigned int configurationRetryNumber;

	// listener for spontaneous events status (default true)
	_bool listeningStatus;
	// write default radio settings at open parameter (default false)
	_bool writeDefaultRadioSettings;

	/* waveport protocol dynamic standard parameters */
	DynamicStandardParams dynamicStandardParams;

	/* waveport protocol pseudo dynamic expert parameters */
	DynamicExpertParams dynamicExpertParams;

	/* waveport protocol static standard parameters */
	StaticStandardParams staticStandardParams;

	/* waveport protocol static expert parameters */
	StaticExpertParams staticExpertParams;

} ProtocolParams;

extern char g_loger_protocol_prop_file_name[];

void get_all_properties( const char* propertyString, ProtocolParams* params );

#endif /* PROTOCOL_PROPERTIES_H_ */
