
/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

/*
 * waveportprotocol.c
 *
 * This file contain entry point function(s) for different platforms.
 * DllMain entry point for Windows OS
 * initialize_waveportprotocol - entry point for POSIX systems (UNIX, Linux, Solaris, Mac OS X)
 * finalize_waveportprotocol   - exit point for POSIX systems (UNIX, Linux, Solaris, Mac OS X)
 */

#include "protocol.h"
#include "loader_logger.h"
void* g_ModuleHandle = 0;

#ifdef  _WINDOWS

#include <Windows.h>

BOOL WINAPI DllMain (HANDLE hDll, DWORD dwReason, LPVOID lpReserved)
{
    switch (dwReason)
    {
        case DLL_PROCESS_ATTACH:
            // Code to run when the DLL is loaded
        	memset( &g_WaveProtocol, 0 , sizeof(g_WaveProtocol) );
        	g_ModuleHandle = hDll;
            break;

        case DLL_PROCESS_DETACH:
            // Code to run when the DLL is freed
        	if ( g_Logger.loggerHandle ) {
        		LOG_DEBUG( g_Logger, "WAVEPORT LOGGING FINISHED", 0);
        		free_logger( &g_Logger );
        	}
            break;

        case DLL_THREAD_ATTACH:
            // Code to run when a thread is created during the DLL's lifetime
            break;

        case DLL_THREAD_DETACH:
            // Code to run when a thread ends normally.
            break;
    }
    return TRUE;
}

#else

__attribute__((constructor))
static void initialize_waveportprotocol() {
	memset( &g_WaveProtocol, 0 , sizeof(g_WaveProtocol) );
}

__attribute__((destructor))
static void finalize_waveportprotocol() {
}

#endif//_WINDOWS
