/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

/** @file i_serial_driver_listener_types.h
 * @brief This interface represents listeners of a i_protocol_service implementation
 * To be notified of event process by a Protocol instance,
 * program has to implement this interface and to registered to the Protocol instance.
 *
 * Note: For a Protocol instance, only one listener could be registered
 */
#ifndef I_PROTOCOL_LISTENER_TYPES_H_
#define I_PROTOCOL_LISTENER_TYPES_H_

#include <protocol_response_types.h>

/**
 * Called method when a RadioExchangeCommand is received
 * on a Protocol instance.
 * @param param A user define data setted during the registration
 * @param receiveCommand The received RadioExchangeCommand
 *
 * Note: For a SerialDriver instance, only one listener could be registered
 * If many listener are needed, serial_driver_spy_t interface must be used.
 */
typedef int   (* protocol_listener_t)         ( void* param, const RadioExchangeCommand* receiveCommand );

/**
 * Called method when a protocol event occurs
 * on a Protocol instance.
 * @param param A user define data setted during the registration
 * @param event The occured protocol_event
 *        IGNORED_COMMAND_RECEIVED, SERIALDRIVER_DISCONNECTION, etc.
 */
typedef int   (* protocol_listener_event_t)   ( void* param, int eventCode );

/**
 * Called method when a valid radio command has been received or
 * has been sent successfully
 * via a Protocol instance.
 * @param param A user define data setted during the registration
 * @param spyIndex The index of the Spy listener
 * @param frame The sent/received RadioExchangeCommand
 *
 * Note: These interface is not supposed to be used in a production context,
 * but is useful for Debug context.
 */
typedef int   (* protocol_spy_listener_t)     ( void* param, int spyIndex, const RadioExchangeCommand* command );


#endif /* I_PROTOCOL_LISTENER_TYPES_H_ */
