/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

/** @file i_serial_driver_export_types.h
 * @brief This interface define the type that must be used
 * to create a dynamic instance of a SerialDriver implementation
 * To use a serial driver implementation in accordance with the API.
 * For example, the Coronis domain module use this definition
 * inside the SerialDriver loading feature.
 */
#ifndef I_SERIAL_DRIVER_EXPORT_TYPES_H_
#define I_SERIAL_DRIVER_EXPORT_TYPES_H_

#include "i_serial_frame.h"

// Name of method that must be called.
#define FN_DRIVER_CREATE                       "driver_create"
#define FN_DRIVER_INIT                         "driver_init"
#define FN_DRIVER_INIT_FROM_FILE               "driver_init_from_file"
#define FN_DRIVER_DESTROY                      "driver_destroy"
#define FN_DRIVER_OPEN                         "driver_open"
#define FN_DRIVER_CLOSE                        "driver_close"
#define FN_DRIVER_IS_OPEN                      "driver_is_open"
#define FN_DRIVER_GET_PORT_ID                  "driver_get_port_id"
#define FN_DRIVER_SET_PORT_ID                  "driver_set_port_id"
#define FN_DRIVER_SET_PORT_PARAMS              "driver_set_port_params"
#define FN_DRIVER_SET_LISTENING_PERIOD_PARAM   "driver_set_listening_period_param"
#define FN_DRIVER_SET_FLOW_CONTROL_MODE_PARAMS "driver_set_flow_control_mode_params"
#define FN_DRIVER_SEND                         "driver_send"
#define FN_DRIVER_ADD_LISTENER            	   "driver_add_listener"
#define FN_DRIVER_REMOVE_LISTENER              "driver_remove_listener"
#define FN_DRIVER_ADD_SPY_LISTENER             "driver_add_spy_listener"
#define FN_DRIVER_REMOVE_SPY_LISTENER          "driver_remove_spy_listener"
#define FN_DRIVER_GET_VERSION      			   "driver_get_version"
#define FN_DRIVER_GET_SERIAL_DRIVER_INSTANCE   "driver_get_serial_driver_instance"



#ifdef __cplusplus
extern "C" {  
#endif

// Function type definition for service offers by the SerialDriver API.
typedef void* (* driver_create_t)                       ( );
typedef int   (* driver_destroy_t)                      ( const void* driver );
typedef int   (* driver_init_t)                         ( const void* driver );
typedef int   (* driver_init_from_file_t)               ( const void* driver, const char* file );
typedef int   (* driver_open_t)                         ( const void* driver );
typedef int   (* driver_close_t)                        ( const void* driver );
typedef int   (* driver_is_open_t)                      ( const void* driver );
typedef int   (* driver_get_port_id_t)                  ( const void* driver, char* id, int idLen );
typedef int   (* driver_set_port_id_t)                  ( const void* driver, const char* id );
typedef int   (* driver_set_port_params_t)              ( const void* driver, int baudRate, int dataBit, int stopBit, int parity );
typedef int   (* driver_set_listening_period_param_t)   ( const void* driver, int receiveTimeout );
typedef int   (* driver_set_flow_control_mode_params_t) ( const void* driver, int flowControlIn, int flowControlOut );
typedef int   (* driver_send_t)                         ( const void* driver, const pSerialExchangeData frame);
typedef int	  (* driver_add_listener_t)                 ( const void* driver, int (*receiveFrame)( void* param, const pSerialExchangeData frame ), int (*notifyListener)( void* param, int notifyCode ), void* param );
typedef int   (* driver_remove_listener_t)              ( const void* driver );
typedef int   (* driver_add_spy_listener_t)             ( const void* driver, int (*spySend)( void* param, int spyIndex, const pSerialExchangeData frame ), int (*spyRecv)( void* param, int spyIndex, const pSerialExchangeData frame ), void* param, int* spyIndex );
typedef int   (* driver_remove_spy_listener_t)          ( const void* driver, int spyIndex );
typedef void* (* driver_get_serial_driver_instance_t)   ( const void* driver );
typedef int   (* driver_get_version_t)                  ( );


#ifdef __cplusplus
}
#endif

#endif /* I_SERIAL_DRIVER_EXPORT_TYPES_H_ */
