/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

/**
 * @file serial_driver_types.h
 * @brief Declaration of serial driver types that are used for
 * Coronis implementation of SerialDriver implementation
 */
#ifndef _SERIAL_DRIVER_TYPES_H_
#define _SERIAL_DRIVER_TYPES_H_

#include <generic_types.h>

#define CORONIS_SDK_SERIAL_DRIVER_CONSTANTS_V_1_0

/**
 * Maximum length used for Coronis SerialDriver implementations (RS232Driver,...)
 * for buffering the serial frame information
 * 256 byte for data storage + 4 for length
 */
#define SERIALFRAME_BUFFER_LEN  260
/**
 * Maximum length of frame that can be send over
 * Coronis SerialDriver implementations (RS232Driver,...)
 */
#define  MAX_SERIALFRAME_LEN  (SERIALFRAME_BUFFER_LEN - 4)

/**
 * Structure used to store serial frame information
 * for Coronis SerialDriver implementations (RS232Driver,...) usage.
 * 		frame: data that contains the frame in byte[] format
 * 		len: length of the data that contains in the frame value
 * 		 - 4 is to make room for frame length (in this concrete implementation)
 */
typedef struct tagISerialFrame {
	byte         frame[MAX_SERIALFRAME_LEN];
	unsigned int len;
} SerialFrame;

#endif //_SERIAL_DRIVER_TYPES_H_
