/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

/**
 * @file serial_driver_types_utils.h
 * @brief Declaration of serialdriver types utility methods to manipulate
 * Coronis implementation of SerialDriver types
 */
#ifndef _SERIAL_DRIVER_TYPES_UTILS_H_
#define _SERIAL_DRIVER_TYPES_UTILS_H_

#include "serial_driver_types.h"
#include <i_serial_frame.h>
#include <serial_driver_status.h>

/**
 * 	Copy a serial frame from a concrete SerialFrame structure to a generic type serial frame.
 * @param frame
 * @return the new SerialExchangeData that contains a generic type serial frame.
 */
SerialExchangeData create_serial_exchange_data( SerialFrame frame );

/**
 * 	Copy an hexadecimal string to a generic type serial frame, using the concrete type SerialFrame.
 * @param frame
 * @return the new SerialExchangeData that contains a generic type serial frame.
 */
SerialExchangeData create_serial_exchange_data_from_string( const char* hex_string );

/**
 *	Copy content of a concrete serial frame to a generic type serial frame.
 * @param dest - the generic type which is the destination
 * @param src - the concrete type SerialFrame which is source.
 */
void set_serial_frame_value(pISerialFrame dest, const SerialFrame *src);

/**
 * Function used to obtain hex string of Serial Frame.
 * @param sf - pointer to Serial Frame.
 * @return hex string of  given Serial Frame.
 */
const char* get_hex_serialframe(char *frameString, const int frameStringSize,
		const SerialFrame* serialFrame );

/**
 *	Convert serial frame from binary to hexadecimal string. The user mus reserve room for the output
 *	hex string. The function is boundary safe by using frameStringSize.
 * @param frameString - the conversion output buffer
 * @param frameStringSize - the maximal allowed output size (in bytes)
 * @param serialExchangeData - the binary SerialFrame.
 * @return a pointer to frameString
 */
const char* get_hex_serialexchangeddata(char *frameString, const int frameStringSize, const pSerialExchangeData serialExchangeData );

/**
 * Function used to obtain string name of SerialDriverStatus enumeration value.
 * @param ss - SerialDriverStatus enumeration value
 * @return string name of ss value or empty string if given value not in SerialDriverStatus enumeration range.
 */
const char* get_sending_status_name( SerialDriverStatus ss );

#endif //_SERIAL_DRIVER_TYPES_UTILS_H_
