/*
 * cs_utils.c
 *
 *  Created on: 30.10.2012
 *      Author: vmacari
 */
#include "protocol_types_utils.h"
#include "utils.h"

_bool is_node_address_valid(const char *node_address) {

	// param validity
	if (node_address == NULL)
		return _false;

	// node address length
	size_t len = strlen(node_address);
	if ((len <= 0) || (len > MAX_RADIOADDRESS_LEN) || ((len % 2) != 0))
		return _false;

	return is_hex(node_address);
}

void set_payload_value(pIPayload dest, const Payload *src) {
	if (!dest || !src) return;

	memcpy(dest, src, sizeof(Payload));
	((Payload *)&dest)->len = src->len;
}

const char* get_hex_payload(char* payloadString, const int payloadStringSize, const Payload* pl ) {
	//static char payloadString[MAX_SERIALFRAME_LEN*2+1] = "";

	if (!payloadString || payloadStringSize <= 0) {
		return payloadString;
	}

	payloadString[0] = 0;

	sd_buffer_to_hex(payloadString, pl->frame, pl->len, payloadStringSize); //sizeof(payloadString));
	return payloadString;
}

LocalNodeServiceCommand create_local_node_service_command(NodeServiceId serviceId, Payload payload) {
	LocalNodeServiceCommand command;
	memset( &command, 0, sizeof(command) );
	command.serviceId = serviceId;
	memcpy(command.command.payload, &payload, sizeof(Payload));
		((Payload*)&command.command.payload)->len = payload.len;
	return command;
}

RadioExchangeCommand create_radio_exchange_command(
		Payload payload,
		RadioAddress destinationAddress) {

	RadioExchangeCommand command;
	memset( &command, 0, sizeof(command) );

	command.dynamicStandardParams.groupId = -1;
	memcpy(command.dynamicStandardParams.nodeAddress, &destinationAddress, sizeof(RadioAddress));
	memcpy(command.command.payload, &payload, sizeof(Payload));
	((Payload*)&command.command.payload)->len = payload.len;

	return command;
}

RadioExchangeCommand create_radio_exchange_command_with_relay(
		Payload payload,
		RadioAddress destinationAddress,
		RadioAddress* relayPath) {
	RadioExchangeCommand command;
	memset( &command, 0, sizeof(command) );
	command.dynamicStandardParams.groupId = -1;
	memcpy(command.dynamicStandardParams.nodeAddress, &destinationAddress, sizeof(RadioAddress));
	memcpy(command.dynamicStandardParams.routerPath, relayPath, sizeof(RadioAddress)*3);

	memcpy(command.command.payload, &payload, sizeof(Payload));
	((Payload*)&command.command.payload)->len = payload.len;

	return command;
}

const char* get_response_status_name( ResponseStatus rs ) {

	static char sendingStatus[100] = "";
	const char* sendingStatusCode = "";
	sendingStatus[0] = 0;

	switch( rs ) {
	case OK_PROTOCOL_SUCCESS:                     							sendingStatusCode = "OK_PROTOCOL_SUCCESS"; break;
	case KO_PAYLOAD_REQUIRED:                     							sendingStatusCode = "KO_PAYLOAD_REQUIRED"; break;
	case KO_NODE_ADDRESS_REQUIRED:                     						sendingStatusCode = "KO_NODE_ADDRESS_REQUIRED"; break;
	case KO_PROTOCOL_NOT_OPEN:                     							sendingStatusCode = "KO_PROTOCOL_NOT_OPEN"; break;// protocol is not opened
	case KO_PROTOCOL_ALREADY_OPEN:                     						sendingStatusCode = "KO_PROTOCOL_ALREADY_OPEN"; break;// protocol already open
	case KO_PROTOCOL_ALREADY_IN_USE:                     					sendingStatusCode = "KO_PROTOCOL_ALREADY_IN_USE"; break;
	case KO_PROTOCOL_WRITING_ERROR:                     					sendingStatusCode = "KO_PROTOCOL_WRITING_ERROR"; break;
	case KO_PROTOCOL_NO_VALID_PARAMETER:                     				sendingStatusCode = "KO_PROTOCOL_NO_VALID_PARAMETER"; break;// A one of command parameters is absent or wrong
	case KO_PROTOCOL_CONFIGURATION_TIMEOUT:                     			sendingStatusCode = "KO_PROTOCOL_CONFIGURATION_TIMEOUT"; break;// Error wile waiting for frame, timeout expired
	case KO_PROTOCOL_UNEXPECTED_FRAME:                     					sendingStatusCode = "KO_PROTOCOL_UNEXPECTED_FRAME"; break;// An unexpected frame received
	case KO_PROTOCOL_PARAM_UPDATE_ERROR:                     				sendingStatusCode = "KO_PROTOCOL_PARAM_UPDATE_ERROR"; break;// An error occurred while writing command ( CMD_STATUS_KO(01) received! )
	case KO_PROTOCOL_PARAM_READ_ERROR:                     					sendingStatusCode = "KO_PROTOCOL_PARAM_READ_ERROR"; break;// An error occurred while reading command ( CMD_STATUS_KO(01) received! )
	case KO_PROTOCOL_NODE_SERVICE_ID_REQUIRED:                     			sendingStatusCode = "KO_PROTOCOL_NODE_SERVICE_ID_REQUIRED"; break;
	case KO_PROTOCOL_NODE_SERVICE_NOT_SUPPORTED:                     		sendingStatusCode = "KO_PROTOCOL_NODE_SERVICE_NOT_SUPPORTED"; break;
	case KO_PROTOCOL_MAX_SPY_LISTENERS_EXCEEDED:                     		sendingStatusCode = "KO_PROTOCOL_MAX_SPY_LISTENERS_EXCEEDED"; break;
	case KO_PROTOCOL_SETTING_ERROR:                     					sendingStatusCode = "KO_PROTOCOL_SETTING_ERROR"; break;
	case KO_PROTOCOL_PROCESS_FAILURE:                     					sendingStatusCode = "KO_PROTOCOL_PROCESS_FAILURE"; break;
	case KO_SERIALDRIVER_SENDING_FAILURE:                     				sendingStatusCode = "KO_SERIALDRIVER_SENDING_FAILURE"; break;// An error occurred in serial driver while sending command
	case KO_SERIALDRIVER_NOT_AVAILABLE:                     				sendingStatusCode = "KO_SERIALDRIVER_NOT_AVAILABLE"; break;// An error occurred while try to connect to serial_port
	case KO_SERIALDRIVER_ALREADY_OPENED:                     				sendingStatusCode = "KO_SERIALDRIVER_ALREADY_OPENED"; break;// Serial Driver instance already open
	case KO_SERIALDRIVER_ID_NOT_EXIST:                     					sendingStatusCode = "KO_SERIALDRIVER_ID_NOT_EXIST"; break;//Serial Driver ID not exist
	case KO_SERIALDRIVER_PORT_NOT_OPENED:                     				sendingStatusCode = "KO_SERIALDRIVER_PORT_NOT_OPENED"; break;// Serial Port is not opened
	case KO_SERIALDRIVER_NOT_INITIALIZED:                     				sendingStatusCode = "KO_SERIALDRIVER_NOT_INITIALIZED"; break;// Serial Driver is not initialized
	case KO_PARAM_WRITING_FAILURE:                     						sendingStatusCode = "KO_PARAM_WRITING_FAILURE"; break;
	case KO_PARAMETER_SERIAL_DRIVER_ID_NOT_VALID:                     		sendingStatusCode = "KO_PARAMETER_SERIAL_DRIVER_ID_NOT_VALID"; break;// Serial Driver instance not loaded
	case KO_PARAMETER_NODE_ADDRESS_NOT_VALID:                     			sendingStatusCode = "KO_PARAMETER_NODE_ADDRESS_NOT_VALID"; break;
	case KO_PARAMETER_RELAY_ROUTE_NOT_VALID:                     			sendingStatusCode = "KO_PARAMETER_RELAY_ROUTE_NOT_VALID"; break;
	case KO_PARAMETER_GROUP_NUMBER_NOT_VALID:                     			sendingStatusCode = "KO_PARAMETER_GROUP_NUMBER_NOT_VALID"; break;
	case KO_PARAMETER_WAKEUP_LENGTH_NOT_VALID:                     			sendingStatusCode = "KO_PARAMETER_WAKEUP_LENGTH_NOT_VALID"; break;
	case KO_PARAMETER_RESPONSE_TIMEOUT_NOT_VALID:                     		sendingStatusCode = "KO_PARAMETER_RESPONSE_TIMEOUT_NOT_VALID"; break;
	case KO_PARAMETER_BROADCAST_SESSION_TIMEOUT_NOT_VALID:                  sendingStatusCode = "KO_PARAMETER_BROADCAST_SESSION_TIMEOUT_NOT_VALID"; break;
	case KO_PARAMETER_BROADCAST_SESSION_MAX_RESPONSES_NB_NOT_VALID:			sendingStatusCode = "KO_PARAMETER_BROADCAST_SESSION_MAX_RESPONSES_NB_NOT_VALID"; break;
	case KO_PARAMETER_TRANSMISSION_RADIO_CHANNEL_NOT_VALID:        			sendingStatusCode = "KO_PARAMETER_TRANSMISSION_RADIO_CHANNEL_NOT_VALID"; break;
	case KO_PARAMETER_TRANSMISSION_POWER_LEVEL_NOT_VALID:           		sendingStatusCode = "KO_PARAMETER_TRANSMISSION_POWER_LEVEL_NOT_VALID"; break;
	case KO_PARAMETER_RADIO_ACK_ACTIVATION_NOT_VALID:               		sendingStatusCode = "KO_PARAMETER_RADIO_ACK_ACTIVATION_NOT_VALID"; break;
	case KO_PARAMETER_FIRST_SCAN_ACTIVITY_PERIOD_NOT_VALID:                 sendingStatusCode = "KO_PARAMETER_FIRST_SCAN_ACTIVITY_PERIOD_NOT_VALID"; break;
	case KO_PARAMETER_SWITCHING_MODE_ON_RADIO_ADDRESS_ACTIVATION_NOT_VALID:	sendingStatusCode = "KO_PARAMETER_SWITCHING_MODE_ON_RADIO_ADDRESS_ACTIVATION_NOT_VALID"; break;
	case KO_PARAMETER_RECEPTION_GROUP_ID_NOT_VALID:                     	sendingStatusCode = "KO_PARAMETER_RECEPTION_GROUP_ID_NOT_VALID"; break;
	case KO_PARAMETER_FIRST_SCAN_ENABLE_ACTIVATION_NOT_VALID:               sendingStatusCode = "KO_PARAMETER_FIRST_SCAN_ENABLE_ACTIVATION_NOT_VALID"; break;
	case KO_PARAMETER_SECOND_SCAN_ENABLE_ACTIVATION_NOT_VALID:              sendingStatusCode = "KO_PARAMETER_SECOND_SCAN_ENABLE_ACTIVATION_NOT_VALID"; break;
	case KO_PARAMETER_RADIO_CRC_ACTIVATION_NOT_VALID:                     	sendingStatusCode = "KO_PARAMETER_RADIO_CRC_ACTIVATION_NOT_VALID"; break;
	case KO_PARAMETER_FIRST_SCAN_PHYSICAL_CONFIG_NOT_VALID:                 sendingStatusCode = "KO_PARAMETER_FIRST_SCAN_PHYSICAL_CONFIG_NOT_VALID"; break;
	case KO_PARAMETER_SECOND_SCAN_PHYSICAL_CONFIG_NOT_VALID:                sendingStatusCode = "KO_PARAMETER_SECOND_SCAN_PHYSICAL_CONFIG_NOT_VALID"; break;
	case KO_PARAMETER_NODE_ADDRESS_VALUE_NOT_SUPPORTED:                     sendingStatusCode = "KO_PARAMETER_NODE_ADDRESS_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_RELAY_ROUTE_VALUE_NOT_SUPPORTED:                     	sendingStatusCode = "KO_PARAMETER_RELAY_ROUTE_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_GROUP_NUMBER_VALUE_NOT_SUPPORTED:                     sendingStatusCode = "KO_PARAMETER_GROUP_NUMBER_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_WAKEUP_LENGTH_VALUE_NOT_SUPPORTED:                    sendingStatusCode = "KO_PARAMETER_WAKEUP_LENGTH_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_RESPONSE_TIMEOUT_VALUE_NOT_SUPPORTED:                 sendingStatusCode = "KO_PARAMETER_RESPONSE_TIMEOUT_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_BROADCAST_SESSION_TIMEOUT_VALUE_NOT_SUPPORTED:        sendingStatusCode = "KO_PARAMETER_BROADCAST_SESSION_TIMEOUT_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_BROADCAST_SESSION_MAX_RESPONSES_NB_VALUE_NOT_SUPPORTED:sendingStatusCode = "KO_PARAMETER_BROADCAST_SESSION_MAX_RESPONSES_NB_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_TRANSMISSION_RADIO_CHANNEL_VALUE_NOT_SUPPORTED:       sendingStatusCode = "KO_PARAMETER_TRANSMISSION_RADIO_CHANNEL_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_TRANSMISSION_POWER_LEVEL_VALUE_NOT_SUPPORTED:         sendingStatusCode = "KO_PARAMETER_TRANSMISSION_POWER_LEVEL_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_RADIO_ACK_ACTIVATION_VALUE_NOT_SUPPORTED:             sendingStatusCode = "KO_PARAMETER_RADIO_ACK_ACTIVATION_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_FIRST_SCAN_ACTIVITY_PERIOD_VALUE_NOT_SUPPORTED:       sendingStatusCode = "KO_PARAMETER_FIRST_SCAN_ACTIVITY_PERIOD_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_SWITCHING_MODE_ON_RADIO_ADDRESS_ACTIVATION_VALUE_NOT_SUPPORTED:                     sendingStatusCode = "KO_PARAMETER_SWITCHING_MODE_ON_RADIO_ADDRESS_ACTIVATION_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_RECEPTION_GROUP_ID_VALUE_NOT_SUPPORTED:               sendingStatusCode = "KO_PARAMETER_RECEPTION_GROUP_ID_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_FIRST_SCAN_ENABLE_ACTIVATION_VALUE_NOT_SUPPORTED:     sendingStatusCode = "KO_PARAMETER_FIRST_SCAN_ENABLE_ACTIVATION_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_SECOND_SCAN_ENABLE_ACTIVATION_VALUE_NOT_SUPPORTED:    sendingStatusCode = "KO_PARAMETER_SECOND_SCAN_ENABLE_ACTIVATION_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_RADIO_CRC_ACTIVATION_VALUE_NOT_SUPPORTED:             sendingStatusCode = "KO_PARAMETER_RADIO_CRC_ACTIVATION_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_FIRST_SCAN_PHYSICAL_CONFIG_VALUE_NOT_SUPPORTED:       sendingStatusCode = "KO_PARAMETER_FIRST_SCAN_PHYSICAL_CONFIG_VALUE_NOT_SUPPORTED"; break;
	case KO_PARAMETER_SECOND_SCAN_PHYSICAL_CONFIG_VALUE_NOT_SUPPORTED:      sendingStatusCode = "KO_PARAMETER_SECOND_SCAN_PHYSICAL_CONFIG_VALUE_NOT_SUPPORTED"; break;
	case KO_RADIO_TRANSMISSION_FAILURE:                     				sendingStatusCode = "KO_RADIO_TRANSMISSION_FAILURE"; break;
	case KO_RADIO_TRANSMISSION_FAILURE_WITH_DESTINATION_NODE:               sendingStatusCode = "KO_RADIO_TRANSMISSION_FAILURE_WITH_DESTINATION_NODE"; break;
	case KO_RADIO_TRANSMISSION_FAILURE_WITH_FIRST_RELAY:                    sendingStatusCode = "KO_RADIO_TRANSMISSION_FAILURE_WITH_FIRST_RELAY"; break;
	case KO_RADIO_TRANSMISSION_FAILURE_WITH_SECOND_RELAY:                   sendingStatusCode = "KO_RADIO_TRANSMISSION_FAILURE_WITH_SECOND_RELAY"; break;
	case KO_RADIO_TRANSMISSION_FAILURE_WITH_THIRD_RELAY:                    sendingStatusCode = "KO_RADIO_TRANSMISSION_FAILURE_WITH_THIRD_RELAY"; break;
	case KO_PROTOCOL_NODE_LOCAL_SERVICE_FAILURE:                     		sendingStatusCode = "KO_PROTOCOL_NODE_LOCAL_SERVICE_FAILURE"; break;
	case KO_PROTOCOL_NO_RESPONSE_SESSION:                     				sendingStatusCode = "KO_PROTOCOL_NO_RESPONSE_SESSION"; break;
	case KO_PAYLOAD_NOT_VALID: 												sendingStatusCode = "KO_PAYLOAD_NOT_VALID"; break;
	case KO_NODE_ADDRESS_NOT_VALID:                     					sendingStatusCode = "KO_NODE_ADDRESS_NOT_VALID"; break;
	case KO_UNEXPECTED_ERROR:												sendingStatusCode = "KO_UNEXPECTED_ERROR"; break;
	default:																sendingStatusCode = "KO_UNEXPECTED_ERROR"; break;
	}

	// case KO_SERIALDRIVER_ALREADY_LOADED: fieldIDName = "KO_SERIALDRIVER_ALREADY_LOADED"; break;
	// case KO_PROTOCOL_NOT_INITIALIZED: fieldIDName = "KO_PROTOCOL_NOT_INITIALIZED"; break;
	// case KO_PARAMETER_RECEPTION_MODE_NOT_VALID: fieldIDName = "KO_PARAMETER_RECEPTION_MODE_NOT_VALID"; break;

	strcpy( sendingStatus, sendingStatusCode );
	return sendingStatus;
}

const char* get_local_node_service_name( NodeServiceId nsid ) {
	static char nodeServiceName[100] = "";
	const char* nodeServiceNameTmp = "";
	nodeServiceName[0] = 0;
	switch( nsid ) {
	case NODE_SERVICE_ID_UNDEFINED:		nodeServiceNameTmp = "NODE_SERVICE_ID_UNDEFINED"; break;
	case STACK_SERVICE:				    nodeServiceNameTmp = "STACK_SERVICE"; break;
	case SDP_SERVICE:					nodeServiceNameTmp = "SDP_SERVICE"; break;
	case FIRMWARE_UPGRADE_SERVICE:		nodeServiceNameTmp = "FIRMWARE_UPGRADE_SERVICE"; break;
	}
	strcpy( nodeServiceName, nodeServiceNameTmp );
	return nodeServiceName;
}

