/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

#include <serial_comm_port_identifier.h>

#ifdef _WINDOWS

/****************************************************************************************************/
void driver_get_serial_port_ids( char* ids, int idsLen ) {
	memset(ids, 0, idsLen);

	//Get the function pointers to "SetupDiGetClassDevs", "SetupDiGetClassDevs", "SetupDiEnumDeviceInfo", "SetupDiOpenDevRegKey"
	//and "SetupDiDestroyDeviceInfoList" in setupapi.dll
	HINSTANCE hSetupAPI = LoadLibraryA( "SETUPAPI.DLL" );
	if (hSetupAPI == NULL)
		return;

	typedef HKEY (__stdcall SETUPDIOPENDEVREGKEY)(HDEVINFO, PSP_DEVINFO_DATA, DWORD, DWORD, DWORD, REGSAM);
	typedef BOOL (__stdcall SETUPDICLASSGUIDSFROMNAME)(LPCSTR, LPGUID, DWORD, PDWORD);
	typedef BOOL (__stdcall SETUPDIDESTROYDEVICEINFOLIST)(HDEVINFO);
	typedef BOOL (__stdcall SETUPDIENUMDEVICEINFO)(HDEVINFO, DWORD, PSP_DEVINFO_DATA);
	typedef HDEVINFO (__stdcall SETUPDIGETCLASSDEVS)(LPGUID, LPCSTR, HWND, DWORD);
	typedef BOOL (__stdcall SETUPDIGETDEVICEREGISTRYPROPERTY)(HDEVINFO, PSP_DEVINFO_DATA, DWORD, PDWORD, PBYTE, DWORD, PDWORD);


	SETUPDIOPENDEVREGKEY* lpfnLPSETUPDIOPENDEVREGKEY = (SETUPDIOPENDEVREGKEY*)(GetProcAddress(hSetupAPI, "SetupDiOpenDevRegKey"));
	SETUPDICLASSGUIDSFROMNAME* lpfnSETUPDICLASSGUIDSFROMNAME = (SETUPDICLASSGUIDSFROMNAME*)(GetProcAddress(hSetupAPI, "SetupDiClassGuidsFromNameA"));
	SETUPDIGETCLASSDEVS* lpfnSETUPDIGETCLASSDEVS = (SETUPDIGETCLASSDEVS*)(GetProcAddress(hSetupAPI, "SetupDiGetClassDevsA"));
	SETUPDIGETDEVICEREGISTRYPROPERTY* lpfnSETUPDIGETDEVICEREGISTRYPROPERTY = (SETUPDIGETDEVICEREGISTRYPROPERTY*)(GetProcAddress(hSetupAPI, "SetupDiGetDeviceRegistryPropertyA"));
	SETUPDIDESTROYDEVICEINFOLIST* lpfnSETUPDIDESTROYDEVICEINFOLIST = (SETUPDIDESTROYDEVICEINFOLIST*)(GetProcAddress(hSetupAPI, "SetupDiDestroyDeviceInfoList"));
	SETUPDIENUMDEVICEINFO* lpfnSETUPDIENUMDEVICEINFO = (SETUPDIENUMDEVICEINFO*)(GetProcAddress(hSetupAPI, "SetupDiEnumDeviceInfo"));

	if ((lpfnLPSETUPDIOPENDEVREGKEY == NULL) || (lpfnSETUPDICLASSGUIDSFROMNAME == NULL) || (lpfnSETUPDIDESTROYDEVICEINFOLIST == NULL) ||
		(lpfnSETUPDIENUMDEVICEINFO == NULL) || (lpfnSETUPDIGETCLASSDEVS == NULL) || (lpfnSETUPDIGETDEVICEREGISTRYPROPERTY == NULL))
	{
		//Unload the setup dll
		FreeLibrary(hSetupAPI);
		return ;
	}

	//First need to convert the name "Ports" to a GUID using SetupDiClassGuidsFromName
	DWORD dwGuids = 0;
	lpfnSETUPDICLASSGUIDSFROMNAME( "Ports", NULL, 0, &dwGuids);
	if (dwGuids == 0)
	{
		//Unload the setup dll
		FreeLibrary(hSetupAPI);
		return ;
	}

	GUID Guids[256];
	//Call the function again
	if (!lpfnSETUPDICLASSGUIDSFROMNAME( "Ports", Guids, dwGuids, &dwGuids))
	{
		//Unload the setup dll
		FreeLibrary(hSetupAPI);
		return ;
	}

	//Now create a "device information set" which is required to enumerate all the ports
	HDEVINFO hDevInfoSet = lpfnSETUPDIGETCLASSDEVS( Guids, NULL, NULL, DIGCF_PRESENT);
	if (hDevInfoSet == INVALID_HANDLE_VALUE)
	{
		//Unload the setup dll
		FreeLibrary(hSetupAPI);
		return ;
	}

	//Finally do the enumeration
	BOOL bMoreItems = TRUE;
	int nIndex = 0;
	int nOffset = 0;
	SP_DEVINFO_DATA devInfo;
	while (bMoreItems)
	{
		//Enumerate the current device
		devInfo.cbSize = sizeof(SP_DEVINFO_DATA);
		bMoreItems = lpfnSETUPDIENUMDEVICEINFO(hDevInfoSet, nIndex, &devInfo);
		if (bMoreItems)
		{
			//Get the registry key which stores the ports settings
			HKEY hDeviceKey = lpfnLPSETUPDIOPENDEVREGKEY(hDevInfoSet, &devInfo, DICS_FLAG_GLOBAL, 0, DIREG_DEV, KEY_QUERY_VALUE);
			if (hDeviceKey)
			{
				//Read in the name of the port
				char szPortName[256] = {0};
				DWORD dwSize = sizeof(szPortName);
				DWORD dwType = 0;
				if ((RegQueryValueExA(hDeviceKey, "PortName", NULL, &dwType, (LPBYTE)szPortName, &dwSize) == ERROR_SUCCESS) && (dwType == REG_SZ))
				{
					//If it looks like "COMX" then
					//add it to the array which will be returned
					size_t nLen = strlen(szPortName);
					if (nLen > 3)
					{
						if ( strncmp( szPortName, "COM", 3) == 0 )
						{
							if ( (nOffset + (strlen(szPortName)+1)) < idsLen ) {
								strcpy( ids + nOffset, szPortName );
								nOffset += strlen(szPortName)+1;
								ids[nOffset] = 0;
							}
						}
					}
				}

				//Close the key now that we are finished with it
				RegCloseKey(hDeviceKey);
			}
			++nIndex;
		}
	}

	//Free up the "device information set" now that we are finished with it
	lpfnSETUPDIDESTROYDEVICEINFOLIST(hDevInfoSet);

	//Unload the setup dll
	FreeLibrary(hSetupAPI);
	return ;
}
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#else

/****************************************************************************************************/
void driver_get_serial_port_ids( char* ids, int idsLen ) {
	int nOffset = 0;
	memset(ids, 0, idsLen);
	DIR *dp;
	struct dirent *dirp;
	if ( (dp  = opendir( "/dev" )) != 0) {
		while ( (dirp = readdir(dp)) != 0) {
	        if ( strncmp( dirp->d_name, "tty", 3 ) == 0 ) {
	        	char id[20] = "";
	        	sprintf( id, "/dev/%s", dirp->d_name);
	        	int fd = open( id, O_RDWR | O_NOCTTY | O_NDELAY);
	        	if (fd != -1)  {
	        		struct serial_struct serinfo;

	        		if ( ioctl (fd, TIOCGSERIAL, &serinfo) >= 0 ) {
	        			if ( (nOffset + (strlen(id)+1)) < idsLen ) {
	        				strcpy( ids + nOffset, id );
	        				nOffset += strlen(id)+1;
	        			}
	        		}

	        		close(fd);
	        	}
	        }
	    }
		closedir(dp);
	}

}
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#endif//_WINDOWS
