/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/

#include "serial_properties.h"
#include "loader_logger.h"
void* g_ModuleHandle = 0;

#ifdef  _WINDOWS

#include <Windows.h>

BOOL WINAPI DllMain (HANDLE hDll, DWORD dwReason, LPVOID lpReserved)
{
    switch (dwReason)
    {
        case DLL_PROCESS_ATTACH:
            // Code to run when the DLL is loaded
        	memset( &drivers, 0, sizeof(drivers) );
        	memset( &g_Logger, 0, sizeof(g_Logger) );
        	g_ModuleHandle = hDll;
            break;

        case DLL_PROCESS_DETACH:
            // Code to run when the DLL is freed
        	if ( g_Logger.loggerHandle ) {
        		LOG_DEBUG( g_Logger, "RS232DRIVER LOGGING FINISHED", 0);
        		free_logger( &g_Logger );
        	}
            break;

        case DLL_THREAD_ATTACH:
            // Code to run when a thread is created during the DLL's lifetime
            break;

        case DLL_THREAD_DETACH:
            // Code to run when a thread ends normally.
            break;
    }
    return TRUE;
}

#else

__attribute__((constructor))
static void initialize_rs232library() {
	memset( &drivers, 0, sizeof(drivers) );
	memset( &g_Logger, 0, sizeof(g_Logger) );
}

__attribute__((destructor))
static void finalize_rs232library() {
	if ( g_Logger.loggerHandle ) {
		LOG_DEBUG( g_Logger, "RS232DRIVER LOGGING FINISHED", 0);
		free_logger( &g_Logger );
	}
	memset( &g_Logger, 0, sizeof(g_Logger) );
}

#endif //_WINDOWS
