/*******************************************************************************
 * Copyright (c) 2012 Coronis
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contributors:
 *     Coronis - initial API and implementation
 *******************************************************************************/
#include "serial_properties.h"

char g_loger_rs232_properties_file_name[1024];

void get_all_properties( const char* propertyString, SerialParams* params ) {
	if ( !(params->isSet & FLAG_SERIAL_PORT_ID ) ) {
		get_property_value( propertyString, SERIAL_DRIVER_PORT_ID, "", PT_STRING, params->id );
		params->isSet |= (strlen(params->id) > 0) ? FLAG_SERIAL_PORT_ID: 0;
	}
	if ( !(params->isSet & FLAG_BAUD_RATE ) ) {
		get_property_value( propertyString, SERIAL_DRIVER_BAUD_RATE, "9600", PT_UINT, &params->baudRate );
		params->isSet |= FLAG_BAUD_RATE;
	}
	if ( !(params->isSet & FLAG_DATA_BIT ) ) {
		get_property_value( propertyString, SERIAL_DRIVER_DATA_BITS, "8", PT_UINT, &params->dataBit );
		params->isSet |= FLAG_DATA_BIT;
	}
	if ( !(params->isSet & FLAG_STOP_BIT ) ) {
		get_property_value( propertyString, SERIAL_DRIVER_STOP_BITS, "1", PT_UINT, &params->stopBit );
		switch (params->stopBit) {
		case 2:
			params->stopBit = TWOSTOPBITS;
			break;
		case 1:
		default:
			params->stopBit = ONESTOPBIT;
			break;
		}
		params->isSet |= FLAG_STOP_BIT;
	}
	if ( !(params->isSet & FLAG_PARITY ) ) {
		get_property_value( propertyString, SERIAL_DRIVER_PARITY, "0", PT_UINT, &params->parity );
		params->isSet |= FLAG_PARITY;
	}
	if ( !(params->isSet & FLAG_LISTENING_PERIOD ) ) {
		get_property_value( propertyString, SERIAL_DRIVER_LISTENING_PERIOD, "20", PT_UINT, &params->listeningPeriod );
		params->isSet |= FLAG_LISTENING_PERIOD;
	}
	if ( !(params->isSet & FLAG_FLOWCONTROL_MODE ) ) {
		get_property_value( propertyString, SERIAL_DRIVER_FLOW_CONTROL_MODE, "0", PT_UINT, &params->flowControlMode );
		params->isSet |= FLAG_FLOWCONTROL_MODE;
	}
	if ( !(params->isSet & FLAG_SERIAL_LISTENING_STATUS ) ) {
		get_property_value( propertyString, SERIAL_DRIVER_LISTENING_STATUS, "true", PT_BOOL, &params->listeningStatus );
		params->isSet |= FLAG_SERIAL_LISTENING_STATUS;
	}

	get_property_value( propertyString, SERIAL_DRIVER_RADIO_BOARD_ACK_STATUS, "true", PT_BOOL, &params->radioBoardAckStatus );
	params->isSet |= FLAG_RADIOBOARD_ACK_STATUS;

	get_property_value( propertyString, SERIAL_DRIVER_RADIO_BOARD_ACK_TIMEOUT, "500", PT_UINT, &params->radioAckTimeout );
	params->isSet |= FLAG_RADIO_ACK_TIMEOUT;

	get_property_value( propertyString, SERIAL_DRIVER_HOST_ACK_STATUS, "true", PT_BOOL, &params->hostAckStatus );
	params->isSet |= FLAG_HOST_ACK_STATUS;

	get_property_value( propertyString, SERIAL_DRIVER_RETRY_NUMBER, "2", PT_UINT, &params->retryCountDef );
	params->isSet |= FLAG_RETRY_COUNT_DEF;

	get_property_value( propertyString, SERIAL_DRIVER_SYNC_CHARACTER_ACTIVATION, "true", PT_BOOL, &params->syncCharacterActivation );
	params->isSet |= FLAG_SYNC_CHARACTER_ACTIVATION;

	get_property_value( propertyString, SERIAL_DRIVER_START_END_CHARACTERS_ACTIVATION, "true", PT_BOOL, &params->startEndCharactersActivation );
	params->isSet |= FLAG_START_END_CHARACTERS_ACTIVATION;

	get_property_value( propertyString, SERIAL_DRIVER_CRC16_ACTIVATION, "true", PT_BOOL, &params->crc16Activation );
	params->isSet |= FLAG_CRC16_ACTIVATION;

	get_property_value( propertyString, SERIAL_DRIVER_LENGTH_CHARACTER_ACTIVATION, "true", PT_BOOL, &params->lengthActivation );
	params->isSet |= FLAG_LENGTH_ACTIVATION;

	get_property_value( propertyString, SERIAL_DRIVER_LOG_PROPERTIES_FILE_NAME, "logger.properties", PT_STRING, g_loger_rs232_properties_file_name );
	params->isSet |= FLAG_LOGGER_PROP_FILE_NAME;
}

DriverInfo* get_driver( const void* driver ) {
	int index = ((int)driver)-1;
	if ( ( index >= 0 ) && ( index < MAX_DRIVER_NUM ) && ( drivers[index].integrity == SERIAL_DRIVER_INTEGRITY ) )
		return &drivers[index];
	return NULL;
}

int copy_frame_body( const DriverInfo* driver, const SerialFrame* frameSrc, SerialFrame* frameDest ) {
	int iRet = KO_SERIALFRAME_REQUIRED;

	if ( driver && frameSrc && frameSrc->len > 0 && frameDest ) {

		int len = driver->params.syncCharacterActivation ? 1 : 0; // + SYNC byte
		len += driver->params.startEndCharactersActivation ? 2 : 0; // + STX & ETX bytes
		int i = -1;

		// verify if SYNC byte is ok
		_bool frameOK = driver->params.syncCharacterActivation ? frameSrc->frame[++i] == SYNC : _true;
		if ( frameOK ) {
			// verify if STX byte is ok
			frameOK = driver->params.startEndCharactersActivation ? frameSrc->frame[++i] == STX : _true;
			if ( frameOK ) {
				// verify if len byte is ok
				frameOK = driver->params.lengthActivation ? frameSrc->frame[++i] == (frameSrc->len - len) : _true;
				if ( frameOK ) {
					// verify if ETX byte is ok
					frameOK = driver->params.startEndCharactersActivation ? frameSrc->frame[frameSrc->len-1] == ETX : _true;
					if ( frameOK && driver->params.crc16Activation ) {
						// verify if CRC bytes are ok
						unsigned int crc = calc_crc( frameSrc->frame + i, (frameSrc->len - len) - 2 );
						frameOK = frameSrc->frame[i + (frameSrc->len - len) - 2 ] == (crc & 0x00FF);
						if ( frameOK )
							frameOK = frameSrc->frame[i + (frameSrc->len - len) - 1] == ((crc & 0xFF00) >> 8);
					}
				}
			}
		}

		if ( frameOK ) {
			//int j = 1; // + length 1 byte
			int j = driver->params.lengthActivation ? 1 : 0;
			j += driver->params.syncCharacterActivation ? 1 : 0;
			j += driver->params.startEndCharactersActivation ? 1 : 0;
			frameDest->len = frameSrc->len - ( j + (driver->params.crc16Activation ? 2 : 0) + (driver->params.startEndCharactersActivation ? 1 : 0));
			if ( frameDest->len > 0 ) {
				memcpy( frameDest->frame, frameSrc->frame + j, frameDest->len );
				iRet = OK_SERIALDRIVER_SUCCESS;
			}
		}
	}
	return iRet;
}
